@@ -30,6 +30,8 @@ NPROC=$(nproc)
 BUILD_RPM=0
 BUILD_DEB=0
 ID=${ID:-"unknown"}
+DISABLE_DOCA_TELEMETRY=0
+DOCA_MODE=
 
 DPDK_SRC_ARG=""
 DPDK_INSTALL_ARG=""
@@ -172,6 +174,10 @@ function execute() {
     fi
 }
 
+function is_bluefield_system() {
+    lspci -s 00:00.0 | grep -qi 'bluefield'
+}
+
 function check_meson_if_rebuild_needed() {
     local src_dir=${1:?missing arg}
     local install_dir=${2:?missing arg}
@@ -373,6 +379,9 @@ function rebuild_doca() {
     fi
 
     if [ "$build" == 1 ]; then
+        if [ "$(uname -m)" = "aarch64" ] && [ "$DOCA_MODE" = "host" ]; then
+            DOCA_MESON_EXTRA+=" -Darm_host_support=true"
+        fi
         execute --into "$DOCA_OUT" \
                 meson setup -Dc_args="$DOCA_CFLAGS" \
                       -Dcpp_args="$DOCA_CFLAGS" \
@@ -385,6 +394,7 @@ function rebuild_doca() {
                       -Ddisable_symbol_hiding=true \
                       -Ddisable_system_tests=true \
                       -Denable_grpc_support=false \
+                      -Denable_gpu_support=false \
                       -Ddisable_tool_flow_tune=true \
                       -Dverification_disable_testsuit=true \
                       $DOCA_MESON_EXTRA --prefix="$DOCA_INSTALL_DIR" build
@@ -398,6 +408,7 @@ function rebuild_openvswitch() {
 
     [ "$ASAN" == 1 ] && OVS_CONFIGURE_EXTRA+=" --with-sanitizer"
     [ "$SPARSE" == 1 ] && OVS_CONFIGURE_EXTRA+=" --enable-sparse"
+    [ "$DISABLE_DOCA_TELEMETRY" == 1 ] && OVS_CONFIGURE_EXTRA+=" --disable-doca-telemetry"
 
     execute pushd "$OVS_SRC_DIR"
 
@@ -433,6 +444,7 @@ function rebuild_openvswitch() {
     if [ "$BUILD_RPM" == 1 ]; then
         RPMBUILD_OPT="--with static --with dpdk --with doca --with nvhws_static --without check"
         [ "$ASAN" == 1 ] && RPMBUILD_OPT+=" --with sanitizer"
+        [ "$DISABLE_DOCA_TELEMETRY" == 1 ] && RPMBUILD_OPT+=" --without doca-telemetry"
         if [ -n "$CFLAGS" ];  then
             local optflags=$(rpm --eval "%optflags" | sed -e 's/-D_FORTIFY_SOURCE=2 -Wp,//')
             optflags+=" $CFLAGS"
@@ -444,6 +456,7 @@ function rebuild_openvswitch() {
         EXTRA_CONFIGURE_OPTS="--with-nvhws=static"
         [ "$ASAN" == 1 ] && EXTRA_CONFIGURE_OPTS+=" --with-sanitizer"
         [ -n "$CFLAGS" ] && export DEB_CFLAGS_APPEND="$CFLAGS"
+        [ "$DISABLE_DOCA_TELEMETRY" == 1 ] && EXTRA_CONFIGURE_OPTS+=" --disable-doca-telemetry"
         execute make debian-deb DEB_BUILD_OPTIONS="$DEB_BUILD_OPTIONS" EXTRA_CONFIGURE_OPTS="$EXTRA_CONFIGURE_OPTS"
     else
         execute make -j$NPROC -s
@@ -469,7 +482,9 @@ libnuma-dev libpcap-dev dh-exec python3-all-dev python3-sortedcontainers
 build-essential devscripts fakeroot python3-netifaces libdbus-1-dev
 python3-sphinx python3-prometheus-client python3-enchant
 libcap-ng-dev libsystemd-dev dh-python sparse
-libarchive-dev libfdt-dev"
+collectx-clxapidev
+libarchive-dev libfdt-dev libdoca-sdk-dpdk-bridge-dev
+nettle-dev libacl1-dev libzstd-dev liblz4-dev libbz2-dev libxml2-dev"
         if [ "$ASAN" -eq 1 ]; then
             if [[ "$VERSION_ID" =~ "20" ]]; then
                 deps+=" libasan5 libubsan1"
@@ -479,16 +494,20 @@ libarchive-dev libfdt-dev"
                 deps+=" libasan8 libubsan1"
             fi
         fi
+        [ "$NV_HWS_SRC_DIR" ] && deps+=" dpacc"
     else
         deps="\
 librdmacm rdma-core-devel meson json-c-devel protobuf-compiler binutils-devel
 zlib-devel libcurl-devel autoconf automake openssl-devel libtool nvhws-devel
 unbound-devel unbound libpcap-devel libbsd-devel elfutils-libelf-devel
 libunwind-devel rpm-build numactl-devel libcap-ng-devel python3-enchant
-python3-sphinx python3-prometheus_client sparse"
+python3-sphinx python3-prometheus_client sparse bc systemd-devel
+collectx-clxapidev
+doca-sdk-dpdk-bridge-devel"
         if [ "$ASAN" -eq 1 ]; then
             deps+=" libasan libubsan"
         fi
+        [ "$NV_HWS_SRC_DIR" ] && deps+=" dpacc"
     fi
 
     echo $deps
@@ -649,6 +668,9 @@ Optional arguments:
     --build-deb         Build ovs deb.
     --info              Build with CFLAGS="$INFO_CFLAGS". Does not change build type.
     --debug             Build with CFLAGS="$DEBUG_CFLAGS". Does not change build type.
+    --doca-mode         Build DOCA in 'host' or 'dpu' mode ($DOCA_MODE)
+
+    --disable-doca-telemetry Disable DOCA Telemetry support and ignore CollectX.
 
     --buildtype ARG         Build type for DPDK, DOCA. [debug, release]. ($DPDK_BUILDTYPE)
                             If set to 'debug', also enables --debug.
@@ -774,6 +796,7 @@ function parse_args() {
             DPDK_BUILDTYPE=$2
             DOCA_BUILDTYPE=$2
             RDMA_CORE_BUILDTYPE=$2
+            NV_HWS_BUILDTYPE=$2
             shift 2
             ;;
 
@@ -847,6 +870,16 @@ function parse_args() {
             shift
             ;;
 
+        --disable-doca-telemetry)
+            DISABLE_DOCA_TELEMETRY=1
+            shift
+            ;;
+
+        --doca-mode)
+            DOCA_MODE="$2"
+            shift 2
+            ;;
+
         --)
             shift
             ;;
@@ -859,6 +892,14 @@ function parse_args() {
       esac
     done
 
+    if [ ! "$DOCA_MODE" ]; then
+        is_bluefield_system && DOCA_MODE=dpu || DOCA_MODE=host
+    fi
+
+    if [ "$DOCA_MODE" != "host" ] && [ "$DOCA_MODE" != "dpu" ]; then
+        err "Invalid DOCA mode: '$DOCA_MODE'. Only 'host' or 'dpu' supported."
+    fi
+
     if [ "$DPDK_BUILDTYPE" == "debug" ] ||
        [ "$DOCA_BUILDTYPE" == "debug" ] ||
        [ "$NV_HWS_BUILDTYPE" == "debug" ] ||
