@@ -10,7 +10,7 @@ rm -rf $compiler_dir
 mkdir -p $compiler_dir
 cd $compiler_dir
 
-compiler_list="clang:clang++:dev/clang-9.0.1 icc:icpc:intel/ics-18.0.4 icc:icpc:intel/ics-19.1.1 gcc:g++:dev/gcc-8.3.0 gcc:g++:dev/gcc-9.3.0 gcc:g++:dev/gcc-10.1.0"
+compiler_list=${COMPILER_SPEC:-"clang:clang++:dev/clang-9.0.1 icc:icpc:intel/ics-18.0.4 icc:icpc:intel/ics-19.1.1 gcc:g++:dev/gcc-8.3.0 gcc:g++:dev/gcc-9.3.0 gcc:g++:dev/gcc-10.1.0"}
 
 compiler_tap=${WORKSPACE}/${prefix}/compiler.tap
 echo "1..$(echo $compiler_list | tr " " "\n" | wc -l)" > $compiler_tap
