@@ -356,7 +356,11 @@ static int fuse_check_folio(struct folio *folio)
 {
 	if (folio_mapped(folio) ||
 	    folio->mapping != NULL ||
+#ifdef HAVE_FOLIO_FLAGS_F
+	    (folio->flags.f & PAGE_FLAGS_CHECK_AT_PREP &
+#else
 	    (folio->flags & PAGE_FLAGS_CHECK_AT_PREP &
+#endif
 	     ~(1 << PG_locked |
 	       1 << PG_referenced |
 	       1 << PG_lru |
