@@ -35,14 +35,11 @@
 #include <linux/errno.h>
 
 #define DRV_NAME	mlx4_ib
-#define DRV_VERSION	"25.10-1.7.1"
-#define DRV_RELDATE	"11 Dec 2025"
+#define DRV_VERSION	"26.01-1.0.0"
+#define DRV_RELDATE	"17 Feb 2026"
 
 MODULE_AUTHOR("Tzafrir Cohen");
 MODULE_LICENSE("Dual BSD/GPL");
-#ifdef RETPOLINE_MLNX
-MODULE_INFO(retpoline, "Y");
-#endif
 MODULE_VERSION(DRV_VERSION);
 
 #define dummy_module_code(name) \
