@@ -37,15 +37,12 @@
 
 #define DRV_NAME	"ib_srp"
 #define PFX		DRV_NAME ": "
-#define DRV_VERSION	"25.10-1.7.1"
-#define DRV_RELDATE	"11 Dec 2025"
+#define DRV_VERSION	"26.01-1.0.0"
+#define DRV_RELDATE	"17 Feb 2026"
 
 MODULE_AUTHOR("Alaa Hleihel");
 MODULE_DESCRIPTION("ib_srp dummy kernel module");
 MODULE_LICENSE("Dual BSD/GPL");
-#ifdef RETPOLINE_MLNX
-MODULE_INFO(retpoline, "Y");
-#endif
 MODULE_VERSION(DRV_VERSION);
 
 static int __init srp_init(void)
