@@ -11,14 +11,13 @@ mlx5e_tc_act_reorder_flow_actions(struct flow_action **flow_action_reorder,
 bool
 mlx5e_tc_act_verify_actions(struct flow_action *flow_action);
 
-#if defined(HAVE_SWITCHDEV_OPS) || defined(HAVE_SWITCHDEV_H_COMPAT)
+#if defined(HAVE_SWITCHDEV_OPS)
 int mlx5e_attr_get(struct net_device *dev, struct switchdev_attr *attr);
 #endif
-void mlx5e_rep_set_sysfs_attr(struct net_device *netdev);
+#if IS_ENABLED(CONFIG_MLX5_CLS_ACT) && defined(HAVE_TC_SETUP_CB_EGDEV_REGISTER)
 int mlx5e_vport_rep_load_compat(struct mlx5e_priv *priv);
-int mlx5e_vport_rep_unload_compat(struct mlx5e_priv *priv);
-#else
-void mlx5e_rep_set_sysfs_attr(struct net_device *netdev) {}
+void mlx5e_vport_rep_unload_compat(struct mlx5e_priv *priv);
+#endif
 #endif /* CONFIG_MLX5_ESWITCH */
 
 #endif /* __MLX5_COMPAT__ */
