@@ -746,57 +746,6 @@ static ssize_t min_tx_rate_store(struct mlx5_sriov_vf *g,
 	((PAGE_SIZE - (int)(p - buf)) <= 0 ? 0 :			\
 	scnprintf(p, PAGE_SIZE - (int)(p - buf), format, ## arg))
 
-static ssize_t trunk_show(struct mlx5_sriov_vf *g,
-			  struct vf_attributes *oa,
-			  char *buf)
-{
-	struct mlx5_core_dev *dev = g->dev;
-	struct mlx5_eswitch *esw = dev->priv.eswitch;
-	struct mlx5_vport *evport = mlx5_eswitch_get_vport(esw, g->vf + 1);
-	u16 vlan_id = 0;
-	char *ret = buf;
-
-	mutex_lock(&esw->state_lock);
-	if (!!bitmap_weight(evport->info.vlan_trunk_8021q_bitmap, VLAN_N_VID)) {
-		ret += _sprintf(ret, buf, "Allowed 802.1Q VLANs:");
-		for_each_set_bit(vlan_id, evport->info.vlan_trunk_8021q_bitmap,
-				VLAN_N_VID)
-			ret += _sprintf(ret, buf, " %d", vlan_id);
-		ret += _sprintf(ret, buf, "\n");
-	}
-	mutex_unlock(&esw->state_lock);
-
-	return (ssize_t)(ret - buf);
-}
-
-static ssize_t trunk_store(struct mlx5_sriov_vf *g,
-			   struct vf_attributes *oa,
-			   const char *buf,
-			   size_t count)
-{
-	struct mlx5_core_dev *dev = g->dev;
-	u16 start_vid, end_vid;
-	char op[5];
-	int err;
-
-	err = sscanf(buf, "%4s %hu %hu", op, &start_vid, &end_vid);
-	if (err != 3)
-		return -EINVAL;
-
-	if (!strcmp(op, "add"))
-		err = mlx5_eswitch_add_vport_trunk_range(dev->priv.eswitch,
-							 g->vf + 1,
-							 start_vid, end_vid);
-	else if (!strcmp(op, "rem"))
-		err = mlx5_eswitch_del_vport_trunk_range(dev->priv.eswitch,
-							 g->vf + 1,
-							 start_vid, end_vid);
-	else
-		return -EINVAL;
-
-	return err ? err : count;
-}
-
 static ssize_t config_show(struct mlx5_sriov_vf *g, struct vf_attributes *oa,
 			   char *buf)
 {
@@ -835,9 +784,6 @@ static ssize_t config_show(struct mlx5_sriov_vf *g, struct vf_attributes *oa,
         } else {
                 p += _sprintf(p, buf, "MinTxRate  : 0\nMaxTxRate  : 0\nRateGroup  : 0\n");
         }
-        p += _sprintf(p, buf, "VGT+       : %s\n",
-                      !!bitmap_weight(ivi->vlan_trunk_8021q_bitmap,
-		      VLAN_N_VID) ? "ON" : "OFF");
 	mutex_unlock(&esw->state_lock);
 
 	return (ssize_t)(p - buf);
@@ -882,7 +828,6 @@ static ssize_t config_group_store(struct mlx5_esw_sched_node *g,
 static ssize_t stats_show(struct mlx5_sriov_vf *g, struct vf_attributes *oa,
 			  char *buf)
 {
-	struct ifla_vf_stats_backport ifi_backport;
 	struct mlx5_core_dev *dev = g->dev;
 	struct mlx5_vport *vport = mlx5_eswitch_get_vport(dev->priv.eswitch, g->vf + 1);
 	struct ifla_vf_stats ifi;
@@ -894,10 +839,6 @@ static ssize_t stats_show(struct mlx5_sriov_vf *g, struct vf_attributes *oa,
 	if (err)
 		return -EINVAL;
 
-	err = mlx5_eswitch_get_vport_stats_backport(dev->priv.eswitch, g->vf + 1, &ifi_backport);
-	if (err)
-		return -EINVAL;
-
 	err = mlx5_esw_query_vport_drop_stats(dev, vport, &stats);
 	if (err)
 		return -EINVAL;
@@ -909,8 +850,6 @@ static ssize_t stats_show(struct mlx5_sriov_vf *g, struct vf_attributes *oa,
 	p += _sprintf(p, buf, "rx_bytes      : %llu\n", ifi.rx_bytes);
 	p += _sprintf(p, buf, "rx_broadcast  : %llu\n", ifi.broadcast);
 	p += _sprintf(p, buf, "rx_multicast  : %llu\n", ifi.multicast);
-	p += _sprintf(p, buf, "tx_broadcast  : %llu\n", ifi_backport.tx_broadcast);
-	p += _sprintf(p, buf, "tx_multicast  : %llu\n", ifi_backport.tx_multicast);
 	p += _sprintf(p, buf, "rx_dropped    : %llu\n", stats.rx_dropped);
 
 	return (ssize_t)(p - buf);
@@ -1185,7 +1124,6 @@ VF_ATTR(num_pages);
 VF_ATTR(max_tx_rate);
 VF_ATTR(min_tx_rate);
 VF_ATTR(config);
-VF_ATTR(trunk);
 VF_ATTR(stats);
 VF_ATTR(group);
 VF_RATE_GROUP_ATTR(min_tx_rate);
@@ -1202,7 +1140,6 @@ static struct attribute *vf_eth_attrs[] = {
 	&vf_attr_max_tx_rate.attr,
 	&vf_attr_min_tx_rate.attr,
 	&vf_attr_config.attr,
-	&vf_attr_trunk.attr,
 	&vf_attr_stats.attr,
 	&vf_attr_group.attr,
 	NULL
