@@ -1897,11 +1897,6 @@ static void fuse_writepage_finish_stat(struct inode *inode, struct folio *folio)
 {
 	struct backing_dev_info *bdi = inode_to_bdi(inode);
 
-	dec_wb_stat(&bdi->wb, WB_WRITEBACK);
-#ifdef HAVE_NR_WRITEBACK_TEMP
-	/* Older kernels: NR_WRITEBACK_TEMP counter exists */
-	node_stat_sub_folio(folio, NR_WRITEBACK_TEMP);
-#endif
 	wb_writeout_inc(&bdi->wb);
 }
 
@@ -2125,20 +2120,6 @@ int fuse_write_inode(struct inode *inode, struct writeback_control *wbc)
 	struct fuse_file *ff;
 	int err;
 
-#ifdef HAVE_WRITEBACK_CONTROL_FOR_RECLAIM
-	/*
-	 * Inode is always written before the last reference is dropped and
-	 * hence this should not be reached from reclaim.
-	 *
-	 * Writing back the inode from reclaim can deadlock if the request
-	 * processing itself needs an allocation.  Allocations triggering
-	 * reclaim while serving a request can't be prevented, because it can
-	 * involve any number of unrelated userspace processes.
-	 */
-	/* Older kernels: for_reclaim field exists in writeback_control */
-	WARN_ON(wbc->for_reclaim);
-#endif
-
 	ff = __fuse_write_file_get(fi);
 	err = fuse_flush_times(inode, ff);
 	if (ff)
@@ -2183,24 +2164,14 @@ static void fuse_writepage_add_to_bucket(struct fuse_conn *fc,
 static void fuse_writepage_args_page_fill(struct fuse_writepage_args *wpa, struct folio *folio,
 					  struct folio *tmp_folio, uint32_t folio_index)
 {
-	struct inode *inode = folio->mapping->host;
 	struct fuse_args_pages *ap = &wpa->ia.ap;
 
-#ifdef HAVE_FOLIO_COPY_EXPORTED
 	folio_copy(tmp_folio, folio);
-#else
-	folio_migrate_copy(tmp_folio, folio);
-#endif
 
 	ap->folios[folio_index] = tmp_folio;
 	ap->descs[folio_index].offset = 0;
 	ap->descs[folio_index].length = PAGE_SIZE;
 
-	inc_wb_stat(&inode_to_bdi(inode)->wb, WB_WRITEBACK);
-#ifdef HAVE_NR_WRITEBACK_TEMP
-	/* Older kernels: NR_WRITEBACK_TEMP counter exists */
-	node_stat_add_folio(tmp_folio, NR_WRITEBACK_TEMP);
-#endif
 }
 
 static struct fuse_writepage_args *fuse_writepage_args_setup(struct folio *folio,
@@ -2406,11 +2377,7 @@ static bool fuse_writepage_need_send(struct fuse_conn *fc, struct folio *folio,
 		return true;
 
 	/* Discontinuity */
-#ifdef HAVE_FOLIO_INDEX
-	if (data->orig_folios[ap->num_folios - 1]->index + 1 != folio_index(folio))
-#else
 	if (data->orig_folios[ap->num_folios - 1]->index + 1 != folio->index)
-#endif
 		return true;
 
 	/* Need to grow the pages array?  If so, did the expansion fail? */
@@ -2498,6 +2465,39 @@ out_unlock:
 	return err;
 }
 
+/**
+ * write_cache_pages - walk the list of dirty pages of the given address space and write all of them.
+ * @mapping: address space structure to write
+ * @wbc: subtract the number of written pages from *@wbc->nr_to_write
+ * @writepage: function called for each page
+ * @data: data passed to writepage function
+ *
+ * Return: %0 on success, negative error code otherwise
+ *
+ * Note: please use writeback_iter() instead.
+ */
+typedef int (*writepage_t)(struct folio *folio, struct writeback_control *wbc,
+                               void *data);
+#ifndef HAVE_WRITE_CACHE_PAGES_4_PARAMS
+static int write_cache_pages(struct address_space *mapping,
+			     struct writeback_control *wbc, writepage_t writepage,
+			     void *data)
+{
+       struct folio *folio = NULL;
+       int error;
+
+       while ((folio = writeback_iter(mapping, wbc, folio, &error))) {
+               error = writepage(folio, wbc, data);
+               if (error == AOP_WRITEPAGE_ACTIVATE) {
+                       folio_unlock(folio);
+                       error = 0;
+               }
+       }
+
+       return error;
+}
+#endif
+
 static int fuse_writepages(struct address_space *mapping,
 			   struct writeback_control *wbc)
 {
@@ -2550,9 +2550,9 @@ static int fuse_write_begin(
 #endif
 			    struct address_space *mapping,
 #ifdef HAVE_WRITE_BEGIN_FOLIO
-	loff_t pos, unsigned len, struct folio **foliop, void **fsdata)
+			    loff_t pos, unsigned len, struct folio **foliop, void **fsdata)
 #else
-	loff_t pos, unsigned len, struct page **pagep, void **fsdata)
+			    loff_t pos, unsigned len, struct page **pagep, void **fsdata)
 #endif
 {
 	pgoff_t index = pos >> PAGE_SHIFT;
@@ -2648,16 +2648,16 @@ error:
 
 static int fuse_write_end(
 #ifdef HAVE_ADDRESS_SPACE_WRITE_BEGIN_KIOCB
-		          const struct kiocb *iocb,
+			  const struct kiocb *iocb,
 #else
-		          struct file *file,
+			  struct file *file,
 #endif
-		          struct address_space *mapping,
-		          loff_t pos, unsigned len, unsigned copied,
+			  struct address_space *mapping,
+			  loff_t pos, unsigned len, unsigned copied,
 #ifdef HAVE_WRITE_BEGIN_FOLIO
-		struct folio *folio, void *fsdata)
+			  struct folio *folio, void *fsdata)
 #else
-		struct page *page, void *fsdata)
+	struct page *page, void *fsdata)
 #endif
 {
 #ifdef HAVE_WRITE_BEGIN_FOLIO
@@ -2676,7 +2676,6 @@ static int fuse_write_end(
 #else
 	if (!PageUptodate(page)) {
 #endif
-
 		/* Zero any unwritten bytes at the end of the page */
 		size_t endoff = pos & ~PAGE_MASK;
 		if (endoff)
