@@ -9,9 +9,10 @@
 #ifndef _FS_FUSE_I_H
 #define _FS_FUSE_I_H
 
-#ifndef pr_fmt
-# define pr_fmt(fmt) "fuse: " fmt
+#ifdef pr_fmt
+#undef pr_fmt
 #endif
+#define pr_fmt(fmt) "fuse: " fmt
 
 /* Workaround for packaging */
 #undef CONFIG_CUSE
