@@ -28,7 +28,6 @@ static struct nvmf_host *nvmf_default_host;
 static struct nvmf_host *nvmf_host_alloc(const char *hostnqn, uuid_t *id)
 {
 	struct nvmf_host *host;
-	ssize_t res;
 
 	host = kmalloc(sizeof(*host), GFP_KERNEL);
 	if (!host)
@@ -36,7 +35,7 @@ static struct nvmf_host *nvmf_host_alloc(const char *hostnqn, uuid_t *id)
 
 	kref_init(&host->ref);
 	uuid_copy(&host->id, id);
-	res = strscpy(host->nqn, hostnqn, NVMF_NQN_SIZE);
+	strscpy(host->nqn, hostnqn, NVMF_NQN_SIZE);
 	return host;
 }
 
@@ -585,7 +584,7 @@ EXPORT_SYMBOL_GPL(nvmf_connect_io_queue);
  * Do not retry when:
  *
  * - the DNR bit is set and the specification states no further connect
- *   attempts with the same set of paramenters should be attempted.
+ *   attempts with the same set of parameters should be attempted.
  *
  * - when the authentication attempt fails, because the key was invalid.
  *   This error code is set on the host side.
@@ -1123,7 +1122,6 @@ out:
 	return ret;
 }
 
-#ifdef HAVE_BLK_MQ_HCTX_TYPE
 void nvmf_set_io_queues(struct nvmf_ctrl_options *opts, u32 nr_io_queues,
 			u32 io_queues[HCTX_MAX_TYPES])
 {
@@ -1199,7 +1197,6 @@ void nvmf_map_queues(struct blk_mq_tag_set *set, struct nvme_ctrl *ctrl,
 		io_queues[HCTX_TYPE_POLL]);
 }
 EXPORT_SYMBOL_GPL(nvmf_map_queues);
-#endif /* HAVE_BLK_MQ_HCTX_TYPE */
 
 static int nvmf_check_required_opts(struct nvmf_ctrl_options *opts,
 		unsigned int required_opts)
@@ -1590,9 +1587,6 @@ static void __exit nvmf_exit(void)
 }
 
 MODULE_LICENSE("GPL v2");
-#ifdef RETPOLINE_MLNX
-MODULE_INFO(retpoline, "Y");
-#endif
 MODULE_DESCRIPTION("NVMe host fabrics library");
 
 module_init(nvmf_init);
