@@ -106,10 +106,8 @@ admin:
  */
 static void __user *nvme_to_user_ptr(uintptr_t ptrval)
 {
-#if defined(HAVE_IN_COMPAT_SYSCALL) && defined(HAVE_COMPAT_UPTR_T)
 	if (in_compat_syscall())
 		ptrval = (compat_uptr_t)ptrval;
-#endif
 	return (void __user *)ptrval;
 }
 
@@ -199,7 +197,7 @@ static int nvme_map_user_request(struct request *req, u64 ubuffer,
 {
 	struct request_queue *q = req->q;
 	struct nvme_ns *ns = q->queuedata;
-#ifdef HAVE_ENUM_BIO_REMAPPED	
+#ifdef HAVE_ENUM_BIO_REMAPPED
 	struct block_device *bdev = ns ? ns->disk->part0 : NULL;
 	bool supports_metadata = bdev && blk_get_integrity(bdev->bd_disk);
 #else
@@ -224,6 +222,7 @@ static int nvme_map_user_request(struct request *req, u64 ubuffer,
 			dev_warn_once(ctrl->device,
 				      "using unchecked metadata buffer\n");
 	}
+
 #ifdef HAVE_NVME_IOCTL_IO64_CMD_VEC
 #ifdef HAVE_BLK_RQ_MAP_USER_IO
 	if (iter)
@@ -254,15 +253,11 @@ static int nvme_map_user_request(struct request *req, u64 ubuffer,
 	ret = blk_rq_map_user(q, req, NULL, nvme_to_user_ptr(ubuffer), bufflen,
 			GFP_KERNEL);
 #endif /* HAVE_NVME_IOCTL_IO64_CMD_VEC */
-
 	if (ret)
 		return ret;
 
 	bio = req->bio;
 #ifdef HAVE_BIO_INTEGRITY_MAP_USER
-	if (bdev)
-		bio_set_dev(bio, bdev);
-
 	if (has_metadata) {
 #ifdef HAVE_BLK_RQ_INTEGRITY_MAP_USER
 #ifdef HAVE_BLK_RQ_INTEGRITY_MAP_USER_GET_3_PARAM
@@ -353,7 +348,7 @@ static int nvme_submit_user_cmd(struct request_queue *q,
 	bio = req->bio;
 	ctrl = nvme_req(req)->ctrl;
 
-#if defined(HAVE_BLK_EXECUTE_RQ_2_PARAM) || defined(HAVE_BLK_EXECUTE_RQ_3_PARAM)	
+#if defined(HAVE_BLK_EXECUTE_RQ_2_PARAM) || defined(HAVE_BLK_EXECUTE_RQ_3_PARAM)
 	effects = nvme_passthru_start(ctrl, ns, cmd->common.opcode);
 #if defined(HAVE_BLK_EXECUTE_RQ_2_PARAM)
 	ret = nvme_execute_rq(req, false);
@@ -608,6 +603,20 @@ static inline struct nvme_uring_cmd_pdu *nvme_uring_cmd_pdu(
 #endif
 }
 
+#ifdef HAVE_IO_URING_CMD_FROM_TW
+/* Newer kernels (6.19+): io_req_tw_func_t callback signature */
+static void nvme_uring_task_cb(struct io_tw_req tw_req, io_tw_token_t tw)
+{
+	struct io_uring_cmd *ioucmd = io_uring_cmd_from_tw(tw_req);
+	struct nvme_uring_cmd_pdu *pdu = nvme_uring_cmd_pdu(ioucmd);
+
+	if (pdu->bio)
+		blk_rq_unmap_user(pdu->bio);
+	io_uring_cmd_done32(ioucmd, pdu->status, pdu->result,
+			    IO_URING_CMD_TASK_WORK_ISSUE_FLAGS);
+}
+#else
+/* Older kernels: io_uring_cmd_tw_t callback signature */
 static void nvme_uring_task_cb(struct io_uring_cmd *ioucmd,
 			       unsigned issue_flags)
 {
@@ -615,8 +624,13 @@ static void nvme_uring_task_cb(struct io_uring_cmd *ioucmd,
 
 	if (pdu->bio)
 		blk_rq_unmap_user(pdu->bio);
+#ifdef HAVE_IO_URING_CMD_DONE32
+	io_uring_cmd_done32(ioucmd, pdu->status, pdu->result, issue_flags);
+#else
 	io_uring_cmd_done(ioucmd, pdu->status, pdu->result, issue_flags);
+#endif
 }
+#endif
 
 static enum rq_end_io_ret nvme_uring_cmd_end_io(struct request *req,
 						blk_status_t err)
@@ -634,25 +648,14 @@ static enum rq_end_io_ret nvme_uring_cmd_end_io(struct request *req,
 	pdu->result = le64_to_cpu(nvme_req(req)->result.u64);
 
 	/*
-	 * For iopoll, complete it directly. Note that using the uring_cmd
-	 * helper for this is safe only because we check blk_rq_is_poll().
-	 * As that returns false if we're NOT on a polled queue, then it's
-	 * safe to use the polled completion helper.
-	 *
-	 * Otherwise, move the completion to task work.
+	 * IOPOLL could potentially complete this request directly, but
+	 * if multiple rings are polling on the same queue, then it's possible
+	 * for one ring to find completions for another ring. Punting the
+	 * completion via task_work will always direct it to the right
+	 * location, rather than potentially complete requests for ringA
+	 * under iopoll invocations from ringB.
 	 */
-	if (blk_rq_is_poll(req)) {
-#ifdef HAVE_IO_URING_CMD_IOPOLL_DONE
-		if (pdu->bio)
-			blk_rq_unmap_user(pdu->bio);
-		io_uring_cmd_iopoll_done(ioucmd, pdu->result, pdu->status);
-#else
-		nvme_uring_task_cb(ioucmd, IO_URING_F_UNLOCKED);
-#endif
-	} else {
-		io_uring_cmd_do_in_task_lazy(ioucmd, nvme_uring_task_cb);
-	}
-
+	io_uring_cmd_do_in_task_lazy(ioucmd, nvme_uring_task_cb);
 	return RQ_END_IO_FREE;
 }
 
@@ -706,16 +709,30 @@ static int nvme_uring_cmd_io(struct nvme_ctrl *ctrl, struct nvme_ns *ns,
 	d.timeout_ms = READ_ONCE(cmd->timeout_ms);
 
 	if (d.data_len && (ioucmd->flags & IORING_URING_CMD_FIXED)) {
-		/* fixedbufs is only for non-vectored io */
+#ifdef HAVE_IO_URING_CMD_IMPORT_FIXED_VEC
+		int ddir = nvme_is_write(&c) ? WRITE : READ;
+
+		if (vec)
+			ret = io_uring_cmd_import_fixed_vec(ioucmd,
+					u64_to_user_ptr(d.addr), d.data_len,
+					ddir, &iter, issue_flags);
+		else
+			ret = io_uring_cmd_import_fixed(d.addr, d.data_len,
+					ddir, &iter, ioucmd
+#ifdef HAVE_IO_URING_CMD_IMPORT_FIXED_6_PARAMS
+							,issue_flags
+#endif
+				);
+#else
 		if (vec)
 			return -EINVAL;
 
 		ret = io_uring_cmd_import_fixed(d.addr, d.data_len,
+						nvme_is_write(&c) ? WRITE : READ, &iter, ioucmd
 #ifdef HAVE_IO_URING_CMD_IMPORT_FIXED_6_PARAMS
-			nvme_is_write(&c) ? WRITE : READ, &iter, ioucmd,
-			issue_flags);
-#else
-			nvme_is_write(&c) ? WRITE : READ, &iter, ioucmd);
+							,issue_flags
+#endif
+						);
 #endif
 		if (ret < 0)
 			return ret;
@@ -738,7 +755,7 @@ static int nvme_uring_cmd_io(struct nvme_ctrl *ctrl, struct nvme_ns *ns,
 	if (d.data_len) {
 		ret = nvme_map_user_request(req, d.addr, d.data_len,
 			nvme_to_user_ptr(d.metadata), d.metadata_len,
-			map_iter, vec);
+			map_iter, vec ? NVME_IOCTL_VEC : 0);
 		if (ret)
 			goto out_free_req;
 	}
@@ -761,10 +778,8 @@ static bool is_ctrl_ioctl(unsigned int cmd)
 {
 	if (cmd == NVME_IOCTL_ADMIN_CMD || cmd == NVME_IOCTL_ADMIN64_CMD)
 		return true;
-#ifdef HAVE_LINUX_SED_OPAL_H
 	if (is_sed_ioctl(cmd))
 		return true;
-#endif
 	return false;
 }
 
@@ -777,11 +792,7 @@ static int nvme_ctrl_ioctl(struct nvme_ctrl *ctrl, unsigned int cmd,
 	case NVME_IOCTL_ADMIN64_CMD:
 		return nvme_user_cmd64(ctrl, NULL, argp, 0, open_for_write);
 	default:
-#ifdef HAVE_LINUX_SED_OPAL_H
 		return sed_ioctl(ctrl->opal_dev, cmd, argp);
-#else
-		return 0;
-#endif
 	}
 }
 
@@ -977,7 +988,7 @@ int nvme_ns_head_ioctl(struct block_device *bdev, fmode_t mode,
 
 	/*
 	 * Handle ioctls that apply to the controller instead of the namespace
-	 * seperately and drop the ns SRCU reference early.  This avoids a
+	 * separately and drop the ns SRCU reference early.  This avoids a
 	 * deadlock when deleting namespaces using the passthrough interface.
 	 */
 	if (is_ctrl_ioctl(cmd))
