@@ -134,9 +134,7 @@ struct nvme_rdma_ctrl {
 
 	struct nvme_ctrl	ctrl;
 	bool			use_inline_data;
-#if defined(HAVE_BLK_MQ_HCTX_TYPE)
 	u32			io_queues[HCTX_MAX_TYPES];
-#endif
 };
 
 static inline struct nvme_rdma_ctrl *to_rdma_ctrl(struct nvme_ctrl *ctrl)
@@ -188,13 +186,9 @@ static inline int nvme_rdma_queue_idx(struct nvme_rdma_queue *queue)
 
 static bool nvme_rdma_poll_queue(struct nvme_rdma_queue *queue)
 {
-#if defined(HAVE_BLK_MQ_HCTX_TYPE)
 	return nvme_rdma_queue_idx(queue) >
 		queue->ctrl->io_queues[HCTX_TYPE_DEFAULT] +
 		queue->ctrl->io_queues[HCTX_TYPE_READ];
-#else
-	return false;
-#endif
 }
 
 static inline size_t nvme_rdma_inline_data_size(struct nvme_rdma_queue *queue)
@@ -250,7 +244,7 @@ static struct nvme_rdma_qe *nvme_rdma_alloc_ring(struct ib_device *ibdev,
 
 	/*
 	 * Bind the CQEs (post recv buffers) DMA mapping to the RDMA queue
-	 * lifetime. It's safe, since any chage in the underlying RDMA device
+	 * lifetime. It's safe, since any change in the underlying RDMA device
 	 * will issue error recovery and queue re-creation.
 	 */
 	for (i = 0; i < ib_queue_size; i++) {
@@ -763,9 +757,7 @@ static int nvme_rdma_alloc_io_queues(struct nvme_rdma_ctrl *ctrl)
 	dev_info(ctrl->ctrl.device,
 		"creating %d I/O queues.\n", nr_io_queues);
 
-#ifdef HAVE_BLK_MQ_HCTX_TYPE
 	nvmf_set_io_queues(opts, nr_io_queues, ctrl->io_queues);
-#endif
 	for (i = 1; i < ctrl->ctrl.queue_count; i++) {
 		ret = nvme_rdma_alloc_queue(ctrl, i,
 				ctrl->ctrl.sqsize + 1);
@@ -793,11 +785,7 @@ static int nvme_rdma_alloc_tag_set(struct nvme_ctrl *ctrl)
 
 	return nvme_alloc_io_tag_set(ctrl, &to_rdma_ctrl(ctrl)->tag_set,
 			&nvme_rdma_mq_ops,
-#ifdef HAVE_BLK_MQ_TAG_SET_HAS_NR_MAP
 			ctrl->opts->nr_poll_queues ? HCTX_MAX_TYPES : 2,
-#else
-			2,
-#endif
 			cmd_size);
 }
 
@@ -838,7 +826,7 @@ static int nvme_rdma_configure_admin_queue(struct nvme_rdma_ctrl *ctrl,
 
 	/*
 	 * Bind the async event SQE DMA mapping to the admin queue lifetime.
-	 * It's safe, since any chage in the underlying RDMA device will issue
+	 * It's safe, since any change in the underlying RDMA device will issue
 	 * error recovery and queue re-creation.
 	 */
 	error = nvme_rdma_alloc_qe(ctrl->device->dev, &ctrl->async_event_sqe,
@@ -915,7 +903,7 @@ static int nvme_rdma_configure_io_queues(struct nvme_rdma_ctrl *ctrl, bool new)
 
 	/*
 	 * Only start IO queues for which we have allocated the tagset
-	 * and limitted it to the available queues. On reconnects, the
+	 * and limited it to the available queues. On reconnects, the
 	 * queue number might have changed.
 	 */
 	nr_queues = min(ctrl->tag_set.nr_hw_queues + 1, ctrl->ctrl.queue_count);
@@ -1273,7 +1261,6 @@ static int nvme_rdma_inv_rkey(struct nvme_rdma_queue *queue,
 #ifdef CONFIG_NVFS
 #include "nvfs-rdma.h"
 #endif
-
 static void nvme_rdma_dma_unmap_req(struct ib_device *ibdev, struct request *rq)
 {
 	struct nvme_rdma_request *req = blk_mq_rq_to_pdu(rq);
@@ -1303,7 +1290,6 @@ static void nvme_rdma_dma_unmap_req(struct ib_device *ibdev, struct request *rq)
 #endif
 }
 
-
 static void nvme_rdma_unmap_data(struct nvme_rdma_queue *queue,
 		struct request *rq)
 {
@@ -1571,13 +1557,12 @@ static int nvme_rdma_dma_map_req(struct ib_device *ibdev, struct request *rq,
         bool is_nvfs_io = false;
         ret = nvme_rdma_nvfs_map_data(ibdev, rq, &is_nvfs_io, count);
         if (is_nvfs_io) {
-                if (ret)
-                    goto out_free_table;
+	        if (ret)
+	               goto out_free_table;
                 return 0;
-        }
+	}
         }
 #endif
-
 #ifdef HAVE_BLK_RQ_MAP_SG_2_PARAMS
 	req->data_sgl.nents = blk_rq_map_sg(rq, req->data_sgl.sg_table.sgl);
 #else
@@ -1669,7 +1654,6 @@ static int nvme_rdma_map_data(struct nvme_rdma_queue *queue,
 	if (!blk_rq_nr_phys_segments(rq))
 		return nvme_rdma_set_sg_null(c);
 
-
 	ret = nvme_rdma_dma_map_req(ibdev, rq, &count, &pi_count);
 	if (unlikely(ret))
 		return ret;
@@ -2289,7 +2273,6 @@ static void nvme_rdma_complete_rq(struct request *rq)
 	nvme_complete_rq(rq);
 }
 
-#if defined(HAVE_BLK_MQ_TAG_SET_HAS_MAP)
 #ifdef HAVE_BLK_MQ_OPS_MAP_QUEUES_RETURN_INT
 static int nvme_rdma_map_queues(struct blk_mq_tag_set *set)
 #else
@@ -2298,16 +2281,11 @@ static void nvme_rdma_map_queues(struct blk_mq_tag_set *set)
 {
 	struct nvme_rdma_ctrl *ctrl = to_rdma_ctrl(set->driver_data);
 
-#if defined(HAVE_BLK_MQ_HCTX_TYPE)
 	nvmf_map_queues(set, &ctrl->ctrl, ctrl->io_queues);
 #ifdef HAVE_BLK_MQ_OPS_MAP_QUEUES_RETURN_INT
 	return 0;
 #endif
-#elif defined HAVE_BLK_MQ_OPS_MAP_QUEUES_RETURN_INT
-	return blk_mq_rdma_map_queues(set, ctrl->device->dev, 0);
-#endif
 }
-#endif
 
 static const struct blk_mq_ops nvme_rdma_mq_ops = {
 	.queue_rq	= nvme_rdma_queue_rq,
@@ -2316,9 +2294,7 @@ static const struct blk_mq_ops nvme_rdma_mq_ops = {
 	.exit_request	= nvme_rdma_exit_request,
 	.init_hctx	= nvme_rdma_init_hctx,
 	.timeout	= nvme_rdma_timeout,
-#if defined(HAVE_BLK_MQ_TAG_SET_HAS_MAP)
 	.map_queues	= nvme_rdma_map_queues,
-#endif
 	.poll		= nvme_rdma_poll,
 };
 
@@ -2613,6 +2589,3 @@ module_exit(nvme_rdma_cleanup_module);
 
 MODULE_DESCRIPTION("NVMe host RDMA transport driver");
 MODULE_LICENSE("GPL v2");
-#ifdef RETPOLINE_MLNX
-MODULE_INFO(retpoline, "Y");
-#endif
