@@ -10,9 +10,11 @@
 #include <linux/module.h>
 #include <linux/init.h>
 #include <linux/slab.h>
+#include <linux/sched/mm.h>
 #include <linux/err.h>
-#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
 #include <linux/crc32.h>
+#ifndef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+#include <crypto/hash.h>
 #endif
 #include <linux/nvme-tcp.h>
 #ifdef CONFIG_NVME_TCP_TLS
@@ -26,12 +28,8 @@
 #include <net/handshake.h>
 #endif
 #include <linux/blk-mq.h>
-#ifndef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
-#include <crypto/hash.h>
-#endif
 #include <net/busy_poll.h>
 #include <trace/events/sock.h>
-#include <linux/sched/mm.h>
 
 #include "nvme.h"
 #include "fabrics.h"
@@ -185,8 +183,8 @@ struct nvme_tcp_queue {
 	u32			rcv_crc;
 	u32			snd_crc;
 #else
-	struct ahash_request	*rcv_hash;
-	struct ahash_request	*snd_hash;
+ 	struct ahash_request	*rcv_hash;
+ 	struct ahash_request	*snd_hash;
 #endif
 	__le32			exp_ddgst;
 	__le32			recv_ddgst;
@@ -214,9 +212,7 @@ struct nvme_tcp_ctrl {
 	struct work_struct	err_work;
 	struct delayed_work	connect_work;
 	struct nvme_tcp_request async_req;
-#ifdef HAVE_BLK_MQ_HCTX_TYPE
 	u32			io_queues[HCTX_MAX_TYPES];
-#endif
 };
 
 static LIST_HEAD(nvme_tcp_ctrl_list);
@@ -389,11 +385,7 @@ static void nvme_tcp_init_iter(struct nvme_tcp_request *req,
 		offset = bio->bi_iter.bi_bvec_done;
 	}
 
-#ifdef HAVE_IOV_ITER_IS_BVEC_SET
 	iov_iter_bvec(&req->iter, dir, vec, nr_bvec, size);
-#else
-	iov_iter_bvec(&req->iter, ITER_BVEC | dir, vec, nr_bvec, size);
-#endif
 	req->iter.iov_offset = offset;
 }
 
@@ -437,7 +429,7 @@ static inline bool nvme_tcp_queue_more(struct nvme_tcp_queue *queue)
 }
 
 static inline void nvme_tcp_queue_request(struct nvme_tcp_request *req,
-		bool sync, bool last)
+		bool last)
 {
 	struct nvme_tcp_queue *queue = req->queue;
 	bool empty;
@@ -451,7 +443,7 @@ static inline void nvme_tcp_queue_request(struct nvme_tcp_request *req,
 	 * are on the same cpu, so we don't introduce contention.
 	 */
 	if (queue->io_cpu == raw_smp_processor_id() &&
-	    sync && empty && mutex_trylock(&queue->send_mutex)) {
+	    empty && mutex_trylock(&queue->send_mutex)) {
 		nvme_tcp_send_all(queue);
 		mutex_unlock(&queue->send_mutex);
 	}
@@ -486,7 +478,8 @@ nvme_tcp_fetch_request(struct nvme_tcp_queue *queue)
 			return NULL;
 	}
 
-	list_del(&req->entry);
+	list_del_init(&req->entry);
+	init_llist_node(&req->lentry);
 	return req;
 }
 
@@ -525,13 +518,6 @@ static inline void nvme_tcp_set_hdgst(void *pdu, size_t len)
 	*(__le32 *)(pdu + len) = nvme_tcp_hdgst(pdu, len);
 }
 #else
-static inline void nvme_tcp_ddgst_final(struct ahash_request *hash,
-		__le32 *dgst)
-{
-	ahash_request_set_crypt(hash, NULL, (u8 *)dgst, 0);
-	crypto_ahash_final(hash);
-}
-
 static inline void nvme_tcp_ddgst_update(struct ahash_request *hash,
 		struct page *page, off_t off, size_t len)
 {
@@ -543,6 +529,13 @@ static inline void nvme_tcp_ddgst_update(struct ahash_request *hash,
 	crypto_ahash_update(hash);
 }
 
+static inline void nvme_tcp_ddgst_final(struct ahash_request *hash,
+		__le32 *dgst)
+{
+	ahash_request_set_crypt(hash, NULL, (u8 *)dgst, 0);
+	crypto_ahash_final(hash);
+}
+
 static inline void nvme_tcp_hdgst(struct ahash_request *hash,
 		void *pdu, size_t len)
 {
@@ -572,8 +565,8 @@ static int nvme_tcp_verify_hdgst(struct nvme_tcp_queue *queue,
 #ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
 	exp_digest = nvme_tcp_hdgst(pdu, pdu_len);
 #else
-	nvme_tcp_hdgst(queue->rcv_hash, pdu, pdu_len);
-	exp_digest = *(__le32 *)(pdu + hdr->hlen);
+ 	nvme_tcp_hdgst(queue->rcv_hash, pdu, pdu_len);
+ 	exp_digest = *(__le32 *)(pdu + hdr->hlen);
 #endif
 	if (recv_digest != exp_digest) {
 		dev_err(queue->ctrl->ctrl.device,
@@ -603,7 +596,7 @@ static int nvme_tcp_check_ddgst(struct nvme_tcp_queue *queue, void *pdu)
 #ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
 	queue->rcv_crc = NVME_TCP_CRC_SEED;
 #else
-	crypto_ahash_init(queue->rcv_hash);
+ 	crypto_ahash_init(queue->rcv_hash);
 #endif
 
 	return 0;
@@ -638,6 +631,8 @@ static int nvme_tcp_init_request(struct blk_mq_tag_set *set,
 	req->queue = queue;
 	nvme_req(rq)->ctrl = &ctrl->ctrl;
 	nvme_req(rq)->cmd = &pdu->cmd;
+	init_llist_node(&req->lentry);
+	INIT_LIST_HEAD(&req->entry);
 
 	return 0;
 }
@@ -842,13 +837,23 @@ static int nvme_tcp_handle_r2t(struct nvme_tcp_queue *queue,
 		return -EPROTO;
 	}
 
+	if (llist_on_list(&req->lentry) ||
+	    !list_empty(&req->entry)) {
+		dev_err(queue->ctrl->ctrl.device,
+			"req %d unexpected r2t while processing request\n",
+			rq->tag);
+		return -EPROTO;
+	}
+
 	req->pdu_len = 0;
 	req->h2cdata_left = r2t_length;
 	req->h2cdata_offset = r2t_offset;
 	req->ttag = pdu->ttag;
 
 	nvme_tcp_setup_h2c_data_pdu(req);
-	nvme_tcp_queue_request(req, false, true);
+
+	llist_add(&req->lentry, &queue->req_list);
+	queue_work_on(queue->io_cpu, nvme_tcp_wq, &queue->io_work);
 
 	return 0;
 }
@@ -1012,8 +1017,8 @@ static int nvme_tcp_recv_data(struct nvme_tcp_queue *queue, struct sk_buff *skb,
 			ret = skb_copy_and_crc32c_datagram_iter(skb, *offset,
 				&req->iter, recv_len, &queue->rcv_crc);
 #else
-			ret = skb_copy_and_hash_datagram_iter(skb, *offset,
-				&req->iter, recv_len, queue->rcv_hash);
+ 			ret = skb_copy_and_hash_datagram_iter(skb, *offset,
+ 				&req->iter, recv_len, queue->rcv_hash);
 #endif
 		else
 			ret = skb_copy_datagram_iter(skb, *offset,
@@ -1156,6 +1161,9 @@ static void nvme_tcp_write_space(struct sock *sk)
 	queue = sk->sk_user_data;
 	if (likely(queue && sk_stream_is_writeable(sk))) {
 		clear_bit(SOCK_NOSPACE, &sk->sk_socket->flags);
+		/* Ensure pending TLS partial records are retried */
+		if (nvme_tcp_queue_tls(queue))
+			queue->write_space(sk);
 		queue_work_on(queue->io_cpu, nvme_tcp_wq, &queue->io_work);
 	}
 	read_unlock_bh(&sk->sk_callback_lock);
@@ -1286,9 +1294,10 @@ static int nvme_tcp_try_send_data(struct nvme_tcp_request *req)
 				req->ddgst =
 					nvme_tcp_ddgst_final(queue->snd_crc);
 #else
-				nvme_tcp_ddgst_final(queue->snd_hash,
-					&req->ddgst);
+ 				nvme_tcp_ddgst_final(queue->snd_hash,
+ 					&req->ddgst);
 #endif
+
 				req->state = NVME_TCP_SEND_DDGST;
 				req->offset = 0;
 			} else {
@@ -1518,7 +1527,7 @@ static int nvme_tcp_try_recv(struct nvme_tcp_queue *queue)
 	queue->nr_cqe = 0;
 	consumed = sock->ops->read_sock(sk, &rd_desc, nvme_tcp_recv_skb);
 	release_sock(sk);
-	return consumed;
+	return consumed == -EAGAIN ? 0 : consumed;
 }
 
 static void nvme_tcp_io_work(struct work_struct *w)
@@ -1546,6 +1555,11 @@ static void nvme_tcp_io_work(struct work_struct *w)
 		else if (unlikely(result < 0))
 			return;
 
+		/* did we get some space after spending time in recv? */
+		if (nvme_tcp_queue_has_pending(queue) &&
+		    sk_stream_is_writeable(queue->sock->sk))
+			pending = true;
+
 		if (!pending || !queue->rd_enabled)
 			return;
 
@@ -1640,6 +1654,7 @@ static void nvme_tcp_free_queue(struct nvme_ctrl *nctrl, int qid)
 		queue->pf_cache.va = NULL;
 	}
 #endif
+
 	noreclaim_flag = memalloc_noreclaim_save();
 	/* ->sock will be released by fput() */
 	fput(queue->sock->file);
@@ -1790,7 +1805,6 @@ free_icreq:
 	return ret;
 }
 
-#ifdef HAVE_BLK_MQ_HCTX_TYPE
 static bool nvme_tcp_admin_queue(struct nvme_tcp_queue *queue)
 {
 	return nvme_tcp_queue_id(queue) == 0;
@@ -1880,7 +1894,6 @@ out:
 	dev_dbg(ctrl->ctrl.device, "queue %d: using cpu %d\n",
 		qid, queue->io_cpu);
 }
-#endif /* HAVE_BLK_MQ_HCTX_TYPE */
 
 #ifdef CONFIG_NVME_TCP_TLS
 static void nvme_tcp_tls_done(void *data, int status, key_serial_t pskid)
@@ -1957,9 +1970,14 @@ static int nvme_tcp_start_tls(struct nvme_ctrl *nctrl,
 			qid, ret);
 		tls_handshake_cancel(queue->sock->sk);
 	} else {
-		dev_dbg(nctrl->device,
-			"queue %d: TLS handshake complete, error %d\n",
-			qid, queue->tls_err);
+		if (queue->tls_err) {
+			dev_err(nctrl->device,
+				"queue %d: TLS handshake complete, error %d\n",
+				qid, queue->tls_err);
+		} else {
+			dev_dbg(nctrl->device,
+				"queue %d: TLS handshake complete\n", qid);
+		}
 		ret = queue->tls_err;
 	}
 	return ret;
@@ -1975,9 +1993,6 @@ static int nvme_tcp_alloc_queue(struct nvme_ctrl *nctrl, int qid,
 	struct nvme_tcp_queue *queue = &ctrl->queues[qid];
 	int ret, rcv_pdu_size;
 	struct file *sock_file;
-#ifndef HAVE_BLK_MQ_HCTX_TYPE
-	int n;
-#endif
 #ifndef HAVE_IP_SOCK_SET_TOS
 	int opt;
 #endif
@@ -2085,7 +2100,14 @@ static int nvme_tcp_alloc_queue(struct nvme_ctrl *nctrl, int qid,
 	sk_set_memalloc(queue->sock->sk);
 
 	if (nctrl->opts->mask & NVMF_OPT_HOST_TRADDR) {
-		ret = kernel_bind(queue->sock, (struct sockaddr *)&ctrl->src_addr,
+		ret = kernel_bind(queue->sock,
+#ifdef HAVE_SOCKADDR_UNSIZED
+/* Newer kernels (6.19-rc6+): sockaddr_unsized parameter */
+			(struct sockaddr_unsized *)&ctrl->src_addr,
+#else
+/* Older kernels: sockaddr parameter */
+			(struct sockaddr *)&ctrl->src_addr,
+#endif
 			sizeof(ctrl->src_addr));
 		if (ret) {
 			dev_err(nctrl->device,
@@ -2123,7 +2145,7 @@ static int nvme_tcp_alloc_queue(struct nvme_ctrl *nctrl, int qid,
 		}
 	}
 #endif
-
+	
 	rcv_pdu_size = sizeof(struct nvme_tcp_rsp_pdu) +
 			nvme_tcp_hdgst_len(queue);
 	queue->pdu = kmalloc(rcv_pdu_size, GFP_KERNEL);
@@ -2139,8 +2161,15 @@ static int nvme_tcp_alloc_queue(struct nvme_ctrl *nctrl, int qid,
 	dev_dbg(nctrl->device, "connecting queue %d\n",
 			nvme_tcp_queue_id(queue));
 
-	ret = kernel_connect(queue->sock, (struct sockaddr *)&ctrl->addr,
-		sizeof(ctrl->addr), 0);
+	ret = kernel_connect(queue->sock,
+#ifdef HAVE_SOCKADDR_UNSIZED
+/* Newer kernels (6.19-rc6+): sockaddr_unsized parameter */
+			(struct sockaddr_unsized *)&ctrl->addr,
+#else
+/* Older kernels: sockaddr parameter */
+			(struct sockaddr *)&ctrl->addr,
+#endif
+			sizeof(ctrl->addr), 0);
 	if (ret) {
 		dev_err(nctrl->device,
 			"failed to connect socket: %d\n", ret);
@@ -2448,11 +2477,7 @@ static int nvme_tcp_configure_io_queues(struct nvme_ctrl *ctrl, bool new)
 	if (new) {
 		ret = nvme_alloc_io_tag_set(ctrl, &to_tcp_ctrl(ctrl)->tag_set,
 				&nvme_tcp_mq_ops,
-#ifdef HAVE_BLK_MQ_TAG_SET_HAS_NR_MAP
 				ctrl->opts->nr_poll_queues ? HCTX_MAX_TYPES : 2,
-#else
-				2,
-#endif
 				sizeof(struct nvme_tcp_request));
 		if (ret)
 			goto out_free_io_queues;
@@ -2460,7 +2485,7 @@ static int nvme_tcp_configure_io_queues(struct nvme_ctrl *ctrl, bool new)
 
 	/*
 	 * Only start IO queues for which we have allocated the tagset
-	 * and limitted it to the available queues. On reconnects, the
+	 * and limited it to the available queues. On reconnects, the
 	 * queue number might have changed.
 	 */
 	nr_queues = min(ctrl->tagset->nr_hw_queues + 1, ctrl->queue_count);
@@ -2531,6 +2556,9 @@ static int nvme_tcp_configure_admin_queue(struct nvme_ctrl *ctrl, bool new)
 	if (error)
 		goto out_cleanup_tagset;
 
+	if (ctrl->opts->concat && !ctrl->tls_pskid)
+		return 0;
+
 	error = nvme_enable_ctrl(ctrl);
 	if (error)
 		goto out_stop_queue;
@@ -2645,14 +2673,14 @@ static int nvme_tcp_setup_ctrl(struct nvme_ctrl *ctrl, bool new)
 	if (ret)
 		return ret;
 
-	if (ctrl->opts && ctrl->opts->concat && !ctrl->tls_pskid) {
+	if (ctrl->opts->concat && !ctrl->tls_pskid) {
 		/* See comments for nvme_tcp_key_revoke_needed() */
 		dev_dbg(ctrl->device, "restart admin queue for secure concatenation\n");
 		nvme_stop_keep_alive(ctrl);
 		nvme_tcp_teardown_admin_queue(ctrl, false);
 		ret = nvme_tcp_configure_admin_queue(ctrl, false);
 		if (ret)
-			return ret;
+			goto destroy_admin;
 	}
 
 	if (ctrl->icdoff) {
@@ -2896,8 +2924,10 @@ static void nvme_tcp_submit_async_event(struct nvme_ctrl *arg)
 	ctrl->async_req.offset = 0;
 	ctrl->async_req.curr_bio = NULL;
 	ctrl->async_req.data_len = 0;
+	init_llist_node(&ctrl->async_req.lentry);
+	INIT_LIST_HEAD(&ctrl->async_req.entry);
 
-	nvme_tcp_queue_request(&ctrl->async_req, true, true);
+	nvme_tcp_queue_request(&ctrl->async_req, true);
 }
 
 static void nvme_tcp_complete_timed_out(struct request *rq)
@@ -3056,7 +3086,7 @@ static blk_status_t nvme_tcp_queue_rq(struct blk_mq_hw_ctx *hctx,
 
 	nvme_start_request(rq);
 
-	nvme_tcp_queue_request(req, true, bd->last);
+	nvme_tcp_queue_request(req, bd->last);
 
 	return BLK_STS_OK;
 }
@@ -3067,13 +3097,9 @@ static int nvme_tcp_map_queues(struct blk_mq_tag_set *set)
 static void nvme_tcp_map_queues(struct blk_mq_tag_set *set)
 #endif
 {
-#ifdef HAVE_BLK_MQ_HCTX_TYPE
 	struct nvme_tcp_ctrl *ctrl = to_tcp_ctrl(set->driver_data);
 
 	nvmf_map_queues(set, &ctrl->ctrl, ctrl->io_queues);
-#else
-	blk_mq_map_queues(set);
-#endif
 
 #ifdef HAVE_BLK_MQ_OPS_MAP_QUEUES_RETURN_INT
 	return 0;
@@ -3098,11 +3124,7 @@ static int nvme_tcp_poll(struct blk_mq_hw_ctx *hctx, unsigned int tag)
 		return 0;
 
 	set_bit(NVME_TCP_Q_POLLING, &queue->flags);
-#ifdef HAVE_SKB_QUEUE_EMPTY_LOCKLESS
 	if (sk_can_busy_loop(sk) && skb_queue_empty_lockless(&sk->sk_receive_queue))
-#else
-	if (sk_can_busy_loop(sk) && skb_queue_empty(&sk->sk_receive_queue))
-#endif
 		sk_busy_loop(sk, true);
 	ret = nvme_tcp_try_recv(queue);
 	clear_bit(NVME_TCP_Q_POLLING, &queue->flags);
