@@ -13,11 +13,6 @@ static inline void ethtool_puts(u8 **data, const char *str)
 }
 #endif
 
-/* EEPROM Standards for plug in modules */
-#ifndef ETH_MODULE_SFF_8079
-#define ETH_MODULE_SFF_8079		0x1
-#define ETH_MODULE_SFF_8079_LEN		256
-#endif
 #ifndef ETH_MODULE_SFF_8436_MAX_LEN
 #define ETH_MODULE_SFF_8636_MAX_LEN     640
 #define ETH_MODULE_SFF_8436_MAX_LEN     640
@@ -45,87 +40,14 @@ struct ethtool_rmon_stats {
 	u64 hist[ETHTOOL_RMON_HIST_MAX];
 	u64 hist_tx[ETHTOOL_RMON_HIST_MAX];
 };
-
 #endif
 
-#ifndef ETHTOOL_FEC_NONE
-enum ethtool_fec_config_bits {
-	ETHTOOL_FEC_NONE_BIT,
-	ETHTOOL_FEC_AUTO_BIT,
-	ETHTOOL_FEC_OFF_BIT,
-	ETHTOOL_FEC_RS_BIT,
-	ETHTOOL_FEC_BASER_BIT,
-	ETHTOOL_FEC_LLRS_BIT,
-};
-
-struct ethtool_fecparam {
-	__u32   cmd;
-	/* bitmask of FEC modes */
-	__u32   active_fec;
-	__u32   fec;
-	__u32   reserved;
-};
-
-#define ETHTOOL_FEC_NONE                (1 << ETHTOOL_FEC_NONE_BIT)
-#define ETHTOOL_FEC_AUTO                (1 << ETHTOOL_FEC_AUTO_BIT)
-#define ETHTOOL_FEC_OFF                 (1 << ETHTOOL_FEC_OFF_BIT)
-#define ETHTOOL_FEC_RS                  (1 << ETHTOOL_FEC_RS_BIT)
-#define ETHTOOL_FEC_BASER               (1 << ETHTOOL_FEC_BASER_BIT)
-#define ETHTOOL_FEC_LLRS                (1 << ETHTOOL_FEC_LLRS_BIT)
-
-#else /* ETHTOOL_FEC_NONE */
-
 /* check whether ethtool_fec_config_bits is defined, but without LLRS bit */
 #ifndef ETHTOOL_FEC_LLRS
 #define ETHTOOL_FEC_LLRS_BIT		(ETHTOOL_FEC_BASER_BIT + 1)
 #define ETHTOOL_FEC_LLRS		(1 << ETHTOOL_FEC_LLRS_BIT)
-/* if SPEED_200000 missing, ETHTOOL_LINK_MODE_FEC_LLRS_BIT is defined below */
-#ifdef SPEED_200000
-#define ETHTOOL_LINK_MODE_FEC_LLRS_BIT 74
-#endif /* SPEED_200000 */
 #endif /* ETHTOOL_FEC_LLRS */
 
-#endif /* ETHTOOL_FEC_NONE */
-
-#ifndef ETH_MODULE_SFF_8472
-#define ETH_MODULE_SFF_8472		0x2
-#define ETH_MODULE_SFF_8472_LEN		512
-#endif
-
-#ifndef ETH_MODULE_SFF_8636
-#define ETH_MODULE_SFF_8636		0x3
-#define ETH_MODULE_SFF_8636_LEN		256
-#endif
-
-#ifndef ETH_MODULE_SFF_8436
-#define ETH_MODULE_SFF_8436		0x4
-#define ETH_MODULE_SFF_8436_LEN		256
-#endif
-
-#ifndef SPEED_20000
-#define SPEED_20000 20000
-#define SUPPORTED_20000baseMLD2_Full    (1 << 21)
-#define SUPPORTED_20000baseKR2_Full     (1 << 22)
-#define ADVERTISED_20000baseMLD2_Full   (1 << 21)
-#define ADVERTISED_20000baseKR2_Full    (1 << 22)
-#endif
-
-#ifndef SPEED_40000
-#define SPEED_40000 40000
-#endif
-
-#ifndef SPEED_56000
-#define SPEED_56000 56000
-#define SUPPORTED_56000baseKR4_Full	(1 << 27)
-#define SUPPORTED_56000baseCR4_Full	(1 << 28)
-#define SUPPORTED_56000baseSR4_Full	(1 << 29)
-#define SUPPORTED_56000baseLR4_Full	(1 << 30)
-#define ADVERTISED_56000baseKR4_Full	(1 << 27)
-#define ADVERTISED_56000baseCR4_Full	(1 << 28)
-#define ADVERTISED_56000baseSR4_Full	(1 << 29)
-#define ADVERTISED_56000baseLR4_Full	(1 << 30)
-#endif
-
 #define SPEED_25000 25000
 #define SPEED_50000 50000
 #define SPEED_100000 100000
@@ -161,8 +83,7 @@ struct ethtool_fecparam {
 #define ETHTOOL_LINK_MODE_FEC_BASER_BIT 51
 #endif
 
-#if !defined(SPEED_200000) || !defined(ETHTOOL_LINK_MODE_400000baseKR8_Full_BIT)
-#define SPEED_200000            200000
+#ifndef ETHTOOL_LINK_MODE_400000baseKR8_Full_BIT
 #define ETHTOOL_LINK_MODE_50000baseKR_Full_BIT 52
 #define ETHTOOL_LINK_MODE_50000baseSR_Full_BIT 53
 #define ETHTOOL_LINK_MODE_50000baseCR_Full_BIT 54
@@ -248,6 +169,14 @@ struct ethtool_fecparam {
 #define ETHTOOL_LINK_MODE_800000baseVR4_Full_BIT	  120
 #endif
 
+#ifndef SPEED_1600000
+#define SPEED_1600000 1600000
+#define ETHTOOL_LINK_MODE_1600000baseCR8_Full_BIT	  121
+#define ETHTOOL_LINK_MODE_1600000baseKR8_Full_BIT	  122
+#define ETHTOOL_LINK_MODE_1600000baseDR8_Full_BIT	  123
+#define ETHTOOL_LINK_MODE_1600000baseDR8_2_Full_BIT	  124
+#endif
+
 #define SUPPORTED_100000baseCR4_Full 0
 #define ADVERTISED_100000baseCR4_Full 0
 #define SUPPORTED_100000baseSR4_Full 0
@@ -269,47 +198,4 @@ struct ethtool_fecparam {
 #define SUPPORTED_50000baseKR2_Full 0
 #define ADVERTISED_50000baseKR2_Full 0
 
-#ifndef SPEED_UNKNOWN
-#define SPEED_UNKNOWN		-1
-#endif
-
-#ifndef DUPLEX_UNKNOWN
-#define DUPLEX_UNKNOWN		-1
-#endif
-
-#ifndef SUPPORTED_40000baseKR4_Full
-/* Add missing defines for supported and advertised speed features */
-#define SUPPORTED_40000baseKR4_Full     (1 << 23)
-#define SUPPORTED_40000baseCR4_Full     (1 << 24)
-#define SUPPORTED_40000baseSR4_Full     (1 << 25)
-#define SUPPORTED_40000baseLR4_Full     (1 << 26)
-#define ADVERTISED_40000baseKR4_Full    (1 << 23)
-#define ADVERTISED_40000baseCR4_Full    (1 << 24)
-#define ADVERTISED_40000baseSR4_Full    (1 << 25)
-#define ADVERTISED_40000baseLR4_Full    (1 << 26)
-#endif
-
-#ifndef FLOW_EXT
-#define FLOW_EXT	0x80000000
-#endif
-
-#ifndef ETHER_FLOW
-#define ETHER_FLOW      0x12    /* spec only (ether_spec) */
-#endif
-
-#ifndef SPEED_5000
-#define SPEED_5000	5000
-#endif
-
-#ifndef SPEED_14000
-#define SPEED_14000	14000
-#endif
-
-#ifndef PFC_STORM_PREVENTION_AUTO
-/* The feature wont work, but this will save backport lines */
-#define PFC_STORM_PREVENTION_AUTO	0xffff
-#define PFC_STORM_PREVENTION_DISABLE	0
-#define ETHTOOL_PFC_PREVENTION_TOUT	3
-#endif
-
 #endif /* _COMPAT_LINUX_ETHTOOL_H */
