@@ -6,7 +6,7 @@
 #include_next <net/ip6_fib.h>
 
 
-#ifndef container_of_const
+#ifndef HAVE_CONTAINER_OF_H
 #if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
 #define container_of_const(ptr, type, member)				\
 	_Generic(ptr,							\
