@@ -3,16 +3,13 @@
 
 #include "../../compat/config.h"
 
-#ifdef HAVE_NET_TLS_H
 #include_next <net/tls.h>
-#endif
 
 #ifndef TLS_DRIVER_STATE_SIZE_TX
 #define TLS_DRIVER_STATE_SIZE_TX	16
 #endif
 
-#if defined(HAVE_KTLS_STRUCTS) && !defined(HAVE_TLS_DRIVER_CTX)
-
+#ifndef HAVE_TLS_DRIVER_CTX
 static inline void *__tls_driver_ctx(struct tls_context *tls_ctx,
 				     enum tls_offload_ctx_dir direction)
 {
