@@ -157,7 +157,13 @@ static void fuse_evict_inode(struct inode *inode)
 	struct fuse_inode *fi = get_fuse_inode(inode);
 
 	/* Will write inode on close/munmap and in all other dirtiers */
+#ifdef HAVE_INODE_STATE_READ_ONCE
+/* Newer kernels (6.19+): i_state is struct, use accessor */
+	WARN_ON(inode_state_read_once(inode) & I_DIRTY_INODE);
+#else
+/* Older kernels: i_state is unsigned long */
 	WARN_ON(inode->i_state & I_DIRTY_INODE);
+#endif
 
 	truncate_inode_pages_final(&inode->i_data);
 	clear_inode(inode);
@@ -517,7 +523,13 @@ retry:
 	if (!inode)
 		return NULL;
 
+#ifdef HAVE_INODE_STATE_READ_ONCE
+/* Newer kernels (6.19+): i_state is struct, use accessor */
+	if ((inode_state_read_once(inode) & I_NEW)) {
+#else
+/* Older kernels: i_state is unsigned long */
 	if ((inode->i_state & I_NEW)) {
+#endif
 		inode->i_flags |= S_NOATIME;
 		if (!fc->writeback_cache || !S_ISREG(attr->mode))
 			inode->i_flags |= S_NOCMTIME;
@@ -1053,7 +1065,11 @@ static const struct super_operations fuse_super_operations = {
 	.free_inode     = fuse_free_inode,
 	.evict_inode	= fuse_evict_inode,
 	.write_inode	= fuse_write_inode,
+#ifdef HAVE_INODE_JUST_DROP
+	.drop_inode	= inode_just_drop,
+#else
 	.drop_inode	= generic_delete_inode,
+#endif
 	.umount_begin	= fuse_umount_begin,
 	.statfs		= fuse_statfs,
 	.sync_fs	= fuse_sync_fs,
@@ -1386,8 +1402,6 @@ static int fuse_bdi_init(struct fuse_conn *fc, struct super_block *sb)
 	if (err)
 		return err;
 
-	/* fuse does it's own writeback accounting */
-	sb->s_bdi->capabilities &= ~BDI_CAP_WRITEBACK_ACCT;
 	sb->s_bdi->capabilities |= BDI_CAP_STRICTLIMIT;
 
 	/*
@@ -1676,17 +1690,6 @@ int fuse_fill_super_common(struct super_block *sb, struct fuse_fs_context *ctx)
 
 	err = -ENOMEM;
 	root = fuse_get_root_inode(sb, ctx->rootmode);
-#ifdef HAVE_SET_DEFAULT_D_OP
-	/* Newer kernels (v6.17-rc1+): use set_default_d_op helper */
-	set_default_d_op(sb, &fuse_root_dentry_operations);
-#else
-	/* Older kernels: direct field assignment */
-	sb->s_d_op = &fuse_root_dentry_operations;
-#endif
-	root_dentry = d_make_root(root);
-	if (!root_dentry)
-		goto err_dev_free;
-	/* Root dentry doesn't have .d_revalidate */
 #ifdef HAVE_SET_DEFAULT_D_OP
 	/* Newer kernels (v6.17-rc1+): use set_default_d_op helper */
 	set_default_d_op(sb, &fuse_dentry_operations);
@@ -1694,6 +1697,9 @@ int fuse_fill_super_common(struct super_block *sb, struct fuse_fs_context *ctx)
 	/* Older kernels: direct field assignment */
 	sb->s_d_op = &fuse_dentry_operations;
 #endif
+	root_dentry = d_make_root(root);
+	if (!root_dentry)
+		goto err_dev_free;
 
 	mutex_lock(&fuse_mutex);
 	err = -EINVAL;
