@@ -290,8 +290,12 @@ static int remap_func(pte_t *pte, pgtable_t token, unsigned long addr,
 	}
 
 	/* Special PTE are not associated with any struct page */
+#if defined(CONFIG_PPC)
+	*pte = pte_mkspecial(pfn_pte(pfn, ctx->vma->vm_page_prot));
+#else
 	set_pte_at(ctx->vma->vm_mm, addr, pte,
 		   pte_mkspecial(pfn_pte(pfn, ctx->vma->vm_page_prot)));
+#endif
 	return 0;
 }
 
