@@ -498,9 +498,7 @@ conntrack_offload_netdev_flush(struct conntrack *ct, struct netdev *netdev)
 
 void
 conntrack_offload_dump(struct ds *ds, struct conn *conn, int dir,
-                       uint32_t ct_match_zone_id,
-                       uint32_t ct_action_label_id,
-                       uint32_t ct_miss_ctx_id)
+                       uint32_t ct_action_label_id)
 {
     struct ct_flow_offload_item items[CT_DIR_NUM];
     struct ct_flow_offload_item *ct_offload;
@@ -544,7 +542,7 @@ conntrack_offload_dump(struct ds *ds, struct conn *conn, int dir,
         }
     }
 
-    ds_put_format(ds, " zone_map=%d", ct_match_zone_id);
+    ds_put_format(ds, " zone_map=%d", conn->key.zone);
 
     /* CT MARK */
     ds_put_format(ds, " mark=0x%08x", ct_offload->mark_key);
@@ -554,9 +552,6 @@ conntrack_offload_dump(struct ds *ds, struct conn *conn, int dir,
 
     /* CT STATE */
     ds_put_format(ds, " state=0x%02x", ct_offload->ct_state);
-
-    /* CT CTX */
-    ds_put_format(ds, " ctx=0x%02x", ct_miss_ctx_id);
 }
 
 void
