@@ -148,9 +148,7 @@ conntrack_offload_netdev_flush(struct conntrack *ct, struct netdev *netdev);
 
 void
 conntrack_offload_dump(struct ds *ds, struct conn *conn, int dir,
-                       uint32_t ct_match_zone_id,
-                       uint32_t ct_action_label_id,
-                       uint32_t ct_miss_ctx_id);
+                       uint32_t ct_action_label_id);
 
 struct ct_offload_handle *
 conntrack_offload_get(struct conn *conn);
