@@ -178,7 +178,7 @@ tcp_bypass_seq_chk(struct conntrack *ct, struct conn *conn)
     int offloaded;
 
     offloaded = conn->offloads.flags &
-        (CT_OFFLOAD_BOTH | CT_OFFLOAD_DELETED | CT_OFFLOAD_TERMINATED);
+        (CT_OFFLOAD_BOTH | CT_OFFLOAD_DELETED | CT_OFFLOAD_BYPASS_TCP_SEQ_CHK);
     if (!conntrack_get_tcp_seq_chk(ct) || offloaded) {
         COVERAGE_INC(conntrack_tcp_seq_chk_bypass);
         return true;
