@@ -1423,7 +1423,7 @@ doca_pipe_group_add_entry(struct netdev *netdev,
     }
 
     ret = doca_offload_add_entry(netdev, pipe_queue, mask_ctx->pipe, match, actions,
-                                 monitor, fwd, DOCA_FLOW_NO_WAIT, entry);
+                                 monitor, fwd, DOCA_FLOW_ENTRY_FLAGS_NO_WAIT, entry);
     if (ret) {
         goto err_add;
     }
