@@ -146,4 +146,10 @@ void
 doca_offload_shared_meter_stats_get(uint32_t meter_id, struct doca_flow_resource_query *red_stats,
                                     struct doca_flow_resource_query *green_stats);
 
+int
+doca_offload_pre_miss_rules_update(struct netdev *netdev, const char *pre_miss_rules);
+
+struct netdev *
+doca_offload_get_esw_netdev(struct netdev *netdev);
+
 #endif /* DPDK_OFFLOAD_DOCA */
