@@ -192,6 +192,11 @@ struct vxlan_data {
 };
 BUILD_ASSERT_DECL(offsetof(struct vxlan_data, conf) == 0);
 
+struct port_id_data {
+    struct rte_flow_item_port_id conf;
+    bool physdev;
+};
+
 struct doca_flow_handle_resources {
     struct doca_pipe_group_mask_ctx *mask_ctx;
     struct doca_pipe_group_ctx *self_group_ctx;
@@ -324,6 +329,8 @@ enum {
     OVS_RTE_FLOW_ACTION_TYPE_SFLOW,
     OVS_RTE_FLOW_ACTION_TYPE_USERSPACE,
     OVS_RTE_FLOW_ACTION_TYPE_NESTED,
+    OVS_RTE_FLOW_ACTION_TYPE_SET_IPV4_TTL,
+    OVS_RTE_FLOW_ACTION_TYPE_SET_IPV6_HOP,
 };
 
 #define OVS_RTE_FLOW_ACTION_TYPE(TYPE) \
@@ -336,6 +343,8 @@ enum {
     OVS_RTE_FLOW_ITEM_TYPE_NV_MP_PID,
     OVS_RTE_FLOW_ITEM_TYPE_NV_MP_PREFERRED,
     OVS_RTE_FLOW_ITEM_TYPE_NV_MP_STRICT,
+    OVS_RTE_FLOW_ITEM_TYPE_L3_TYPE_HINT,
+    OVS_RTE_FLOW_ITEM_TYPE_L4_TYPE_HINT,
 };
 
 #define OVS_RTE_FLOW_ITEM_TYPE(TYPE) \
@@ -354,8 +363,6 @@ struct reg_tags {
 struct dpdk_offload_recovery_info {
     struct reg_tags reg_tags;
     uint32_t flow_miss_id;
-    uint32_t ct_miss_id;
-    uint32_t sflow_id;
     uint32_t dp_hash;
 };
 
@@ -366,8 +373,6 @@ enum dpdk_reg_id {
     REG_FIELD_CT_LABEL_ID,
     REG_FIELD_TUN_INFO,
     REG_FIELD_PORT_METER_COLOR = REG_FIELD_TUN_INFO,
-    REG_FIELD_CT_CTX,
-    REG_FIELD_SFLOW_CTX,
     REG_FIELD_FLOW_INFO,
     REG_FIELD_DP_HASH,
     REG_FIELD_SCRATCH,
@@ -448,7 +453,6 @@ struct dpdk_offload_api {
     int (*packet_hw_entropy)(struct netdev *, struct dp_packet *, uint16_t *);
 };
 
-extern struct dpdk_offload_api dpdk_offload_api_rte;
 extern struct dpdk_offload_api dpdk_offload_api_doca;
 
 bool
