@@ -171,7 +171,7 @@ construct_dpdk_args(const struct smap *ovs_other_config, struct svec *args)
     const char *extra_configuration;
 
     extra_configuration = smap_get_def(ovs_other_config, "dpdk-extra",
-                                       "-a 0000:00:00.0");
+                                       "-a 0000:00:00.0 -a auxiliary:mlx5_core.sf.0");
     if (extra_configuration) {
         svec_parse_words(args, extra_configuration);
     }
@@ -418,17 +418,6 @@ dpdk_init__(const struct smap *ovs_other_config)
      * some arguments from it. '+1' to copy the terminating NULL.  */
     argv = xmemdup(args.names, (args.n + 1) * sizeof args.names[0]);
 
-#ifdef DOCA_OFFLOAD
-    /* DOCA has a constructor to disable dpdk steering. This is a WA to enable it back in case
-     * of dpdk configuration.
-     */
-    if (!ovs_doca_enabled()) {
-        int rte_pmd_mlx5_driver_enable_steering(void);
-
-        rte_pmd_mlx5_driver_enable_steering();
-    }
-#endif
-
     /* Make sure things are initialized ... */
     result = rte_eal_init(args.n, argv);
 
@@ -487,10 +476,13 @@ dpdk_init__(const struct smap *ovs_other_config)
     RTE_PER_LCORE(_lcore_id) = NON_PMD_CORE_ID;
 
 
+#ifdef DOCA_OFFLOAD
+    ovs_doca_register_classes(ovs_other_config);
+#else
     /* Finally, register the dpdk classes */
     netdev_dpdk_register(ovs_other_config);
     netdev_register_flow_api_provider(&netdev_offload_dpdk);
-    ovs_doca_register_classes(ovs_other_config);
+#endif
     return true;
 }
 
