@@ -271,13 +271,6 @@ dp_doca_netdev_conns_add(struct netdev *netdevs[CT_DIR_NUM], struct batch *conns
         }
     }
 
-    if (!ovs_doca_initialized()) {
-        netdev_offload_upkeep(netdevs[CT_DIR_INIT], false);
-        if (netdevs[CT_DIR_REP]) {
-            netdev_offload_upkeep(netdevs[CT_DIR_REP], false);
-        }
-    }
-
     return flow_apis[CT_DIR_INIT]->conns_add(netdevs, conns);
 }
 
@@ -315,14 +308,6 @@ dp_doca_netdev_conns_del(struct netdev *netdevs[CT_DIR_NUM], struct batch *conns
         return EOPNOTSUPP;
     }
 
-    if (!ovs_doca_initialized()) {
-        for (int dir = 0; dir < CT_DIR_NUM; dir++) {
-            if (netdevs[dir]) {
-                netdev_offload_upkeep(netdevs[dir], false);
-            }
-        }
-    }
-
     return flow_apis[CT_DIR_INIT]->conns_del(netdevs, conns);
 }
 
