@@ -51,6 +51,7 @@ struct dpif_doca_tcpdump_pkt_hdr {
 OVS_PACKED(
 struct dpif_doca_tcpdump_batch_hdr {
     uint32_t batch_size;
+    uint32_t hook;
     char dev_name[DPIF_DOCA_TCPDUMP_MAX_IFACE_NAME];
 });
 
@@ -362,6 +363,7 @@ dpif_doca_tcpdump_send_batch__(const struct dp_packet_batch *batch,
 
     /* First entry: batch header */
     batch_hdr.batch_size = htonl(batch_size);
+    batch_hdr.hook = htonl(hook);
     snprintf(batch_hdr.dev_name, sizeof(batch_hdr.dev_name), "%s", dev_name);
     iov[0].iov_base = &batch_hdr;
     iov[0].iov_len = sizeof(batch_hdr);
