@@ -885,6 +885,7 @@ netdev_doca_generate_devargs(const char *name, char *devargs, size_t maxlen,
     char iface_tmp[IFNAMSIZ];
     char device[PATH_MAX];
     char *mlx5_devargs;
+    char *rep_part;
     bool mpesw;
     bool is_pf;
     char *pci;
@@ -932,12 +933,7 @@ netdev_doca_generate_devargs(const char *name, char *devargs, size_t maxlen,
     if (ovs_doca_enabled()) {
         mlx5_devargs =
             "dv_xmeta_en=4,"
-            "dv_flow_en=2,"
-            "reuse_tag=1,"
-            "fdb_def_rule_en=0,"
-            "repr_matching_en=0,"
-            "vport_match=1,"
-            "mpesw_suppress_pf_rep=1";
+            "dv_flow_en=2";
     } else {
         mlx5_devargs = "dv_xmeta_en=1";
     }
@@ -945,7 +941,7 @@ netdev_doca_generate_devargs(const char *name, char *devargs, size_t maxlen,
     len = strlen(phys_port_name);
     /* hpf's phys_port_name is pf0/pf1. */
     if (len == 3 && !strncmp(phys_port_name, "pf", 2)) {
-        if (snprintf(devargs, maxlen, "%s,%s,representor=pf%dvf65535", pci,
+        if (snprintf(devargs, maxlen, "%s,%s,representor=(pf%d)vf65535", pci,
                      mlx5_devargs, port) < 0) {
             return NULL;
         }
@@ -967,8 +963,17 @@ netdev_doca_generate_devargs(const char *name, char *devargs, size_t maxlen,
     }
 
     /* representors */
-    if (snprintf(devargs, maxlen, "%s,%s,representor=%s", pci, mlx5_devargs,
-                 phys_port_name) < 0) {
+    rep_part = strstr(phys_port_name, "vf");
+    if (!rep_part) {
+        rep_part = strstr(phys_port_name, "sf");
+    }
+    if (!rep_part) {
+        return NULL;
+    }
+    /* Format as (pfX)vfY or (pfX)sfY */
+    if (snprintf(devargs, maxlen, "%s,%s,representor=(%.*s)%s", pci,
+                 mlx5_devargs, (int) (rep_part - phys_port_name),
+                 phys_port_name, rep_part) < 0) {
         return NULL;
     }
     return devargs;
@@ -1013,39 +1018,6 @@ netdev_doca_setup_hairpin_queues(struct netdev *netdev, dpdk_port_t port_id,
     return diag;
 }
 
-const char *
-netdev_doca_adapt_doca_mlx5_devargs(const char *devargs, char *new_devargs, size_t maxlen)
-{
-    struct {
-        const char *key;
-        const char *value;
-    } args[] = {
-        { "reuse_tag", "1", },
-        { "fdb_def_rule_en", "0", },
-        { "repr_matching_en", "0", },
-        { "vport_match", "1", },
-        { "mpesw_suppress_pf_rep", "1", },
-    };
-    char *tail = new_devargs;
-
-    if (!devargs || !new_devargs || maxlen == 0) {
-        return devargs;
-    }
-
-    ovs_strcat(new_devargs, maxlen, &tail, devargs);
-
-    for (int k = 0; k < ARRAY_SIZE(args); k++) {
-        if (!strstr(devargs, args[k].key)) {
-            ovs_strcat(new_devargs, maxlen, &tail, ",");
-            ovs_strcat(new_devargs, maxlen, &tail, args[k].key);
-            ovs_strcat(new_devargs, maxlen, &tail, "=");
-            ovs_strcat(new_devargs, maxlen, &tail, args[k].value);
-        }
-    }
-
-    return new_devargs;
-}
-
 void
 netdev_doca_get_custom_stats_sw_meter(const struct netdev *netdev,
                                       struct netdev_custom_stats *custom_stats)
