@@ -166,9 +166,6 @@ netdev_doca_setup_hairpin_queues(struct netdev *netdev, dpdk_port_t port_id,
                                  int n_rxq, int n_txq,
                                  struct rte_eth_hairpin_conf *hairpin_conf);
 
-const char *
-netdev_doca_adapt_doca_mlx5_devargs(const char *devargs, char *new_devargs, size_t maxlen);
-
 void
 netdev_doca_get_custom_stats_sw_meter(const struct netdev *netdev,
                                       struct netdev_custom_stats *custom_stats);
