@@ -35,16 +35,11 @@ struct act_resources {
     uint32_t tnl_id;
     uint32_t flow_id;
     bool associated_flow_id;
-    uint32_t ct_miss_ctx_id;
-    uint32_t ct_match_zone_id;
-    uint32_t ct_action_zone_id;
     uint32_t ct_match_label_id;
     uint32_t ct_action_label_id;
     uint32_t ct_action_label_ctx_id;
-    struct ovs_shared_ctx *shared_age_ctx;
     struct ovs_shared_ctx *shared_count_ctx;
     struct ovs_shared_ctx *shared_ct_actions_ctx[CT_DIR_NUM];
-    uint32_t sflow_id;
     uint32_t meter_ids[DPDK_OFFLOAD_MAX_METERS_PER_FLOW];
 };
 
@@ -157,8 +152,7 @@ ct_offload_handle_active_set(struct ct_offload_handle *coh, enum ct_offload_flag
 
 void
 netdev_offload_doca_get_conns_res(struct netdev *netdevs[CT_DIR_NUM], struct batch *conns,
-                                  bool with_miss_ctx, bool with_shared_counters,
-                                  bool with_shared_ct_actions);
+                                  bool with_shared_counters, bool with_shared_ct_actions);
 
 void
 netdev_offload_doca_free_conn_res(struct act_resources *act_resources);
@@ -244,7 +238,6 @@ netdev_offload_doca_aux_tables_uninit(struct netdev *netdev);
 int
 netdev_offload_doca_hw_miss_packet_recover(struct netdev *netdev,
                                            struct dp_packet *packet,
-                                           struct dpif_sflow_attr *sflow_attr,
                                            bool dump);
 int
 netdev_offload_doca_init_flow_api(struct netdev *netdev);
@@ -271,11 +264,13 @@ struct act_vars {
     bool is_ct_conn;
     rte_be16_t vlan_tpid;
     uint8_t vlan_pcp;
+    ovs_be16 eth_type;
     uint8_t proto;
     bool has_dp_hash;
     bool has_decap;
     odp_port_t tnl_push_out_port;
     struct ds offload_error;
+    uint32_t ct_action_zone;
 };
 
 #define IS_REWRITE_ACTION(act_type) (\
@@ -283,10 +278,10 @@ struct act_vars {
     ((act_type) == RTE_FLOW_ACTION_TYPE_SET_MAC_DST) || \
     ((act_type) == RTE_FLOW_ACTION_TYPE_SET_IPV4_SRC) || \
     ((act_type) == RTE_FLOW_ACTION_TYPE_SET_IPV4_DST) || \
-    ((act_type) == RTE_FLOW_ACTION_TYPE_SET_IPV4_TTL) || \
     ((act_type) == RTE_FLOW_ACTION_TYPE_SET_IPV6_SRC) || \
     ((act_type) == RTE_FLOW_ACTION_TYPE_SET_IPV6_DST) || \
-    ((act_type) == RTE_FLOW_ACTION_TYPE_SET_IPV6_HOP) || \
+    ((act_type) == OVS_RTE_FLOW_ACTION_TYPE(SET_IPV4_TTL)) || \
+    ((act_type) == OVS_RTE_FLOW_ACTION_TYPE(SET_IPV6_HOP)) || \
     ((act_type) == OVS_RTE_FLOW_ACTION_TYPE(SET_UDP_SRC)) || \
     ((act_type) == OVS_RTE_FLOW_ACTION_TYPE(SET_UDP_DST)) || \
     ((act_type) == OVS_RTE_FLOW_ACTION_TYPE(SET_TCP_SRC)) || \
@@ -332,12 +327,6 @@ netdev_offload_doca_get_table_ctx(odp_port_t vport,
                                   struct netdev *physdev,
                                   struct table_ctx **ctx);
 
-int
-netdev_offload_doca_hw_miss_packet_recover_sflow(struct netdev *netdev,
-                                                 struct dp_packet *packet,
-                                                 int sflow_id,
-                                                 struct dpif_sflow_attr *sflow_attr);
-
 int
 netdev_offload_doca_hw_miss_packet_recover_ext(struct netdev *netdev,
                                                struct dp_packet *packet,
@@ -398,8 +387,6 @@ add_recirc_action(struct netdev *netdev,
 int
 parse_sample_action(struct flow_actions *actions,
                     const struct nlattr *nl_actions,
-                    struct dpif_sflow_attr *sflow_attr,
-                    struct act_resources *act_resources,
                     struct act_vars *act_vars);
 
 
@@ -411,9 +398,7 @@ add_meter_action(struct flow_actions *actions,
 
 int
 add_sflow_action(struct flow_actions *actions,
-                 const struct nlattr *nla,
-                 struct act_resources *act_resources,
-                 struct act_vars *act_vars);
+                 const struct nlattr *nla);
 
 int
 add_ct_clear_action(struct flow_actions *actions,
@@ -438,9 +423,6 @@ parse_tnl_match_recirc(struct flow_patterns *patterns,
                        struct act_resources *act_resources,
                        struct act_vars *act_vars);
 
-void
-offload_provider_api_init(void);
-
 int
 netdev_offload_doca_flow_flush_esw_mgr_members(struct netdev *netdev);
 
@@ -525,6 +507,23 @@ void netdev_offload_flow_mark_free(uint32_t mark);
 
 struct reg_field *netdev_offload_doca_get_reg_field(uint32_t reg_field_id);
 
+void
+netdev_offload_doca_dump_flow(struct netdev *netdev,
+                              const struct rte_flow_attr *attr,
+                              struct flow_patterns *flow_patterns,
+                              struct flow_actions *flow_actions,
+                              struct dpdk_offload_handle *doh,
+                              bool trace,
+                              struct rte_flow_error *error);
+
+void
+netdev_offload_doca_items2flow_patterns(struct rte_flow_item *items,
+                                        struct flow_patterns *flow_patterns);
+
+void
+netdev_offload_doca_actions2flow_actions(struct rte_flow_action *actions,
+                                         struct flow_actions *flow_actions);
+
 #else /* DPDK_NETDEV */
 
 static inline void
