@@ -362,7 +362,6 @@ netdev_flow_put(struct netdev *netdev, struct match *match,
 int
 netdev_hw_miss_packet_recover(struct netdev *netdev,
                               struct dp_packet *packet,
-                              struct dpif_sflow_attr *sflow_attr,
                               bool dump)
 {
     if (OVS_UNLIKELY(!netdev->netdev_class->hw_miss_packet_recover)) {
@@ -370,7 +369,7 @@ netdev_hw_miss_packet_recover(struct netdev *netdev,
     }
 
     packet->orig_netdev = netdev;
-    return netdev->netdev_class->hw_miss_packet_recover(netdev, packet, sflow_attr, dump);
+    return netdev->netdev_class->hw_miss_packet_recover(netdev, packet, dump);
 }
 
 int
