@@ -889,8 +889,7 @@ struct netdev_class {
      * in software.
      * Return 0 if successful, otherwise returns a positive errno value and
      * takes ownership of a packet if errno != EOPNOTSUPP. */
-    int (*hw_miss_packet_recover)(struct netdev *, struct dp_packet *,
-                                  struct dpif_sflow_attr *, bool);
+    int (*hw_miss_packet_recover)(struct netdev *, struct dp_packet *, bool);
 };
 
 int netdev_register_provider(const struct netdev_class *);
