@@ -60,6 +60,7 @@
 #include "flow.h"
 #include "userspace-tso.h"
 #include "util.h"
+#include "ovs-doca.h"
 #ifdef __linux__
 #include "tc.h"
 #endif
@@ -386,17 +387,13 @@ netdev_is_reserved_name(const char *name)
 const char *
 netdev_type_normalize(const char *type)
 {
-    if (type && type[0] && !strcmp(type, "dpdk")) {
-        return "doca";
-    }
-    if (type && type[0] && !strcmp(type, "dpdkvdpa")) {
-        return "docavdpa";
-    }
-    if (type && type[0] && !strcmp(type, "dpdkvhostuser")) {
-        return "docavhostuser";
-    }
-    if (type && type[0] && !strcmp(type, "dpdkvhostuserclient")) {
-        return "docavhostuserclient";
+    if (ovs_doca_initialized()) {
+        if (type && type[0] && !strcmp(type, "dpdk")) {
+            return "doca";
+        }
+        if (type && type[0] && !strcmp(type, "dpdkvdpa")) {
+            return "docavdpa";
+        }
     }
     return type && type[0] ? type : "system";
 }
