@@ -86,7 +86,6 @@ static struct refmap *ovs_doca_esw_rfm;
 struct ovs_mutex mgmt_queue_lock = OVS_MUTEX_INITIALIZER;
 static enum doca_flow_port_operation_state init_op_state = DOCA_FLOW_PORT_OPERATION_STATE_ACTIVE;
 
-unsigned int ovs_doca_max_eswitch_num = OVS_DOCA_DEFAULT_MAX_ESW;
 unsigned int doca_congestion_threshold = OVS_DOCA_CONGESTION_THRESHOLD_DEFAULT;
 /* Size of control pipes. If zero, DOCA uses its default value. */
 uint32_t ctl_pipe_size = 1024;
@@ -98,10 +97,20 @@ static unsigned int ovs_doca_max_megaflows_counters;
 static enum ovs_doca_gpr_mode gpr_mode = OVS_DOCA_GPR_MODE_DISABLED;
 atomic_bool ovs_doca_eswitch_active_ids[OVS_DOCA_MAX_SUPPORTED_ESW];
 static unsigned int n_offload_queues;
+static unsigned int ovs_doca_max_shared_encaps;
 
 #define OVS_DOCA_MAX_SW_METER_COUNTERS RTE_MAX_ETHPORTS
 #define OVS_DOCA_SLOWPATH_COUNTERS ((OVS_DOCA_RSS_NUM_ENTRIES + 1) * RTE_MAX_ETHPORTS)
-#define OVS_DOCA_MAX_SAMPLE_COUNTERS (4 * ovs_doca_max_eswitch_num_get())
+#define OVS_DOCA_MAX_SAMPLE_COUNTERS (4 * OVS_DOCA_MAX_SUPPORTED_ESW)
+
+/* Number of supported tunnel configurations:
+ * GRE-IPv4/GRE-IPv6/GRE-no-key-IPv4/GRE-no-key-IPv6
+ * VXLAN-IPv4/VXLAN-IPv6/VXLAN-GBP-IPv4/VXLAN-GBP-IPv6
+ * GENEVE-IPv4/GENEVE-IPv6/GENVE-OPTIONS-IPv4/GENEVE-OPTIONS-IPv6
+ */
+#define OVS_DOCA_MAX_TUNNELS 12
+/* 16K different tunnel header + number of different tunnel templates */
+#define OVS_DOCA_DEFAULT_MAX_SHARED_ENCAPS ((1 << 14) + OVS_DOCA_MAX_TUNNELS)
 
 static const char *
 ovs_doca_stats_name(enum ovs_doca_rss_type type);
@@ -124,10 +133,10 @@ ovs_doca_initialized(void)
     return initialized;
 }
 
-unsigned int
-ovs_doca_max_eswitch_num_get(void)
+int
+ovs_doca_pre_miss_rules_update(struct netdev *netdev, const char *pre_miss_rules)
 {
-    return ovs_doca_max_eswitch_num;
+    return doca_offload_pre_miss_rules_update(netdev, pre_miss_rules);
 }
 
 unsigned int
@@ -482,7 +491,7 @@ ovs_doca_entry_uninit(struct ovs_doca_netdev_data *dev_data,
     }
 
     ovs_doca_mgmt_queue_lock();
-    doca_flow_pipe_remove_entry(AUX_QUEUE, DOCA_FLOW_NO_WAIT, *pentry);
+    doca_flow_pipe_remove_entry(AUX_QUEUE, DOCA_FLOW_ENTRY_FLAGS_NO_WAIT, *pentry);
     ovs_doca_mgmt_queue_unlock();
     *pentry = NULL;
     dev_data->esw_ctx->queue.n_waiting_entries++;
@@ -693,8 +702,10 @@ ovs_doca_rss_entries_init(struct ovs_doca_netdev_data *dev_data)
         fwd.rss.inner_flags = ovs_doca_rss_flags(i);
 
         ovs_doca_mgmt_queue_lock();
-        entry_ret = doca_flow_pipe_add_entry(AUX_QUEUE, ctx->rss_pipe, &match.d, 0, &actions.d,
-                                             NULL, &fwd, DOCA_FLOW_NO_WAIT, &ctx->queue, &entry);
+        entry_ret = doca_flow_pipe_basic_add_entry(AUX_QUEUE, ctx->rss_pipe, &match.d, 0,
+                                                   &actions.d, NULL, &fwd,
+                                                   DOCA_FLOW_ENTRY_FLAGS_NO_WAIT,
+                                                   &ctx->queue, &entry);
         ovs_doca_mgmt_queue_unlock();
         if (entry_ret) {
             VLOG_ERR("%s: Failed to create '%s' rss entry: Error %d (%s)", dev_data->devargs,
@@ -913,6 +924,15 @@ ovs_doca_flow_ct_init(void)
 
     n_ct_sessions = ovs_doca_max_ct_conns();
     if (n_ct_queues > 1) {
+        double queues_factor;
+
+        /* RSS works by powers of 2. Some queues will get N/ROUND_DOWN_POW2(Q) while others
+         * N/ROUND_UP_POW2(Q). Increase N to compensate that.
+         */
+        queues_factor = (double) n_ct_queues / (double) ROUND_DOWN_POW2(n_ct_queues);
+        n_ct_sessions *= queues_factor;
+
+        /* RSS may not be evenly distributed. Increase by a some factor. */
         n_ct_sessions *= OVS_DOCA_CT_ENTRIES_RSS_FACTOR;
     }
     if (n_ct_sessions > MAX_DOCA_CT_CONNECTIONS) {
@@ -931,6 +951,8 @@ ovs_doca_flow_ct_init(void)
     if (conntrack_offload_doca_ct_ipv6_enabled) {
         n_ct_sessions_ipv6 = n_ct_sessions;
     }
+    VLOG_INFO("Number of connections: requested %d, effective %d", ovs_doca_max_ct_conns(),
+              n_ct_sessions);
 
     doca_offload_set_reg_mask(&zone_match_mask, REG_FIELD_CT_ZONE);
     /* DOCA-CT is configured by default in managed mode. */
@@ -1061,7 +1083,6 @@ ovs_doca_init(const struct smap *ovs_other_config)
     struct doca_flow_definitions_cfg *defs_cfg = NULL;
     unsigned int req_doca_congestion_threshold;
     struct doca_flow_definitions *defs = NULL;
-    unsigned int req_doca_max_eswitch_num;
     uint32_t req_ctl_pipe_infra_size;
     static bool enabled = false;
     struct doca_flow_cfg *cfg;
@@ -1091,8 +1112,6 @@ ovs_doca_init(const struct smap *ovs_other_config)
         return 0;
     }
 
-    offload_provider_api_init();
-
     if (enabled) {
         goto out;
     }
@@ -1196,18 +1215,9 @@ ovs_doca_init(const struct smap *ovs_other_config)
                        doca_congestion_threshold);
     }
 
-    req_doca_max_eswitch_num = smap_get_uint(ovs_other_config, "doca-eswitch-max",
-                                             OVS_DOCA_DEFAULT_MAX_ESW);
-    if (req_doca_max_eswitch_num != ovs_doca_max_eswitch_num) {
-        if (req_doca_max_eswitch_num == 0 ||
-            req_doca_max_eswitch_num > OVS_DOCA_MAX_SUPPORTED_ESW) {
-            VLOG_WARN("other_config doca-eswitch-max=%u not supported, setting as default=%u",
-                      req_doca_max_eswitch_num, OVS_DOCA_DEFAULT_MAX_ESW);
-        } else {
-            ovs_doca_max_eswitch_num = req_doca_max_eswitch_num;
-        }
-    }
-    VLOG_INFO("Maximum supported E-Switches: %u", ovs_doca_max_eswitch_num);
+    ovs_doca_max_shared_encaps = smap_get_uint(ovs_other_config, "doca-max-shared-encaps",
+                                               OVS_DOCA_DEFAULT_MAX_SHARED_ENCAPS);
+    VLOG_INFO("Maximum supported shared encaps: %u", ovs_doca_max_shared_encaps);
 
     ovs_assert(!doca_flow_cfg_create(&cfg));
     /* Offload threads, PMD threads, one management thread. */
@@ -1303,9 +1313,9 @@ ovs_doca_egress_entry_init(struct ovs_doca_netdev_data *dev_data,
     fwd.port_id = port_id;
 
     ovs_doca_mgmt_queue_lock();
-    err = doca_flow_pipe_add_entry(AUX_QUEUE, pipe, &match.d, 0, NULL, NULL, &fwd,
-                                   DOCA_FLOW_NO_WAIT, &dev_data->esw_ctx->queue,
-                                   &entry);
+    err = doca_flow_pipe_basic_add_entry(AUX_QUEUE, pipe, &match.d, 0, NULL, NULL, &fwd,
+                                         DOCA_FLOW_ENTRY_FLAGS_NO_WAIT, &dev_data->esw_ctx->queue,
+                                         &entry);
     ovs_doca_mgmt_queue_unlock();
 
     if (err) {
@@ -1483,9 +1493,11 @@ ovs_doca_port_start(struct netdev *netdev, struct ovs_doca_netdev_data *dev_data
                                                   ovs_doca_max_counters() +
                                                   ovs_doca_max_shared_counters_per_esw());
         ovs_assert(!err);
-        err = doca_flow_port_cfg_set_nr_resources(port_cfg, DOCA_FLOW_RESOURCE_ENCAP,
-                                                  OVS_DOCA_MAX_SHARED_ENCAPS);
-        ovs_assert(!err);
+        if (ovs_doca_max_shared_encaps) {
+            err = doca_flow_port_cfg_set_nr_resources(port_cfg, DOCA_FLOW_RESOURCE_ENCAP,
+                                                      ovs_doca_max_shared_encaps);
+            ovs_assert(!err);
+        }
         err = doca_flow_port_cfg_set_nr_resources(port_cfg, DOCA_FLOW_RESOURCE_METER,
                                                   OVS_DOCA_MAX_METERS_PER_ESW);
         ovs_assert(!err);
@@ -2166,3 +2178,9 @@ ovs_doca_get_esw_n_rxq(struct ovs_doca_netdev_data *dev_data)
 
     return dev_data->esw_ctx->n_rxq;
 }
+
+unsigned int
+ovs_doca_get_max_shared_encaps(void)
+{
+    return ovs_doca_max_shared_encaps;
+}
