@@ -61,6 +61,7 @@ struct clsmap_node {
 struct router_rule {
     uint32_t prio;
     bool invert;
+    bool ipv4;
     bool user;
     uint8_t src_len;
     struct in6_addr from_addr;
@@ -72,6 +73,7 @@ static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(1, 5);
 static struct ovs_mutex mutex = OVS_MUTEX_INITIALIZER;
 static struct cmap clsmap OVS_GUARDED_BY(mutex) = CMAP_INITIALIZER;
 static struct pvector rules;
+static bool rules_enabled = true;
 
 /* By default, use the system routing table.  For system-independent testing,
  * the unit tests disable using the system routing table. */
@@ -196,9 +198,13 @@ ovs_router_lookup(uint32_t mark, const struct in6_addr *ip6_dst,
     }
 
     PVECTOR_FOR_EACH (rule, &rules) {
-        bool matched = !!(!rule->src_len ||
-                          ipv6_addr_equals_masked(&rule->from_addr,
-                                                  from_src, rule->src_len));
+        uint8_t plen;
+        bool matched;
+
+        plen = rule->ipv4 ? rule->src_len + 96 : rule->src_len;
+        matched = (IN6_IS_ADDR_V4MAPPED(from_src) ? rule->ipv4 : true) &&
+                  (!rule->src_len ||
+                   ipv6_addr_equals_masked(&rule->from_addr, from_src, plen));
 
         if (rule->invert) {
             matched = !matched;
@@ -212,9 +218,18 @@ ovs_router_lookup(uint32_t mark, const struct in6_addr *ip6_dst,
                 continue;
             }
             cr = classifier_lookup(cls, OVS_VERSION_MAX, &flow, NULL,
-                                    NULL);
+                                   NULL);
             if (cr) {
-                break;
+                struct ovs_router_entry *p = ovs_router_entry_cast(cr);
+
+                /* Avoid matching mapped IPv4 of a packet against default IPv6
+                 * route entry. Either packet dst is IPv6 or both packet and
+                 * route entry dst are mapped IPv4.
+                 */
+                if (!IN6_IS_ADDR_V4MAPPED(ip6_dst) ||
+                    IN6_IS_ADDR_V4MAPPED(&p->nw_addr)) {
+                    break;
+                }
             }
         }
     }
@@ -884,9 +899,9 @@ ovs_router_rules_show_text(struct ds *ds)
 
     PVECTOR_FOR_EACH (rule, &rules) {
         if (rule->user) {
-            ds_put_format(ds, "User: ");
+            ds_put_format(ds, rule->ipv4 ? "User: " : "User6: ");
         } else {
-            ds_put_format(ds, "Cached: ");
+            ds_put_format(ds, rule->ipv4 ? "Cached: " : "Cached6: ");
         }
         ds_put_format(ds, "%"PRIu32": ", rule->prio);
         if (rule->invert) {
@@ -938,10 +953,15 @@ ovs_router_rule_add_cmd(struct unixctl_conn *conn, int argc OVS_UNUSED,
     struct in6_addr from;
     bool invert = false;
     uint32_t prio = 0;
+    bool ipv4 = true;
     uint32_t table;
     ovs_be32 ip;
     int i = 1;
 
+    if (ovs_scan(argv[i], "-6")) {
+        ipv4 = false;
+        i++;
+    }
     if (ovs_scan(argv[i], "not")) {
         invert = true;
         i++;
@@ -954,8 +974,10 @@ ovs_router_rule_add_cmd(struct unixctl_conn *conn, int argc OVS_UNUSED,
 
         if (scan_ipv4_route(arg, &ip, &src_len)) {
             in6_addr_set_mapped_ipv4(&from, ip);
-            src_len += 96;
-        } else if (!scan_ipv6_route(arg, &from, &src_len)) {
+            ipv4 = true;
+        } else if (scan_ipv6_route(arg, &from, &src_len)) {
+            ipv4 = false;
+        } else {
             unixctl_command_reply_error(conn, "Invalid from=ip/plen");
             return;
         }
@@ -1000,7 +1022,7 @@ ovs_router_rule_add_cmd(struct unixctl_conn *conn, int argc OVS_UNUSED,
     }
 
     ovs_mutex_lock(&mutex);
-    ovs_router_rule_add(prio, invert, true, src_len, &from, table);
+    ovs_router_rule_add(prio, invert, true, src_len, &from, table, ipv4);
     ovs_mutex_unlock(&mutex);
     unixctl_command_reply(conn, "OK");
 }
@@ -1013,11 +1035,16 @@ ovs_router_rule_del_cmd(struct unixctl_conn *conn, int argc OVS_UNUSED,
     struct in6_addr from;
     bool invert = false;
     uint32_t prio = 0;
+    bool ipv4 = true;
     uint32_t table;
     ovs_be32 ip;
     int i = 1;
     int err;
 
+    if (ovs_scan(argv[i], "-6")) {
+        ipv4 = false;
+        i++;
+    }
     if (ovs_scan(argv[i], "not")) {
         invert = true;
         i++;
@@ -1030,8 +1057,10 @@ ovs_router_rule_del_cmd(struct unixctl_conn *conn, int argc OVS_UNUSED,
 
         if (scan_ipv4_route(arg, &ip, &src_len)) {
             in6_addr_set_mapped_ipv4(&from, ip);
-            src_len += 96;
-        } else if (!scan_ipv6_route(arg, &from, &src_len)) {
+            ipv4 = true;
+        } else if (scan_ipv6_route(arg, &from, &src_len)) {
+            ipv4 = false;
+        } else {
             unixctl_command_reply_error(conn, "Invalid from=ip/plen");
             return;
         }
@@ -1063,7 +1092,7 @@ ovs_router_rule_del_cmd(struct unixctl_conn *conn, int argc OVS_UNUSED,
     }
 
     ovs_mutex_lock(&mutex);
-    err = ovs_router_rule_del(prio, invert, src_len, &from, table);
+    err = ovs_router_rule_del(prio, invert, src_len, &from, table, ipv4);
     ovs_mutex_unlock(&mutex);
 
     if (err) {
@@ -1171,9 +1200,10 @@ static void
 init_standard_rules(void)
 {
     /* Add default rules using same priorities as Linux kernel does. */
-    ovs_router_rule_add(0, false, false, 0, &in6addr_any, CLS_LOCAL);
-    ovs_router_rule_add(0x7FFE, false, false, 0, &in6addr_any, CLS_MAIN);
-    ovs_router_rule_add(0x7FFF, false, false, 0, &in6addr_any, CLS_DEFAULT);
+    ovs_router_rule_add(0, false, false, 0, &in6addr_any, CLS_LOCAL, true);
+    ovs_router_rule_add(0x7FFE, false, false, 0, &in6addr_any, CLS_MAIN, true);
+    ovs_router_rule_add(0x7FFF, false, false, 0, &in6addr_any, CLS_DEFAULT,
+                        true);
 }
 
 static void
@@ -1246,7 +1276,8 @@ rule_pvec_prio(uint32_t prio)
 
 void
 ovs_router_rule_add(uint32_t prio, bool invert, bool user, uint8_t src_len,
-                    const struct in6_addr *from, uint32_t lookup_table)
+                    const struct in6_addr *from, uint32_t lookup_table,
+                    bool ipv4)
 {
     struct router_rule *rule = xzalloc(sizeof *rule);
 
@@ -1256,6 +1287,7 @@ ovs_router_rule_add(uint32_t prio, bool invert, bool user, uint8_t src_len,
     rule->src_len = src_len;
     rule->from_addr = *from;
     rule->lookup_table = lookup_table;
+    rule->ipv4 = ipv4;
 
     pvector_insert(&rules, rule, rule_pvec_prio(prio));
     pvector_publish(&rules);
@@ -1263,7 +1295,8 @@ ovs_router_rule_add(uint32_t prio, bool invert, bool user, uint8_t src_len,
 
 int
 ovs_router_rule_del(uint32_t prio, bool invert, uint8_t src_len,
-                    const struct in6_addr *from, uint32_t lookup_table)
+                    const struct in6_addr *from, uint32_t lookup_table,
+                    bool ipv4)
 {
     struct router_rule *rule;
 
@@ -1274,6 +1307,7 @@ ovs_router_rule_del(uint32_t prio, bool invert, uint8_t src_len,
         if (rule->user
             && rule->invert == invert
             && (!prio || rule->prio == prio)
+            && rule->ipv4 == ipv4
             && rule->src_len == src_len
             && ipv6_addr_equals(&rule->from_addr, from)
             && rule->lookup_table == lookup_table) {
@@ -1287,6 +1321,18 @@ ovs_router_rule_del(uint32_t prio, bool invert, uint8_t src_len,
     return -ENOENT;
 }
 
+void
+ovs_router_rules_enable(bool enable)
+{
+    rules_enabled = enable;
+}
+
+bool
+ovs_router_rules_enabled(void)
+{
+    return rules_enabled;
+}
+
 void
 ovs_router_init(void)
 {
@@ -1314,11 +1360,11 @@ ovs_router_init(void)
         unixctl_command_register("ovs/route/rule/show", "", 0, 0,
                                  ovs_router_rules_show, NULL);
         unixctl_command_register("ovs/route/rule/add",
-                                 "[not] from=all|ip/plen [prio=num] "
+                                 "[-6] [not] from=all|ip/plen [prio=num] "
                                  "table=local|main|default|id",
                                  2, 5, ovs_router_rule_add_cmd, NULL);
         unixctl_command_register("ovs/route/rule/del",
-                                 "[not] from=all|ip/plen [prio=num] "
+                                 "[-6] [not] from=all|ip/plen [prio=num] "
                                  "table=local|main|default|id",
                                  2, 5, ovs_router_rule_del_cmd, NULL);
         ovsthread_once_done(&once);
