@@ -149,20 +149,28 @@ BUILD_ASSERT_DECL(offsetof(struct pkt_metadata, cacheline2) ==
 static inline void
 pkt_metadata_init_tnl(struct pkt_metadata *md)
 {
+    struct nvidia_multiplane_info nv_mp;
     odp_port_t orig_in_port;
     bool partial;
     bool sample;
 
     /* Zero up through the tunnel metadata options. The length and table
      * are before this and as long as they are empty, the options won't
-     * be looked at. Keep the orig_in_port and sample fields. */
+     * be looked at.
+     * Preserve some fields:
+     * - orig_in_port.
+     * - sample.
+     * - nv_mp.
+     */
     orig_in_port = md->orig_in_port;
     sample = md->sample;
     partial = md->partial;
+    memcpy(&nv_mp, &md->nv_mp, sizeof nv_mp);
     memset(md, 0, offsetof(struct pkt_metadata, tunnel.metadata.opts));
     md->orig_in_port = orig_in_port;
     md->sample = sample;
     md->partial = partial;
+    memcpy(&md->nv_mp, &nv_mp, sizeof nv_mp);
 }
 
 static inline void
@@ -199,9 +207,6 @@ pkt_metadata_reset(struct pkt_metadata *md, odp_port_t port)
     md->pkt_mark = 0;
     md->ct_state = 0;
     md->partial = false;
-    md->nv_mp.pid = 0;
-    md->nv_mp.preferred = 0;
-    md->nv_mp.strict = 0;
 
     /* It can be expensive to zero out all of the tunnel metadata. However,
      * we can just zero out ip_dst and the rest of the data will never be
@@ -218,6 +223,7 @@ pkt_metadata_init(struct pkt_metadata *md, odp_port_t port)
     pkt_metadata_reset(md, port);
     md->orig_in_port = port;
     md->sample = false;
+    memset(&md->nv_mp, 0, sizeof md->nv_mp);
 }
 
 /* This function prefetches the cachelines touched by pkt_metadata_init()
