@@ -18,6 +18,7 @@ int check_pci_id_range(unsigned short pci_device_id, unsigned short id_range_sta
 int is_connectx(unsigned short pci_device_id);
 int is_connectx3(unsigned short pci_device_id);
 int is_bluefield(unsigned short pci_device_id);
+int is_bluefield4(unsigned short pci_device_id);
 int is_pcie_switch(unsigned short pci_device_id);
 int is_quantum(unsigned short pci_device_id);
 int is_spectrum(unsigned short pci_device_id);
@@ -285,7 +286,15 @@ int is_connectx3(unsigned short pci_device_id)
 int is_bluefield(unsigned short pci_device_id)
 {
     return check_pci_id_range(pci_device_id, BLUEFIELD_PCI_ID) ||
-           check_pci_id_range(pci_device_id, BLUEFIELD_DPU_AUX_PCI_ID);
+           check_pci_id_range(pci_device_id, BLUEFIELD_DPU_AUX_PCI_ID) || is_bluefield4(pci_device_id);
+}
+
+int is_bluefield4(unsigned short pci_device_id)
+{
+    return check_pci_id_range(pci_device_id, BLUEFIELD4_CRYPTO_ENABLED_PCI_ID) ||
+           check_pci_id_range(pci_device_id, BLUEFIELD4_CRYPTO_DISABLED_PCI_ID) ||
+           check_pci_id_range(pci_device_id, BLUEFIELD4_NETWORK_CONTROLLER_PCI_ID) ||
+           check_pci_id_range(pci_device_id, BLUEFIELD4_MANAGMENT_INTERFACE_PCI_ID);
 }
 
 int is_pcie_switch(unsigned short pci_device_id)
@@ -377,13 +386,13 @@ int is_pciconf_device(unsigned short pci_device_id)
 int is_pcicr_device(unsigned short pci_device_id)
 {
     int result = (is_gpu_pci_device(pci_device_id) || is_switch(pci_device_id) || is_toolspf(pci_device_id) ||
-                  is_connectx3(pci_device_id)) &&
-                 !(is_livefish_device(pci_device_id) && !is_livefish_device_with_supported_bar_access(pci_device_id));
+                  is_connectx3(pci_device_id)) ||
+                 (is_livefish_device(pci_device_id) && is_livefish_device_with_supported_bar_access(pci_device_id));
 
     DEBUG_PRINTK(
-      "is_pcicr_device check for device 0x%x: result=%d (is_gpu: %d, is_toolspf: %d, is_connectx3: %d, !is_livefish: %d, !is_livefish_device_with_supported_bar_access: %d)\n",
+      "is_pcicr_device check for device 0x%x: result=%d (is_gpu: %d, is_toolspf: %d, is_connectx3: %d, is_livefish: %d, is_livefish_device_with_supported_bar_access: %d)\n",
       pci_device_id, result, is_gpu_pci_device(pci_device_id), is_toolspf(pci_device_id), is_connectx3(pci_device_id),
-      !is_livefish_device(pci_device_id), !is_livefish_device_with_supported_bar_access(pci_device_id));
+      is_livefish_device(pci_device_id), is_livefish_device_with_supported_bar_access(pci_device_id));
 
     return result;
 }
