@@ -13,12 +13,10 @@ typedef struct pci_id_range_st
     unsigned int upper_bound;
 } pci_id_range;
 
-pci_id_range GB100_PCI_IDS[] = {{0x2900, 0x29FF}, {0x2B00, 0x2BFF}, {0x2C00, 0x2CFF}, {0x2D00, 0x2DFF},
-                                {0x2E00, 0x2EFF}, {0x2F00, 0x2F7F}, {0x3180, 0x31FF}, {0x3200, 0x327F},
-                                {0x3300, 0x33FF}, {0x3400, 0x347F}};
+pci_id_range GB100_PCI_IDS[] = {{0x2900, 0x29FF}, {0x2B00, 0x2B7F}, {0x3180, 0x31FF},
+                                {0x3200, 0x327F}, {0x3300, 0x33FF}, {0x3400, 0x347F}};
 
-pci_id_range GR100_PCI_IDS[] = {{0x3000, 0x30FF}, {0x3280, 0x32FF}, {0x3900, 0x397F}, {0x3580, 0x35FF},
-                                {0x3600, 0x36FF}, {0x3700, 0x377F}, {0x3880, 0x38FF}};
+pci_id_range GR100_PCI_IDS[] = {{0x3000, 0x30FF}};
 
 pci_id_range GR150_PCI_IDS[] = {{0x3480, 0x34BF}, {0x34C0, 0x34FF}, {0x3500, 0x353F}, {0x3540, 0x357F}};
 
