@@ -439,6 +439,7 @@ tnl_port_send(const struct ofport_dpif *ofport, struct flow *flow,
             flow->tunnel.ipv6_dst = in6addr_any;
         }
     }
+    flow->tunnel.tp_src = 0; /* Do not carry from a previous tunnel. */
     if (!cfg->is_explicit) {
         flow->tunnel.tp_dst = cfg->dst_port;
         if (!cfg->out_key_flow) {
@@ -469,6 +470,7 @@ tnl_port_send(const struct ofport_dpif *ofport, struct flow *flow,
         } else {
             flow->tunnel.ip_tos |= flow->nw_tos & IP_ECN_MASK;
         }
+        wc->masks.tunnel.ip_tos |= IP_ECN_MASK;
     }
 
     flow->tunnel.flags &= ~(FLOW_TNL_F_MASK & ~FLOW_TNL_PUB_F_MASK);
