@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -194,12 +194,13 @@ int hw_queue_tx::configure(const slave_data_t *slave)
 
     // In case of enabled TSO we need to take into account amount of SGE together with header inline
     // Per PRM maximum of CTRL + ETH + ETH_HEADER_INLINE+DATA_PTR*NUM_SGE+MAX_INLINE+INLINE_SIZE
-    // MLX5 return 32678 WQEBBs at max so minimal number
+    // Converting WQEBBs to WQEs for ibv_create_qp API.
     int max_wqe_sz =
         16 + 14 + 16 * qp_init_attr.cap.max_send_sge + qp_init_attr.cap.max_inline_data + 4;
     max_wqe_sz += (m_p_ring->is_tso() ? m_p_ring->m_tso.max_header_sz : 94);
-    int num_wr = 32678 * 64 / max_wqe_sz;
-    hwqtx_logdbg("calculated max_wqe_sz=%d num_wr=%d", max_wqe_sz, num_wr);
+    int num_wr = m_p_ib_ctx_handler->get_max_sq_wqebbs() * 64 / max_wqe_sz;
+    hwqtx_logdbg("calculated max_wqe_sz=%d num_wr=%d, m_tx_num_wr=%u", max_wqe_sz, num_wr,
+                 m_tx_num_wr);
     if (num_wr < (signed)m_tx_num_wr) {
         qp_init_attr.cap.max_send_wr =
             num_wr; // force min for create_qp or you will have error of memory allocation
@@ -420,6 +421,7 @@ void hw_queue_tx::init_queue()
 
     // We use the min between CQ size and the QP size (that might be increases by ibv creation).
     m_sq_free_credits = std::min(m_tx_num_wr, old_wr_val);
+    hwqtx_logdbg("SQ total credits: %u", m_sq_free_credits);
 
     /* Maximum BF inlining consists of:
      * - CTRL:
@@ -440,7 +442,7 @@ void hw_queue_tx::init_queue()
             hwqtx_logerr("Failed allocating m_sq_wqe_idx_to_prop (errno=%d %m)", errno);
             return;
         }
-        m_sq_wqe_prop_last_signalled = m_tx_num_wr - 1;
+        m_last_sq_wqe_prop_to_complete = m_sq_wqe_idx_to_prop;
         m_sq_wqe_prop_last = nullptr;
     }
 
@@ -465,13 +467,6 @@ void hw_queue_tx::init_device_memory()
     }
 }
 
-void hw_queue_tx::update_next_wqe_hot()
-{
-    // Preparing pointer to the next WQE after a doorbell
-    m_sq_wqe_hot = &(*m_sq_wqes)[m_sq_wqe_counter & (m_tx_num_wr - 1)];
-    m_sq_wqe_hot_index = m_sq_wqe_counter & (m_tx_num_wr - 1);
-}
-
 cq_mgr_tx *hw_queue_tx::init_tx_cq_mgr()
 {
     m_tx_num_wr = align32pow2(m_tx_num_wr);
@@ -479,7 +474,7 @@ cq_mgr_tx *hw_queue_tx::init_tx_cq_mgr()
                          m_p_ring->get_tx_comp_event_channel());
 }
 
-inline void hw_queue_tx::ring_doorbell(int num_wqebb, bool skip_comp /*=false*/)
+inline void hw_queue_tx::ring_doorbell(uint8_t num_wqebb, bool skip_comp /*=false*/)
 {
     uint64_t *dst = (uint64_t *)m_mlx5_qp.bf.reg;
     uint64_t *src = reinterpret_cast<uint64_t *>(m_sq_wqe_hot);
@@ -537,7 +532,7 @@ inline int hw_queue_tx::fill_inl_segment(sg_array &sga, uint8_t *cur_seg, uint8_
 }
 
 //! Fill WQE dynamically, based on amount of free WQEBB in SQ
-inline int hw_queue_tx::fill_wqe(xlio_ibv_send_wr *pswr)
+inline uint8_t hw_queue_tx::fill_wqe(xlio_ibv_send_wr *pswr)
 {
     // control segment is mostly filled by preset after previous packet
     // we always inline ETH header
@@ -550,6 +545,7 @@ inline int hw_queue_tx::fill_wqe(xlio_ibv_send_wr *pswr)
 
     // assume packet is full inline
     if (data_len <= max_inline_len && xlio_send_wr_opcode(*pswr) == XLIO_IBV_WR_SEND) {
+        /* coverity[returned_null] */
         uint8_t *data_addr = sga.get_data(&inline_len); // data for inlining in ETH header
         data_len -= inline_len;
         hwqtx_logfunc(
@@ -562,6 +558,7 @@ inline int hw_queue_tx::fill_wqe(xlio_ibv_send_wr *pswr)
 
         // Fill Ethernet segment with header inline, static data
         // were populated in preset after previous packet send
+        /* coverity[null_deref] */
         memcpy(cur_seg + offsetof(struct mlx5_wqe_eth_seg, inline_hdr_start), data_addr,
                MLX5_ETH_INLINE_HEADER_SIZE);
         cur_seg += sizeof(struct mlx5_wqe_eth_seg);
@@ -587,14 +584,9 @@ inline int hw_queue_tx::fill_wqe(xlio_ibv_send_wr *pswr)
                           sizeof(struct mlx5_wqe_eth_seg)) = htonl(0x80000000 | inline_len);
             rest_space = align_to_octoword_up(inline_len + 4); // align to OCTOWORDs
             wqe_size += rest_space / OCTOWORD;
-            // assert((data_len-inline_len)==0);
-            // configuring control
-            m_sq_wqe_hot->ctrl.data[1] = htonl((m_mlx5_qp.qpn << 8) | wqe_size);
-            rest_space = align_to_WQEBB_up(wqe_size) / 4;
-            hwqtx_logfunc("data_len: %d inline_len: %d wqe_size: %d wqebbs: %d",
-                          data_len - inline_len, inline_len, wqe_size, rest_space);
-            ring_doorbell(rest_space);
-            return rest_space;
+
+            hwqtx_logfunc("data_len: %d inline_len: %d wqe_size: %d", data_len - inline_len,
+                          inline_len, wqe_size * OCTOWORD);
         } else {
             // wrap around case, first filling till the end of m_sq_wqes
             int wrap_up_size = max_inline_len - rest_space;
@@ -608,14 +600,11 @@ inline int hw_queue_tx::fill_wqe(xlio_ibv_send_wr *pswr)
             data_len -= inline_len;
             rest_space = align_to_octoword_up(inline_len + 4);
             wqe_size += rest_space / OCTOWORD;
-            rest_space =
-                align_to_WQEBB_up(rest_space / OCTOWORD) / 4; // size of 1st chunk at the end
 
-            hwqtx_logfunc(
-                "END chunk data_addr: %p data_len: %d inline_len: %d wqe_size: %d wqebbs: %d",
-                data_addr, data_len, inline_len, wqe_size, rest_space);
+            hwqtx_logfunc("END chunk data_addr: %p data_len: %d inline_len: %d wqe_size: %d",
+                          data_addr, data_len, inline_len, wqe_size * OCTOWORD);
+
             // Wrap around
-            //
             cur_seg = (uint8_t *)m_sq_wqes;
             data_addr = sga.get_data(&wrap_up_size);
 
@@ -623,40 +612,35 @@ inline int hw_queue_tx::fill_wqe(xlio_ibv_send_wr *pswr)
             inline_len += wrap_up_size;
             max_inline_len = align_to_octoword_up(wrap_up_size);
             wqe_size += max_inline_len / OCTOWORD;
-            max_inline_len = align_to_WQEBB_up(max_inline_len / OCTOWORD) / 4;
-            // store inline data size
+
+            // Store inline data size
             *(uint32_t *)((uint8_t *)m_sq_wqe_hot + sizeof(struct mlx5_wqe_ctrl_seg) +
                           sizeof(struct mlx5_wqe_eth_seg)) = htonl(0x80000000 | inline_len);
-            hwqtx_logfunc("BEGIN_CHUNK data_addr: %p data_len: %d wqe_size: %d inline_len: %d "
-                          "end_wqebbs: %d wqebbs: %d",
-                          data_addr, data_len - wrap_up_size, wqe_size, inline_len + wrap_up_size,
-                          rest_space, max_inline_len);
-            // assert((data_len-wrap_up_size)==0);
-            // configuring control
-            m_sq_wqe_hot->ctrl.data[1] = htonl((m_mlx5_qp.qpn << 8) | wqe_size);
 
-            dbg_dump_wqe((uint32_t *)m_sq_wqe_hot, rest_space * 4 * 16);
-            dbg_dump_wqe((uint32_t *)m_sq_wqes, max_inline_len * 4 * 16);
+            hwqtx_logfunc("BEGIN_CHUNK data_addr: %p data_len: %d wqe_size: %d inline_len: %d",
+                          data_addr, data_len - wrap_up_size, wqe_size * OCTOWORD,
+                          inline_len + wrap_up_size);
 
-            ring_doorbell(rest_space + max_inline_len);
-            return rest_space + max_inline_len;
+            dbg_dump_wqe(
+                (uint32_t *)m_sq_wqe_hot,
+                sizeof(struct mlx5_wqe_ctrl_seg) + sizeof(struct mlx5_wqe_eth_seg) + rest_space);
+            dbg_dump_wqe((uint32_t *)m_sq_wqes, max_inline_len);
         }
+    } else if (xlio_send_wr_opcode(*pswr) == XLIO_IBV_WR_SEND) {
+        // Data is bigger than max to inline we inlined only ETH header + uint from IP (18
+        // bytes) the rest will be in data pointer segment adding data seg with pointer if there
+        // still data to transfer
+        wqe_size = fill_wqe_send(pswr);
     } else {
-        if (xlio_send_wr_opcode(*pswr) == XLIO_IBV_WR_SEND) {
-            /* data is bigger than max to inline we inlined only ETH header + uint from IP (18
-             * bytes) the rest will be in data pointer segment adding data seg with pointer if there
-             * still data to transfer
-             */
-            wqe_size = fill_wqe_send(pswr);
-            return wqe_size;
-        } else {
-            /* Support XLIO_IBV_WR_SEND_TSO operation
-             */
-            wqe_size = fill_wqe_lso(pswr, data_len);
-            return wqe_size;
-        }
+        // Support XLIO_IBV_WR_SEND_TSO operation
+        wqe_size = fill_wqe_lso(pswr, data_len);
     }
-    return 1;
+
+    m_sq_wqe_hot->ctrl.data[1] = htonl((m_mlx5_qp.qpn << 8) | wqe_size);
+
+    uint8_t wqebbs = static_cast<uint8_t>(align_to_WQEBB_up(wqe_size) / 4);
+
+    return wqebbs;
 }
 
 inline int hw_queue_tx::fill_wqe_send(xlio_ibv_send_wr *pswr)
@@ -694,11 +678,7 @@ inline int hw_queue_tx::fill_wqe_send(xlio_ibv_send_wr *pswr)
         }
     }
 
-    m_sq_wqe_hot->ctrl.data[1] = htonl((m_mlx5_qp.qpn << 8) | wqe_size);
-    int wqebbs = align_to_WQEBB_up(wqe_size) / 4;
-    ring_doorbell(wqebbs);
-
-    return wqebbs;
+    return wqe_size;
 }
 
 //! Filling wqe for LSO
@@ -769,25 +749,8 @@ inline int hw_queue_tx::fill_wqe_lso(xlio_ibv_send_wr *pswr, int data_len)
         dpseg++;
         wqe_size += sizeof(struct mlx5_wqe_data_seg) / OCTOWORD;
     }
-    m_sq_wqe_hot->ctrl.data[1] = htonl((m_mlx5_qp.qpn << 8) | wqe_size);
 
-    int wqebbs = align_to_WQEBB_up(wqe_size) / 4;
-    ring_doorbell(wqebbs);
-    return wqebbs;
-}
-
-void hw_queue_tx::store_current_wqe_prop(mem_buf_desc_t *buf, unsigned credits, xlio_ti *ti)
-{
-    m_sq_wqe_idx_to_prop[m_sq_wqe_hot_index] = sq_wqe_prop {
-        .buf = buf,
-        .credits = credits,
-        .ti = ti,
-        .next = m_sq_wqe_prop_last,
-    };
-    m_sq_wqe_prop_last = &m_sq_wqe_idx_to_prop[m_sq_wqe_hot_index];
-    if (ti) {
-        ti->get();
-    }
+    return wqe_size;
 }
 
 //! Send one RAW packet
@@ -820,20 +783,39 @@ void hw_queue_tx::send_to_wire(xlio_ibv_send_wr *p_send_wqe, xlio_wr_tx_packet_a
     eseg->rsvd2 = 0;
     eseg->cs_flags = (uint8_t)(attr & (XLIO_TX_PACKET_L3_CSUM | XLIO_TX_PACKET_L4_CSUM) & 0xff);
 
+    submit_wqe(reinterpret_cast<mem_buf_desc_t *>(p_send_wqe->wr_id), credits, fill_wqe(p_send_wqe),
+               tis, false);
+}
+
+inline void hw_queue_tx::submit_wqe(mem_buf_desc_t *buf, unsigned credits, uint8_t wqebbs,
+                                    xlio_ti *ti, bool skip_comp)
+{
     /* Store buffer descriptor */
-    store_current_wqe_prop(reinterpret_cast<mem_buf_desc_t *>(p_send_wqe->wr_id), credits, tis);
+    m_sq_wqe_idx_to_prop[m_sq_wqe_hot_index] = sq_wqe_prop {
+        .buf = buf,
+        .credits = credits,
+        .wqebbs = wqebbs,
+        .ti = ti,
+        .next = m_sq_wqe_prop_last,
+    };
 
-    /* Complete WQE */
-    int wqebbs = fill_wqe(p_send_wqe);
-    assert(wqebbs > 0 && (unsigned)wqebbs <= credits);
-    NOT_IN_USE(wqebbs);
+    m_sq_wqe_prop_last = &m_sq_wqe_idx_to_prop[m_sq_wqe_hot_index];
 
-    update_next_wqe_hot();
+    if (ti) {
+        ti->get();
+    }
+
+    assert(wqebbs > 0 && wqebbs <= credits);
+    ring_doorbell(wqebbs, skip_comp);
+
+    // Preparing pointer to the next WQE after a doorbell
+    m_sq_wqe_hot = &(*m_sq_wqes)[m_sq_wqe_counter & (m_tx_num_wr - 1)];
+    m_sq_wqe_hot_index = m_sq_wqe_counter & (m_tx_num_wr - 1);
 
     hwqtx_logfunc(
-        "m_sq_wqe_hot: %p m_sq_wqe_hot_index: %d wqe_counter: %d new_hot_index: %d wr_id: %llx",
+        "m_sq_wqe_hot: %p m_sq_wqe_hot_index: %d wqe_counter: %d new_hot_index: %d buf: %p",
         m_sq_wqe_hot, m_sq_wqe_hot_index, m_sq_wqe_counter, (m_sq_wqe_counter & (m_tx_num_wr - 1)),
-        p_send_wqe->wr_id);
+        buf);
 }
 
 std::unique_ptr<xlio_tis> hw_queue_tx::create_tis(uint32_t flags)
@@ -1151,12 +1133,10 @@ inline void hw_queue_tx::tls_post_static_params_wqe(xlio_ti *ti, const struct xl
     memset(tspseg, 0, sizeof(*tspseg));
 
     tls_fill_static_params_wqe(tspseg, info, key_id, resync_tcp_sn);
-    store_current_wqe_prop(nullptr, SQ_CREDITS_UMR, ti);
 
-    ring_doorbell(TLS_SET_STATIC_PARAMS_WQEBBS, true);
     dbg_dump_wqe((uint32_t *)m_sq_wqe_hot, sizeof(mlx5_set_tls_static_params_wqe));
 
-    update_next_wqe_hot();
+    submit_wqe(nullptr, SQ_CREDITS_UMR, TLS_SET_STATIC_PARAMS_WQEBBS, ti, true);
 }
 
 inline void hw_queue_tx::tls_fill_progress_params_wqe(
@@ -1195,12 +1175,10 @@ inline void hw_queue_tx::tls_post_progress_params_wqe(xlio_ti *ti, uint32_t tis_
         (fence ? MLX5_FENCE_MODE_INITIATOR_SMALL : 0) | (is_tx ? 0 : MLX5_WQE_CTRL_CQ_UPDATE);
 
     tls_fill_progress_params_wqe(&wqe->params, tis_tir_number, next_record_tcp_sn);
-    store_current_wqe_prop(nullptr, SQ_CREDITS_SET_PSV, ti);
 
-    ring_doorbell(TLS_SET_PROGRESS_PARAMS_WQEBBS);
     dbg_dump_wqe((uint32_t *)m_sq_wqe_hot, sizeof(mlx5_set_tls_progress_params_wqe));
 
-    update_next_wqe_hot();
+    submit_wqe(nullptr, SQ_CREDITS_SET_PSV, TLS_SET_PROGRESS_PARAMS_WQEBBS, ti, false);
 }
 
 inline void hw_queue_tx::tls_get_progress_params_wqe(xlio_ti *ti, uint32_t tirn, void *buf,
@@ -1226,11 +1204,7 @@ inline void hw_queue_tx::tls_get_progress_params_wqe(xlio_ti *ti, uint32_t tirn,
     psv->psv_index[0] = htobe32(tirn);
     psv->va = htobe64((uintptr_t)buf);
 
-    store_current_wqe_prop(nullptr, SQ_CREDITS_GET_PSV, ti);
-
-    ring_doorbell(TLS_GET_PROGRESS_WQEBBS);
-
-    update_next_wqe_hot();
+    submit_wqe(nullptr, SQ_CREDITS_GET_PSV, TLS_GET_PROGRESS_WQEBBS, ti, false);
 }
 
 void hw_queue_tx::tls_tx_post_dump_wqe(xlio_tis *tis, void *addr, uint32_t len, uint32_t lkey,
@@ -1290,11 +1264,7 @@ void hw_queue_tx::post_nop_fence(void)
     cseg->qpn_ds = htobe32((m_mlx5_qp.qpn << MLX5_WQE_CTRL_QPN_SHIFT) | 0x01);
     cseg->fm_ce_se = MLX5_FENCE_MODE_INITIATOR_SMALL;
 
-    store_current_wqe_prop(nullptr, SQ_CREDITS_NOP, nullptr);
-
-    ring_doorbell(1);
-
-    update_next_wqe_hot();
+    submit_wqe(nullptr, SQ_CREDITS_NOP, XLIO_NOP_WQEBBS, nullptr, false);
 }
 
 void hw_queue_tx::post_dump_wqe(xlio_tis *tis, void *addr, uint32_t len, uint32_t lkey,
@@ -1317,11 +1287,7 @@ void hw_queue_tx::post_dump_wqe(xlio_tis *tis, void *addr, uint32_t len, uint32_
     dseg->lkey = htobe32(lkey);
     dseg->byte_count = htobe32(len);
 
-    store_current_wqe_prop(nullptr, SQ_CREDITS_DUMP, tis);
-
-    ring_doorbell(XLIO_DUMP_WQEBBS, true);
-
-    update_next_wqe_hot();
+    submit_wqe(nullptr, SQ_CREDITS_DUMP, XLIO_DUMP_WQEBBS, tis, true);
 }
 
 //! Handle releasing of Tx buffers
@@ -1384,26 +1350,6 @@ void hw_queue_tx::trigger_completion_for_all_sent_packets()
     }
 }
 
-void hw_queue_tx::reset_inflight_zc_buffers_ctx(void *ctx)
-{
-    sq_wqe_prop *p = m_sq_wqe_prop_last;
-    sq_wqe_prop *prev;
-    if (p) {
-        unsigned p_i = p - m_sq_wqe_idx_to_prop;
-        if (p_i == m_sq_wqe_prop_last_signalled) {
-            return;
-        }
-        do {
-            mem_buf_desc_t *desc = p->buf;
-            if (desc && desc->tx.zc.ctx == ctx) {
-                desc->tx.zc.ctx = nullptr;
-            }
-            prev = p;
-            p = p->next;
-        } while (p && is_sq_wqe_prop_valid(p, prev));
-    }
-}
-
 uint32_t hw_queue_tx::is_ratelimit_change(struct xlio_rate_limit_t &rate_limit)
 {
     uint32_t rl_changes = 0;
