@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -42,6 +42,8 @@ struct sq_wqe_prop {
     mem_buf_desc_t *buf;
     /* Number of credits (usually number of WQEBBs). */
     unsigned credits;
+    /* Size of the WQE in WQEBBs. */
+    uint8_t wqebbs;
     /* Transport interface (TIS/TIR) current WQE holds reference to. */
     xlio_ti *ti;
     struct sq_wqe_prop *next;
@@ -101,8 +103,6 @@ public:
     void put_tls_dek(std::unique_ptr<dpcp::tls_dek> &&dek_obj);
 #endif
 
-    void reset_inflight_zc_buffers_ctx(void *ctx);
-
     void credits_return(unsigned credits) { m_sq_free_credits += credits; }
 
     bool credits_get(unsigned credits)
@@ -129,23 +129,40 @@ public:
          *  3. TSO packet with inline headers
          *
          * Formulas details:
-         *  1. WQEBB is 64 bytes, the 1st WQEBB contains ctrl segment, eth segment and 18 bytes of
-         *     inline data. So, we take the 1st WQEBB and number of WQEBBs for the packet minus 18
-         *     bytes.
-         *  2. Data segment for each scatter-gather element is 16 bytes. Therefore, WQEBB can hold
-         *     up to 4 data segments. The 1st element fits into the 1st WQEBB after the eth segment.
-         *     So, we take the 1st WQEBB and number of WQEBBs for scatter-gather elements minus 1.
-         *  3. Inline header starts from offset 46 in WQE (2 bytes before 16 bytes alignment).
-         *     Decrease inline header size by 2 to align it to 16 bytes boundary at the right edge.
-         *     This compensates data segments alignment. Add the 2 bytes back and length of
-         *     scatter-gather elements. Take into account that 18 bytes goes to the 1st WQEBB and
-         *     add the 1st WQEBB to the result.
+         *  1. Inline path (≤204 bytes): 18 bytes of ETH header are inlined into the eth segment.
+         *     WQEBB is 64 bytes, the 1st WQEBB contains ctrl segment (16B), eth segment (32B with
+         *     inline header), and the remaining packet data goes into inline data segment.
+         *     Formula (length + 63 - 18) / 64 + 1 accounts for the 18 bytes already inlined.
+         *
+         *  2. Scatter-gather path (>204 bytes): NO inline header (inline_hdr_sz = 0).
+         *     ETH segment is just 16 bytes (1 octoword). Data segment for each scatter-gather
+         *     element is 16 bytes. Therefore, WQEBB can hold up to 4 data segments.
+         *     The 1st data segment fits into the 1st WQEBB after ctrl (1 octoword) + eth (1
+         * octoword). Formula: (num_sge + 3 - 1) / 4 + 1 accounts for ctrl + eth + N data segments.
          */
         if (xlio_send_wr_opcode(*p_send_wqe) != XLIO_IBV_WR_TSO) {
-            if (p_send_wqe->num_sge == 1 && p_send_wqe->sg_list->length <= 204) {
-                return (p_send_wqe->sg_list->length + 63U - 18U) / 64U + 1U;
+            if (p_send_wqe->num_sge == 1) {
+                if (p_send_wqe->sg_list->length <= 204) {
+                    // Single-SGE inline path (fast path for small packets)
+                    return (p_send_wqe->sg_list->length + 63U - 18U) / 64U + 1U;
+                } else {
+                    // 1 data segment fits into 1 WQEBB
+                    return 1U;
+                }
             } else {
-                return (p_send_wqe->num_sge + 3U - 1U) / 4U + 1U;
+                // Multi-SGE: Calculate total to differentiate inline vs scatter-gather
+                uint32_t total_len = 0;
+                for (int i = 0; i < p_send_wqe->num_sge; i++) {
+                    total_len += p_send_wqe->sg_list[i].length;
+                }
+
+                if (total_len <= 204) {
+                    // Multi-SGE inline path
+                    return (total_len + 63U - 18U) / 64U + 1U;
+                } else {
+                    // Multi-SGE scatter-gather: 1 ctrl + 1 eth + N data segments
+                    return (p_send_wqe->num_sge + 3U - 1U) / 4U + 1U;
+                }
             }
         } else {
             return (((p_send_wqe->tso.hdr_sz + 15U - 2U) & ~15U) + 2U + p_send_wqe->num_sge * 16U -
@@ -163,7 +180,6 @@ private:
     void init_queue();
     void init_device_memory();
     void trigger_completion_for_all_sent_packets();
-    void update_next_wqe_hot();
     void destroy_tis_cache();
     void put_tls_tis_in_cache(xlio_tis *tis);
 
@@ -189,15 +205,6 @@ private:
         }
     }
 
-    bool is_sq_wqe_prop_valid(sq_wqe_prop *p, sq_wqe_prop *prev)
-    {
-        unsigned p_i = p - m_sq_wqe_idx_to_prop;
-        unsigned prev_i = prev - m_sq_wqe_idx_to_prop;
-        return (p_i != m_sq_wqe_prop_last_signalled) &&
-            ((m_tx_num_wr + p_i - m_sq_wqe_prop_last_signalled) % m_tx_num_wr <
-             (m_tx_num_wr + prev_i - m_sq_wqe_prop_last_signalled) % m_tx_num_wr);
-    }
-
 #if defined(DEFINED_UTLS)
     inline void tls_fill_static_params_wqe(struct mlx5_wqe_tls_static_params_seg *params,
                                            const struct xlio_tls_info *info, uint32_t key_id,
@@ -212,13 +219,14 @@ private:
     inline void tls_get_progress_params_wqe(xlio_ti *ti, uint32_t tirn, void *buf, uint32_t lkey);
 #endif /* DEFINED_UTLS */
 
-    inline void store_current_wqe_prop(mem_buf_desc_t *wr_id, unsigned credits, xlio_ti *ti);
-    inline int fill_wqe(xlio_ibv_send_wr *p_send_wqe);
+    inline void submit_wqe(mem_buf_desc_t *buf, unsigned credits, uint8_t wqebbs, xlio_ti *ti,
+                           bool skip_comp);
+    inline uint8_t fill_wqe(xlio_ibv_send_wr *p_send_wqe);
     inline int fill_wqe_send(xlio_ibv_send_wr *pswr);
     inline int fill_wqe_lso(xlio_ibv_send_wr *pswr, int data_len);
     inline int fill_inl_segment(sg_array &sga, uint8_t *cur_seg, uint8_t *data_addr,
                                 int max_inline_len, int inline_len);
-    inline void ring_doorbell(int num_wqebb, bool skip_comp = false);
+    inline void ring_doorbell(uint8_t num_wqebb, bool skip_comp = false);
 
     struct xlio_rate_limit_t m_rate_limit;
     xlio_ib_mlx5_qp_t m_mlx5_qp;
@@ -229,13 +237,24 @@ private:
     sq_wqe_prop *m_sq_wqe_idx_to_prop = nullptr;
     sq_wqe_prop *m_sq_wqe_prop_last = nullptr;
 
+    /**
+     * @brief Keeps track of the last Send Queue WQE (Work Queue Element) to be completed.
+     *
+     * This pointer is used to mark the last WQE in the chain that needs to be completed
+     * by a completion event (CQE). For example, if WQEs are posted in order | WQE1 | WQE2 | WQE3 |,
+     * m_last_sq_wqe_prop_to_complete will initially point to WQE1. When a CQE is received for WQE3,
+     * the completion logic will process and complete WQE3, WQE2, WQE1 (from right to left) until
+     * reaching m_last_sq_wqe_prop_to_complete. After that, m_last_sq_wqe_prop_to_complete is
+     * updated to point to the index where the next WQE (e.g., WQE4) should be tracked.
+     */
+    sq_wqe_prop *m_last_sq_wqe_prop_to_complete = nullptr;
+
     struct mlx5_eth_wqe (*m_sq_wqes)[] = nullptr;
     struct mlx5_eth_wqe *m_sq_wqe_hot = nullptr;
     uint8_t *m_sq_wqes_end = nullptr;
 
     const uint32_t m_n_sysvar_tx_num_wr_to_signal;
     uint32_t m_tx_num_wr;
-    unsigned m_sq_wqe_prop_last_signalled = 0U;
     unsigned m_sq_free_credits = 0U;
     uint32_t m_n_unsignaled_count = 0U;
     int m_sq_wqe_hot_index = 0;
