@@ -1,11 +1,12 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
 #include <array>
 #include <mutex>
+#include <cinttypes>
 #include <infiniband/verbs.h>
 
 #include "utils/bullseye.h"
@@ -30,11 +31,13 @@
 #define ibch_logfuncall __log_info_funcall
 
 ib_ctx_handler::ib_ctx_handler(struct ib_ctx_handler_desc *desc)
-    : m_flow_tag_enabled(false)
+    : m_p_ibv_pd(nullptr)
+    , m_flow_tag_enabled(false)
     , m_on_device_memory(0)
     , m_removed(false)
     , m_lock_umr("spin_lock_umr")
     , m_p_ctx_time_converter(nullptr)
+    , m_str {}
 {
     if (!desc) {
         ibch_logpanic("Invalid ib_ctx_handler");
@@ -195,7 +198,7 @@ int parse_dpcp_version(const char *dpcp_ver)
 
 dpcp::adapter *ib_ctx_handler::set_dpcp_adapter()
 {
-    dpcp::status status = dpcp::DPCP_ERR_NO_SUPPORT;
+    dpcp::status status;
     dpcp::provider *p_provider = nullptr;
     dpcp::adapter_info *dpcp_lst = nullptr;
     size_t adapters_num = 0;
@@ -275,6 +278,7 @@ dpcp::adapter *ib_ctx_handler::set_dpcp_adapter()
 
                 ret = xlio_ib_mlx5dv_init_obj(&mlx5_obj, MLX5DV_OBJ_PD);
                 if (ret) {
+                    /* coverity[uninit_use_in_call] */
                     ibch_logerr("failed getting mlx5_pd for %p (errno=%d %m) ", m_p_ibv_pd, errno);
                     ibv_dealloc_pd(pd);
                     delete adapter;
@@ -317,6 +321,9 @@ void ib_ctx_handler::check_capabilities()
     if (rc == dpcp::DPCP_OK) {
         set_flow_tag_capability(caps.flow_table_caps.receive.is_flow_action_tag_supported);
         ibch_logdbg("Flow Tag Support: %s", get_flow_tag_capability() ? "Yes" : "No");
+
+        m_max_sq_wqebbs = (1U << caps.log_max_qp_sz);
+        ibch_logdbg("Max SQ WQEBBs: %" PRIu32, m_max_sq_wqebbs);
     }
 }
 
