@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -80,10 +80,6 @@ public:
     virtual bool is_tso(void);
     void slave_create(int if_index);
 
-    void reset_inflight_zc_buffers_ctx(ring_user_id_t id, void *ctx)
-    {
-        m_xmit_rings[id]->reset_inflight_zc_buffers_ctx(id, ctx);
-    }
     virtual uint64_t get_rx_cq_out_of_buffer_drop();
 
 protected:
