@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -72,7 +72,9 @@ ring_simple::ring_simple(int if_index, ring *parent, bool use_locks)
     BULLSEYE_EXCLUDE_BLOCK_END
 
     const slave_data_t *p_slave = p_ndev->get_slave(get_if_index());
-
+    if (unlikely(!p_slave)) {
+        ring_logpanic("Cannot find slave for a ring");
+    }
     ring_logdbg("new ring_simple()");
 
     BULLSEYE_EXCLUDE_BLOCK_START
@@ -198,6 +200,9 @@ void ring_simple::create_resources()
     BULLSEYE_EXCLUDE_BLOCK_END
 
     const slave_data_t *p_slave = p_ndev->get_slave(get_if_index());
+    if (unlikely(!p_slave)) {
+        ring_logpanic("Cannot find slave for a ring");
+    }
 
     save_l2_address(p_slave->p_L2_addr);
     m_p_tx_comp_event_channel = ibv_create_comp_channel(m_p_ib_ctx->get_ibv_context());
@@ -217,13 +222,13 @@ void ring_simple::create_resources()
     BULLSEYE_EXCLUDE_BLOCK_END
     VALGRIND_MAKE_MEM_DEFINED(m_p_tx_comp_event_channel, sizeof(struct ibv_comp_channel));
     // Check device capabilities for max QP work requests
-    uint32_t max_qp_wr = ALIGN_WR_DOWN(m_p_ib_ctx->get_ibv_device_attr()->max_qp_wr);
+
     m_tx_num_wr = safe_mce_sys().tx_num_wr;
-    if (m_tx_num_wr > max_qp_wr) {
-        ring_logwarn(
-            "Allocating only %d Tx QP work requests while user requested %s=%d for QP on interface",
-            max_qp_wr, SYS_VAR_TX_NUM_WRE, m_tx_num_wr);
-        m_tx_num_wr = max_qp_wr;
+    if (m_tx_num_wr > m_p_ib_ctx->get_max_sq_wqebbs()) {
+        ring_logwarn("Max supported SQ size is %" PRIu32 " while the requested size is %s=%d."
+                     " Using the maximum supported size.",
+                     m_p_ib_ctx->get_max_sq_wqebbs(), SYS_VAR_TX_NUM_WRE, m_tx_num_wr);
+        m_tx_num_wr = m_p_ib_ctx->get_max_sq_wqebbs();
     }
     ring_logdbg("ring attributes: m_tx_num_wr = %d", m_tx_num_wr);
 
@@ -376,7 +381,7 @@ void ring_simple::create_resources()
 
 int ring_simple::request_notification(cq_type_t cq_type, uint64_t poll_sn)
 {
-    int ret = 1;
+    int ret;
     if (likely(CQT_RX == cq_type)) {
         m_lock_ring_rx.lock();
         ret = m_p_cq_mgr_rx->request_notification(poll_sn);
@@ -787,8 +792,9 @@ mem_buf_desc_t *ring_simple::get_tx_buffers(pbuf_type type, uint32_t n_num_mem_b
             return nullptr;
         }
     }
-
+    /* coverity[returned_null] */
     head = pool.get_and_pop_back();
+    /* coverity[null_deref] */
     head->lwip_pbuf.ref = 1;
     assert(head->lwip_pbuf.type == type);
     head->lwip_pbuf.type = type;
