@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -311,7 +311,7 @@ protected:
     template <typename KEY4T, typename KEY2T, typename HDR> friend class steering_handler;
 
 private:
-    uint8_t padding[58]; // make class size up to a whole cache line
+    uint8_t padding[58] = {}; // make class size up to a whole cache line
 };
 
 static_assert(sizeof(ring_slave) % 64 == 0, "ring_slave size is not a multiple of cache line size");
