@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -31,8 +31,14 @@ select_call::select_call(int *off_fds_buffer, offloaded_mode_t *off_modes_buffer
     , m_writefds(writefds)
     , m_exceptfds(exceptfds)
     , m_timeout(timeout)
+    , m_orig_readfds {}
+    , m_orig_writefds {}
+    , m_orig_exceptfds {}
     , m_nfds_with_cq(0)
     , m_b_run_prepare_to_poll(false)
+    , m_os_rfds {0}
+    , m_os_wfds {0}
+    , m_cq_rfds {0}
 {
     int fd;
 
