@@ -46,11 +46,7 @@ extern "C" {
 typedef u32_t (*sys_now_fn)(void);
 void register_sys_now(sys_now_fn fn);
 
-#define LWIP_MEM_ALIGN_SIZE(size) (((size) + MEM_ALIGNMENT - 1) & ~(MEM_ALIGNMENT - 1))
-
 extern u16_t lwip_tcp_mss;
-extern u32_t lwip_tcp_snd_buf;
-extern u32_t lwip_zc_tx_size;
 extern u32_t lwip_tcp_nodelay_treshold;
 
 struct tcp_seg;
@@ -62,8 +58,10 @@ typedef struct pbuf *(*tcp_tx_pbuf_alloc_fn)(void *p_conn, pbuf_type type, pbuf_
 void register_tcp_tx_pbuf_alloc(tcp_tx_pbuf_alloc_fn fn);
 
 typedef void (*tcp_tx_pbuf_free_fn)(void *p_conn, struct pbuf *p);
+typedef void (*tcp_rx_pbuf_free_fn)(struct pbuf *p);
 
 void register_tcp_tx_pbuf_free(tcp_tx_pbuf_free_fn fn);
+void register_tcp_rx_pbuf_free(tcp_rx_pbuf_free_fn fn);
 
 typedef struct tcp_seg *(*tcp_seg_alloc_fn)(void *p_conn);
 
@@ -75,6 +73,7 @@ void register_tcp_seg_free(tcp_seg_free_fn fn);
 
 extern tcp_tx_pbuf_alloc_fn external_tcp_tx_pbuf_alloc;
 extern tcp_tx_pbuf_free_fn external_tcp_tx_pbuf_free;
+extern tcp_rx_pbuf_free_fn external_tcp_rx_pbuf_free;
 extern tcp_seg_alloc_fn external_tcp_seg_alloc;
 extern tcp_seg_free_fn external_tcp_seg_free;
 
@@ -128,18 +127,18 @@ typedef void (*tcp_accepted_pcb_fn)(struct tcp_pcb *accepted_pcb);
  */
 typedef err_t (*tcp_recv_fn)(void *arg, struct tcp_pcb *tpcb, struct pbuf *p, err_t err);
 
-/** Function prototype for tcp sent callback functions. Called when sent data has
+/** Function prototype for tcp acked callback functions. Called when data has
  * been acknowledged by the remote side. Use it to free corresponding resources.
  * This also means that the pcb has now space available to send new data.
  *
  * @param arg Additional argument to pass to the callback function (@see tcp_arg())
  * @param tpcb The connection pcb for which data has been acknowledged
- * @param len The amount of bytes acknowledged
+ * @param acked The amount of bytes acknowledged
  * @return ERR_OK: try to send some data by calling tcp_output
  *            Only return ERR_ABRT if you have called tcp_abort from within the
  *            callback function!
  */
-typedef err_t (*tcp_sent_fn)(void *arg, struct tcp_pcb *tpcb, u16_t len);
+typedef err_t (*tcp_acked_fn)(void *arg, struct tcp_pcb *tpcb, u32_t acked);
 
 /** Function prototype for tcp error callback functions. Called when the pcb
  * receives a RST or is unexpectedly closed for any other reason.
@@ -220,13 +219,6 @@ extern tcp_state_observer_fn external_tcp_state_observer;
 #define SND_WND_SCALE(pcb, wnd) ((u32_t)(wnd) << (pcb)->snd_scale)
 #define TCPWND_MIN16(x)         ((u16_t)LWIP_MIN((x), 0xFFFF))
 
-/* Note: max_tcp_snd_queuelen is now a multiple by 16 (was 4 before) to match max_unsent_len */
-#define UPDATE_PCB_BY_MSS(pcb, snd_mss)                                                            \
-    (pcb)->mss = (snd_mss);                                                                        \
-    (pcb)->max_tcp_snd_queuelen = (16 * ((pcb)->max_snd_buff) / ((pcb)->mss));                     \
-    (pcb)->max_unsent_len = (16 * ((pcb)->max_snd_buff) / ((pcb)->mss));                           \
-    (pcb)->tcp_oversize_val = (pcb)->mss;
-
 /* the TCP protocol control block */
 struct tcp_pcb {
     /** IP specific PCB members */
@@ -330,23 +322,9 @@ struct tcp_pcb {
 
     u32_t acked;
 
-    u32_t snd_buf; /* Available buffer space for sending (in bytes). */
-    u32_t max_snd_buff;
-
     u32_t snd_sml_snt; /* maintain state for minshall's algorithm */
     u32_t snd_sml_add; /* maintain state for minshall's algorithm */
 
-#define TCP_SNDQUEUELEN_OVERFLOW (0xffffffU - 3)
-    u32_t snd_queuelen; /* Available buffer space for sending (in tcp_segs). */
-    u32_t max_tcp_snd_queuelen;
-
-#if TCP_OVERSIZE
-    /* Extra bytes available at the end of the last pbuf in unsent. */
-    u16_t unsent_oversize;
-    u16_t tcp_oversize_val;
-#endif /* TCP_OVERSIZE */
-    u16_t max_unsent_len;
-
     /* These are ordered by sequence number: */
     struct tcp_seg *unsent; /* Unsent (queued) segments. */
     struct tcp_seg *last_unsent; /* Last unsent (queued) segment. */
@@ -356,12 +334,10 @@ struct tcp_pcb {
     struct tcp_seg *ooseq; /* Received out of sequence segments. */
 #endif /* TCP_QUEUE_OOSEQ */
 
-    struct pbuf *refused_data; /* Data previously received but not yet taken by upper layer */
     struct tcp_seg *seg_alloc; /* Available tcp_seg element for use */
-    struct pbuf *pbuf_alloc; /* Available pbuf element for use */
 
-    /* Function to be called when more send buffer space is available. */
-    tcp_sent_fn sent;
+    /* Function to be called when data is acknowledged. */
+    tcp_acked_fn acked_cb;
     /* Function to be called when (in-sequence) data has arrived. */
     tcp_recv_fn recv;
     /* Function to be called when a connection has been set up. */
@@ -417,8 +393,6 @@ struct tcp_pcb {
         /* Maximum number of SGE */
         u32_t max_send_sge;
     } tso;
-
-    u32_t max_send_sge;
 };
 
 typedef u16_t (*ip_route_mtu_fn)(struct tcp_pcb *pcb);
@@ -439,23 +413,17 @@ void tcp_syn_handled(struct tcp_pcb *pcb, tcp_syn_handled_fn syn_handled);
 void tcp_clone_conn(struct tcp_pcb *pcb, tcp_clone_conn_fn clone_conn);
 void tcp_accepted_pcb(struct tcp_pcb *pcb, tcp_accepted_pcb_fn accepted_pcb);
 void tcp_recv(struct tcp_pcb *pcb, tcp_recv_fn recv);
-void tcp_sent(struct tcp_pcb *pcb, tcp_sent_fn sent);
+void tcp_acked(struct tcp_pcb *pcb, tcp_acked_fn sent);
 void tcp_err(struct tcp_pcb *pcb, tcp_err_fn err);
 
 #define tcp_mss(pcb)            (((pcb)->flags & TF_TIMESTAMP) ? ((pcb)->mss - 12) : (pcb)->mss)
-#define tcp_sndbuf(pcb)         ((pcb)->snd_buf)
-#define tcp_sndqueuelen(pcb)    ((pcb)->snd_queuelen)
 #define tcp_nagle_disable(pcb)  ((pcb)->flags |= TF_NODELAY)
 #define tcp_nagle_enable(pcb)   ((pcb)->flags &= ~TF_NODELAY)
 #define tcp_nagle_disabled(pcb) (((pcb)->flags & TF_NODELAY) != 0)
 
 #define tcp_tso(pcb) ((pcb)->tso.max_payload_sz)
 
-#define tcp_accepted(pcb)                                                                          \
-    LWIP_ASSERT("get_tcp_state(pcb) == LISTEN (called for wrong pcb?)",                            \
-                get_tcp_state(pcb) == LISTEN)
-
-void tcp_recved(struct tcp_pcb *pcb, u32_t len);
+bool tcp_recved(struct tcp_pcb *pcb, u32_t len, bool do_output);
 err_t tcp_bind(struct tcp_pcb *pcb, const ip_addr_t *ipaddr, u16_t port, bool is_ipv6);
 err_t tcp_connect(struct tcp_pcb *pcb, const ip_addr_t *ipaddr, u16_t port, bool is_ipv6,
                   tcp_connected_fn connected);
@@ -467,16 +435,15 @@ err_t tcp_close(struct tcp_pcb *pcb);
 err_t tcp_shutdown(struct tcp_pcb *pcb, int shut_rx, int shut_tx);
 
 /* Flags for "apiflags" parameter in tcp_write */
-#define TCP_WRITE_FLAG_COPY 0x01
-#define TCP_WRITE_FLAG_MORE 0x02
-#define TCP_WRITE_REXMIT    0x08
-#define TCP_WRITE_DUMMY     0x10
-#define TCP_WRITE_TSO       0x20
-#define TCP_WRITE_FILE      0x40
-#define TCP_WRITE_ZEROCOPY  0x80
+#define TCP_WRITE_REXMIT   0x08
+#define TCP_WRITE_TSO      0x20
+#define TCP_WRITE_FILE     0x40
+#define TCP_WRITE_ZEROCOPY 0x80
 
 err_t tcp_write(struct tcp_pcb *pcb, const void *dataptr, u32_t len, u16_t apiflags,
                 pbuf_desc *desc);
+err_t tcp_write_express(struct tcp_pcb *pcb, const struct iovec *iov, u32_t iovcnt,
+                        pbuf_desc *desc);
 
 #define TCP_PRIO_MIN    1
 #define TCP_PRIO_NORMAL 64
