@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -9,7 +9,7 @@
 #endif
 
 #include "main.h"
-
+#include "config_printer.h"
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/resource.h>
@@ -54,6 +54,7 @@
 #include "sock/sockinfo_udp.h"
 #include "sock/bind_no_port.h"
 #include "iomux/io_mux_call.h"
+#include "config/descriptors/config_descriptor.h"
 
 #include "util/instrumentation.h"
 #include "util/agent.h"
@@ -84,6 +85,7 @@ const char *xlio_version_str = "XLIO_VERSION: " PACKAGE_VERSION "-" STR(PRJ_LIBR
 
 bool g_b_exit = false;
 bool g_init_ibv_fork_done = false;
+enum ibv_fork_status g_ibv_fork_status = IBV_FORK_DISABLED;
 bool g_is_forked_child = false;
 bool g_init_global_ctors_done = false;
 bool g_init_xlio_init_done = false;
@@ -422,11 +424,13 @@ void print_env_vars_xlio_global_settings()
     }
     vlog_printf(VLOG_INFO, "Cmd Line: %s\n", safe_mce_sys().app_name);
 
-    // Use DEBUG level logging with more details in RPM release builds
-    vlog_levels_t log_level = VLOG_DEBUG;
+    vlog_levels_t log_level;
 #if !defined(PRJ_LIBRARY_RELEASE) || (PRJ_LIBRARY_RELEASE == 0)
     // If non RPM (development builds) use more verbosity
     log_level = VLOG_DEFAULT;
+#else
+    // Use DEBUG level logging with more details in RPM release builds
+    log_level = VLOG_DEBUG;
 #endif
     vlog_printf(log_level, "Current Time: %s", ctime(&clock));
     vlog_printf(log_level, "Pid: %5u\n", getpid());
@@ -616,6 +620,7 @@ void print_env_vars_xlio_global_settings()
     VLOG_PARAM_STRING("Force Flowtag for MC", safe_mce_sys().mc_force_flowtag,
                       MCE_DEFAULT_MC_FORCE_FLOWTAG, SYS_VAR_MC_FORCE_FLOWTAG,
                       safe_mce_sys().mc_force_flowtag ? "Enabled " : "Disabled");
+    /* coverity[returned_null][dereference] */
     VLOG_STR_PARAM_STRING("Striding RQ", option_3::to_str(safe_mce_sys().enable_strq_env),
                           option_3::to_str(MCE_DEFAULT_STRQ), SYS_VAR_STRQ,
                           option_3::to_str(safe_mce_sys().enable_strq_env));
@@ -745,11 +750,13 @@ void print_env_vars_xlio_global_settings()
                       MCE_DEFAULT_NEIGH_UC_ARP_DELAY_MSEC, SYS_VAR_NEIGH_UC_ARP_DELAY_MSEC);
     VLOG_PARAM_NUMBER("Num of neigh restart retries", safe_mce_sys().neigh_num_err_retries,
                       MCE_DEFAULT_NEIGH_NUM_ERR_RETRIES, SYS_VAR_NEIGH_NUM_ERR_RETRIES);
+    /* coverity[returned_null][dereference] */
     VLOG_STR_PARAM_STRING("TSO support", option_3::to_str(safe_mce_sys().enable_tso),
                           option_3::to_str(MCE_DEFAULT_TSO), SYS_VAR_TSO,
                           option_3::to_str(safe_mce_sys().enable_tso));
     VLOG_PARAM_STRING("TSO max size", safe_mce_sys().max_tso_sz, MCE_DEFAULT_MAX_TSO_SIZE,
                       SYS_VAR_MAX_TSO_SIZE, option_size::to_str(safe_mce_sys().max_tso_sz));
+    /* coverity[returned_null][dereference] */
     VLOG_STR_PARAM_STRING("LRO support", option_3::to_str(safe_mce_sys().enable_lro),
                           option_3::to_str(MCE_DEFAULT_LRO), SYS_VAR_LRO,
                           option_3::to_str(safe_mce_sys().enable_lro));
@@ -855,11 +862,13 @@ void print_xlio_global_settings()
     }
     vlog_printf(VLOG_INFO, "Cmd Line: %s\n", safe_mce_sys().app_name);
 
-    // Use DEBUG level logging with more details in RPM release builds
-    vlog_levels_t log_level = VLOG_DEBUG;
+    vlog_levels_t log_level;
 #if !defined(PRJ_LIBRARY_RELEASE) || (PRJ_LIBRARY_RELEASE == 0)
     // If non RPM (development builds) use more verbosity
     log_level = VLOG_DEFAULT;
+#else
+    // Use DEBUG level logging with more details in RPM release builds
+    log_level = VLOG_DEBUG;
 #endif
     vlog_printf(log_level, "Current Time: %s", ctime(&clock));
     vlog_printf(log_level, "Pid: %5u\n", getpid());
@@ -879,48 +888,11 @@ void print_xlio_global_settings()
     vlog_printf(VLOG_INFO,
                 "---------------------------------------------------------------------------\n");
 
-    if (safe_mce_sys().mce_spec != MCE_SPEC_NONE) {
-        vlog_printf(VLOG_INFO, FORMAT_STRING, "Spec",
-                    xlio_spec::to_str((xlio_spec_t)safe_mce_sys().mce_spec), CONFIG_VAR_SPEC);
-    }
+    config_printer printer = config_printer(safe_mce_sys());
+    printer.print_to_log();
+    // Once we are done with the registry, destroy it to free the memory
+    safe_mce_sys().destroy_registry();
 
-    VLOG_STR_PARAM_STRING("Log Level", log_level::to_str(safe_mce_sys().log_level), "",
-                          CONFIG_VAR_LOG_LEVEL, log_level::to_str(safe_mce_sys().log_level));
-    VLOG_PARAM_NUMBER("Log Details", safe_mce_sys().log_details, MCE_DEFAULT_LOG_DETAILS,
-                      CONFIG_VAR_LOG_DETAILS);
-    VLOG_PARAM_BOOL("Log Colors", safe_mce_sys().log_colors, MCE_DEFAULT_LOG_COLORS,
-                    CONFIG_VAR_LOG_COLORS);
-    VLOG_STR_PARAM_STRING("Log File", safe_mce_sys().log_filename, MCE_DEFAULT_LOG_FILE,
-                          CONFIG_VAR_LOG_FILENAME, safe_mce_sys().log_filename);
-    VLOG_STR_PARAM_STRING("Stats File", safe_mce_sys().stats_filename, MCE_DEFAULT_STATS_FILE,
-                          CONFIG_VAR_STATS_FILENAME, safe_mce_sys().stats_filename);
-    VLOG_STR_PARAM_STRING("Stats shared memory directory", safe_mce_sys().stats_shmem_dirname,
-                          MCE_DEFAULT_STATS_SHMEM_DIR, CONFIG_VAR_STATS_SHMEM_DIRNAME,
-                          safe_mce_sys().stats_shmem_dirname);
-    VLOG_STR_PARAM_STRING("SERVICE output directory", safe_mce_sys().service_notify_dir,
-                          MCE_DEFAULT_SERVICE_FOLDER, CONFIG_VAR_SERVICE_DIR,
-                          safe_mce_sys().service_notify_dir);
-    VLOG_PARAM_NUMBER("Stats FD Num (max)", safe_mce_sys().stats_fd_num_max,
-                      MCE_DEFAULT_STATS_FD_NUM, CONFIG_VAR_STATS_FD_NUM);
-    VLOG_STR_PARAM_STRING("Application ID", safe_mce_sys().app_id, MCE_DEFAULT_APP_ID,
-                          CONFIG_VAR_APPLICATION_ID, safe_mce_sys().app_id);
-    VLOG_PARAM_BOOL("Polling CPU idle usage", safe_mce_sys().select_handle_cpu_usage_stats,
-                    MCE_DEFAULT_SELECT_CPU_USAGE_STATS, CONFIG_VAR_SELECT_CPU_USAGE_STATS);
-    VLOG_PARAM_BOOL("SigIntr Ctrl-C Handle", safe_mce_sys().handle_sigintr,
-                    MCE_DEFAULT_HANDLE_SIGINTR, CONFIG_VAR_HANDLE_SIGINTR);
-    VLOG_PARAM_BOOL("SegFault Backtrace", safe_mce_sys().handle_segfault,
-                    MCE_DEFAULT_HANDLE_SIGFAULT, CONFIG_VAR_HANDLE_SIGSEGV);
-    VLOG_PARAM_STRING("Print a report", safe_mce_sys().print_report, MCE_DEFAULT_PRINT_REPORT,
-                      CONFIG_VAR_PRINT_REPORT, option_3::to_str(safe_mce_sys().print_report));
-    VLOG_PARAM_BOOL("Quick start", safe_mce_sys().quick_start, MCE_DEFAULT_QUICK_START,
-                    CONFIG_VAR_QUICK_START);
-
-    VLOG_PARAM_NUMSTR("Ring allocation logic TX", safe_mce_sys().ring_allocation_logic_tx,
-                      MCE_DEFAULT_RING_ALLOCATION_LOGIC_TX, CONFIG_VAR_RING_ALLOCATION_LOGIC_TX,
-                      ring_logic_str(safe_mce_sys().ring_allocation_logic_tx));
-    VLOG_PARAM_NUMSTR("Ring allocation logic RX", safe_mce_sys().ring_allocation_logic_rx,
-                      MCE_DEFAULT_RING_ALLOCATION_LOGIC_RX, CONFIG_VAR_RING_ALLOCATION_LOGIC_RX,
-                      ring_logic_str(safe_mce_sys().ring_allocation_logic_rx));
     if (safe_mce_sys().ring_allocation_logic_rx == RING_LOGIC_PER_USER_ID) {
         vlog_printf(VLOG_WARNING,
                     "user_id is not supported using "
@@ -935,328 +907,6 @@ void print_xlio_global_settings()
         safe_mce_sys().ring_allocation_logic_tx = MCE_DEFAULT_RING_ALLOCATION_LOGIC_TX;
     }
 
-    VLOG_PARAM_NUMBER("Ring migration ratio TX", safe_mce_sys().ring_migration_ratio_tx,
-                      (safe_mce_sys().enable_tso ? -1 : MCE_DEFAULT_RING_MIGRATION_RATIO_TX),
-                      CONFIG_VAR_RING_MIGRATION_RATIO_TX);
-    VLOG_PARAM_NUMBER("Ring migration ratio RX", safe_mce_sys().ring_migration_ratio_rx,
-                      MCE_DEFAULT_RING_MIGRATION_RATIO_RX, CONFIG_VAR_RING_MIGRATION_RATIO_RX);
-
-    if (safe_mce_sys().ring_limit_per_interface) {
-        VLOG_PARAM_NUMBER("Ring limit per interface", safe_mce_sys().ring_limit_per_interface,
-                          MCE_DEFAULT_RING_LIMIT_PER_INTERFACE,
-                          CONFIG_VAR_RING_LIMIT_PER_INTERFACE);
-    } else {
-        VLOG_PARAM_NUMSTR("Ring limit per interface", safe_mce_sys().ring_limit_per_interface,
-                          MCE_DEFAULT_RING_LIMIT_PER_INTERFACE, CONFIG_VAR_RING_LIMIT_PER_INTERFACE,
-                          "(no limit)");
-    }
-
-    VLOG_PARAM_NUMBER("Ring On Device Memory TX", safe_mce_sys().ring_dev_mem_tx,
-                      MCE_DEFAULT_RING_DEV_MEM_TX, CONFIG_VAR_RING_DEV_MEM_TX);
-
-    VLOG_PARAM_STRING("Zerocopy Cache Threshold", safe_mce_sys().zc_cache_threshold,
-                      MCE_DEFAULT_ZC_CACHE_THRESHOLD, CONFIG_VAR_ZC_CACHE_THRESHOLD,
-                      option_size::to_str(safe_mce_sys().zc_cache_threshold));
-    VLOG_PARAM_STRING("Tx Mem Buf size", safe_mce_sys().tx_buf_size, MCE_DEFAULT_TX_BUF_SIZE,
-                      CONFIG_VAR_TX_BUF_SIZE, option_size::to_str(safe_mce_sys().tx_buf_size));
-    VLOG_PARAM_NUMBER("Tx QP WRE", safe_mce_sys().tx_num_wr, MCE_DEFAULT_TX_NUM_WRE,
-                      CONFIG_VAR_TX_NUM_WRE);
-    VLOG_PARAM_NUMBER("Tx QP WRE Batching", safe_mce_sys().tx_num_wr_to_signal,
-                      MCE_DEFAULT_TX_NUM_WRE_TO_SIGNAL, CONFIG_VAR_TX_NUM_WRE_TO_SIGNAL);
-    VLOG_PARAM_NUMBER("Tx Max QP INLINE", safe_mce_sys().tx_max_inline, MCE_DEFAULT_TX_MAX_INLINE,
-                      CONFIG_VAR_TX_MAX_INLINE);
-    VLOG_PARAM_BOOL("Tx MC Loopback", safe_mce_sys().tx_mc_loopback_default,
-                    MCE_DEFAULT_TX_MC_LOOPBACK, CONFIG_VAR_TX_MC_LOOPBACK);
-    VLOG_PARAM_BOOL("Tx non-blocked eagains", safe_mce_sys().tx_nonblocked_eagains,
-                    MCE_DEFAULT_TX_NONBLOCKED_EAGAINS, CONFIG_VAR_TX_NONBLOCKED_EAGAINS);
-    VLOG_PARAM_NUMBER("Tx Prefetch Bytes", safe_mce_sys().tx_prefetch_bytes,
-                      MCE_DEFAULT_TX_PREFETCH_BYTES, CONFIG_VAR_TX_PREFETCH_BYTES);
-    VLOG_PARAM_NUMBER("Tx Bufs Batch TCP", safe_mce_sys().tx_bufs_batch_tcp,
-                      MCE_DEFAULT_TX_BUFS_BATCH_TCP, CONFIG_VAR_TX_BUFS_BATCH_TCP);
-    VLOG_PARAM_NUMBER("Tx Segs Batch TCP", safe_mce_sys().tx_segs_batch_tcp,
-                      MCE_DEFAULT_TX_SEGS_BATCH_TCP, CONFIG_VAR_TX_SEGS_BATCH_TCP);
-    VLOG_PARAM_NUMBER("Tx Segs Ring Batch TCP", safe_mce_sys().tx_segs_ring_batch_tcp,
-                      MCE_DEFAULT_TX_SEGS_RING_BATCH_TCP, CONFIG_VAR_TX_SEGS_RING_BATCH_TCP);
-    VLOG_PARAM_STRING("TCP Send Buffer size", safe_mce_sys().tcp_send_buffer_size,
-                      MCE_DEFAULT_TCP_SEND_BUFFER_SIZE, CONFIG_VAR_TCP_SEND_BUFFER_SIZE,
-                      option_size::to_str(safe_mce_sys().tcp_send_buffer_size));
-    VLOG_PARAM_NUMBER(
-        "Rx QP WRE", safe_mce_sys().rx_num_wr,
-        (safe_mce_sys().enable_striding_rq ? MCE_DEFAULT_STRQ_NUM_WRE : MCE_DEFAULT_RX_NUM_WRE),
-        CONFIG_VAR_RX_NUM_WRE);
-    VLOG_PARAM_NUMBER("Rx QP WRE Batching", safe_mce_sys().rx_num_wr_to_post_recv,
-                      (safe_mce_sys().enable_striding_rq ? MCE_DEFAULT_STRQ_NUM_WRE_TO_POST_RECV
-                                                         : MCE_DEFAULT_RX_NUM_WRE_TO_POST_RECV),
-                      CONFIG_VAR_RX_NUM_WRE_TO_POST_RECV);
-    VLOG_PARAM_NUMBER("Rx Byte Min Limit", safe_mce_sys().rx_ready_byte_min_limit,
-                      MCE_DEFAULT_RX_BYTE_MIN_LIMIT, CONFIG_VAR_RX_BYTE_MIN_LIMIT);
-    VLOG_PARAM_NUMBER("Rx Poll Loops", safe_mce_sys().rx_poll_num, MCE_DEFAULT_RX_NUM_POLLS,
-                      CONFIG_VAR_RX_NUM_POLLS);
-    VLOG_PARAM_NUMBER("Rx Poll Init Loops", safe_mce_sys().rx_poll_num_init,
-                      MCE_DEFAULT_RX_NUM_POLLS_INIT, CONFIG_VAR_RX_NUM_POLLS_INIT);
-    if (safe_mce_sys().rx_udp_poll_os_ratio) {
-        VLOG_PARAM_NUMBER("Rx UDP Poll OS Ratio", safe_mce_sys().rx_udp_poll_os_ratio,
-                          MCE_DEFAULT_RX_UDP_POLL_OS_RATIO, CONFIG_VAR_RX_UDP_POLL_OS_RATIO);
-    } else {
-        VLOG_PARAM_NUMSTR("Rx UDP Poll OS Ratio", safe_mce_sys().rx_udp_poll_os_ratio,
-                          MCE_DEFAULT_RX_UDP_POLL_OS_RATIO, CONFIG_VAR_RX_UDP_POLL_OS_RATIO,
-                          "(Disabled)");
-    }
-
-    VLOG_PARAM_NUMBER("HW TS Conversion", safe_mce_sys().hw_ts_conversion_mode,
-                      MCE_DEFAULT_HW_TS_CONVERSION_MODE, CONFIG_VAR_HW_TS_CONVERSION_MODE);
-
-    if (safe_mce_sys().rx_poll_yield_loops) {
-        VLOG_PARAM_NUMBER("Rx Poll Yield", safe_mce_sys().rx_poll_yield_loops,
-                          MCE_DEFAULT_RX_POLL_YIELD, CONFIG_VAR_RX_POLL_YIELD);
-    } else {
-        VLOG_PARAM_STRING("Rx Poll Yield", safe_mce_sys().rx_poll_yield_loops,
-                          MCE_DEFAULT_RX_POLL_YIELD, CONFIG_VAR_RX_POLL_YIELD, "Disabled");
-    }
-    VLOG_PARAM_NUMBER("Rx Prefetch Bytes", safe_mce_sys().rx_prefetch_bytes,
-                      MCE_DEFAULT_RX_PREFETCH_BYTES, CONFIG_VAR_RX_PREFETCH_BYTES);
-
-    VLOG_PARAM_NUMBER("Rx Prefetch Bytes Before Poll", safe_mce_sys().rx_prefetch_bytes_before_poll,
-                      MCE_DEFAULT_RX_PREFETCH_BYTES_BEFORE_POLL,
-                      CONFIG_VAR_RX_PREFETCH_BYTES_BEFORE_POLL);
-
-    if (safe_mce_sys().rx_cq_drain_rate_nsec == MCE_RX_CQ_DRAIN_RATE_DISABLED) {
-        VLOG_PARAM_STRING("Rx CQ Drain Rate", safe_mce_sys().rx_cq_drain_rate_nsec,
-                          MCE_DEFAULT_RX_CQ_DRAIN_RATE, CONFIG_VAR_RX_CQ_DRAIN_RATE_NSEC,
-                          "Disabled");
-    } else {
-        VLOG_PARAM_NUMBER("Rx CQ Drain Rate (nsec)", safe_mce_sys().rx_cq_drain_rate_nsec,
-                          MCE_DEFAULT_RX_CQ_DRAIN_RATE, CONFIG_VAR_RX_CQ_DRAIN_RATE_NSEC);
-    }
-
-    VLOG_PARAM_NUMBER("GRO max streams", safe_mce_sys().gro_streams_max,
-                      MCE_DEFAULT_GRO_STREAMS_MAX, CONFIG_VAR_GRO_STREAMS_MAX);
-    VLOG_PARAM_BOOL("Disable flow tag", safe_mce_sys().disable_flow_tag,
-                    MCE_DEFAULT_DISABLE_FLOW_TAG, CONFIG_VAR_DISABLE_FLOW_TAG);
-
-    VLOG_PARAM_BOOL("TCP 2T rules", safe_mce_sys().tcp_2t_rules, MCE_DEFAULT_TCP_2T_RULES,
-                    CONFIG_VAR_TCP_2T_RULES);
-    VLOG_PARAM_BOOL("TCP 3T rules", safe_mce_sys().tcp_3t_rules, MCE_DEFAULT_TCP_3T_RULES,
-                    CONFIG_VAR_TCP_3T_RULES);
-    VLOG_PARAM_BOOL("UDP 3T rules", safe_mce_sys().udp_3t_rules, MCE_DEFAULT_UDP_3T_RULES,
-                    CONFIG_VAR_UDP_3T_RULES);
-    VLOG_PARAM_BOOL("ETH MC L2 only rules", safe_mce_sys().eth_mc_l2_only_rules,
-                    MCE_DEFAULT_ETH_MC_L2_ONLY_RULES, CONFIG_VAR_ETH_MC_L2_ONLY_RULES);
-    VLOG_PARAM_BOOL("Force Flowtag for MC", safe_mce_sys().mc_force_flowtag,
-                    MCE_DEFAULT_MC_FORCE_FLOWTAG, CONFIG_VAR_MC_FORCE_FLOWTAG);
-    VLOG_PARAM_BOOL("Striding RQ", safe_mce_sys().enable_strq_env, MCE_DEFAULT_STRQ,
-                    CONFIG_VAR_STRQ);
-    VLOG_PARAM_NUMBER("STRQ Strides per RWQE", safe_mce_sys().strq_stride_num_per_rwqe,
-                      MCE_DEFAULT_STRQ_NUM_STRIDES, CONFIG_VAR_STRQ_NUM_STRIDES);
-    VLOG_PARAM_NUMBER("STRQ Stride Size (Bytes)", safe_mce_sys().strq_stride_size_bytes,
-                      MCE_DEFAULT_STRQ_STRIDE_SIZE_BYTES, CONFIG_VAR_STRQ_STRIDE_SIZE_BYTES);
-    VLOG_PARAM_NUMBER(
-        "STRQ Strides Compensation Level", safe_mce_sys().strq_strides_compensation_level,
-        MCE_DEFAULT_STRQ_STRIDES_COMPENSATION_LEVEL, CONFIG_VAR_STRQ_STRIDES_COMPENSATION_LEVEL);
-    VLOG_PARAM_NUMBER("Select Poll (usec)", safe_mce_sys().select_poll_num,
-                      MCE_DEFAULT_SELECT_NUM_POLLS, CONFIG_VAR_SELECT_NUM_POLLS);
-
-    if (safe_mce_sys().select_poll_os_ratio) {
-        VLOG_PARAM_NUMBER("Select Poll OS Ratio", safe_mce_sys().select_poll_os_ratio,
-                          MCE_DEFAULT_SELECT_POLL_OS_RATIO, CONFIG_VAR_SELECT_POLL_OS_RATIO);
-    } else {
-        VLOG_PARAM_NUMSTR("Select Poll OS Ratio", safe_mce_sys().select_poll_os_ratio,
-                          MCE_DEFAULT_SELECT_POLL_OS_RATIO, CONFIG_VAR_SELECT_POLL_OS_RATIO,
-                          "(Disabled)");
-    }
-
-    if (safe_mce_sys().select_skip_os_fd_check) {
-        VLOG_PARAM_NUMBER("Select Skip OS", safe_mce_sys().select_skip_os_fd_check,
-                          MCE_DEFAULT_SELECT_SKIP_OS, CONFIG_VAR_SELECT_SKIP_OS);
-    } else {
-        VLOG_PARAM_NUMSTR("Select Skip OS", safe_mce_sys().select_skip_os_fd_check,
-                          MCE_DEFAULT_SELECT_SKIP_OS, CONFIG_VAR_SELECT_SKIP_OS, "(Disabled)");
-    }
-
-    if (safe_mce_sys().progress_engine_interval_msec == MCE_CQ_DRAIN_INTERVAL_DISABLED ||
-        safe_mce_sys().progress_engine_wce_max == 0) {
-        vlog_printf(VLOG_INFO, FORMAT_NUMSTR, "CQ Drain Interval (msec)",
-                    safe_mce_sys().progress_engine_interval_msec, "(Disabled)",
-                    CONFIG_VAR_PROGRESS_ENGINE_INTERVAL);
-    } else {
-        VLOG_PARAM_NUMBER("CQ Drain Interval (msec)", safe_mce_sys().progress_engine_interval_msec,
-                          MCE_DEFAULT_PROGRESS_ENGINE_INTERVAL_MSEC,
-                          CONFIG_VAR_PROGRESS_ENGINE_INTERVAL);
-        VLOG_PARAM_NUMBER("CQ Drain WCE (max)", safe_mce_sys().progress_engine_wce_max,
-                          MCE_DEFAULT_PROGRESS_ENGINE_WCE_MAX, CONFIG_VAR_PROGRESS_ENGINE_WCE_MAX);
-    }
-
-    VLOG_PARAM_BOOL("CQ Interrupts Moderation", safe_mce_sys().cq_moderation_enable,
-                    MCE_DEFAULT_CQ_MODERATION_ENABLE, CONFIG_VAR_CQ_MODERATION_ENABLE);
-    VLOG_PARAM_NUMBER("CQ Moderation Count", safe_mce_sys().cq_moderation_count,
-                      MCE_DEFAULT_CQ_MODERATION_COUNT, CONFIG_VAR_CQ_MODERATION_COUNT);
-    VLOG_PARAM_NUMBER("CQ Moderation Period (usec)", safe_mce_sys().cq_moderation_period_usec,
-                      MCE_DEFAULT_CQ_MODERATION_PERIOD_USEC, CONFIG_VAR_CQ_MODERATION_PERIOD_USEC);
-    VLOG_PARAM_NUMBER("CQ AIM Max Count", safe_mce_sys().cq_aim_max_count,
-                      MCE_DEFAULT_CQ_AIM_MAX_COUNT, CONFIG_VAR_CQ_AIM_MAX_COUNT);
-    VLOG_PARAM_NUMBER("CQ AIM Max Period (usec)", safe_mce_sys().cq_aim_max_period_usec,
-                      MCE_DEFAULT_CQ_AIM_MAX_PERIOD_USEC, CONFIG_VAR_CQ_AIM_MAX_PERIOD_USEC);
-    if (safe_mce_sys().cq_aim_interval_msec == MCE_CQ_ADAPTIVE_MODERATION_DISABLED) {
-        vlog_printf(VLOG_INFO, FORMAT_STRING, "CQ Adaptive Moderation", "Disabled",
-                    CONFIG_VAR_CQ_AIM_INTERVAL_MSEC);
-    } else {
-        VLOG_PARAM_NUMBER("CQ AIM Interval (msec)", safe_mce_sys().cq_aim_interval_msec,
-                          MCE_DEFAULT_CQ_AIM_INTERVAL_MSEC, CONFIG_VAR_CQ_AIM_INTERVAL_MSEC);
-    }
-    VLOG_PARAM_NUMBER(
-        "CQ AIM Interrupts Rate (per sec)", safe_mce_sys().cq_aim_interrupts_rate_per_sec,
-        MCE_DEFAULT_CQ_AIM_INTERRUPTS_RATE_PER_SEC, CONFIG_VAR_CQ_AIM_INTERRUPTS_RATE_PER_SEC);
-
-    VLOG_PARAM_NUMBER("CQ Poll Batch (max)", safe_mce_sys().cq_poll_batch_max,
-                      MCE_DEFAULT_CQ_POLL_BATCH, CONFIG_VAR_CQ_POLL_BATCH_MAX);
-    VLOG_PARAM_BOOL("CQ Keeps QP Full", safe_mce_sys().cq_keep_qp_full, MCE_DEFAULT_CQ_KEEP_QP_FULL,
-                    CONFIG_VAR_CQ_KEEP_QP_FULL);
-    VLOG_PARAM_NUMBER("QP Compensation Level", safe_mce_sys().qp_compensation_level,
-                      safe_mce_sys().rx_num_wr / 2U, CONFIG_VAR_QP_COMPENSATION_LEVEL);
-    VLOG_PARAM_BOOL("Offloaded Sockets", safe_mce_sys().offloaded_sockets,
-                    MCE_DEFAULT_OFFLOADED_SOCKETS, CONFIG_VAR_OFFLOADED_SOCKETS);
-    VLOG_PARAM_NUMBER("Timer Resolution (msec)", safe_mce_sys().timer_resolution_msec,
-                      MCE_DEFAULT_TIMER_RESOLUTION_MSEC, CONFIG_VAR_TIMER_RESOLUTION_MSEC);
-    VLOG_PARAM_NUMBER("TCP Timer Resolution (msec)", safe_mce_sys().tcp_timer_resolution_msec,
-                      MCE_DEFAULT_TCP_TIMER_RESOLUTION_MSEC, CONFIG_VAR_TCP_TIMER_RESOLUTION_MSEC);
-    VLOG_PARAM_STRING(
-        "TCP control thread", option_tcp_ctl_thread::to_str(safe_mce_sys().tcp_ctl_thread),
-        option_tcp_ctl_thread::to_str(MCE_DEFAULT_TCP_CTL_THREAD), CONFIG_VAR_TCP_CTL_THREAD,
-        option_tcp_ctl_thread::to_str(safe_mce_sys().tcp_ctl_thread));
-    VLOG_PARAM_NUMBER("TCP timestamp option", safe_mce_sys().tcp_ts_opt,
-                      MCE_DEFAULT_TCP_TIMESTAMP_OPTION, CONFIG_VAR_TCP_TIMESTAMP_OPTION);
-    VLOG_PARAM_BOOL("TCP nodelay", safe_mce_sys().tcp_nodelay, MCE_DEFAULT_TCP_NODELAY,
-                    CONFIG_VAR_TCP_NODELAY);
-    VLOG_PARAM_NUMBER("TCP nodelay treshold", safe_mce_sys().tcp_nodelay_treshold,
-                      MCE_DEFAULT_TCP_NODELAY_TRESHOLD, CONFIG_VAR_TCP_NODELAY_TRESHOLD);
-    VLOG_PARAM_BOOL("TCP quickack", safe_mce_sys().tcp_quickack, MCE_DEFAULT_TCP_QUICKACK,
-                    CONFIG_VAR_TCP_QUICKACK);
-    VLOG_PARAM_NUMSTR(xlio_exception_handling::getName(), (int)safe_mce_sys().exception_handling,
-                      xlio_exception_handling::MODE_DEFAULT, xlio_exception_handling::getSysVar(),
-                      safe_mce_sys().exception_handling.to_str());
-    VLOG_PARAM_BOOL("Avoid sys-calls on tcp fd", safe_mce_sys().avoid_sys_calls_on_tcp_fd,
-                    MCE_DEFAULT_AVOID_SYS_CALLS_ON_TCP_FD, CONFIG_VAR_AVOID_SYS_CALLS_ON_TCP_FD);
-    VLOG_PARAM_BOOL("Allow privileged sock opt", safe_mce_sys().allow_privileged_sock_opt,
-                    MCE_DEFAULT_ALLOW_PRIVILEGED_SOCK_OPT, CONFIG_VAR_ALLOW_PRIVILEGED_SOCK_OPT);
-    VLOG_PARAM_NUMBER("Delay after join (msec)", safe_mce_sys().wait_after_join_msec,
-                      MCE_DEFAULT_WAIT_AFTER_JOIN_MSEC, CONFIG_VAR_WAIT_AFTER_JOIN_MSEC);
-    VLOG_STR_PARAM_STRING("Internal Thread Affinity", safe_mce_sys().internal_thread_affinity_str,
-                          MCE_DEFAULT_INTERNAL_THREAD_AFFINITY_STR,
-                          CONFIG_VAR_INTERNAL_THREAD_AFFINITY,
-                          safe_mce_sys().internal_thread_affinity_str);
-    VLOG_STR_PARAM_STRING("Internal Thread Cpuset", safe_mce_sys().internal_thread_cpuset,
-                          MCE_DEFAULT_INTERNAL_THREAD_CPUSET, CONFIG_VAR_INTERNAL_THREAD_CPUSET,
-                          safe_mce_sys().internal_thread_cpuset);
-    VLOG_PARAM_NUMSTR("Buffer batching mode", safe_mce_sys().buffer_batching_mode,
-                      MCE_DEFAULT_BUFFER_BATCHING_MODE, CONFIG_VAR_BUFFER_BATCHING_MODE,
-                      buffer_batching_mode_str(safe_mce_sys().buffer_batching_mode));
-    VLOG_PARAM_STRING(
-        "Use hugepages", safe_mce_sys().mem_alloc_type, MCE_DEFAULT_MEM_ALLOC_TYPE,
-        CONFIG_VAR_MEM_ALLOC_TYPE,
-        (safe_mce_sys().mem_alloc_type == option_alloc_type::HUGE) ? "true" : "false");
-    VLOG_PARAM_STRING("Memory limit", safe_mce_sys().memory_limit, MCE_DEFAULT_MEMORY_LIMIT,
-                      CONFIG_VAR_MEMORY_LIMIT, option_size::to_str(safe_mce_sys().memory_limit));
-    VLOG_PARAM_STRING("Memory limit (user allocator)", safe_mce_sys().memory_limit_user,
-                      MCE_DEFAULT_MEMORY_LIMIT_USER, CONFIG_VAR_MEMORY_LIMIT_USER,
-                      option_size::to_str(safe_mce_sys().memory_limit_user));
-    VLOG_PARAM_STRING("Hugepage size", safe_mce_sys().hugepage_size, MCE_DEFAULT_HUGEPAGE_SIZE,
-                      CONFIG_VAR_HUGEPAGE_SIZE, option_size::to_str(safe_mce_sys().hugepage_size));
-
-    VLOG_PARAM_NUMBER("Num of UC ARPs", safe_mce_sys().neigh_uc_arp_quata,
-                      MCE_DEFAULT_NEIGH_UC_ARP_QUATA, CONFIG_VAR_NEIGH_UC_ARP_QUATA);
-    VLOG_PARAM_NUMBER("UC ARP delay (msec)", safe_mce_sys().neigh_wait_till_send_arp_msec,
-                      MCE_DEFAULT_NEIGH_UC_ARP_DELAY_MSEC, CONFIG_VAR_NEIGH_UC_ARP_DELAY_MSEC);
-    VLOG_PARAM_NUMBER("Num of neigh restart retries", safe_mce_sys().neigh_num_err_retries,
-                      MCE_DEFAULT_NEIGH_NUM_ERR_RETRIES, CONFIG_VAR_NEIGH_NUM_ERR_RETRIES);
-    VLOG_STR_PARAM_STRING("TSO support", option_3::to_str(safe_mce_sys().enable_tso),
-                          option_3::to_str(MCE_DEFAULT_TSO), CONFIG_VAR_TSO,
-                          option_3::to_str(safe_mce_sys().enable_tso));
-    VLOG_PARAM_STRING("TSO max size", safe_mce_sys().max_tso_sz, MCE_DEFAULT_MAX_TSO_SIZE,
-                      CONFIG_VAR_MAX_TSO_SIZE, option_size::to_str(safe_mce_sys().max_tso_sz));
-    VLOG_STR_PARAM_STRING("LRO support", option_3::to_str(safe_mce_sys().enable_lro),
-                          option_3::to_str(MCE_DEFAULT_LRO), CONFIG_VAR_LRO,
-                          option_3::to_str(safe_mce_sys().enable_lro));
-#ifdef DEFINED_UTLS
-    VLOG_PARAM_BOOL("UTLS RX support", safe_mce_sys().enable_utls_rx, MCE_DEFAULT_UTLS_RX,
-                    CONFIG_VAR_UTLS_RX);
-    VLOG_PARAM_BOOL("UTLS TX support", safe_mce_sys().enable_utls_tx, MCE_DEFAULT_UTLS_TX,
-                    CONFIG_VAR_UTLS_TX);
-    VLOG_PARAM_NUMBER("UTLS high watermark DEK cache size",
-                      static_cast<uint32_t>(safe_mce_sys().utls_high_wmark_dek_cache_size),
-                      MCE_DEFAULT_UTLS_HIGH_WMARK_DEK_CACHE_SIZE,
-                      CONFIG_VAR_UTLS_HIGH_WMARK_DEK_CACHE_SIZE);
-    VLOG_PARAM_NUMBER("UTLS low watermark DEK cache size",
-                      static_cast<uint32_t>(safe_mce_sys().utls_low_wmark_dek_cache_size),
-                      MCE_DEFAULT_UTLS_LOW_WMARK_DEK_CACHE_SIZE,
-                      CONFIG_VAR_UTLS_LOW_WMARK_DEK_CACHE_SIZE);
-#endif /* DEFINED_UTLS */
-#if defined(DEFINED_NGINX)
-    VLOG_PARAM_NUMBER("Number of Nginx workers",
-                      (safe_mce_sys().app.type == APP_NGINX ? safe_mce_sys().app.workers_num
-                                                            : MCE_DEFAULT_APP_WORKERS_NUM),
-                      MCE_DEFAULT_APP_WORKERS_NUM, CONFIG_VAR_NGINX_WORKERS_NUM);
-    VLOG_PARAM_NUMBER("Size of UDP socket pool", safe_mce_sys().nginx_udp_socket_pool_size,
-                      MCE_DEFAULT_NGINX_UDP_POOL_SIZE, CONFIG_VAR_NGINX_UDP_POOL_SIZE);
-    VLOG_PARAM_NUMBER("Max RX reuse buffs UDP pool",
-                      safe_mce_sys().nginx_udp_socket_pool_rx_num_buffs_reuse,
-                      MCE_DEFAULT_NGINX_UDP_POOL_RX_NUM_BUFFS_REUSE,
-                      CONFIG_VAR_NGINX_UDP_POOL_RX_NUM_BUFFS_REUSE);
-#endif
-#if defined(DEFINED_ENVOY)
-    VLOG_PARAM_NUMBER("Number of Envoy workers",
-                      (safe_mce_sys().app.type == APP_ENVOY ? safe_mce_sys().app.workers_num
-                                                            : MCE_DEFAULT_APP_WORKERS_NUM),
-                      MCE_DEFAULT_APP_WORKERS_NUM, CONFIG_VAR_ENVOY_WORKERS_NUM);
-#endif /* DEFINED_ENVOY */
-#if defined(DEFINED_NGINX) || defined(DEFINED_ENVOY)
-    VLOG_PARAM_NUMBER("Src port stirde", safe_mce_sys().app.src_port_stride,
-                      MCE_DEFAULT_SRC_PORT_STRIDE, CONFIG_VAR_SRC_PORT_STRIDE);
-#endif
-    VLOG_PARAM_BOOL("fork() support", safe_mce_sys().handle_fork, MCE_DEFAULT_FORK_SUPPORT,
-                    CONFIG_VAR_FORK);
-    VLOG_PARAM_BOOL("close on dup2()", safe_mce_sys().close_on_dup2, MCE_DEFAULT_CLOSE_ON_DUP2,
-                    CONFIG_VAR_CLOSE_ON_DUP2);
-    switch (safe_mce_sys().mtu) {
-    case MTU_FOLLOW_INTERFACE:
-        VLOG_PARAM_NUMSTR("MTU", safe_mce_sys().mtu, MCE_DEFAULT_MTU, CONFIG_VAR_MTU,
-                          "(follow actual MTU)");
-        break;
-    default:
-        VLOG_PARAM_NUMBER("MTU", safe_mce_sys().mtu, MCE_DEFAULT_MTU, CONFIG_VAR_MTU);
-        break;
-    }
-    switch (safe_mce_sys().lwip_mss) {
-    case MSS_FOLLOW_MTU:
-        VLOG_PARAM_NUMSTR("MSS", safe_mce_sys().lwip_mss, MCE_DEFAULT_MSS, CONFIG_VAR_MSS,
-                          "(follow XLIO_MTU)");
-        break;
-    default:
-        VLOG_PARAM_NUMBER("MSS", safe_mce_sys().lwip_mss, MCE_DEFAULT_MSS, CONFIG_VAR_MSS);
-        break;
-    }
-    VLOG_PARAM_NUMSTR("TCP CC Algorithm", safe_mce_sys().lwip_cc_algo_mod,
-                      MCE_DEFAULT_LWIP_CC_ALGO_MOD, CONFIG_VAR_TCP_CC_ALGO,
-                      lwip_cc_algo_str(safe_mce_sys().lwip_cc_algo_mod));
-    VLOG_PARAM_BOOL("Deferred close", safe_mce_sys().deferred_close, MCE_DEFAULT_DEFERRED_CLOSE,
-                    CONFIG_VAR_DEFERRED_CLOSE);
-    VLOG_PARAM_BOOL("TCP abort on close", safe_mce_sys().tcp_abort_on_close,
-                    MCE_DEFAULT_TCP_ABORT_ON_CLOSE, CONFIG_VAR_TCP_ABORT_ON_CLOSE);
-    VLOG_PARAM_BOOL("Polling Rx on Tx TCP", safe_mce_sys().rx_poll_on_tx_tcp,
-                    MCE_DEFAULT_RX_POLL_ON_TX_TCP, CONFIG_VAR_RX_POLL_ON_TX_TCP);
-    VLOG_PARAM_BOOL("RX CQ wait control", safe_mce_sys().rx_cq_wait_ctrl,
-                    MCE_DEFAULT_RX_CQ_WAIT_CTRL, CONFIG_VAR_RX_CQ_WAIT_CTRL);
-    VLOG_PARAM_STRING(
-        "Skip CQ polling in rx", safe_mce_sys().skip_poll_in_rx, MCE_DEFAULT_SKIP_POLL_IN_RX,
-        CONFIG_VAR_SKIP_POLL_IN_RX,
-        safe_mce_sys().skip_poll_in_rx
-            ? (safe_mce_sys().skip_poll_in_rx == SKIP_POLL_IN_RX_ENABLE ? "enable"
-                                                                        : "enable_epoll_only")
-            : "disable");
-    VLOG_PARAM_STRING("Mutex over spinlock", safe_mce_sys().multilock, MCE_DEFAULT_MULTILOCK,
-                      CONFIG_VAR_MULTILOCK,
-                      (safe_mce_sys().multilock != MULTILOCK_SPIN) ? "true" : "false");
-    VLOG_PARAM_NUMBER("Worker Threads", safe_mce_sys().worker_threads, MCE_DEFAULT_WORKER_THREADS,
-                      CONFIG_VAR_WORKER_THREADS);
     vlog_printf(VLOG_INFO,
                 "---------------------------------------------------------------------------\n");
 }
@@ -1284,6 +934,15 @@ void prepare_fork()
         {
             g_init_ibv_fork_done = true;
             vlog_printf(VLOG_DEBUG, "ibv_fork_init() succeeded, fork() may be used safely!!\n");
+
+            g_ibv_fork_status = ibv_is_fork_initialized();
+            const char *status_str = (g_ibv_fork_status == IBV_FORK_DISABLED)
+                ? "DISABLED (libibverbs will NOT protect from fork)"
+                : (g_ibv_fork_status == IBV_FORK_ENABLED)
+                ? "ENABLED (libibverbs using madvise)"
+                : "UNNEEDED (libibverbs will NOT protect from fork)";
+            vlog_printf(VLOG_DEBUG, "ibv_is_fork_initialized() returned: %d (%s)\n",
+                        g_ibv_fork_status, status_str);
         }
         ENDIF_VERBS_FAILURE;
     }
@@ -1403,6 +1062,7 @@ static void do_global_ctors_helper()
 {
     static lock_spin_recursive g_globals_lock;
     std::lock_guard<decltype(g_globals_lock)> lock(g_globals_lock);
+    int rc;
 
     if (!g_init_xlio_init_done || g_init_global_ctors_done) {
         return;
@@ -1438,6 +1098,12 @@ static void do_global_ctors_helper()
 
     // Create all global management objects
     NEW_CTOR(g_p_event_handler_manager, event_handler_manager());
+    rc = g_p_event_handler_manager->start_thread();
+    if (rc != 0) {
+        BULLSEYE_EXCLUDE_BLOCK_START
+        throw_xlio_exception("Failed to start event handler thread.\n");
+        BULLSEYE_EXCLUDE_BLOCK_END
+    }
 
     xlio_shmem_stats_open(&g_p_vlogger_level, &g_p_vlogger_details);
     *g_p_vlogger_level = g_vlogger_level;
