@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -18,6 +18,9 @@
 #include <netinet/ether.h>
 #include <arpa/inet.h>
 
+#include <netlink/route/route.h>
+#include <netlink/netlink.h>
+
 #include "utils/bullseye.h"
 #include "utils/lock_wrapper.h"
 #include "vlogger/vlogger.h"
@@ -124,113 +127,122 @@ void route_table_mgr::update_tbl(nl_data_t data_type)
     netlink_socket_mgr::update_tbl(data_type);
 }
 
-void route_table_mgr::parse_entry(struct nlmsghdr *nl_header)
+void route_table_mgr::parse_entry(struct nl_object *nl_obj)
 {
-    int len;
-    struct rtmsg *rt_msg;
-    struct rtattr *rt_attribute;
     route_val val;
 
-    // get route entry header
-    rt_msg = (struct rtmsg *)NLMSG_DATA(nl_header);
+    // Cast the generic nl_object to a specific route or rule object
+    struct rtnl_route *route = reinterpret_cast<struct rtnl_route *>(nl_obj);
 
-    if (rt_msg->rtm_family != AF_INET && rt_msg->rtm_family != AF_INET6) {
-        return;
+    val.set_family(rtnl_route_get_family(route));
+    val.set_protocol(rtnl_route_get_protocol(route));
+    val.set_scope(rtnl_route_get_scope(route));
+    val.set_type(rtnl_route_get_type(route));
+
+    int table_id = rtnl_route_get_table(route);
+    if (table_id > 0) {
+        val.set_table_id(table_id);
     }
 
-    val.set_family(rt_msg->rtm_family);
-    val.set_protocol(rt_msg->rtm_protocol);
-    val.set_scope(rt_msg->rtm_scope);
-    val.set_type(rt_msg->rtm_type);
-    val.set_table_id(rt_msg->rtm_table);
-    val.set_dst_pref_len(rt_msg->rtm_dst_len);
-
-    len = RTM_PAYLOAD(nl_header);
-    rt_attribute = (struct rtattr *)RTM_RTA(rt_msg);
-
-    for (; RTA_OK(rt_attribute, len); rt_attribute = RTA_NEXT(rt_attribute, len)) {
-        parse_attr(rt_attribute, val);
+    // Set destination mask and prefix length
+    struct nl_addr *dst = rtnl_route_get_dst(route);
+    if (dst) {
+        val.set_dst_pref_len(nl_addr_get_prefixlen(dst));
     }
+
+    parse_attr(route, val);
+
     val.set_state(true);
 
     route_table_t &table = val.get_family() == AF_INET ? m_table_in4 : m_table_in6;
     table.push_back(val);
 }
 
-void route_table_mgr::parse_attr(struct rtattr *rt_attribute, route_val &val)
+void route_table_mgr::parse_attr(struct rtnl_route *route, route_val &val)
 {
-    char if_name[IFNAMSIZ];
+    struct nl_addr *addr;
 
-    switch (rt_attribute->rta_type) {
-    case RTA_DST:
-        val.set_dst_addr(ip_address((void *)RTA_DATA(rt_attribute), val.get_family()));
-        break;
-    // next hop address
-    case RTA_GATEWAY:
-        val.set_gw(ip_address((void *)RTA_DATA(rt_attribute), val.get_family()));
-        break;
-    // unique ID associated with the network interface
-    case RTA_OIF:
-        val.set_if_index(*(int *)RTA_DATA(rt_attribute));
-        if_indextoname(val.get_if_index(), if_name);
-        val.set_if_name(if_name);
-        break;
-    case RTA_SRC:
-    case RTA_PREFSRC:
-        val.set_src_addr(ip_address((void *)RTA_DATA(rt_attribute), val.get_family()));
-        break;
-    case RTA_TABLE:
-        val.set_table_id(*(uint32_t *)RTA_DATA(rt_attribute));
-        break;
-    case RTA_METRICS: {
-        struct rtattr *rta = (struct rtattr *)RTA_DATA(rt_attribute);
-        int len = RTA_PAYLOAD(rt_attribute);
-        uint16_t type;
-        while (RTA_OK(rta, len)) {
-            type = rta->rta_type;
-            switch (type) {
-            case RTAX_MTU:
-                val.set_mtu(*(uint32_t *)RTA_DATA(rta));
-                break;
-            default:
-                rt_mgr_logdbg("got unexpected METRICS %d %x", type, *(uint32_t *)RTA_DATA(rta));
-                break;
-            }
-            rta = RTA_NEXT(rta, len);
-        }
-        break;
+    // Destination Address
+    addr = rtnl_route_get_dst(route);
+    if (addr && is_valid_addr(addr)) {
+        val.set_dst_addr(ip_address(nl_addr_get_binary_addr(addr), nl_addr_get_family(addr)));
     }
-    case RTA_MULTIPATH: {
-        struct rtnexthop *rtnh = (struct rtnexthop *)RTA_DATA(rt_attribute);
-        int rtnh_len = RTA_PAYLOAD(rt_attribute);
-        while (RTNH_OK(rtnh, rtnh_len)) {
-            val.set_if_index(rtnh->rtnh_ifindex);
-            if_indextoname(val.get_if_index(), if_name);
-            val.set_if_name(if_name);
 
-            int len = rtnh->rtnh_len - sizeof(*rtnh);
-            for (struct rtattr *rta = RTNH_DATA(rtnh); RTA_OK(rta, len); rta = RTA_NEXT(rta, len)) {
-                parse_attr(rta, val);
-            }
-
-            const ip_address &gw_addr = val.get_gw_addr();
-            if (!gw_addr.is_anyaddr() && !gw_addr.is_linklocal(val.get_family())) {
-                // Currently, we support only a single nexthop per multipath route
-                // and we found a good one.
-                // If the gw is link-local we will check the next nexthop if present.
-                // FIXME We cannot rely on that the next entry overwrites all the attributes.
-                break;
-            }
-
-            rtnh = RTNH_NEXT(rtnh);
-            rtnh_len -= RTNH_ALIGN(rtnh->rtnh_len);
-        }
-        break;
+    // Source Address
+    addr = rtnl_route_get_pref_src(route);
+    if (addr && is_valid_addr(addr)) {
+        val.set_src_addr(ip_address(nl_addr_get_binary_addr(addr), nl_addr_get_family(addr)));
     }
-    default:
-        rt_mgr_logdbg("got unexpected type %d %x", rt_attribute->rta_type,
-                      *(uint32_t *)RTA_DATA(rt_attribute));
-        break;
+
+    // Metrics (e.g., MTU)
+    uint32_t mtu = 0;
+    int get_metric_result = rtnl_route_get_metric(route, RTAX_MTU, &mtu);
+    if (get_metric_result == 0) {
+        if (mtu > 0) {
+            val.set_mtu(mtu);
+        }
+    }
+
+    // Nexthop handling: Extract interface and gateway from nexthops
+    // Try foreach_nexthop first for multipath/weight-based selection,
+    // then fall back to nexthop_n(0) for routes that don't iterate
+    struct nexthop_iterator_context {
+        struct rtnl_nexthop *best_next_hop;
+        uint8_t best_next_hop_weight;
+        sa_family_t family;
+
+    } best_next_hop_context = {.best_next_hop = nullptr,
+                               .best_next_hop_weight = 0,
+                               .family = static_cast<sa_family_t>(val.get_family())};
+
+    rtnl_route_foreach_nexthop(
+        route,
+        [](struct rtnl_nexthop *next_hop, void *context) {
+            nexthop_iterator_context *ctx = (nexthop_iterator_context *)context;
+            const uint8_t current_nh_weight = rtnl_route_nh_get_weight(next_hop);
+
+            // Check gateway - skip link-local gateways as they're not usable for routing
+            struct nl_addr *gw = rtnl_route_nh_get_gateway(next_hop);
+            if (gw && is_valid_addr(gw)) {
+                ip_address gw_ip(nl_addr_get_binary_addr(gw), nl_addr_get_family(gw));
+                // Skip link-local gateways (matching old RTA_MULTIPATH behavior)
+                if (gw_ip.is_linklocal(ctx->family)) {
+                    return;
+                }
+            }
+
+            // Normalize weight: Linux defaults to 1 when unspecified (weight 0 is non-standard)
+            // For multipath routes, select the nexthop with highest weight (most traffic share)
+            const uint8_t normalized_weight = (current_nh_weight == 0) ? 1 : current_nh_weight;
+            if (normalized_weight > ctx->best_next_hop_weight) {
+                ctx->best_next_hop_weight = normalized_weight;
+                ctx->best_next_hop = next_hop;
+            }
+        },
+        &best_next_hop_context);
+
+    // If foreach didn't find anything, try direct nexthop access (for routes without iteration)
+    struct rtnl_nexthop *nh = best_next_hop_context.best_next_hop;
+    if (!nh) {
+        nh = rtnl_route_nexthop_n(route, 0);
+    }
+
+    if (nh) {
+        // Gateway Address
+        const auto nh_gateway = rtnl_route_nh_get_gateway(nh);
+        if (nh_gateway && is_valid_addr(nh_gateway)) {
+            val.set_gw(
+                ip_address(nl_addr_get_binary_addr(nh_gateway), nl_addr_get_family(nh_gateway)));
+        }
+
+        // Output Interface Index and Name
+        const int if_index = rtnl_route_nh_get_ifindex(nh);
+        if (if_index > 0) {
+            val.set_if_index(if_index);
+            char nh_if_name[IFNAMSIZ] = {0};
+            if_indextoname(if_index, nh_if_name);
+            val.set_if_name(nh_if_name);
+        }
     }
 }
 
@@ -305,12 +317,11 @@ bool route_table_mgr::route_resolve(IN route_rule_table_key key, OUT route_resul
 
 void route_table_mgr::update_entry(INOUT route_entry *p_ent, bool b_register_to_net_dev /*= false*/)
 {
-    rt_mgr_logdbg("entry [%p]", p_ent);
-
-    route_table_t &rt = p_ent->get_key().get_family() == AF_INET ? m_table_in4 : m_table_in6;
 
     std::lock_guard<decltype(m_lock)> lock(m_lock);
     if (p_ent && !p_ent->is_valid()) { // if entry is found in the collection and is not valid
+        rt_mgr_logdbg("entry [%p]", p_ent);
+        route_table_t &rt = p_ent->get_key().get_family() == AF_INET ? m_table_in4 : m_table_in6;
         rt_mgr_logdbg("route_entry is not valid-> update value");
         rule_entry *p_rr_entry = p_ent->get_rule_entry();
         std::deque<rule_val *> *p_rr_val;
