@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2023-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2023-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -26,27 +26,14 @@
 
 #define SET_EXTRA_API(__dst, __func, __mask)                                                       \
     do {                                                                                           \
-        xlio_api->__dst = __func;                                                                  \
-        xlio_api->cap_mask |= __mask;                                                              \
+        xlio_api.__dst = __func;                                                                   \
+        xlio_api.cap_mask |= __mask;                                                               \
     } while (0);
 
 //-----------------------------------------------------------------------------
 // extended API functions
 //-----------------------------------------------------------------------------
 
-extern "C" int xlio_register_recv_callback(int __fd, xlio_recv_callback_t __callback,
-                                           void *__context)
-{
-    sockinfo *p_socket_object = nullptr;
-    p_socket_object = fd_collection_get_sockfd(__fd);
-    if (p_socket_object && p_socket_object->get_protocol() == PROTO_UDP) {
-        reinterpret_cast<sockinfo_udp *>(p_socket_object)->register_callback(__callback, __context);
-        return 0;
-    }
-    errno = EINVAL;
-    return -1;
-}
-
 extern "C" int xlio_add_conf_rule(const char *config_line)
 {
     int ret = __xlio_parse_config_line(config_line);
@@ -80,16 +67,13 @@ extern "C" int xlio_dump_fd_stats(int fd, int log_level)
 
 struct xlio_api_t *extra_api()
 {
-    static struct xlio_api_t *xlio_api = nullptr;
+    // xlio_api is zerod-out by linker
+    static struct xlio_api_t xlio_api;
 
-    if (!xlio_api) {
-        xlio_api = new struct xlio_api_t();
+    if (xlio_api.magic != XLIO_MAGIC_NUMBER) {
+        memset(&xlio_api, 0, sizeof(struct xlio_api_t));
+        xlio_api.magic = XLIO_MAGIC_NUMBER;
 
-        memset(xlio_api, 0, sizeof(struct xlio_api_t));
-        xlio_api->magic = XLIO_MAGIC_NUMBER;
-        xlio_api->cap_mask = 0;
-        SET_EXTRA_API(register_recv_callback, xlio_register_recv_callback,
-                      XLIO_EXTRA_API_REGISTER_RECV_CALLBACK);
         SET_EXTRA_API(add_conf_rule, xlio_add_conf_rule, XLIO_EXTRA_API_ADD_CONF_RULE);
         SET_EXTRA_API(thread_offload, xlio_thread_offload, XLIO_EXTRA_API_THREAD_OFFLOAD);
         SET_EXTRA_API(dump_fd_stats, xlio_dump_fd_stats, XLIO_EXTRA_API_DUMP_FD_STATS);
@@ -123,7 +107,7 @@ struct xlio_api_t *extra_api()
                       XLIO_EXTRA_API_XLIO_ULTRA);
     }
 
-    return xlio_api;
+    return &xlio_api;
 }
 
 /*
