@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -878,10 +878,12 @@ bool sockinfo::attach_receiver(flow_tuple_with_local_if &flow_key)
                         flow_key.get_protocol(), flow_key.get_family(), flow_key.get_local_if());
                     p_nd_resources =
                         create_nd_resources(ip_addr(new_key.get_local_if(), new_key.get_family()));
-                    if (!p_nd_resources->p_ring->attach_flow(new_key, this, false)) {
+                    if (!p_nd_resources ||
+                        !p_nd_resources->p_ring->attach_flow(new_key, this, false)) {
                         lock_rx_q();
-                        si_logerr("Failed to attach %s to ring %p", new_key.to_str().c_str(),
-                                  p_nd_resources->p_ring);
+                        si_logerr("Failed to %s for %s",
+                                  p_nd_resources ? "attach flow" : "create ND resources",
+                                  new_key.to_str().c_str());
                         g_p_app->add_second_4t_rule = false;
                         return false;
                     }
@@ -911,7 +913,9 @@ bool sockinfo::attach_receiver(flow_tuple_with_local_if &flow_key)
         rx_flow_map_t::iterator rx_flow_iter = m_rx_flow_map.find(flow_key_3t);
         if (rx_flow_iter != m_rx_flow_map.end()) {
             si_logdbg("Removing (and detaching) 3 tuple now that we added a stronger 5 tuple");
-            detach_receiver(flow_key_3t);
+            if (unlikely(!detach_receiver(flow_key_3t))) {
+                si_logwarn("Failed to detach 3 tuple flow key %s", flow_key_3t.to_str().c_str());
+            }
         }
     }
 
@@ -1170,9 +1174,7 @@ void sockinfo::do_rings_migration_rx(resource_allocation_key &old_key)
         unlock_rx_q();
         m_rx_ring_map_lock.lock();
         lock_rx_q();
-        if (!m_p_rx_ring && m_rx_ring_map.size() == 1) {
-            m_p_rx_ring = m_rx_ring_map.begin()->first;
-        }
+        update_rx_ring_ptr();
         unlock_rx_q();
         m_rx_ring_map_lock.unlock();
 
@@ -1530,9 +1532,7 @@ void sockinfo::rx_add_ring_cb(ring *p_ring)
          *  - rx_del_ring_cb()
          *  - do_rings_migration_rx()
          */
-        if (m_rx_ring_map.size() == 1) {
-            m_p_rx_ring = m_rx_ring_map.begin()->first;
-        }
+        update_rx_ring_ptr();
 
         notify_epoll = true;
 
@@ -1620,11 +1620,7 @@ void sockinfo::rx_del_ring_cb(ring *p_ring)
             delete p_ring_info;
 
             if (m_p_rx_ring == base_ring) {
-                if (m_rx_ring_map.size() == 1) {
-                    m_p_rx_ring = m_rx_ring_map.begin()->first;
-                } else {
-                    m_p_rx_ring = nullptr;
-                }
+                update_rx_ring_ptr();
 
                 move_descs(base_ring, &temp_rx_reuse, &m_rx_reuse_buff.rx_reuse, true);
                 move_descs(base_ring, &temp_rx_reuse_global, &m_rx_reuse_buff.rx_reuse, false);
@@ -1666,8 +1662,10 @@ void sockinfo::move_descs(ring *p_ring, descq_t *toq, descq_t *fromq, bool own)
     mem_buf_desc_t *temp;
     const size_t size = fromq->size();
     for (size_t i = 0; i < size; i++) {
+        /* coverity[returned_null] */
         temp = fromq->front();
         fromq->pop_front();
+        /* coverity[nonnull] */
         if (!__xor(own, p_ring->is_member(temp->p_desc_owner))) {
             toq->push_back(temp);
         } else {
@@ -1702,13 +1700,16 @@ void sockinfo::pop_descs_rx_ready(descq_t *cache, ring *p_ring)
 void sockinfo::push_descs_rx_ready(descq_t *cache)
 {
     // Assume locked by owner!!!
+    /* coverity[returned_null] */
     mem_buf_desc_t *temp;
     const size_t size = (cache ? cache->size() : 0);
 
     for (size_t i = 0; i < size; i++) {
+        /* coverity[returned_null] */
         temp = cache->front();
         cache->pop_front();
         m_n_rx_pkt_ready_list_count++;
+        /* coverity[nonnull] */
         m_rx_ready_byte_count += temp->rx.sz_payload;
         if (m_p_socket_stats) {
             m_p_socket_stats->n_rx_ready_pkt_count++;
@@ -1828,6 +1829,15 @@ bool sockinfo::attach_as_uc_receiver_anyip(sa_family_t family, role_t role, bool
     return ret;
 }
 
+void sockinfo::update_rx_ring_ptr()
+{
+    if (m_rx_ring_map.size() == 1) {
+        m_p_rx_ring = m_rx_ring_map.begin()->first;
+    } else {
+        m_p_rx_ring = nullptr;
+    }
+}
+
 transport_t sockinfo::find_target_family(role_t role, const struct sockaddr *sock_addr_first,
                                          const struct sockaddr *sock_addr_second /* = NULL */)
 {
@@ -1869,7 +1879,9 @@ void sockinfo::shutdown_rx()
     rx_flow_map_t::iterator rx_flow_iter = m_rx_flow_map.begin();
     while (rx_flow_iter != m_rx_flow_map.end()) {
         flow_tuple_with_local_if detach_key = rx_flow_iter->first;
-        detach_receiver(detach_key);
+        if (unlikely(!detach_receiver(detach_key))) {
+            si_logwarn("Failed to detach receiver for socket %p", this);
+        }
         rx_flow_iter = m_rx_flow_map.begin(); // Pop next flow rule
     }
 
@@ -2132,18 +2144,22 @@ ssize_t sockinfo::tx_os(const tx_call_t call_type, const iovec *p_iov, const ssi
     switch (call_type) {
     case TX_WRITE:
         __log_info_func("calling os transmit with orig write");
+        /* coverity[null_dereference][forward_null]*/
         return SYSCALL(write, m_fd, p_iov[0].iov_base, p_iov[0].iov_len);
 
     case TX_WRITEV:
         __log_info_func("calling os transmit with orig writev");
+        /* coverity[null_dereference][forward_null]*/
         return SYSCALL(writev, m_fd, p_iov, sz_iov);
 
     case TX_SEND:
         __log_info_func("calling os transmit with orig send");
+        /* coverity[null_dereference][forward_null]*/
         return SYSCALL(send, m_fd, p_iov[0].iov_base, p_iov[0].iov_len, __flags);
 
     case TX_SENDTO:
         __log_info_func("calling os transmit with orig sendto");
+        /* coverity[null_dereference][forward_null]*/
         return SYSCALL(sendto, m_fd, p_iov[0].iov_base, p_iov[0].iov_len, __flags, __to, __tolen);
 
     case TX_SENDMSG: {
