@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -444,6 +444,20 @@ private:
     int fcntl_helper(int __cmd, unsigned long int __arg, bool &bexit);
     bool attach_as_uc_receiver_anyip(sa_family_t family, role_t role, bool skip_rules);
 
+    /**
+     * @brief Update the RX ring fast-path pointer based on ring map size.
+     *
+     * This method maintains the m_p_rx_ring optimization pointer for single-ring sockets.
+     * Performance optimization: When a socket has exactly one RX ring (the common case),
+     * m_p_rx_ring provides direct pointer access, avoiding map iteration overhead.
+     *
+     * Must be called after any operation that modifies m_rx_ring_map:
+     * - Adding rings (rx_add_ring_cb)
+     * - Removing rings (rx_del_ring_cb)
+     * - Ring migration (do_rings_migration_rx)
+     */
+    void update_rx_ring_ptr();
+
 protected:
     dst_entry *m_p_connected_dst_entry = nullptr;
     sockinfo_state m_state = SOCKINFO_OPENED; // socket current state
