@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -71,6 +71,8 @@ agent::agent()
     : m_state(AGENT_CLOSED)
     , m_sock_fd(-1)
     , m_pid_fd(-1)
+    , m_sock_file {}
+    , m_pid_file {}
     , m_msg_num(AGENT_DEFAULT_MSG_NUM)
 {
     int rc = 0;
@@ -169,7 +171,9 @@ err:
     if (m_pid_fd > 0) {
         int ret = 0;
         NOT_IN_USE(ret);
+        /* coverity[RESOURCE_LEAK] */
         sys_call(ret, close, m_pid_fd);
+        /* coverity[leaked_handle] */
         m_pid_fd = -1;
         unlink(m_pid_file);
     }
