@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -311,7 +311,7 @@ static inline int get_procname(int pid, char *proc, size_t size)
     char app_full_name[PATH_MAX] = {0};
     char proccess_proc_dir[FILE_NAME_MAX_SIZE] = {0};
     char *app_base_name = nullptr;
-    int n = -1;
+    int n;
 
     if (!proc) {
         return -1;
@@ -443,9 +443,11 @@ public:
         m_timer_countdown = m_interval_it;
 
         if (!ts_isset(&m_start)) {
+            // coverity[check_return]
             gettime(&m_start);
         }
         // update timer
+        // coverity[check_return]
         gettime(&m_current);
         ts_sub(&m_current, &m_start, &m_elapsed);
         vlog_printf(VLOG_FUNC_ALL, "update loops_timer (elapsed time=%ld sec %ld usec\n",
