@@ -224,7 +224,7 @@ char *rshim_static_dev_name;
 /* Default configuration file. */
 const char *rshim_cfg_file = DEFAULT_RSHIM_CONFIG_FILE;
 static int rshim_display_level;
-static int rshim_boot_timeout = 300;
+int rshim_boot_timeout = 300;
 int rshim_drop_mode = -1;
 int rshim_usb_reset_delay = 1;
 bool rshim_has_usb_reset_delay;
@@ -925,6 +925,9 @@ int rshim_boot_write(rshim_backend_t *bd, const char *user_buffer, size_t count,
   bd->is_in_boot_write = 0;
   pthread_mutex_unlock(&bd->mutex);
 
+  if (rc == -ETIMEDOUT)
+    return rc;
+
   if (bytes_written > 0 || count == 0)
     return bytes_written;
   else
@@ -2770,6 +2773,15 @@ void rshim_deregister(rshim_backend_t *bd)
   rshim_fifo_free(bd);
 
   rshim_devs[bd->index] = NULL;
+
+#ifdef __arm__
+  // Clear the device name to allow index reuse on BMC.
+  if ((bd->type == RSH_BACKEND_USB) && rshim_dev_names[bd->index]) {
+    free(rshim_dev_names[bd->index]);
+    rshim_dev_names[bd->index] = NULL;
+  }
+#endif
+
   bd->registered = 0;
 }
 
@@ -3059,6 +3071,14 @@ static void rshim_main(int argc, char *argv[])
     }
   }
 
+  for (i = 0; i < RSHIM_MAX_DEV; i++) {
+    bd = rshim_devs[i];
+    if (bd && bd->type == RSH_BACKEND_PCIE) {
+      rshim_pcie_exit();
+      break;
+    }
+  }
+
   rshim_stop();
 }
 
@@ -3438,6 +3458,9 @@ int main(int argc, char *argv[])
     }
   }
 
+  /* Load configuration. */
+  rshim_load_cfg();
+
   /* Handle command mode separately. */
   if (rshim_cmdmode) {
     return rshim_cmdmode_run(argc, argv);
@@ -3483,8 +3506,6 @@ int main(int argc, char *argv[])
     }
   }
 
-  rshim_load_cfg();
-
   /* In force mode, we will send a one-time ownership request command for each
    * rshim backend if they are found to be detached (aka. in drop mode) */
   if (rshim_force_mode) {
