@@ -1,4 +1,4 @@
-// SPDX-License-Identifier: GPL-2.0-only
+// SPDX-License-Identifier: GPL-2.0 or BSD-3-Clause
 /*
  * Copyright (C) 2019 Mellanox Technologies. All Rights Reserved.
  *
@@ -26,7 +26,27 @@
 #include <fuse/cuse_lowlevel.h>
 #include <fuse/fuse_opt.h>
 #endif
+#include <features.h>
 #include <unistd.h>
+/*
+ * glibc 2.42+ no longer pulls in struct termio; provide a minimal definition so
+ * we can keep using the legacy layout expected by existing userspace tools.
+ */
+#if !__GLIBC_PREREQ(2, 42)   // if glibc is before 2.42
+#include <termio.h>
+#else
+#ifndef NCC
+#define NCC 8
+#endif
+struct termio {
+  unsigned short c_iflag;
+  unsigned short c_oflag;
+  unsigned short c_cflag;
+  unsigned short c_lflag;
+  unsigned char c_line;
+  unsigned char c_cc[NCC];
+};
+#endif
 #elif defined(__FreeBSD__)
 #include <termios.h>
 #include <sys/stat.h>
@@ -171,7 +191,8 @@ void rshim_fuse_input_notify(rshim_backend_t *bd)
 {
   int chan = bd->rx_chan;
 
-  RSHIM_DBG("rshim_fifo_input: woke up readable chan %d\n", chan);
+  RSHIM_DBG("rshim%d(fuse_input_notify) woke up readable chan %d\n",
+            bd->index, chan);
 
 #ifdef __linux__
   if (bd->fuse_poll_handle[chan])
@@ -605,6 +626,25 @@ static int rshim_fuse_misc_open(struct cuse_dev *cdev, int fflags)
 }
 #endif
 
+const char* bf3_bf_mode_str[] = {
+  "Unknown",
+  "DPU mode",
+  "NIC mode",
+  "Reserved"
+};
+
+const char* bf3_get_bf_mode(rshim_backend_t* bd) {
+  uint64_t value;
+
+  int rc = bd->read_rshim(bd, RSHIM_CHANNEL, bd->regs->scratchpad2, &value,
+                          RSHIM_REG_SIZE_8B);
+  if (rc || RSHIM_BAD_CTRL_REG(value)) {
+    RSHIM_ERR("rshim%d failed to read SCRATCHPAD2 (%d)\n", bd->index, rc);
+    return bf3_bf_mode_str[0];
+  }
+  return bf3_bf_mode_str[BF3_RSH_SCRATCHPAD2__BF_MODE(value)];
+}
+
 #ifdef __linux__
 static void rshim_fuse_misc_read(fuse_req_t req, size_t size, off_t off,
                                  struct fuse_file_info *fi)
@@ -651,9 +691,10 @@ static int rshim_fuse_misc_read(struct cuse_dev *cdev, int fflags,
   pthread_mutex_lock(&bd->mutex);
 
   /* Boot mode. */
-  rc = bd->read_rshim(bd, RSHIM_CHANNEL, bd->regs->boot_control, &value, RSHIM_REG_SIZE_8B);
+  rc = bd->read_rshim(bd, RSHIM_CHANNEL, bd->regs->boot_control, &value,
+                      RSHIM_REG_SIZE_8B);
   if (rc) {
-    RSHIM_ERR("couldn't read BOOT_CONTROL register\n");
+    RSHIM_ERR("rshim%d failed to read BOOT_CONTROL(%d)\n", bd->index, rc);
     value = 0;
   }
 
@@ -664,6 +705,14 @@ static int rshim_fuse_misc_read(struct cuse_dev *cdev, int fflags,
   p += n;
   len -= n;
 
+  if (bd->ver_id == RSHIM_BLUEFIELD_3) {
+    n = snprintf(p, len, "%-16s%s\n",
+        "BF_MODE",
+        bf3_get_bf_mode(bd));
+    p += n;
+    len -= n;
+  }
+
   n = snprintf(p, len, "%-16s%lld (0:rshim, 1:emmc, 2:emmc-boot-swap)\n",
                "BOOT_MODE",
                (unsigned long long)value & RSH_BOOT_CONTROL__BOOT_MODE_MASK);
@@ -675,6 +724,11 @@ static int rshim_fuse_misc_read(struct cuse_dev *cdev, int fflags,
   p += n;
   len -= n;
 
+  n = snprintf(p, len, "%-16s%d (seconds)\n", "USB_TIMEOUT",
+               rshim_usb_timeout);
+  p += n;
+  len -= n;
+
   n = snprintf(p, len, "%-16s%d (0:normal, 1:drop)\n", "DROP_MODE",
                bd->drop_mode);
   p += n;
@@ -710,12 +764,23 @@ static int rshim_fuse_misc_read(struct cuse_dev *cdev, int fflags,
     rc = bd->read_rshim(bd, RSHIM_CHANNEL, bd->regs->uptime,
                         &value, RSHIM_REG_SIZE_8B);
     if (!rc) {
-      n = snprintf(p, len, "%-16s%ld(s)\n", "UP_TIME", value/BF3_REF_CLK_IN_HZ);
+      n = snprintf(p, len, "%-16s%lld(s)\n", "UP_TIME",
+                   (unsigned long long)value/BF3_REF_CLK_IN_HZ);
       p += n;
       len -= n;
     }
+
+    n = snprintf(p, len, "%-16s%d (0:no, 1:yes)\n", "SECURE_NIC_MODE",
+                 bd->locked_mode);
+    p += n;
+    len -= n;
   }
 
+  n = snprintf(p, len, "%-16s%d (1: send Force command)\n", "FORCE_CMD",
+      rshim_force_cmd_pending[bd->index]);
+  p += n;
+  len -= n;
+
   if (bd->display_level == 1) {
     gettimeofday(&tp, NULL);
 
@@ -739,7 +804,7 @@ static int rshim_fuse_misc_read(struct cuse_dev *cdev, int fflags,
     ts.tv_nsec = tp.tv_usec * 1000;
     rc = pthread_cond_timedwait(&bd->ctrl_wait_cond, &bd->mutex, &ts);
     if (rc)
-      RSHIM_DBG("Timeout in getting peer response.\n");
+      RSHIM_DBG("rshim%d timeout in getting peer response\n", bd->index);
     n = snprintf(p, len, "%-16s%02x:%02x:%02x:%02x:%02x:%02x (rw)\n",
                    "PEER_MAC", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
     p += n;
@@ -753,6 +818,11 @@ static int rshim_fuse_misc_read(struct cuse_dev *cdev, int fflags,
     n = snprintf(p, len, "%-16s%d %d (rw)\n",
                    "VLAN_ID", bd->vlan[0], bd->vlan[1]);
     p += n;
+    len -= n;
+
+    n = snprintf(p, len, "%-16s%d (0:no, 1:yes)\n", "CLEAR_ON_READ",
+                 bd->clear_on_read);
+    p += n;
   } else if (bd->display_level == 2) {
     n = rshim_log_show(bd, p, len);
     p += n;
@@ -798,7 +868,7 @@ static int rshim_fuse_misc_write(struct cuse_dev *cdev, int fflags,
   rshim_backend_t *bd = cuse_dev_get_priv0(cdev);
   const char *p = buf;
 #endif
-  int i, rc = 0, value = 0, mac[6], vlan[2] = {0}, old_value;
+  int i, rc = 0, value = 0, mac[6], vlan[2] = {0};
   char opn[RSHIM_YU_BOOT_RECORD_OPN_SIZE + 1] = "";
   uint64_t val64 = 0;
   char key[32];
@@ -838,41 +908,25 @@ static int rshim_fuse_misc_write(struct cuse_dev *cdev, int fflags,
     if (sscanf(p, "%d", &value) != 1)
       goto invalid;
     bd->boot_timeout = value;
+  } else if (strcmp(key, "USB_TIMEOUT") == 0) {
+    if (sscanf(p, "%d", &value) != 1)
+      goto invalid;
+    if (value < 0 || value > 300)
+      goto invalid;
+    if (value == 0) {
+      // restore default value
+      rshim_usb_timeout = RSHIM_USB_TIMEOUT;
+    } else {
+      rshim_usb_timeout = value;
+    }
   } else if (strcmp(key, "DROP_MODE") == 0) {
     if (sscanf(p, "%d", &value) != 1)
       goto invalid;
-
-    pthread_mutex_lock(&bd->mutex);
-    old_value = (int)bd->drop_mode;
-    value = !!value;
-    if (value == old_value) {
-      pthread_mutex_unlock(&bd->mutex);
-      goto done;
-    }
-
-    bd->drop_mode = 0;
-    if (bd->enable_device && bd->enable_device(bd, value ? false : true))
-      bd->drop_mode = 1;
-    else
-      bd->drop_mode = value;
-
-    if (bd->drop_mode)
-      bd->drop_pkt = 1;
-    pthread_mutex_unlock(&bd->mutex);
-    /*
-     * Check if another endpoint driver has already attached to the
-     * same rshim device before enabling it.
-     */
-    if (!bd->drop_mode) {
-      rshim_lock();
-      pthread_mutex_lock(&bd->mutex);
-      if (rshim_access_check(bd)) {
-        RSHIM_WARN("rshim%d is not accessible\n", bd->index);
-        bd->drop_mode = 1;
-      }
-      pthread_mutex_unlock(&bd->mutex);
-      rshim_unlock();
-    }
+    rc = rshim_set_drop_mode(bd, value);
+  } else if (strcmp(key, "CLEAR_ON_READ") == 0) {
+    if (sscanf(p, "%d", &value) != 1)
+      goto invalid;
+    bd->clear_on_read = !!value;
   } else if (strcmp(key, "BOOT_MODE") == 0) {
     if (sscanf(p, "%x", &value) != 1)
       goto invalid;
@@ -899,6 +953,7 @@ static int rshim_fuse_misc_write(struct cuse_dev *cdev, int fflags,
       }
 
       /* SW reset. */
+      RSHIM_INFO("rshim%d SW RESET\n", bd->index);
       pthread_mutex_lock(&bd->mutex);
       rc = rshim_reset_control(bd);
       pthread_mutex_unlock(&bd->mutex);
@@ -951,7 +1006,7 @@ static int rshim_fuse_misc_write(struct cuse_dev *cdev, int fflags,
       goto invalid;
     rshim_set_opn(bd, opn, RSHIM_YU_BOOT_RECORD_OPN_SIZE);
   } else if (!strcmp(key, "DEBUG_CODE")) {
-    if (sscanf(p, " 0x%lx", &val64) != 1)
+    if (sscanf(p, " 0x%llx", (unsigned long long *)&val64) != 1)
       goto invalid;
     pthread_mutex_lock(&bd->mutex);
     rc = bd->write_rshim(bd, RSHIM_CHANNEL, bd->regs->scratchpad1,
@@ -959,6 +1014,41 @@ static int rshim_fuse_misc_write(struct cuse_dev *cdev, int fflags,
     if (!rc)
       bd->debug_code = val64;
     pthread_mutex_unlock(&bd->mutex);
+  } else if (strcmp(key, "FORCE_CMD") == 0) {
+    if (sscanf(p, "%x", &value) != 1)
+      goto invalid;
+    if (value) {
+      if (!bd->drop_mode)
+        rc = -EINVAL;
+      else
+        rshim_force_cmd_pending[bd->index] = 1;
+    }
+  } else if (strcmp(key, "DEV_NAME") == 0) {
+    if (bd->type != RSH_BACKEND_PCIE) {
+      rc = -EINVAL;
+      goto invalid;
+    }
+
+    if (strncmp(bd->dev_name, "pcie-", 5)) {
+      RSHIM_ERR("rshim%d device name %s doesn't match expected PCIe format\n",
+                bd->index, bd->dev_name);
+      rc = -EINVAL;
+      goto invalid;
+    }
+
+    RSHIM_INFO("rshim%d starting device refresh (current: %s)\n", bd->index,
+               bd->dev_name);
+
+    pthread_mutex_lock(&bd->mutex);
+    rc = bd->pcie_refresh_dev(bd);
+    pthread_mutex_unlock(&bd->mutex);
+    if (rc) {
+      RSHIM_ERR("rshim%d failed to refresh PCIe device name for %s\n", 
+                bd->index, bd->dev_name);
+    } else {
+      RSHIM_INFO("rshim%d device refresh success (new: %s)\n",
+                 bd->index, bd->dev_name);
+    }
   } else {
 invalid:
 #ifdef __linux__
@@ -969,7 +1059,6 @@ invalid:
 #endif
   }
 
-done:
 #ifdef __linux__
   if (!rc)
     fuse_reply_write(req, size);
@@ -1022,28 +1111,6 @@ static const struct cuse_methods rshim_misc_fops = {
 
 /* Rshim file operations routines */
 
-/* ioctl message header. */
-typedef struct {
-  uint32_t addr;
-  uint64_t data;
-} __attribute__((packed)) rshim_ioctl_msg;
-
-/* ioctl message header for Mustang. Unlike BF1 and BF2, Mustang
- * HW enables different USB transfer sizes: 1B, 2B, 4B and 8B.
- */
-typedef struct {
-  uint32_t addr;
-  uint64_t data;
-  uint8_t data_size;
-} __attribute__((packed)) rshim_ioctl_msg2;
-
-enum {
-  RSHIM_IOC_READ = _IOWR('R', 0, rshim_ioctl_msg),
-  RSHIM_IOC_WRITE = _IOWR('R', 1, rshim_ioctl_msg),
-  RSHIM_IOC_READ2 = _IOWR('R', 0, rshim_ioctl_msg2),
-  RSHIM_IOC_WRITE2 = _IOWR('R', 1, rshim_ioctl_msg2),
-};
-
 #ifdef __linux__
 static void rshim_fuse_rshim_ioctl(fuse_req_t req, int cmd, void *arg,
                                    struct fuse_file_info *fi,
@@ -1094,9 +1161,8 @@ static void rshim_fuse_rshim_ioctl(fuse_req_t req, int cmd, void *arg,
      */
     chan = msg.addr >> 16;
     offset = msg.addr & 0xFFFF;
-    if (bd->ver_id <= RSHIM_BLUEFIELD_2 || strncmp(bd->dev_name, "usb", 3)) {
+    if ((bd->ver_id <= RSHIM_BLUEFIELD_2) || (bd->type != RSH_BACKEND_USB))
       chan &= 0xF;
-    }
 
     if (cmd == RSHIM_IOC_WRITE) {
       pthread_mutex_lock(&bd->mutex);
@@ -1339,14 +1405,14 @@ int rshim_fuse_init(rshim_backend_t *bd)
                                       (char **)argv,
                                       &ci, ops[i], &multithreaded, bd);
     if (!bd->fuse_session[i]) {
-      RSHIM_ERR("Failed to setup CUSE %s\n", name);
+      RSHIM_ERR("rshim%d failed to setup CUSE %s\n", bd->index, name);
       return -1;
     }
     fuse_remove_signal_handlers(bd->fuse_session[i]);
     rc = pthread_create(&bd->fuse_thread[i], NULL, cuse_worker,
                         bd->fuse_session[i]);
     if (rc) {
-      RSHIM_ERR("Failed to create cuse thread %m\n");
+      RSHIM_ERR("rshim%d failed to create cuse thread\n", bd->index);
       return rc;
     }
 #elif defined(__FreeBSD__)
@@ -1358,13 +1424,13 @@ int rshim_fuse_init(rshim_backend_t *bd)
       cuse_dev_create(ops[i], bd, NULL, 0 /* UID_ROOT */, 0 /* GID_WHEEL */,
                       0600, "rshim%d/%s", bd->index, name);
     if (!bd->fuse_session[i]) {
-      RSHIM_ERR("Failed to setup CUSE %s\n", name);
+      RSHIM_ERR("rshim%d failed to setup CUSE %s\n", bd->index, name);
       return -1;
     }
     rc = pthread_create(&bd->fuse_thread[i], NULL, cuse_worker,
                         bd->fuse_session[i]);
     if (rc) {
-      RSHIM_ERR("Failed to create cuse thread %m");
+      RSHIM_ERR("rshim%d failed to create cuse thread\n", bd->index);
       return rc;
     }
 #endif
