@@ -17,6 +17,7 @@
 #include <ucs/sys/compiler.h>
 #include <ucs/sys/string.h>
 #include <ucs/vfs/base/vfs_obj.h>
+#include <ucs/vfs/sock/vfs_sock.h>
 #include <sys/signal.h>
 
 
@@ -74,6 +75,22 @@ static UCS_CONFIG_DEFINE_ARRAY(signo,
                                UCS_CONFIG_TYPE_SIGNO);
 
 
+#define UCS_DISTANCE_KEYS_DESCRIPTION(_field) \
+    {"phb", \
+     "connection traversing PCIe as well as a PCIe Host Bridge (typically " \
+     "the CPU)", \
+     ucs_offsetof(ucs_global_opts_t, dist.phb._field)}, \
+    {"node", \
+     "connection traversing PCIe as well as the interconnect between PCIe " \
+     "Host Bridges within a NUMA node", \
+     ucs_offsetof(ucs_global_opts_t, dist.node._field)}, \
+    {"sys", \
+     "connection traversing PCIe as well as the SMP interconnect between " \
+     "NUMA nodes", \
+     ucs_offsetof(ucs_global_opts_t, dist.sys._field)}, \
+    {NULL}
+
+
 static ucs_config_field_t ucs_global_opts_table[] = {
  {"LOG_LEVEL", "warn",
   "UCS logging level. Messages with a level higher or equal to the selected "
@@ -112,7 +129,7 @@ static ucs_config_field_t ucs_global_opts_table[] = {
 #if ENABLE_DEBUG_DATA
   "bt,freeze",
 #else
-  "bt",
+  "none",
 #endif
   "Error signal handling mode. Either 'none' to disable signal interception,\n"
   "or a combination of:\n"
@@ -175,7 +192,7 @@ static ucs_config_field_t ucs_global_opts_table[] = {
   "Directory to search for loadable modules",
   ucs_offsetof(ucs_global_opts_t, module_dir), UCS_CONFIG_TYPE_STRING},
 
- {"MODULE_LOG_LEVEL", "trace",
+ {"MODULE_LOG_LEVEL", "debug",
   "Logging level for module loader",
   ucs_offsetof(ucs_global_opts_t, module_log_level), UCS_CONFIG_TYPE_ENUM(ucs_log_level_names)},
 
@@ -191,6 +208,16 @@ static ucs_config_field_t ucs_global_opts_table[] = {
   "The list order decides the priority of the providers.",
   ucs_offsetof(ucs_global_opts_t, topo_prio), UCS_CONFIG_TYPE_STRING_ARRAY},
 
+ {"DISTANCE_LAT", "phb:300ns,node:300ns,sys:500ns",
+  "Estimated latency between system devices", 0,
+  UCS_CONFIG_TYPE_KEY_VALUE(UCS_CONFIG_TYPE_TIME,
+                            UCS_DISTANCE_KEYS_DESCRIPTION(latency))},
+
+ {"DISTANCE_BW", "auto",
+  "Estimated bandwidth between system devices", 0,
+  UCS_CONFIG_TYPE_KEY_VALUE(UCS_CONFIG_TYPE_BW,
+                            UCS_DISTANCE_KEYS_DESCRIPTION(bandwidth))},
+
  {NULL}
 };
 
@@ -225,6 +252,10 @@ static ucs_config_field_t ucs_global_opts_read_only_table[] = {
   "Enable virtual monitoring filesystem",
   ucs_offsetof(ucs_global_opts_t, vfs_enable), UCS_CONFIG_TYPE_BOOL},
 
+  {"VFS_SOCK_PATH", UCX_VFS_SOCK_DEFAULT_PATH,
+   "Listening UNIX socket path of the VFS daemon.",
+   ucs_offsetof(ucs_global_opts_t, vfs_sock_path), UCS_CONFIG_TYPE_STRING},
+
  {"VFS_THREAD_AFFINITY", "n",
   "Enable inheriting main process affinity for virtual monitoring filesystem\n"
   "service thread. Setting this value to 'n' will allow the service thread to\n"
@@ -316,26 +347,24 @@ UCS_CONFIG_DECLARE_TABLE(ucs_global_opts_read_only_table,
                          "UCS global (runtime read-only)", NULL,
                          ucs_global_opts_t)
 
-
 ucs_status_t ucs_global_opts_set_value(const char *name, const char *value)
 {
-    ucs_status_t status = ucs_global_opts_set_value_modifiable(name, value);
-
+    ucs_status_t status = ucs_config_parser_set_value(&ucs_global_opts,
+                                                      ucs_global_opts_table,
+                                                      NULL, name, value);
     if (status != UCS_ERR_NO_ELEM) {
         return status;
     }
 
     return ucs_config_parser_set_value(&ucs_global_opts,
-                                       ucs_global_opts_read_only_table,
+                                       ucs_global_opts_read_only_table, NULL,
                                        name, value);
 }
 
-ucs_status_t ucs_global_opts_set_value_modifiable(const char *name,
-                                                  const char *value)
+int ucs_global_opts_is_read_only(const char *name)
 {
-    return ucs_config_parser_set_value(&ucs_global_opts,
-                                       ucs_global_opts_table, name,
-                                       value);
+    return ucs_config_parser_has_field(ucs_global_opts_read_only_table, NULL,
+                                       name);
 }
 
 ucs_status_t ucs_global_opts_get_value(const char *name, char *value,
@@ -383,12 +412,12 @@ void ucs_global_opts_print(FILE *stream, ucs_config_print_flags_t print_flags)
 {
     ucs_config_parser_print_opts(stream, "Global configuration",
                                  &ucs_global_opts, ucs_global_opts_table, NULL,
-                                 UCS_DEFAULT_ENV_PREFIX, print_flags);
+                                 UCS_DEFAULT_ENV_PREFIX, print_flags, NULL);
     ucs_config_parser_print_opts(stream,
                                  "Global configuration (runtime read-only)",
                                  &ucs_global_opts,
                                  ucs_global_opts_read_only_table, NULL,
-                                 UCS_DEFAULT_ENV_PREFIX, print_flags);
+                                 UCS_DEFAULT_ENV_PREFIX, print_flags, NULL);
 }
 
 static void ucs_vfs_read_log_level(void *obj, ucs_string_buffer_t *strb,
