@@ -13,8 +13,8 @@
 #define UCT_VERNO_PATCH            0
 #define UCT_VERNO_EXTRA            ""
 #define UCT_VERNO_STRING           "1.20.0"
-#define UCT_SCM_VERSION            "03898fe"
-#define UCT_SCM_BRANCH             "master"
+#define UCT_SCM_VERSION            "d9a4f35"
+#define UCT_SCM_BRANCH             ""
 
 #define UCT_MINOR_BIT              (16UL)
 #define UCT_MAJOR_BIT              (24UL)
