@@ -216,13 +216,31 @@ static void uct_ud_ep_purge_outstanding(uct_ud_ep_t *ep)
     uct_ud_iface_t *iface = ucs_derived_of(ep->super.super.iface, uct_ud_iface_t);
     uct_ud_ctl_desc_t *cdesc;
     ucs_queue_iter_t iter;
+    khint_t khiter;
 
     ucs_trace_func("ep=%p", ep);
 
-    ucs_queue_for_each_safe(cdesc, iter, &iface->tx.outstanding_q, queue) {
-        if (cdesc->ep == ep) {
-            ucs_queue_del_iter(&iface->tx.outstanding_q, iter);
-            uct_ud_iface_ctl_skb_complete(iface, cdesc, 0);
+    if (uct_ib_iface_device(&iface->super)->ordered_send_comp) {
+        ucs_queue_for_each_safe(cdesc, iter, &iface->tx.outstanding.queue,
+                                queue) {
+            if (cdesc->ep == ep) {
+                ucs_queue_del_iter(&iface->tx.outstanding.queue, iter);
+                uct_ud_iface_ctl_skb_complete(iface, cdesc, 0);
+            }
+        }
+    } else {
+        for (khiter = kh_begin(&iface->tx.outstanding.map);
+             khiter != kh_end(&iface->tx.outstanding.map); ++khiter) {
+            if (!kh_exist(&iface->tx.outstanding.map, khiter)) {
+                continue;
+            }
+
+            cdesc = kh_value(&iface->tx.outstanding.map, khiter);
+            if (cdesc->ep == ep) {
+                kh_del(uct_ud_iface_ctl_desc_hash, &iface->tx.outstanding.map,
+                       khiter);
+                uct_ud_iface_ctl_skb_complete(iface, cdesc, 0);
+            }
         }
     }
 
@@ -710,6 +728,15 @@ uct_ud_ep_process_ack(uct_ud_iface_t *iface, uct_ud_ep_t *ep,
         return;
     }
 
+    /* Ignore stale ACKs for unsent packets (e.g., after endpoint reset).
+     * Valid ACK PSN must be in (acked_psn, psn). */
+    if (ucs_unlikely(UCT_UD_PSN_COMPARE(ep->tx.psn, <=, ack_psn))) {
+        ucs_debug("ep %p: ignoring invalid ack_psn=%u (tx.psn=%u acked_psn=%u)"
+                  " - possibly stale from previous connection",
+                  ep, ack_psn, ep->tx.psn, ep->tx.acked_psn);
+        return;
+    }
+
     ep->tx.acked_psn = ack_psn;
     ucs_assertv(UCT_UD_PSN_COMPARE(ep->tx.acked_psn, <, ep->tx.psn),
                 "ep %p: flags=0x%x acked_psn=%u must be smaller than"
@@ -811,21 +838,30 @@ static void uct_ud_ep_rx_creq(uct_ud_iface_t *iface, uct_ud_neth_t *neth)
         ep->rx.ooo_pkts.head_sn = neth->psn;
         uct_ud_peer_copy(&ep->peer, ucs_unaligned_ptr(&ctl->peer));
         uct_ud_ep_ctl_op_add(iface, ep, UCT_UD_EP_OP_CREP);
-    } else if (ep->dest_ep_id == UCT_UD_EP_NULL_ID) {
-        /* simultaneous CREQ */
+    } else if (uct_ib_unpack_uint24(ctl->conn_req.ep_addr.ep_id) != ep->dest_ep_id) {
+        if (ep->dest_ep_id == UCT_UD_EP_NULL_ID) {
+            /* simultaneous CREQ */
+            ucs_debug("simultaneous CREQ ep=%p"
+                      "(iface=%p conn_sn=%d ep_id=%d, dest_ep_id=%d rx_psn=%u)",
+                      ep, iface, ep->conn_sn, ep->ep_id,
+                      ep->dest_ep_id, ep->rx.ooo_pkts.head_sn);
+            if (UCT_UD_PSN_COMPARE(ep->tx.psn, >, UCT_UD_INITIAL_PSN)) {
+                /* our own creq was sent, treat incoming creq as ack and remove our
+                 * own from tx window
+                 */
+                uct_ud_ep_process_ack(iface, ep, UCT_UD_INITIAL_PSN, 0);
+            }
+        } else {
+            /* stale EP reuse */
+            ucs_debug("iface=%p: detected stale EP reuse (ep=%p conn_sn=%d "
+                      "old_dest_ep_id=%d new_ep_id=%d), updating dest_ep_id",
+                      iface, ep, ep->conn_sn, ep->dest_ep_id,
+                      uct_ib_unpack_uint24(ctl->conn_req.ep_addr.ep_id));
+        }
+        /* Update dest_ep_id */
         uct_ud_ep_set_dest_ep_id(ep, uct_ib_unpack_uint24(ctl->conn_req.ep_addr.ep_id));
         ep->rx.ooo_pkts.head_sn = neth->psn;
         uct_ud_peer_copy(&ep->peer, ucs_unaligned_ptr(&ctl->peer));
-        ucs_debug("simultaneous CREQ ep=%p"
-                  "(iface=%p conn_sn=%d ep_id=%d, dest_ep_id=%d rx_psn=%u)",
-                  ep, iface, ep->conn_sn, ep->ep_id,
-                  ep->dest_ep_id, ep->rx.ooo_pkts.head_sn);
-        if (UCT_UD_PSN_COMPARE(ep->tx.psn, >, UCT_UD_INITIAL_PSN)) {
-            /* our own creq was sent, treat incoming creq as ack and remove our
-             * own from tx window
-             */
-            uct_ud_ep_process_ack(iface, ep, UCT_UD_INITIAL_PSN, 0);
-        }
         uct_ud_ep_ctl_op_add(iface, ep, UCT_UD_EP_OP_CREP);
     }
 
@@ -1259,9 +1295,8 @@ static uct_ud_send_skb_t *uct_ud_ep_prepare_crep(uct_ud_ep_t *ep)
     ucs_assert_always(ep->dest_ep_id != UCT_UD_EP_NULL_ID);
     ucs_assert_always(ep->ep_id != UCT_UD_EP_NULL_ID);
 
-    /* Check that CREQ is neither scheduled nor waiting for CREP ack */
-    ucs_assertv_always(!uct_ud_ep_ctl_op_check(ep, UCT_UD_EP_OP_CREQ) &&
-                       uct_ud_ep_is_last_ack_received(ep),
+    /* Check that CREQ is not scheduled */
+    ucs_assertv_always(!uct_ud_ep_ctl_op_check(ep, UCT_UD_EP_OP_CREQ),
                        "iface=%p ep=%p conn_sn=%d ep_id=%d, dest_ep_id=%d rx_psn=%u "
                        "ep_flags=0x%x ctl_ops=0x%x rx_creq_count=%d",
                        iface, ep, ep->conn_sn, ep->ep_id, ep->dest_ep_id,
