@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -120,7 +120,7 @@ void buffer_pool::free_bpool_resources()
 
 void buffer_pool::register_memory(ib_ctx_handler *p_ib_ctx_h)
 {
-	m_allocator.register_memory(m_size, p_ib_ctx_h, VMA_IBV_ACCESS_LOCAL_WRITE);
+	m_allocator.register_memory(m_size, p_ib_ctx_h, IBV_ACCESS_LOCAL_WRITE);
 }
 
 void buffer_pool::print_val_tbl()
