@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -60,11 +60,9 @@ cq_mgr::cq_mgr(ring_simple* p_ring, ib_ctx_handler* p_ib_ctx_handler, int cq_siz
 	,m_n_sysvar_cq_poll_batch_max(safe_mce_sys().cq_poll_batch_max)
 	,m_n_sysvar_progress_engine_wce_max(safe_mce_sys().progress_engine_wce_max)
 	,m_p_cq_stat(&m_cq_stat_static) // use local copy of stats by default (on rx cq get shared memory stats)
-	,m_transport_type(m_p_ring->get_transport_type())
 	,m_p_next_rx_desc_poll(NULL)
 	,m_n_sysvar_rx_prefetch_bytes_before_poll(safe_mce_sys().rx_prefetch_bytes_before_poll)
 	,m_n_sysvar_rx_prefetch_bytes(safe_mce_sys().rx_prefetch_bytes)
-	,m_sz_transport_header(0)
 	,m_p_ib_ctx_handler(p_ib_ctx_handler)
 	,m_n_sysvar_rx_num_wr_to_post_recv(safe_mce_sys().rx_num_wr_to_post_recv)
 	,m_comp_event_channel(p_comp_event_channel)
@@ -93,38 +91,17 @@ cq_mgr::cq_mgr(ring_simple* p_ring, ib_ctx_handler* p_ib_ctx_handler, int cq_siz
 
 void cq_mgr::configure(int cq_size)
 {
-	vma_ibv_cq_init_attr attr;
-	memset(&attr, 0, sizeof(attr));
-
-	prep_ibv_cq(attr);
-
-	m_p_ibv_cq = vma_ibv_create_cq(m_p_ib_ctx_handler->get_ibv_context(),
-			cq_size - 1, (void *)this, m_comp_event_channel, 0, &attr);
+	m_p_ibv_cq = ibv_create_cq(m_p_ib_ctx_handler->get_ibv_context(),
+			cq_size - 1, (void *)this, m_comp_event_channel, 0);
 	BULLSEYE_EXCLUDE_BLOCK_START
 	if (!m_p_ibv_cq) {
 		throw_vma_exception("ibv_create_cq failed");
 	}
 	BULLSEYE_EXCLUDE_BLOCK_END
 	VALGRIND_MAKE_MEM_DEFINED(m_p_ibv_cq, sizeof(ibv_cq));
-	switch (m_transport_type) {
-	case VMA_TRANSPORT_IB:
-		m_sz_transport_header = GRH_HDR_LEN;
-		break;
-	case VMA_TRANSPORT_ETH:
-		m_sz_transport_header = ETH_HDR_LEN;
-		break;
-	BULLSEYE_EXCLUDE_BLOCK_START
-	default:
-		cq_logpanic("Unknown transport type: %d", m_transport_type);
-		break;
-	BULLSEYE_EXCLUDE_BLOCK_END
-	}
 
 	if (m_b_is_rx) {
 		vma_stats_instance_create_cq_block(m_p_cq_stat);
-	}
-	
-	if (m_b_is_rx) {
 		m_b_is_rx_hw_csum_on = vma_is_rx_hw_csum_supported(m_p_ib_ctx_handler->get_ibv_device_attr());
 		cq_logdbg("RX CSUM support = %d", m_b_is_rx_hw_csum_on);
 	}
@@ -132,13 +109,6 @@ void cq_mgr::configure(int cq_size)
 	cq_logdbg("Created CQ as %s with fd[%d] and of size %d elements (ibv_cq_hndl=%p)", (m_b_is_rx?"Rx":"Tx"), get_channel_fd(), cq_size, m_p_ibv_cq);
 }
 
-void cq_mgr::prep_ibv_cq(vma_ibv_cq_init_attr& attr) const
-{
-	if (m_p_ib_ctx_handler->get_ctx_time_converter_status()) {
-		vma_ibv_cq_init_ts_attr(&attr);
-	}
-}
-
 uint32_t cq_mgr::clean_cq()
 {
 	uint32_t ret_total = 0;
@@ -146,7 +116,7 @@ uint32_t cq_mgr::clean_cq()
 	uint64_t cq_poll_sn = 0;
 	mem_buf_desc_t* buff = NULL;
 	/* coverity[stack_use_local_overflow] */
-	vma_ibv_wc wce[MCE_MAX_CQ_POLL_BATCH];
+	ibv_wc wce[MCE_MAX_CQ_POLL_BATCH];
 	while ((ret = poll(wce, MCE_MAX_CQ_POLL_BATCH, &cq_poll_sn)) > 0) {
 		for (int i = 0; i < ret; i++) {
 			if (m_b_is_rx) {
@@ -304,7 +274,7 @@ void cq_mgr::return_extra_buffers()
 	m_p_cq_stat->n_buffer_pool_len = m_rx_pool.size();
 }
 
-int cq_mgr::poll(vma_ibv_wc* p_wce, int num_entries, uint64_t* p_cq_poll_sn)
+int cq_mgr::poll(ibv_wc* p_wce, int num_entries, uint64_t* p_cq_poll_sn)
 {
 	// Assume locked!!!
 	cq_logfuncall("");
@@ -320,7 +290,7 @@ int cq_mgr::poll(vma_ibv_wc* p_wce, int num_entries, uint64_t* p_cq_poll_sn)
 #ifdef RDTSC_MEASURE_RX_VMA_TCP_IDLE_POLL
 	RDTSC_TAKE_END(g_rdtsc_instr_info_arr[RDTSC_FLOW_RX_VMA_TCP_IDLE_POLL]);
 #endif //RDTSC_MEASURE_RX_VMA_TCP_IDLE_POLL
-	int ret = vma_ibv_poll_cq(m_p_ibv_cq, num_entries, p_wce);
+	int ret = ibv_poll_cq(m_p_ibv_cq, num_entries, p_wce);
 	if (ret <= 0) {
 #ifdef RDTSC_MEASURE_RX_VERBS_IDLE_POLL
 		RDTSC_TAKE_END(g_rdtsc_instr_info_arr[RDTSC_FLOW_RX_VERBS_IDLE_POLL]);
@@ -377,7 +347,7 @@ int cq_mgr::poll(vma_ibv_wc* p_wce, int num_entries, uint64_t* p_cq_poll_sn)
 	return ret;
 }
 
-void cq_mgr::process_cq_element_log_helper(mem_buf_desc_t* p_mem_buf_desc, vma_ibv_wc* p_wce)
+void cq_mgr::process_cq_element_log_helper(mem_buf_desc_t* p_mem_buf_desc, ibv_wc* p_wce)
 {
 	BULLSEYE_EXCLUDE_BLOCK_START
 	// wce with bad status value
@@ -402,7 +372,7 @@ void cq_mgr::process_cq_element_log_helper(mem_buf_desc_t* p_mem_buf_desc, vma_i
 	cq_logfunc("wce error status '%s' [%d] (wr_id=%p, qp_num=%x)", priv_ibv_wc_status_str(p_wce->status), p_wce->status, p_wce->wr_id, p_wce->qp_num);
 }
 
-mem_buf_desc_t* cq_mgr::process_cq_element_tx(vma_ibv_wc* p_wce)
+mem_buf_desc_t* cq_mgr::process_cq_element_tx(ibv_wc* p_wce)
 {
 	// Assume locked!!!
 	cq_logfuncall("");
@@ -436,7 +406,7 @@ mem_buf_desc_t* cq_mgr::process_cq_element_tx(vma_ibv_wc* p_wce)
 	return p_mem_buf_desc;
 }
 
-mem_buf_desc_t* cq_mgr::process_cq_element_rx(vma_ibv_wc* p_wce)
+mem_buf_desc_t* cq_mgr::process_cq_element_rx(ibv_wc* p_wce)
 {
 	// Assume locked!!!
 	cq_logfuncall("");
@@ -482,7 +452,7 @@ mem_buf_desc_t* cq_mgr::process_cq_element_rx(vma_ibv_wc* p_wce)
 
 	p_mem_buf_desc->rx.is_sw_csum_need = !(m_b_is_rx_hw_csum_on && vma_wc_rx_hw_csum_ok(*p_wce));
 
-	if (likely(vma_wc_opcode(*p_wce) & VMA_IBV_WC_RECV)) {
+	if (likely(vma_wc_opcode(*p_wce) & IBV_WC_RECV)) {
 		// Save recevied total bytes
 		p_mem_buf_desc->sz_data = p_wce->byte_len;
 
@@ -491,17 +461,10 @@ mem_buf_desc_t* cq_mgr::process_cq_element_rx(vma_ibv_wc* p_wce)
 		p_mem_buf_desc->rx.context = this;
 		p_mem_buf_desc->rx.socketxtreme_polled = false;
 
-		//this is not a deadcode if timestamping is defined in verbs API
-		// coverity[dead_error_condition]
-		if (vma_wc_flags(*p_wce) & VMA_IBV_WC_WITH_TIMESTAMP) {
-			p_mem_buf_desc->rx.hw_raw_timestamp = vma_wc_timestamp(*p_wce);
-		}
-
 		VALGRIND_MAKE_MEM_DEFINED(p_mem_buf_desc->p_buffer, p_mem_buf_desc->sz_data);
 
-		prefetch_range((uint8_t*)p_mem_buf_desc->p_buffer + m_sz_transport_header, 
-				std::min(p_mem_buf_desc->sz_data - m_sz_transport_header, (size_t)m_n_sysvar_rx_prefetch_bytes));
-		//prefetch((uint8_t*)p_mem_buf_desc->p_buffer + m_sz_transport_header);
+		prefetch_range((uint8_t*)p_mem_buf_desc->p_buffer + ETH_HDR_LEN, 
+				std::min(p_mem_buf_desc->sz_data - ETH_HDR_LEN, (size_t)m_n_sysvar_rx_prefetch_bytes));
 	}
 
 	return p_mem_buf_desc;
@@ -616,7 +579,7 @@ int cq_mgr::poll_and_process_element_rx(uint64_t* p_cq_poll_sn, void* pv_fd_read
 	cq_logfuncall("");
 
 	/* coverity[stack_use_local_overflow] */
-	vma_ibv_wc wce[MCE_MAX_CQ_POLL_BATCH];
+	ibv_wc wce[MCE_MAX_CQ_POLL_BATCH];
 
 	int ret;
 	uint32_t ret_rx_processed = process_recv_queue(pv_fd_ready_array);
@@ -638,7 +601,7 @@ int cq_mgr::poll_and_process_element_rx(uint64_t* p_cq_poll_sn, void* pv_fd_read
 		for (int i = 0; i < ret; i++) {
 			mem_buf_desc_t *buff = process_cq_element_rx((&wce[i]));
 			if (buff) {
-				if (vma_wc_opcode(wce[i]) & VMA_IBV_WC_RECV) {
+				if (vma_wc_opcode(wce[i]) & IBV_WC_RECV) {
 					if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
 							!compensate_qp_poll_success(buff)) {
 						process_recv_buffer(buff, pv_fd_ready_array);
@@ -661,7 +624,7 @@ int cq_mgr::poll_and_process_element_tx(uint64_t* p_cq_poll_sn)
 	cq_logfuncall("");
 	
 	/* coverity[stack_use_local_overflow] */
-	vma_ibv_wc wce[MCE_MAX_CQ_POLL_BATCH];
+	ibv_wc wce[MCE_MAX_CQ_POLL_BATCH];
 	int ret = poll(wce, m_n_sysvar_cq_poll_batch_max, p_cq_poll_sn);
 	if (ret > 0) {
 		m_n_wce_counter += ret;
@@ -772,7 +735,7 @@ int cq_mgr::drain_and_proccess(uintptr_t* p_recycle_buffers_last_wr_id /*=NULL*/
 		(p_recycle_buffers_last_wr_id)) {
 
 		/* coverity[stack_use_local_overflow] */
-		vma_ibv_wc wce[MCE_MAX_CQ_POLL_BATCH];
+		ibv_wc wce[MCE_MAX_CQ_POLL_BATCH];
 		int ret = poll(wce, MCE_MAX_CQ_POLL_BATCH, &cq_poll_sn);
 		if (ret <= 0) {
 			m_b_was_drained = true;
@@ -790,29 +753,19 @@ int cq_mgr::drain_and_proccess(uintptr_t* p_recycle_buffers_last_wr_id /*=NULL*/
 				if (p_recycle_buffers_last_wr_id) {
 					m_p_cq_stat->n_rx_pkt_drop++;
 					reclaim_recv_buffer_helper(buff);
-				} else {
-					bool procces_now = false;
-					if (m_transport_type == VMA_TRANSPORT_ETH) {
-						procces_now = is_eth_tcp_frame(buff);
-					}
-					if (m_transport_type == VMA_TRANSPORT_IB) {
-						procces_now = is_ib_tcp_frame(buff);
-					}
+				} else if (is_eth_tcp_frame(buff)) {
 					// We process immediately all non udp/ip traffic..
-					if (procces_now) {
-						buff->rx.is_vma_thr = true;
-						if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
-								!compensate_qp_poll_success(buff)) {
-							process_recv_buffer(buff, NULL);
-						}
+					buff->rx.is_vma_thr = true;
+					if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
+							!compensate_qp_poll_success(buff)) {
+						process_recv_buffer(buff, NULL);
 					}
-					else { //udp/ip traffic we just put in the cq's rx queue
-						m_rx_queue.push_back(buff);
-						mem_buf_desc_t* buff_cur = m_rx_queue.get_and_pop_front();
-						if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
-								!compensate_qp_poll_success(buff_cur)) {
-							m_rx_queue.push_front(buff_cur);
-						}
+				} else { //udp/ip traffic we just put in the cq's rx queue
+					m_rx_queue.push_back(buff);
+					mem_buf_desc_t* buff_cur = m_rx_queue.get_and_pop_front();
+					if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
+							!compensate_qp_poll_success(buff_cur)) {
+						m_rx_queue.push_front(buff_cur);
 					}
 				}
 			}
