@@ -1,13 +1,11 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
 #include "cq_mgr_mlx5.h"
 
-#if defined(DEFINED_DIRECT_VERBS)
-
 #include <vma/util/valgrind.h>
 #include "cq_mgr.inl"
 #include "cq_mgr_mlx5.inl"
@@ -62,7 +60,7 @@ uint32_t cq_mgr_mlx5::clean_cq()
 	} else {//Tx
 		int ret = 0;
 		/* coverity[stack_use_local_overflow] */
-		vma_ibv_wc wce[MCE_MAX_CQ_POLL_BATCH];
+		ibv_wc wce[MCE_MAX_CQ_POLL_BATCH];
 		while ((ret = cq_mgr::poll(wce, MCE_MAX_CQ_POLL_BATCH, &cq_poll_sn)) > 0) {
 			for (int i = 0; i < ret; i++) {
 				buff = process_cq_element_tx(&wce[i]);
@@ -267,7 +265,7 @@ int cq_mgr_mlx5::drain_and_proccess(uintptr_t* p_recycle_buffers_last_wr_id /*=N
 			for (int i = 0; i < ret; i++) {
 				uint32_t wqe_sz = 0;
 				mlx5_cqe64 *cqe = cqe_arr[i];
-				vma_ibv_wc wce;
+				ibv_wc wce;
 
 				uint16_t wqe_ctr = ntohs(cqe->wqe_counter);
 				if (m_b_is_rx) {
@@ -294,29 +292,19 @@ int cq_mgr_mlx5::drain_and_proccess(uintptr_t* p_recycle_buffers_last_wr_id /*=N
 					if (p_recycle_buffers_last_wr_id) {
 						m_p_cq_stat->n_rx_pkt_drop++;
 						reclaim_recv_buffer_helper(m_rx_hot_buffer);
-					} else {
-						bool procces_now = false;
-						if (m_transport_type == VMA_TRANSPORT_ETH) {
-							procces_now = is_eth_tcp_frame(m_rx_hot_buffer);
-						}
-						if (m_transport_type == VMA_TRANSPORT_IB) {
-							procces_now = is_ib_tcp_frame(m_rx_hot_buffer);
-						}
+					} else if (is_eth_tcp_frame(m_rx_hot_buffer)) {
 						// We process immediately all non udp/ip traffic..
-						if (procces_now) {
-							m_rx_hot_buffer->rx.is_vma_thr = true;
-							if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
-								!compensate_qp_poll_success(m_rx_hot_buffer)) {
-								process_recv_buffer(m_rx_hot_buffer, NULL);
-							}
+						m_rx_hot_buffer->rx.is_vma_thr = true;
+						if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
+							!compensate_qp_poll_success(m_rx_hot_buffer)) {
+							process_recv_buffer(m_rx_hot_buffer, NULL);
 						}
-						else { //udp/ip traffic we just put in the cq's rx queue
-							m_rx_queue.push_back(m_rx_hot_buffer);
-							mem_buf_desc_t* buff_cur = m_rx_queue.get_and_pop_front();
-							if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
-								!compensate_qp_poll_success(buff_cur)) {
-								m_rx_queue.push_front(buff_cur);
-							}
+					} else { //udp/ip traffic we just put in the cq's rx queue
+						m_rx_queue.push_back(m_rx_hot_buffer);
+						mem_buf_desc_t* buff_cur = m_rx_queue.get_and_pop_front();
+						if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
+							!compensate_qp_poll_success(buff_cur)) {
+							m_rx_queue.push_front(buff_cur);
 						}
 					}
 				}
@@ -344,30 +332,20 @@ int cq_mgr_mlx5::drain_and_proccess(uintptr_t* p_recycle_buffers_last_wr_id /*=N
 				if (p_recycle_buffers_last_wr_id) {
 					m_p_cq_stat->n_rx_pkt_drop++;
 					reclaim_recv_buffer_helper(buff);
-				} else {
-					bool procces_now = false;
-					if (m_transport_type == VMA_TRANSPORT_ETH) {
-						procces_now = is_eth_tcp_frame(buff);
-					}
-					if (m_transport_type == VMA_TRANSPORT_IB) {
-						procces_now = is_ib_tcp_frame(buff);
-					}
+				} else if (is_eth_tcp_frame(buff)) {
 					/* We process immediately all non udp/ip traffic.. */
-					if (procces_now) {
-						buff->rx.is_vma_thr = true;
-						if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
-								!compensate_qp_poll_success(buff)) {
-							process_recv_buffer(buff, NULL);
-						}
+					buff->rx.is_vma_thr = true;
+					if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
+							!compensate_qp_poll_success(buff)) {
+						process_recv_buffer(buff, NULL);
 					}
-					else { /* udp/ip traffic we just put in the cq's rx queue */
-						m_rx_queue.push_back(buff);
-						mem_buf_desc_t* buff_cur = m_rx_queue.front();
-						m_rx_queue.pop_front();
-						if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
-								!compensate_qp_poll_success(buff_cur)) {
-							m_rx_queue.push_front(buff_cur);
-						}
+				} else { /* udp/ip traffic we just put in the cq's rx queue */
+					m_rx_queue.push_back(buff);
+					mem_buf_desc_t* buff_cur = m_rx_queue.front();
+					m_rx_queue.pop_front();
+					if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
+							!compensate_qp_poll_success(buff_cur)) {
+						m_rx_queue.push_front(buff_cur);
 					}
 				}
 			}
@@ -444,8 +422,8 @@ mem_buf_desc_t* cq_mgr_mlx5::process_cq_element_rx(mem_buf_desc_t* p_mem_buf_des
 
 	VALGRIND_MAKE_MEM_DEFINED(p_mem_buf_desc->p_buffer, p_mem_buf_desc->sz_data);
 
-	prefetch_range((uint8_t*)p_mem_buf_desc->p_buffer + m_sz_transport_header,
-	std::min(p_mem_buf_desc->sz_data - m_sz_transport_header, (size_t)m_n_sysvar_rx_prefetch_bytes));
+	prefetch_range((uint8_t*)p_mem_buf_desc->p_buffer + ETH_HDR_LEN,
+	std::min(p_mem_buf_desc->sz_data - ETH_HDR_LEN, (size_t)m_n_sysvar_rx_prefetch_bytes));
 
 
 	return p_mem_buf_desc;
@@ -601,7 +579,7 @@ int cq_mgr_mlx5::poll_and_process_element_rx(mem_buf_desc_t **p_desc_lst)
 
 }
 
-inline void cq_mgr_mlx5::cqe64_to_vma_wc(struct mlx5_cqe64 *cqe, vma_ibv_wc *wc)
+inline void cq_mgr_mlx5::cqe64_to_vma_wc(struct mlx5_cqe64 *cqe, ibv_wc *wc)
 {
 	struct mlx5_err_cqe* ecqe = (struct mlx5_err_cqe *)cqe;
 
@@ -612,7 +590,7 @@ inline void cq_mgr_mlx5::cqe64_to_vma_wc(struct mlx5_cqe64 *cqe, vma_ibv_wc *wc)
 	case MLX5_CQE_RESP_SEND:
 	case MLX5_CQE_RESP_SEND_IMM:
 	case MLX5_CQE_RESP_SEND_INV:
-		vma_wc_opcode(*wc) = VMA_IBV_WC_RECV;
+		vma_wc_opcode(*wc) = IBV_WC_RECV;
 		wc->byte_len = ntohl(cqe->byte_cnt);
 		wc->status = IBV_WC_SUCCESS;
 		return;
@@ -680,7 +658,7 @@ int cq_mgr_mlx5::poll_and_process_error_element_tx(struct mlx5_cqe64 *cqe, uint6
 	uint16_t wqe_ctr = ntohs(cqe->wqe_counter);
 	int index = wqe_ctr & (m_qp->m_tx_num_wr - 1);
 	mem_buf_desc_t* buff = NULL;
-	vma_ibv_wc wce;
+	ibv_wc wce;
 
 	// spoil the global sn if we have packets ready
 	union __attribute__((packed)) {
@@ -803,7 +781,7 @@ bool cq_mgr_mlx5::fill_cq_hw_descriptors(struct hw_cq_data &data)
 
 int cq_mgr_mlx5::poll_and_process_error_element_rx(struct mlx5_cqe64 *cqe, void* pv_fd_ready_array)
 {
-	vma_ibv_wc wce;
+	ibv_wc wce;
 
 	memset(&wce, 0, sizeof(wce));
 	wce.wr_id = (uintptr_t)m_rx_hot_buffer;
@@ -814,7 +792,7 @@ int cq_mgr_mlx5::poll_and_process_error_element_rx(struct mlx5_cqe64 *cqe, void*
 
 	m_rx_hot_buffer = cq_mgr::process_cq_element_rx(&wce);
 	if (m_rx_hot_buffer) {
-		if (vma_wc_opcode(wce) & VMA_IBV_WC_RECV) {
+		if (vma_wc_opcode(wce) & IBV_WC_RECV) {
 			if ((++m_qp_rec.debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
 				!compensate_qp_poll_success(m_rx_hot_buffer)) {
 					process_recv_buffer(m_rx_hot_buffer, pv_fd_ready_array);
@@ -825,5 +803,3 @@ int cq_mgr_mlx5::poll_and_process_error_element_rx(struct mlx5_cqe64 *cqe, void*
 
 	return 1;
 }
-
-#endif /* DEFINED_DIRECT_VERBS */
