@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -11,7 +11,6 @@
 #include "cq_mgr.h"
 #include "qp_mgr_eth_mlx5.h"
 
-#if defined(DEFINED_DIRECT_VERBS)
 class qp_mgr_eth_mlx5;
 
 /* Get CQE opcode. */
@@ -57,7 +56,7 @@ private:
 
 	inline struct mlx5_cqe64*   get_cqe64(struct mlx5_cqe64 **cqe_err = NULL);
 	inline void                 cqe64_to_mem_buff_desc(struct mlx5_cqe64 *cqe, mem_buf_desc_t* p_rx_wc_buf_desc, enum buff_status_e& status);
-	void                        cqe64_to_vma_wc(struct mlx5_cqe64 *cqe, vma_ibv_wc *wc);
+	void                        cqe64_to_vma_wc(struct mlx5_cqe64 *cqe, ibv_wc *wc);
 	inline struct mlx5_cqe64*   check_error_completion(struct mlx5_cqe64 *cqe, uint32_t *ci, uint8_t op_own);
 	inline void                 update_global_sn(uint64_t& cq_poll_sn, uint32_t rettotal);
 
@@ -66,5 +65,4 @@ private:
 	};
 };
 
-#endif /* DEFINED_DIRECT_VERBS */
 #endif //CQ_MGR_MLX5_H
