@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -82,7 +82,7 @@ net_device_table_mgr::net_device_table_mgr() :
 	if (m_net_device_map_index.empty()) {
 		int num_devices = 0;
 		struct ibv_device **dev_list = NULL;
-		dev_list = vma_ibv_get_device_list(&num_devices);
+		dev_list = ibv_get_device_list(&num_devices);
 		if (dev_list && num_devices == 0) {
 			ibv_free_device_list(dev_list);
 			ndtm_logdbg("net_device_map is empty %d", num_devices);
@@ -147,8 +147,8 @@ net_device_table_mgr::~net_device_table_mgr()
 
 void net_device_table_mgr::update_tbl()
 {
-	int rc = 0;
-	int fd = -1;
+	int rc;
+	int fd;
 	struct {
 		struct nlmsghdr hdr;
 		struct ifinfomsg infomsg;
@@ -214,9 +214,6 @@ void net_device_table_mgr::update_tbl()
 				case ARPHRD_ETHER:
 					p_net_device_val = new net_device_val_eth(&desc);
 					break;
-				case ARPHRD_INFINIBAND:
-					p_net_device_val = new net_device_val_ib(&desc);
-					break;
 				default:
 					goto next;
 				}
