@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -152,7 +152,12 @@ const char* ring_alloc_logic_attr::to_str()
 	return m_str;
 }
 
-net_device_val::net_device_val(struct net_device_val_desc *desc) : m_lock("net_device_val lock")
+net_device_val::net_device_val(struct net_device_val_desc *desc)
+    : m_lock("net_device_val lock")
+    , m_l2_if_addr {}
+    , m_l2_bc_addr {}
+	, m_str {}
+    , m_base_name {}
 {
 	bool valid = false;
 	ib_ctx_handler* ib_ctx;
@@ -173,7 +178,6 @@ net_device_val::net_device_val(struct net_device_val_desc *desc) : m_lock("net_d
 	m_if_active = 0;
 	m_bond_xmit_hash_policy = XHP_LAYER_2;
 	m_bond_fail_over_mac = 0;
-	m_transport_type = VMA_TRANSPORT_UNKNOWN;
 
 	if (NULL == desc) {
 		nd_logerr("Invalid net_device_val name=%s", "NA");
@@ -252,10 +256,10 @@ net_device_val::net_device_val(struct net_device_val_desc *desc) : m_lock("net_d
 	case LAG_8023ad:
 	case ACTIVE_BACKUP:
 		// this is a bond interface (or a vlan/alias over bond), find the slaves
-		valid = verify_bond_ipoib_or_eth_qp_creation();
+		valid = verify_bond_eth_qp_creation();
 		break;
 	default:
-		valid = (bool)(ib_ctx && verify_ipoib_or_eth_qp_creation(get_ifname_link()));
+		valid = (bool)(ib_ctx && verify_eth_qp_creation(get_ifname_link()));
 		break;
 	}
 
@@ -338,8 +342,8 @@ net_device_val::~net_device_val()
 
 void net_device_val::set_ip_array()
 {
-	int rc = 0;
-	int fd = -1;
+	int rc;
+	int fd;
 	struct {
 		struct nlmsghdr hdr;
 		struct ifaddrmsg addrmsg;
@@ -474,9 +478,6 @@ void net_device_val::set_str()
 	case ARPHRD_ETHER:
 		sprintf(str_x, " type %s", "ether");
 		break;
-	case ARPHRD_INFINIBAND:
-		sprintf(str_x, " type %s", "infiniband");
-		break;
 	default:
 		sprintf(str_x, " type %s", "unknown");
 		break;
@@ -1366,7 +1367,7 @@ L2_address* net_device_val_eth::create_L2_address(const char* ifname)
 		delete m_p_L2_addr;
 		m_p_L2_addr = NULL;
 	}
-	unsigned char hw_addr[ETH_ALEN];
+	unsigned char hw_addr[ETH_ALEN] = {0};
 	get_local_ll_addr(ifname, hw_addr, ETH_ALEN, false);
 	return new ETH_addr(hw_addr);
 }
@@ -1377,7 +1378,7 @@ void net_device_val_eth::create_br_address(const char* ifname)
 		delete m_p_br_addr;
 		m_p_br_addr = NULL;
 	}
-	uint8_t hw_addr[ETH_ALEN];
+	uint8_t hw_addr[ETH_ALEN] = {0};
 	get_local_ll_addr(ifname, hw_addr, ETH_ALEN, true);
 	m_p_br_addr = new ETH_addr(hw_addr);
 
@@ -1392,106 +1393,7 @@ std::string net_device_val_eth::to_str()
 	return std::string("ETH: ") + net_device_val::to_str();
 }
 
-net_device_val_ib::~net_device_val_ib()
-{
-	struct in_addr in;
-	if (1 == inet_pton(AF_INET, BROADCAST_IP, &in)) {
-		g_p_neigh_table_mgr->unregister_observer(neigh_key(ip_address(in.s_addr), this), this);
-	}
-}
-
-void net_device_val_ib::configure()
-{
-	ib_ctx_handler* p_ib_ctx = NULL;
-	struct in_addr in;
-
-	m_p_L2_addr = create_L2_address(get_ifname());
-
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if(m_p_L2_addr == NULL) {
-		nd_logpanic("m_p_L2_addr allocation error");
-	}
-	BULLSEYE_EXCLUDE_BLOCK_END
-
-	create_br_address(get_ifname());
-
-	if (1 == inet_pton(AF_INET, BROADCAST_IP, &in)) {
-		g_p_neigh_table_mgr->unregister_observer(neigh_key(ip_address(in.s_addr), this), this);
-	}
-
-	//Register to IB BR neigh
-	cache_entry_subject<neigh_key, neigh_val*>* p_ces = NULL;
-	if (1 == inet_pton(AF_INET, BROADCAST_IP, &in)) {
-		g_p_neigh_table_mgr->register_observer(neigh_key(ip_address(in.s_addr), this), this, &p_ces);
-	}
-	m_br_neigh = dynamic_cast<neigh_ib_broadcast*>(p_ces);
-
-	p_ib_ctx = g_p_ib_ctx_handler_collection->get_ib_ctx(get_ifname_link());
-	if (!p_ib_ctx || ibv_query_pkey(p_ib_ctx->get_ibv_context(), get_port_from_ifname(get_ifname_link()), 0, &m_pkey)) {
-		nd_logerr("failed querying pkey");
-	}
-	nd_logdbg("pkey: %d", m_pkey);
-}
-
-ring* net_device_val_ib::create_ring(resource_allocation_key *key)
-{
-	ring* ring = NULL;
-
-	NOT_IN_USE(key);
-	try {
-		switch (m_bond) {
-		case NO_BOND:
-			ring = new ring_ib(get_if_idx());
-			break;
-		case ACTIVE_BACKUP:
-		case LAG_8023ad:
-			ring = new ring_bond_ib(get_if_idx());
-			break;
-		default:
-			nd_logdbg("Unknown ring type");
-			break;
-		}
-	} catch (vma_error &error) {
-		nd_logdbg("failed creating ring %s", error.message);
-	}
-
-	return ring;
-}
-
-L2_address* net_device_val_ib::create_L2_address(const char* ifname)
-{
-	if (m_p_L2_addr) {
-		delete m_p_L2_addr;
-		m_p_L2_addr = NULL;
-	}
-	unsigned char hw_addr[IPOIB_HW_ADDR_LEN];
-	get_local_ll_addr(ifname, hw_addr, IPOIB_HW_ADDR_LEN, false);
-	return new IPoIB_addr(hw_addr);
-}
-
-void net_device_val_ib::create_br_address(const char* ifname)
-{
-	if (m_p_br_addr) {
-		delete m_p_br_addr;
-		m_p_br_addr = NULL;
-	}
-	unsigned char hw_addr[IPOIB_HW_ADDR_LEN];
-	get_local_ll_addr(ifname, hw_addr, IPOIB_HW_ADDR_LEN, true);
-	m_p_br_addr = new IPoIB_addr(hw_addr);
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if (m_p_br_addr == NULL) {
-		nd_logpanic("m_p_br_addr allocation error");
-	}
-	BULLSEYE_EXCLUDE_BLOCK_END
-}
-
-std::string net_device_val_ib::to_str()
-{
-	return std::string("IB: ") + net_device_val::to_str();
-}
-
-
-bool net_device_val::verify_bond_ipoib_or_eth_qp_creation()
+bool net_device_val::verify_bond_eth_qp_creation()
 {
 	char slaves[IFNAMSIZ * MAX_SLAVES] = {0};
 
@@ -1510,7 +1412,7 @@ bool net_device_val::verify_bond_ipoib_or_eth_qp_creation()
 	{
 		char* p = strchr(slave_name, '\n');
 		if (p) *p = '\0'; // Remove the tailing 'new line" char
-		if (!verify_ipoib_or_eth_qp_creation(slave_name)) {
+		if (!verify_eth_qp_creation(slave_name)) {
 			//check all slaves but print only once for bond
 			bond_ok =  false;
 		}
@@ -1533,11 +1435,8 @@ bool net_device_val::verify_bond_ipoib_or_eth_qp_creation()
 		 */
 		sys_image_guid_map_t::iterator guid_iter;
 		for (guid_iter = m_sys_image_guid_map.begin(); guid_iter != m_sys_image_guid_map.end(); guid_iter++) {
-			char bond_roce_lag_path[256] = {0};
-			if (guid_iter->second.size() > 1 &&
-					check_bond_roce_lag_exist(bond_roce_lag_path, sizeof(bond_roce_lag_path), guid_iter->second.front().c_str()) &&
-					check_bond_roce_lag_exist(bond_roce_lag_path, sizeof(bond_roce_lag_path), guid_iter->second.back().c_str())) {
-				print_roce_lag_warnings(get_ifname_link(), bond_roce_lag_path, guid_iter->second.front().c_str(), guid_iter->second.back().c_str());
+			if (guid_iter->second.size() > 1) {
+				print_roce_lag_warnings(get_ifname_link(), guid_iter->second.front().c_str(), guid_iter->second.back().c_str());
 			}
 		}
 #endif /* DEFINED_ROCE_LAG */
@@ -1546,76 +1445,15 @@ bool net_device_val::verify_bond_ipoib_or_eth_qp_creation()
 }
 
 //interface name can be slave while ifa struct can describe bond
-bool net_device_val::verify_ipoib_or_eth_qp_creation(const char* interface_name)
+bool net_device_val::verify_eth_qp_creation(const char* interface_name)
 {
-	if (m_type == ARPHRD_INFINIBAND) {
-		if (verify_enable_ipoib(interface_name) && verify_qp_creation(interface_name, IBV_QPT_UD)) {
-			return true;
-		}
-	} else {
-		if (verify_qp_creation(interface_name, IBV_QPT_RAW_PACKET)) {
-			return true;
-		}
-	}
-	return false;
-}
-
-bool net_device_val::verify_enable_ipoib(const char* interface_name)
-{
-	char filename[256] = "\0";
-	char ifname[IFNAMSIZ] = "\0";
-	NOT_IN_USE(interface_name); // Suppress --enable-opt-log=high warning
-
-	if(!safe_mce_sys().enable_ipoib) {
-		nd_logdbg("Blocking offload: IPoIB interfaces ('%s')", interface_name);
-		return false;
-	}
-
-#ifndef DEFINED_IBV_QP_INIT_SOURCE_QPN
-	// Note: mlx4 does not support this capability
-	ib_ctx_handler* ib_ctx = g_p_ib_ctx_handler_collection->get_ib_ctx(get_ifname_link());
-	if (!ib_ctx->is_mlx4()) {
-		nd_logwarn("Blocking offload: SOURCE_QPN is not supported for this driver ('%s')", interface_name);
-		return false;
-	}
-#endif
-
-	// Verify IPoIB is in 'datagram mode' for proper VMA with flow steering operation
-	if (validate_ipoib_prop(get_ifname(), m_flags, IPOIB_MODE_PARAM_FILE, "datagram", 8, filename, ifname)) {
-		vlog_printf(VLOG_WARNING,"*******************************************************************************************************\n");
-		vlog_printf(VLOG_WARNING,"* IPoIB mode of interface '%s' is \"connected\" !\n", get_ifname());
-		vlog_printf(VLOG_WARNING,"* Please change it to datagram: \"echo datagram > %s\" before loading your application with VMA library\n", filename);
-		vlog_printf(VLOG_WARNING,"* VMA doesn't support IPoIB in connected mode.\n");
-		vlog_printf(VLOG_WARNING,"* Please refer to VMA Release Notes for more information\n");
-		vlog_printf(VLOG_WARNING,"*******************************************************************************************************\n");
-		return false;
-	}
-	else {
-		nd_logdbg("verified interface '%s' is running in datagram mode", get_ifname());
-	}
-
-	// Verify umcast is disabled for IB flow
-	if (validate_ipoib_prop(get_ifname(), m_flags, UMCAST_PARAM_FILE, "0", 1, filename, ifname)) { // Extract UMCAST flag (only for IB transport types)
-		vlog_printf(VLOG_WARNING,"*******************************************************************************************************\n");
-		vlog_printf(VLOG_WARNING,"* UMCAST flag is Enabled for interface %s !\n", get_ifname());
-		vlog_printf(VLOG_WARNING,"* Please disable it: \"echo 0 > %s\" before loading your application with VMA library\n", filename);
-		vlog_printf(VLOG_WARNING,"* This option in no longer needed in this version\n");
-		vlog_printf(VLOG_WARNING,"* Please refer to Release Notes for more information\n");
-		vlog_printf(VLOG_WARNING,"*******************************************************************************************************\n");
-		return false;
-	}
-	else {
-		nd_logdbg("verified interface '%s' is running with umcast disabled", get_ifname());
-	}
-
-	return true;
+	return verify_qp_creation(interface_name, IBV_QPT_RAW_PACKET);
 }
 
 //ifname should point to a physical device
 bool net_device_val::verify_qp_creation(const char* ifname, enum ibv_qp_type qp_type)
 {
 	bool success = false;
-	char bond_roce_lag_path[256] = {0};
 	struct ibv_cq* cq = NULL;
 	struct ibv_comp_channel *channel = NULL;
 	struct ibv_qp* qp = NULL;
@@ -1623,9 +1461,6 @@ bool net_device_val::verify_qp_creation(const char* ifname, enum ibv_qp_type qp_
 	vma_ibv_qp_init_attr qp_init_attr;
 	memset(&qp_init_attr, 0, sizeof(qp_init_attr));
 
-	vma_ibv_cq_init_attr attr;
-	memset(&attr, 0, sizeof(attr));
-
 	qp_init_attr.cap.max_send_wr = MCE_DEFAULT_TX_NUM_WRE;
 	qp_init_attr.cap.max_recv_wr = MCE_DEFAULT_RX_NUM_WRE;
 	qp_init_attr.cap.max_inline_data = MCE_DEFAULT_TX_MAX_INLINE;
@@ -1635,7 +1470,7 @@ bool net_device_val::verify_qp_creation(const char* ifname, enum ibv_qp_type qp_
 	qp_init_attr.qp_type = qp_type;
 
 	//find ib_cxt
-	char base_ifname[IFNAMSIZ];
+	char base_ifname[IFNAMSIZ] = {0};
 	get_base_interface_name((const char*)(ifname), base_ifname, sizeof(base_ifname));
 	int port_num = get_port_from_ifname(base_ifname);
 	ib_ctx_handler* p_ib_ctx = g_p_ib_ctx_handler_collection->get_ib_ctx(base_ifname);
@@ -1643,12 +1478,7 @@ bool net_device_val::verify_qp_creation(const char* ifname, enum ibv_qp_type qp_
 	if (!p_ib_ctx) {
 		nd_logdbg("Cant find ib_ctx for interface %s", base_ifname);
 		if (qp_type == IBV_QPT_RAW_PACKET && m_bond != NO_BOND) {
-			if (check_bond_roce_lag_exist(bond_roce_lag_path, sizeof(bond_roce_lag_path), ifname)) {
-				print_roce_lag_warnings(get_ifname_link(), bond_roce_lag_path);
-			} else if ((p_ib_ctx = g_p_ib_ctx_handler_collection->get_ib_ctx(get_ifname_link()))
-					&& strstr(p_ib_ctx->get_ibname(), "bond")) {
-				print_roce_lag_warnings(get_ifname_link());
-			}
+			print_roce_lag_warnings(get_ifname_link());
 		}
 		goto release_resources;
 	} else if (port_num > p_ib_ctx->get_ibv_device_attr()->phys_port_cnt) {
@@ -1671,7 +1501,7 @@ bool net_device_val::verify_qp_creation(const char* ifname, enum ibv_qp_type qp_
 		goto release_resources;
 	}
 	VALGRIND_MAKE_MEM_DEFINED(channel, sizeof(ibv_comp_channel));
-	cq = vma_ibv_create_cq(p_ib_ctx->get_ibv_context(), safe_mce_sys().tx_num_wr, (void*)this, channel, 0, &attr);
+	cq = ibv_create_cq(p_ib_ctx->get_ibv_context(), safe_mce_sys().tx_num_wr, (void*)this, channel, 0);
 	if (!cq) {
 		nd_logdbg("cq creation failed for interface %s (errno=%d %s)", ifname, errno, strerror(errno));
 		goto release_resources;
@@ -1681,14 +1511,6 @@ bool net_device_val::verify_qp_creation(const char* ifname, enum ibv_qp_type qp_
 	qp_init_attr.recv_cq = cq;
 	qp_init_attr.send_cq = cq;
 
-	// Set source qpn for non mlx4 IPoIB devices
-	if (qp_type == IBV_QPT_UD && !p_ib_ctx->is_mlx4()) {
-		unsigned char hw_addr[IPOIB_HW_ADDR_LEN];
-		get_local_ll_addr(ifname, hw_addr, IPOIB_HW_ADDR_LEN, false);
-		IPoIB_addr ipoib_addr(hw_addr);
-		ibv_source_qpn_set(qp_init_attr, ipoib_addr.get_qpn());
-	}
-
 	qp = vma_ibv_create_qp(p_ib_ctx->get_ibv_pd(), &qp_init_attr);
 	if (qp) {
 		if (qp_type == IBV_QPT_UD && priv_ibv_create_flow_supported(qp, port_num) == -1) {
@@ -1711,19 +1533,6 @@ bool net_device_val::verify_qp_creation(const char* ifname, enum ibv_qp_type qp_
 		nd_logdbg("QP creation failed on interface %s (errno=%d %s), Traffic will not be offloaded", ifname, errno, strerror(errno));
 qp_failure:
 		int err = errno; //verify_raw_qp_privliges can overwrite errno so keep it before the call
-#if defined(DEFINED_VERBS_VERSION) && (DEFINED_VERBS_VERSION == 2)
-		if (validate_raw_qp_privliges() == 0) {
-			// MLNX_OFED raw_qp_privliges file exist with bad value
-			vlog_printf(VLOG_WARNING,"*******************************************************************************************************\n");
-			vlog_printf(VLOG_WARNING,"* Interface %s will not be offloaded.\n", ifname);
-			vlog_printf(VLOG_WARNING,"* Working in this mode might causes VMA malfunction over Ethernet/InfiniBand interfaces\n");
-			vlog_printf(VLOG_WARNING,"* WARNING: the following steps will restart your network interface!\n");
-			vlog_printf(VLOG_WARNING,"* 1. \"echo options ib_uverbs disable_raw_qp_enforcement=1 > /etc/modprobe.d/ib_uverbs.conf\"\n");
-			vlog_printf(VLOG_WARNING,"* 2. Restart openibd or rdma service depending on your system configuration\n");
-			vlog_printf(VLOG_WARNING,"* Read the RAW_PACKET QP root access enforcement section in the VMA's User Manual for more information\n");
-			vlog_printf(VLOG_WARNING,"******************************************************************************************************\n");
-		} else
-#endif /* DEFINED_VERBS_VERSION */
 		if (validate_user_has_cap_net_raw_privliges() == 0 || err == EPERM) {
 			vlog_printf(VLOG_WARNING,"*******************************************************************************************************\n");
 			vlog_printf(VLOG_WARNING,"* Interface %s will not be offloaded.\n", ifname);
