@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -32,8 +32,6 @@
 
 #define FICTIVE_REMOTE_QPN	0x48
 #define FICTIVE_REMOTE_QKEY	0x01234567
-#define FICTIVE_AH_SL		5
-#define FICTIVE_AH_DLID		0x3
 
 #define MAX_UPSTREAM_CQ_MSHV_SIZE 8192
 
@@ -60,16 +58,7 @@ qp_mgr::qp_mgr(struct qp_mgr_desc *desc, const uint32_t tx_num_wr):
 	,m_n_ip_id_base(0)
 	,m_n_ip_id_offset(0)
 {
-#ifdef DEFINED_TSO
-	memset(&m_qp_cap, 0, sizeof(m_qp_cap));
-	m_qp_cap.max_inline_data = safe_mce_sys().tx_max_inline;
-	m_qp_cap.max_send_sge = (m_p_ring->is_tso() ?
-		m_p_ib_ctx_handler->get_ibv_device_attr()->max_sge : MCE_DEFAULT_TX_NUM_SGE);
-        m_qp_cap.max_recv_sge = (m_p_ring->is_socketxtreme()) ? 1 : MCE_DEFAULT_RX_NUM_SGE;
-#else
 	m_max_inline_data = 0;
-#endif /* DEFINED_TSO */
-
 	m_ibv_rx_sg_array = new ibv_sge[m_n_sysvar_rx_num_wr_to_post_recv];
 	m_ibv_rx_wr_array = new ibv_recv_wr[m_n_sysvar_rx_num_wr_to_post_recv];
 
@@ -148,8 +137,7 @@ cq_mgr* qp_mgr::init_tx_cq_mgr()
 
 int qp_mgr::configure(struct qp_mgr_desc *desc)
 {
-	qp_logdbg("Creating QP of transport type '%s' on ibv device '%s' [%p] on port %d",
-			priv_vma_transport_type_str(m_p_ring->get_transport_type()),
+	qp_logdbg("Creating QP on ibv device '%s' [%p] on port %d",
 			m_p_ib_ctx_handler->get_ibname(), m_p_ib_ctx_handler->get_ibv_device(), m_port_num);
 
 	// Check device capabilities for max QP work requests
@@ -188,78 +176,6 @@ int qp_mgr::configure(struct qp_mgr_desc *desc)
 	vma_ibv_qp_init_attr qp_init_attr;
 	memset(&qp_init_attr, 0, sizeof(qp_init_attr));
 
-#ifdef DEFINED_TSO
-	// TODO: m_tx_num_wr and m_rx_num_wr should be part of m_qp_cap
-	// and assigned as a result of ibv_query_qp()
-	m_qp_cap.max_send_wr = m_tx_num_wr;
-	m_qp_cap.max_recv_wr = m_rx_num_wr;
-
-	memcpy(&qp_init_attr.cap, &m_qp_cap, sizeof(qp_init_attr.cap));
-	qp_init_attr.recv_cq = m_p_cq_mgr_rx->get_ibv_cq_hndl();
-	qp_init_attr.send_cq = m_p_cq_mgr_tx->get_ibv_cq_hndl();
-	qp_init_attr.sq_sig_all = 0;
-
-	// In case of enabled TSO we need to take into account amount of SGE together with header inline
-	// Per PRM maximum of CTRL + ETH + ETH_HEADER_INLINE+DATA_PTR*NUM_SGE+MAX_INLINE+INLINE_SIZE
-	// MLX5 return 32678 WQEBBs at max so minimal number 
-	int max_wqe_sz = 16+14+m_p_ring->m_tso.max_header_sz+16*qp_init_attr.cap.max_send_sge+qp_init_attr.cap.max_inline_data+4;
-	int num_wr = 32678*64/max_wqe_sz;
-	qp_logdbg("calculated max_wqe_sz=%d num_wr=%d", max_wqe_sz, num_wr);
-	if (num_wr < (signed)m_tx_num_wr) {
-		qp_init_attr.cap.max_send_wr = num_wr; // force min for create_qp or you will have error of memory allocation
-	}
-
-	qp_logdbg("Requested QP parameters: "
-			"wre: tx = %d rx = %d "
-			"sge: tx = %d rx = %d "
-			"inline: %d",
-			qp_init_attr.cap.max_send_wr, qp_init_attr.cap.max_recv_wr,
-			qp_init_attr.cap.max_send_sge, qp_init_attr.cap.max_recv_sge,
-			qp_init_attr.cap.max_inline_data);
-
-	// Create the QP
-	if (prepare_ibv_qp(qp_init_attr)) {
-		return -1;
-	}
-
-	qp_logdbg("Configured QP parameters: "
-			"wre: tx = %d rx = %d "
-			"sge: tx = %d rx = %d "
-			"inline: %d",
-			qp_init_attr.cap.max_send_wr, qp_init_attr.cap.max_recv_wr,
-			qp_init_attr.cap.max_send_sge, qp_init_attr.cap.max_recv_sge,
-			qp_init_attr.cap.max_inline_data);
-
-	/* Check initial parameters with actual */
-	enum ibv_qp_attr_mask attr_mask = IBV_QP_CAP;
-	struct ibv_qp_attr tmp_ibv_qp_attr;
-	struct ibv_qp_init_attr tmp_ibv_qp_init_attr;
-	IF_VERBS_FAILURE(ibv_query_qp(m_qp, &tmp_ibv_qp_attr, attr_mask,
-			 &tmp_ibv_qp_init_attr)) {
-			qp_logerr("ibv_query_qp failed (errno=%d %m)", errno);
-			return -1;
-	} ENDIF_VERBS_FAILURE;
-	m_qp_cap.max_send_wr = min(tmp_ibv_qp_attr.cap.max_send_wr, m_qp_cap.max_send_wr);
-	m_qp_cap.max_recv_wr = min(tmp_ibv_qp_attr.cap.max_recv_wr, m_qp_cap.max_recv_wr);
-	m_qp_cap.max_send_sge = min(tmp_ibv_qp_attr.cap.max_send_sge, m_qp_cap.max_send_sge);
-	m_qp_cap.max_recv_sge = min(tmp_ibv_qp_attr.cap.max_recv_sge, m_qp_cap.max_recv_sge);
-	m_qp_cap.max_inline_data = min(tmp_ibv_qp_attr.cap.max_inline_data, m_qp_cap.max_inline_data);
-
-	if ( m_qp_cap.max_send_wr < m_tx_num_wr ) {
-		qp_logwarn("Amount of requested TX_WRE %d lowered to %d", m_tx_num_wr, m_qp_cap.max_send_wr);
-		m_tx_num_wr = m_qp_cap.max_send_wr;
-		m_p_ring->set_tx_num_wr( m_tx_num_wr );
-	}
-
-	qp_logdbg("Used QP (num=%d) "
-			"wre: tx = %d rx = %d "
-			"sge: tx = %d rx = %d "
-			"inline: %d",
-			m_qp->qp_num,
-			m_qp_cap.max_send_wr, m_qp_cap.max_recv_wr,
-			m_qp_cap.max_send_sge, m_qp_cap.max_recv_sge,
-			m_qp_cap.max_inline_data);
-#else
 	// Check device capabilities for max SG elements
 	uint32_t tx_max_inline = safe_mce_sys().tx_max_inline;
 	uint32_t rx_num_sge = (m_p_ring->is_socketxtreme() ? 1 : MCE_DEFAULT_RX_NUM_SGE);
@@ -282,7 +198,6 @@ int qp_mgr::configure(struct qp_mgr_desc *desc)
 	qp_logdbg("Created QP (num=%d) with %d tx wre and inline=%d and %d rx "
 		"wre and %d sge", m_qp->qp_num, m_tx_num_wr, m_max_inline_data,
 		m_rx_num_wr, rx_num_sge);
-#endif /* DEFINED_TSO */
 
 #if defined(DEFINED_ROCE_LAG)
 	if (desc->slave && desc->slave->lag_tx_port_affinity > 0) {
@@ -422,7 +337,7 @@ void qp_mgr::release_tx_buffers()
 
 void qp_mgr::trigger_completion_for_all_sent_packets()
 {
-	vma_ibv_send_wr send_wr;
+	ibv_send_wr send_wr;
 	ibv_sge sge[1];
 
 	// Handle releasing of Tx buffers
@@ -445,7 +360,6 @@ void qp_mgr::trigger_completion_for_all_sent_packets()
 
 		// Prepare dummy packet: zeroed payload ('0000').
 		// For ETH it replaces the MAC header!! (Nothing is going on the wire, QP in error state)
-		// For IB it replaces the IPoIB header.
 
 		/* need to send at least eth+ip, since libmlx5 will drop just eth header */
 		ethhdr* p_buffer_ethhdr = (ethhdr *)p_mem_buf_desc->p_buffer;
@@ -457,38 +371,13 @@ void qp_mgr::trigger_completion_for_all_sent_packets()
 		sge[0].addr = (uintptr_t)(p_mem_buf_desc->p_buffer);
 		sge[0].lkey = m_p_ring->m_tx_lkey;
 
-		struct ibv_ah *p_ah = NULL;
-		ibv_ah_attr ah_attr;
-
-		if (m_p_ring->get_transport_type() == VMA_TRANSPORT_IB) {
-			memset(&ah_attr, 0, sizeof(ah_attr));
-			ah_attr.dlid	=	FICTIVE_AH_DLID;
-			ah_attr.sl	=	FICTIVE_AH_SL;
-			ah_attr.src_path_bits	= 0;
-			ah_attr.static_rate	= 0;
-			ah_attr.is_global	= 0;
-			ah_attr.port_num	= m_port_num; // Do we need it?
-
-			p_ah = ibv_create_ah(m_p_ib_ctx_handler->get_ibv_pd(), &ah_attr);
-			BULLSEYE_EXCLUDE_BLOCK_START
-			if (!p_ah && (errno != EIO)) {
-				qp_logpanic("failed creating address handler (errno=%d %m)", errno);
-			}
-			BULLSEYE_EXCLUDE_BLOCK_END
-		}
-
-		// Prepare send wr for (does not care if it is UD/IB or RAW/ETH)
-		// UD requires AH+qkey, RAW requires minimal payload instead of MAC header.
-
 		memset(&send_wr, 0, sizeof(send_wr));
 		send_wr.wr_id = (uintptr_t)p_mem_buf_desc;
-		send_wr.wr.ud.ah = p_ah;
-		send_wr.wr.ud.remote_qpn = FICTIVE_REMOTE_QPN;
-		send_wr.wr.ud.remote_qkey = FICTIVE_REMOTE_QKEY;
+		send_wr.wr.ud.ah = nullptr;
 		send_wr.sg_list = sge;
 		send_wr.num_sge = 1;
 		send_wr.next = NULL;
-		vma_send_wr_opcode(send_wr) = VMA_IBV_WR_SEND;
+		vma_send_wr_opcode(send_wr) = IBV_WR_SEND;
 		qp_logdbg("IBV_SEND_SIGNALED");
 
 		// Close the Tx unsignaled send list
@@ -502,12 +391,6 @@ void qp_mgr::trigger_completion_for_all_sent_packets()
 		m_p_ring->m_tx_num_wr_free--;
 
 		send_to_wire(&send_wr, (vma_wr_tx_packet_attr)(VMA_TX_PACKET_L3_CSUM|VMA_TX_PACKET_L4_CSUM), true);
-		if (p_ah) {
-			IF_VERBS_FAILURE_EX(ibv_destroy_ah(p_ah), EIO)
-			{
-				qp_logpanic("failed destroying address handle (errno=%d %m)", errno);
-			}ENDIF_VERBS_FAILURE;
-		}
 	}
 }
 
@@ -567,18 +450,18 @@ void qp_mgr::post_recv_buffers(descq_t* p_buffers, size_t count)
 	}
 }
 
-inline int qp_mgr::send_to_wire(vma_ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr, bool request_comp)
+inline int qp_mgr::send_to_wire(ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr, bool request_comp)
 {
 	NOT_IN_USE(attr);
 	int ret = 0;
-	vma_ibv_send_wr *bad_wr = NULL;
+	ibv_send_wr *bad_wr = NULL;
 
 	if (request_comp) {
-		vma_send_wr_send_flags(*p_send_wqe) = (vma_ibv_send_flags)(vma_send_wr_send_flags(*p_send_wqe) | VMA_IBV_SEND_SIGNALED);
+		vma_send_wr_send_flags(*p_send_wqe) = (ibv_send_flags)(vma_send_wr_send_flags(*p_send_wqe) | IBV_SEND_SIGNALED);
 	}
 
-	IF_VERBS_FAILURE(vma_ibv_post_send(m_qp, p_send_wqe, &bad_wr)) {
-		qp_logerr("failed post_send%s (errno=%d %m)\n", ((vma_send_wr_send_flags(*p_send_wqe) & VMA_IBV_SEND_INLINE)?"(+inline)":""), errno);
+	IF_VERBS_FAILURE(ibv_post_send(m_qp, p_send_wqe, &bad_wr)) {
+		qp_logerr("failed post_send%s (errno=%d %m)\n", ((vma_send_wr_send_flags(*p_send_wqe) & IBV_SEND_INLINE)?"(+inline)":""), errno);
 		if (bad_wr) {
 			qp_logerr("bad_wr info: wr_id=%#lx, send_flags=%#lx, addr=%#lx, length=%d, lkey=%#x, max_inline_data=%d",
 			bad_wr->wr_id, (unsigned long)vma_send_wr_send_flags(*bad_wr), bad_wr->sg_list[0].addr, bad_wr->sg_list[0].length, bad_wr->sg_list[0].lkey, get_max_inline_data());
@@ -587,12 +470,12 @@ inline int qp_mgr::send_to_wire(vma_ibv_send_wr* p_send_wqe, vma_wr_tx_packet_at
 	} ENDIF_VERBS_FAILURE;
 
 	// Clear the SINGAL request
-	vma_send_wr_send_flags(*p_send_wqe) = (vma_ibv_send_flags)(vma_send_wr_send_flags(*p_send_wqe) & ~VMA_IBV_SEND_SIGNALED);
+	vma_send_wr_send_flags(*p_send_wqe) = (ibv_send_flags)(vma_send_wr_send_flags(*p_send_wqe) & ~IBV_SEND_SIGNALED);
 
 	return ret;
 }
 
-int qp_mgr::send(vma_ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr)
+int qp_mgr::send(ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr)
 {
 	mem_buf_desc_t* p_mem_buf_desc = (mem_buf_desc_t *)p_send_wqe->wr_id;
 
@@ -680,13 +563,6 @@ int qp_mgr_eth::prepare_ibv_qp(vma_ibv_qp_init_attr& qp_init_attr)
 	qp_init_attr.qp_type = IBV_QPT_RAW_PACKET;
 	vma_ibv_qp_init_attr_comp_mask(m_p_ib_ctx_handler->get_ibv_pd(), qp_init_attr);
 
-#ifdef DEFINED_TSO
-	if (m_p_ring->is_tso()) {
-		vma_ibv_qp_init_attr_tso(qp_init_attr, m_p_ring->get_max_header_sz());
-		qp_logdbg("create qp with max_tso_header = %d", m_p_ring->get_max_header_sz());
-	}
-#endif /* DEFINED_TSO */
-
 	m_qp = vma_ibv_create_qp(m_p_ib_ctx_handler->get_ibv_pd(), &qp_init_attr);
 
 	BULLSEYE_EXCLUDE_BLOCK_START
@@ -701,8 +577,6 @@ int qp_mgr_eth::prepare_ibv_qp(vma_ibv_qp_init_attr& qp_init_attr)
 	}
 	BULLSEYE_EXCLUDE_BLOCK_END
 
-#ifdef DEFINED_TSO
-#else
 	enum ibv_qp_attr_mask attr_mask = IBV_QP_CAP;
 	struct ibv_qp_attr tmp_ibv_qp_attr;
 	struct ibv_qp_init_attr tmp_ibv_qp_init_attr;
@@ -711,8 +585,10 @@ int qp_mgr_eth::prepare_ibv_qp(vma_ibv_qp_init_attr& qp_init_attr)
 			qp_logerr("ibv_query_qp failed (errno=%d %m)", errno);
 			return -1;
 	} ENDIF_VERBS_FAILURE;
+
 	uint32_t tx_max_inline = safe_mce_sys().tx_max_inline;
 	m_max_inline_data = min(tmp_ibv_qp_attr.cap.max_inline_data, tx_max_inline);
+
 	qp_logdbg("requested max inline = %d QP, actual max inline = %d, "
 		"VMA max inline set to %d, max_send_wr=%d, max_recv_wr=%d, "
 		"max_recv_sge=%d, max_send_sge=%d",
@@ -720,120 +596,10 @@ int qp_mgr_eth::prepare_ibv_qp(vma_ibv_qp_init_attr& qp_init_attr)
 		m_max_inline_data, tmp_ibv_qp_attr.cap.max_send_wr,
 		tmp_ibv_qp_attr.cap.max_recv_wr, tmp_ibv_qp_attr.cap.max_recv_sge,
 		tmp_ibv_qp_attr.cap.max_send_sge);
-#endif /* DEFINED_TSO */
-	return 0;
-}
-
-void qp_mgr_ib::modify_qp_to_ready_state()
-{
-	qp_logdbg("");
-	int ret = 0;
-	int qp_state = priv_ibv_query_qp_state(m_qp);
-
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if (qp_state !=  IBV_QPS_INIT) {
-		if ((ret = priv_ibv_modify_qp_from_err_to_init_ud(m_qp, m_port_num, m_pkey_index, m_underly_qpn)) != 0) {
-			qp_logpanic("failed to modify QP from %d to RTS state (ret = %d)", qp_state, ret);
-		}
-	}
-	if ((ret = priv_ibv_modify_qp_from_init_to_rts(m_qp, m_underly_qpn)) != 0) {
-		qp_logpanic("failed to modify QP from INIT to RTS state (ret = %d)", ret);
-	}
-	BULLSEYE_EXCLUDE_BLOCK_END
-}
-
-int qp_mgr_ib::prepare_ibv_qp(vma_ibv_qp_init_attr& qp_init_attr)
-{
-	qp_logdbg("");
-	int ret = 0;
-
-	qp_init_attr.qp_type = IBV_QPT_UD;
-	vma_ibv_qp_init_attr_comp_mask(m_p_ib_ctx_handler->get_ibv_pd(), qp_init_attr);
-
-#ifdef DEFINED_TSO
-	if (m_p_ring->is_tso()) {
-		vma_ibv_qp_init_attr_tso(qp_init_attr, m_p_ring->get_max_header_sz());
-		qp_logdbg("create qp with max_tso_header = %d", m_p_ring->get_max_header_sz());
-	}
-#endif /* DEFINED_TSO */
-
-	if (m_underly_qpn) {
-		ibv_source_qpn_set(qp_init_attr, m_underly_qpn);
-		qp_logdbg("create qp using underly qpn = 0x%X", m_underly_qpn);
-	}
-
-	m_qp = vma_ibv_create_qp(m_p_ib_ctx_handler->get_ibv_pd(), &qp_init_attr);
-
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if (!m_qp) {
-		qp_logerr("ibv_create_qp failed (errno=%d %m)", errno);
-		return -1;
-	}
-
-	if ((ret = priv_ibv_modify_qp_from_err_to_init_ud(m_qp, m_port_num,
-							  m_pkey_index,
-							  m_underly_qpn)) != 0) {
-		VLOG_PRINTF_INFO_ONCE_THEN_ALWAYS(
-				VLOG_ERROR, VLOG_DEBUG,
-				"failed to modify QP from ERR to INIT state (ret = %d) check number of available fds (ulimit -n)",
-				ret);
-		return ret;
-	}
-	BULLSEYE_EXCLUDE_BLOCK_END
-
-#ifdef DEFINED_TSO
-#else
-	enum ibv_qp_attr_mask attr_mask = IBV_QP_CAP;
-	struct ibv_qp_attr tmp_ibv_qp_attr;
-	struct ibv_qp_init_attr tmp_ibv_qp_init_attr;
-	IF_VERBS_FAILURE(ibv_query_qp(m_qp, &tmp_ibv_qp_attr, attr_mask,
-			 &tmp_ibv_qp_init_attr)) {
-			qp_logerr("ibv_query_qp failed (errno=%d %m)", errno);
-			return -1;
-	} ENDIF_VERBS_FAILURE;
-	uint32_t tx_max_inline = safe_mce_sys().tx_max_inline;
-	m_max_inline_data = min(tmp_ibv_qp_attr.cap.max_inline_data, tx_max_inline);
-	qp_logdbg("requested max inline = %d QP, actual max inline = %d, "
-		"VMA max inline set to %d, max_send_wr=%d, max_recv_wr=%d, "
-		"max_recv_sge=%d, max_send_sge=%d",
-		tx_max_inline, tmp_ibv_qp_init_attr.cap.max_inline_data,
-		m_max_inline_data, tmp_ibv_qp_attr.cap.max_send_wr,
-		tmp_ibv_qp_attr.cap.max_recv_wr, tmp_ibv_qp_attr.cap.max_recv_sge,
-		tmp_ibv_qp_attr.cap.max_send_sge);
-#endif /* DEFINED_TSO */
 
 	return 0;
 }
 
-void qp_mgr_ib::update_pkey_index()
-{
-	qp_logdbg("");
-	VALGRIND_MAKE_MEM_DEFINED(&m_pkey, sizeof(m_pkey));
-	if (priv_ibv_find_pkey_index(m_p_ib_ctx_handler->get_ibv_context(), get_port_num(), m_pkey, &m_pkey_index)) {
-		qp_logdbg("IB: Can't find correct pkey_index for pkey '%d'", m_pkey);
-		m_pkey_index = (uint16_t)-1;
-	}
-	else {
-		qp_logdbg("IB: Found correct pkey_index (%d) for pkey '%d'", m_pkey_index, m_pkey);
-	}
-#ifdef DEFINED_IBV_QP_INIT_SOURCE_QPN
-	/* m_underly_qpn is introduced to detect if current qp_mgr is able to
-	 * use associated qp.
-	 * It is set to non zero value if OFED supports such possibility only but final
-	 * decision can be made just after attempt to create qp. The value of
-	 * m_underly_qpn is reverted to zero if function to qp creation returns
-	 * failure.
-	 * So zero value for this field means no such capability.
-	 * Note: mlx4 does not support this capability. Disable it explicitly because dynamic check
-	 * using ibv_create_qp does not help
-	 */
-	if (!m_p_ib_ctx_handler->is_mlx4()) {
-		m_underly_qpn = m_p_ring->get_qpn();
-	}
-	qp_logdbg("IB: Use qpn = 0x%X for device: %s", m_underly_qpn, m_p_ib_ctx_handler->get_ibname());
-#endif /* DEFINED_IBV_QP_INIT_SOURCE_QPN */
-}
-
 uint32_t qp_mgr::is_ratelimit_change(struct vma_rate_limit_t &rate_limit)
 {
 	uint32_t rl_changes = 0;
