@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -73,21 +73,10 @@ public:
 
 	virtual void        post_recv_buffer(mem_buf_desc_t* p_mem_buf_desc); // Post for receive single mem_buf_desc
 	void                post_recv_buffers(descq_t* p_buffers, size_t count); // Post for receive a list of mem_buf_desc
-	int                 send(vma_ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr);
-
-#ifdef DEFINED_TSO
-	inline uint32_t     get_max_inline_data() const {
-		return m_qp_cap.max_inline_data;
-	}
-	inline uint32_t     get_max_send_sge() const {
-		return m_qp_cap.max_send_sge;
-	}
-#else
+	int                 send(ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr);
 	uint32_t            get_max_inline_data() const {return m_max_inline_data; }
-#endif /* DEFINED_TSO */
 	int                 get_port_num() const { return m_port_num; }
 	virtual uint16_t    get_partiton() const { return 0; };
-	virtual uint32_t    get_underly_qpn() const { return 0; };
 	struct ibv_qp*      get_ibv_qp() const { return m_qp; };
 	class cq_mgr*       get_tx_cq_mgr() const { return m_p_cq_mgr_tx; }
 	class cq_mgr*       get_rx_cq_mgr() const { return m_p_cq_mgr_rx; }
@@ -107,29 +96,20 @@ public:
 	static inline bool  is_lib_mlx5(const char* device_name) {return strstr(device_name, "mlx5");}
 	virtual void        dm_release_data(mem_buf_desc_t* buff) { NOT_IN_USE(buff); }
 	virtual bool        fill_hw_descriptors(vma_mlx_hw_device_data &data) {NOT_IN_USE(data);return false;};
+
 protected:
 	struct ibv_qp*      m_qp;
 	uint64_t*           m_rq_wqe_idx_to_wrid;
-
 	ring_simple*        m_p_ring;
 	uint8_t             m_port_num;
 	ib_ctx_handler*     m_p_ib_ctx_handler;
-
-#ifdef DEFINED_TSO
-	struct ibv_qp_cap   m_qp_cap;
-#else
 	uint32_t            m_max_inline_data;
-#endif /* DEFINED_TSO */
 	uint32_t            m_max_qp_wr;
-
 	cq_mgr*             m_p_cq_mgr_rx;
 	cq_mgr*             m_p_cq_mgr_tx;
-
 	uint32_t            m_rx_num_wr;
 	uint32_t            m_tx_num_wr;
-
 	bool                m_hw_dummy_send_support;
-
 	uint32_t            m_n_sysvar_rx_num_wr_to_post_recv;
 	const uint32_t      m_n_sysvar_tx_num_wr_to_signal;
 	const uint32_t      m_n_sysvar_rx_prefetch_bytes_before_poll;
@@ -143,7 +123,6 @@ protected:
 	// send wr
 	uint32_t            m_n_unsignaled_count;
 	mem_buf_desc_t*     m_p_last_tx_mem_buf_desc; // Remembered so we can list several mem_buf_desc_t on a single notification request
-
 	mem_buf_desc_t*     m_p_prev_rx_desc_pushed;
 
 	// generating packet IDs
@@ -160,7 +139,7 @@ protected:
 
 	cq_mgr* handle_cq_initialization(uint32_t *num_wr, struct ibv_comp_channel* comp_event_channel, bool is_rx);
 
-	virtual int     send_to_wire(vma_ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr, bool request_comp);
+	virtual int     send_to_wire(ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr, bool request_comp);
 	virtual bool    is_completion_need() { return !m_n_unsignaled_count; };
 	virtual bool    is_rq_empty() const { return false; }
 };
@@ -187,28 +166,4 @@ private:
 	const uint16_t 		m_vlan;
 };
 
-class qp_mgr_ib : public qp_mgr
-{
-public:
-	qp_mgr_ib(struct qp_mgr_desc *desc,
-			const uint32_t tx_num_wr, const uint16_t pkey):
-	qp_mgr(desc, tx_num_wr), m_pkey(pkey), m_underly_qpn(0) {
-		update_pkey_index();
-		if(configure(desc)) throw_vma_exception("failed creating qp"); };
-
-	virtual void 		modify_qp_to_ready_state();
-	virtual uint16_t	get_partiton() const { return m_pkey; };
-	virtual uint32_t	get_underly_qpn() const { return m_underly_qpn; };
-
-protected:
-	virtual int		prepare_ibv_qp(vma_ibv_qp_init_attr& qp_init_attr);
-
-private:
-	const uint16_t 		m_pkey;
-	uint16_t 		m_pkey_index;
-	uint32_t 		m_underly_qpn;
-
-	void 			update_pkey_index();
-};
-
 #endif
