@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -10,9 +10,7 @@
 #include "vma/util/valgrind.h"
 #include "vma/util/sg_array.h"
 #include "vma/sock/fd_collection.h"
-#if defined(DEFINED_DIRECT_VERBS)
 #include "vma/dev/qp_mgr_eth_mlx5.h"
-#endif
 
 #undef  MODULE_NAME
 #define MODULE_NAME "ring_simple"
@@ -36,7 +34,7 @@
 /** inlining functions can only help if they are implemented before their usage **/
 /**/
 
-inline void ring_simple::send_status_handler(int ret, vma_ibv_send_wr* p_send_wqe)
+inline void ring_simple::send_status_handler(int ret, ibv_send_wr* p_send_wqe)
 {
 	BULLSEYE_EXCLUDE_BLOCK_START
 	if (unlikely(ret)) {
@@ -61,19 +59,12 @@ inline void ring_simple::send_status_handler(int ret, vma_ibv_send_wr* p_send_wq
 
 qp_mgr* ring_eth::create_qp_mgr(struct qp_mgr_desc *desc)
 {
-#if defined(DEFINED_DIRECT_VERBS)
 	if (qp_mgr::is_lib_mlx5(((ib_ctx_handler*)desc->slave->p_ib_ctx)->get_ibname())) {
 		return new qp_mgr_eth_mlx5(desc,
 				get_tx_num_wr(), m_partition);
 	}
-#endif
-	return new qp_mgr_eth(desc,
-			get_tx_num_wr(), m_partition);
-}
 
-qp_mgr* ring_ib::create_qp_mgr(struct qp_mgr_desc *desc)
-{
-	return new qp_mgr_ib(desc,
+	return new qp_mgr_eth(desc,
 			get_tx_num_wr(), m_partition);
 }
 
@@ -90,8 +81,13 @@ ring_simple::ring_simple(int if_index, ring* parent, ring_type_t type):
 	m_p_rx_comp_event_channel(NULL), m_p_tx_comp_event_channel(NULL), m_p_l2_addr(NULL)
 {
 	net_device_val* p_ndev = g_p_net_device_table_mgr->get_net_device_val(m_parent->get_if_index());
+	if (unlikely(!p_ndev)) {
+		ring_logpanic("Cannot find net_device_val for a ring if_index = %d", get_if_index());
+	}
 	const slave_data_t * p_slave = p_ndev->get_slave(get_if_index());
-
+	if (unlikely(!p_slave)) {
+        ring_logpanic("Cannot find slave for a ring if_index = %d", get_if_index());
+    }
 	ring_logdbg("new ring_simple()");
 
 	/* m_p_ib_ctx, m_tx_lkey should be initialized to be used
@@ -113,9 +109,6 @@ ring_simple::ring_simple(int if_index, ring* parent, ring_type_t type):
 	m_mtu = p_ndev->get_mtu();
 
 	memset(&m_cq_moderation_info, 0, sizeof(m_cq_moderation_info));
-#ifdef DEFINED_TSO
-	memset(&m_tso, 0, sizeof(m_tso));
-#endif /* DEFINED_TSO */
 
 	m_socketxtreme.active = safe_mce_sys().enable_socketxtreme;
 	INIT_LIST_HEAD(&m_socketxtreme.ec_list);
@@ -209,7 +202,13 @@ ring_simple::~ring_simple()
 void ring_simple::create_resources()
 {
 	net_device_val* p_ndev = g_p_net_device_table_mgr->get_net_device_val(m_parent->get_if_index());
+	if (unlikely(!p_ndev)) {
+		ring_logpanic("Cannot find net_device_val for a ring if_index = %d", get_if_index());
+	}
 	const slave_data_t * p_slave = p_ndev->get_slave(get_if_index());
+	if (unlikely(!p_slave)) {
+        ring_logpanic("Cannot find slave for a ring if_index = %d", get_if_index());
+    }
 
 	save_l2_address(p_slave->p_L2_addr);
 	m_p_tx_comp_event_channel = ibv_create_comp_channel(m_p_ib_ctx->get_ibv_context());
@@ -237,24 +236,6 @@ void ring_simple::create_resources()
 	// Temporary fix for rd #4213201
 	m_tx_num_wr_free--;
 
-#ifdef DEFINED_TSO
-	memset(&m_tso, 0, sizeof(m_tso));
-	if (safe_mce_sys().enable_tso && (1 == validate_tso(get_if_index()))) {
-		if (vma_check_dev_attr_tso(m_p_ib_ctx->get_ibv_device_attr())) {
-			const vma_ibv_tso_caps *caps = &vma_get_tso_caps(m_p_ib_ctx->get_ibv_device_attr_ex());
-			if (ibv_is_qpt_supported(caps->supported_qpts, IBV_QPT_RAW_PACKET) ||
-				ibv_is_qpt_supported(caps->supported_qpts, IBV_QPT_UD)) {
-				m_tso.max_payload_sz = caps->max_tso;
-				/* ETH(14) + IP(20) + TCP(20) + TCP OPTIONS(40) */
-				m_tso.max_header_sz = 94;
-			}
-		}
-	}
-	ring_logdbg("ring attributes: m_tso = %d", is_tso());
-	ring_logdbg("ring attributes: m_tso:max_payload_sz = %d", get_max_payload_sz());
-	ring_logdbg("ring attributes: m_tso:max_header_sz = %d", get_max_header_sz());
-#endif /* DEFINED_TSO */
-
 	m_flow_tag_enabled = m_p_ib_ctx->get_flow_tag_capability();
 	ring_logdbg("ring attributes: m_flow_tag_enabled = %d", m_flow_tag_enabled);
 
@@ -374,6 +355,7 @@ int ring_simple::socketxtreme_poll(struct vma_completion_t *vma_completions, uns
 				mem_buf_desc_t *desc;
 				if (likely(m_p_cq_mgr_rx->poll_and_process_element_rx(&desc))) {
 					desc->rx.socketxtreme_polled = true;
+					/* coverity[check_return] */
 					rx_process_buffer(desc, NULL);
 					if (m_socketxtreme.completion->events) {
 						m_socketxtreme.completion++;
@@ -608,7 +590,7 @@ int ring_simple::mem_buf_tx_release(mem_buf_desc_t* p_mem_buf_desc_list, bool b_
 }
 
 /* note that this function is inline, so keep it above the functions using it */
-inline int ring_simple::send_buffer(vma_ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr)
+inline int ring_simple::send_buffer(ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr)
 {
 	//Note: this is debatable logic as it count of WQEs waiting completion but
 	//our SQ is cyclic buffer so in reality only last WQE is still being sent
@@ -630,7 +612,7 @@ inline int ring_simple::send_buffer(vma_ibv_send_wr* p_send_wqe, vma_wr_tx_packe
 	return ret;
 }
 
-bool ring_simple::get_hw_dummy_send_support(ring_user_id_t id, vma_ibv_send_wr* p_send_wqe)
+bool ring_simple::get_hw_dummy_send_support(ring_user_id_t id, ibv_send_wr* p_send_wqe)
 {
 	NOT_IN_USE(id);
 	NOT_IN_USE(p_send_wqe);
@@ -638,7 +620,7 @@ bool ring_simple::get_hw_dummy_send_support(ring_user_id_t id, vma_ibv_send_wr*
 	return m_p_qp_mgr->get_hw_dummy_send_support();
 }
 
-void ring_simple::send_ring_buffer(ring_user_id_t id, vma_ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr)
+void ring_simple::send_ring_buffer(ring_user_id_t id, ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr)
 {
 	NOT_IN_USE(id);
 
@@ -652,15 +634,13 @@ void ring_simple::send_ring_buffer(ring_user_id_t id, vma_ibv_send_wr* p_send_wq
 	}
 
 	auto_unlocker lock(m_lock_ring_tx);
-#ifdef DEFINED_TSO
-#else
 	p_send_wqe->sg_list[0].lkey = m_tx_lkey;
-#endif /* DEFINED_TSO */
+
 	int ret = send_buffer(p_send_wqe, attr);
 	send_status_handler(ret, p_send_wqe);
 }
 
-void ring_simple::send_lwip_buffer(ring_user_id_t id, vma_ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr)
+void ring_simple::send_lwip_buffer(ring_user_id_t id, ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr)
 {
 	NOT_IN_USE(id);
 
@@ -670,12 +650,10 @@ void ring_simple::send_lwip_buffer(ring_user_id_t id, vma_ibv_send_wr* p_send_wq
 #endif
 
 	auto_unlocker lock(m_lock_ring_tx);
-#ifdef DEFINED_TSO
-#else
 	p_send_wqe->sg_list[0].lkey = m_tx_lkey;
 	mem_buf_desc_t* p_mem_buf_desc = (mem_buf_desc_t*)(p_send_wqe->wr_id);
 	p_mem_buf_desc->lwip_pbuf.pbuf.ref++;
-#endif /* DEFINED_TSO */
+
 	int ret = send_buffer(p_send_wqe, attr);
 	send_status_handler(ret, p_send_wqe);
 }
@@ -804,6 +782,7 @@ mem_buf_desc_t* ring_simple::get_tx_buffers(uint32_t n_num_mem_bufs)
 	}
 
 	head = m_tx_pool.get_and_pop_back();
+	/* coverity[dereference:FALSE] */
 	head->lwip_pbuf.pbuf.ref = 1;
 	n_num_mem_bufs--;
 
@@ -1045,25 +1024,3 @@ uint32_t ring_simple::get_max_inline_data()
 {
 	return m_p_qp_mgr->get_max_inline_data();
 }
-
-#ifdef DEFINED_TSO
-uint32_t ring_simple::get_max_send_sge(void)
-{
-	return m_p_qp_mgr->get_max_send_sge();
-}
-
-uint32_t ring_simple::get_max_payload_sz(void)
-{
-	return m_tso.max_payload_sz;
-}
-
-uint16_t ring_simple::get_max_header_sz(void)
-{
-	return m_tso.max_header_sz;
-}
-
-bool ring_simple::is_tso(void)
-{
-	return (m_tso.max_payload_sz && m_tso.max_header_sz);
-}
-#endif /* DEFINED_TSO */
