@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -25,12 +25,12 @@ public:
 	virtual bool reclaim_recv_buffers(descq_t *rx_reuse);
 	virtual bool reclaim_recv_buffers(mem_buf_desc_t *buff);
 	virtual int reclaim_recv_single_buffer(mem_buf_desc_t* rx_reuse) { NOT_IN_USE(rx_reuse); return -1; }
-	virtual void send_ring_buffer(ring_user_id_t id, vma_ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr);
-	virtual void send_lwip_buffer(ring_user_id_t id, vma_ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr);
+	virtual void send_ring_buffer(ring_user_id_t id, ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr);
+	virtual void send_lwip_buffer(ring_user_id_t id, ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr);
 	virtual void mem_buf_desc_return_single_to_owner_tx(mem_buf_desc_t* p_mem_buf_desc);
 	virtual mem_buf_desc_t* mem_buf_tx_get(ring_user_id_t id, bool b_block, int n_num_mem_bufs = 1);
 	virtual int mem_buf_tx_release(mem_buf_desc_t* p_mem_buf_desc_list, bool b_accounting, bool trylock = false);
-	virtual bool get_hw_dummy_send_support(ring_user_id_t id, vma_ibv_send_wr* p_send_wqe) { NOT_IN_USE(id); NOT_IN_USE(p_send_wqe); return false; }
+	virtual bool get_hw_dummy_send_support(ring_user_id_t id, ibv_send_wr* p_send_wqe) { NOT_IN_USE(id); NOT_IN_USE(p_send_wqe); return false; }
 	virtual int request_notification(cq_type_t cq_type, uint64_t poll_sn) { NOT_IN_USE(cq_type); NOT_IN_USE(poll_sn); return 0; }
 	virtual void adapt_cq_moderation() {}
 
@@ -43,15 +43,7 @@ public:
 
 	virtual int modify_ratelimit(struct vma_rate_limit_t &rate_limit) { NOT_IN_USE(rate_limit); return 0; }
 	void inc_cq_moderation_stats(size_t sz_data) { NOT_IN_USE(sz_data); }
-	virtual uint32_t get_underly_qpn() { return -1; }
-        virtual uint32_t get_max_inline_data() { return 0; }
-#ifdef DEFINED_TSO
-        virtual uint32_t get_max_send_sge(void) { return 1; }
-        virtual uint32_t get_max_payload_sz(void) { return 0; }
-        virtual uint16_t get_max_header_sz(void) { return 0; }
-	virtual uint32_t get_tx_lkey(ring_user_id_t id) { NOT_IN_USE(id); return 0; }
-        virtual bool is_tso(void) { return false; }
-#endif /* DEFINED_TSO */
+    virtual uint32_t get_max_inline_data() { return 0; }
 
 	inline void set_tap_data_available() { m_tap_data_available = true; }
 	inline void set_vf_ring(ring_slave *p_ring) { m_vf_ring = p_ring; }
@@ -63,8 +55,8 @@ private:
 	int prepare_flow_message(vma_msg_flow& data, msg_flow_t flow_action);
 	int process_element_rx(void* pv_fd_ready_array);
 	bool request_more_rx_buffers();
-	int send_buffer(vma_ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr);
-	void send_status_handler(int ret, vma_ibv_send_wr* p_send_wqe);
+	int send_buffer(ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr);
+	void send_status_handler(int ret, ibv_send_wr* p_send_wqe);
 	void tap_create(net_device_val* p_ndev);
 	void tap_destroy();
 
