@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -27,7 +27,7 @@ public:
 private:
 	struct ibv_context*       m_p_ibv_context;
 
-	vma_ibv_clock_info        m_clock_values[2];
+	mlx5dv_clock_info         m_clock_values[2];
 	int                       m_clock_values_id;
 };
 
