@@ -359,10 +359,12 @@ tcp_abandon(struct tcp_pcb *pcb, int reset)
     if (pcb->unacked != NULL) {
       tcp_tx_segs_free(pcb, pcb->unacked);
       pcb->unacked = NULL;
+      pcb->last_unacked = NULL;
     }
     if (pcb->unsent != NULL) {
       tcp_tx_segs_free(pcb, pcb->unsent);
       pcb->unsent = NULL;
+      pcb->last_unsent = NULL;
     }
 #if TCP_QUEUE_OOSEQ    
     if (pcb->ooseq != NULL) {
@@ -1234,6 +1236,18 @@ tcp_syn_handled(struct tcp_pcb_listen *pcb, tcp_syn_handled_fn syn_handled)
   pcb->syn_handled_cb = syn_handled;
 }
 
+/**
+ * Used for specifying the function that should be when a accepted pcb is ready.
+ *
+ * @param pcb          Listen pcb
+ * @param accepted_pcb Callback function to call when the accepted pcb is ready.
+ */
+void
+tcp_accepted_pcb(struct tcp_pcb_listen *pcb, tcp_accepted_pcb_fn accepted_pcb)
+{
+  pcb->accepted_pcb = accepted_pcb;
+}
+
 /**
  * Used for specifying the function that should be called to clone pcb
  *
@@ -1306,7 +1320,7 @@ tcp_pcb_purge(struct tcp_pcb *pcb)
     tcp_tx_segs_free(pcb, pcb->unsent);
     tcp_tx_segs_free(pcb, pcb->unacked);
     pcb->unacked = pcb->unsent = NULL;
-    pcb->last_unsent = NULL;
+    pcb->last_unacked = pcb->last_unsent = NULL;
 #if TCP_OVERSIZE
     pcb->unsent_oversize = 0;
 #endif /* TCP_OVERSIZE */
