@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -36,8 +36,6 @@
 #include "vma/proto/vma_lwip.h"
 #include "vma/proto/route_table_mgr.h"
 #include "vma/proto/rule_table_mgr.h"
-#include "vma/proto/igmp_mgr.h"
-
 #include "vma/proto/neighbour_table_mgr.h"
 #include "vma/netlink/netlink_wrapper.h"
 #include "vma/event/command.h"
@@ -106,13 +104,6 @@ static int free_libvma_resources()
 		g_p_net_device_table_mgr->global_ring_drain_and_procces();
 	}
 
-	if(g_p_igmp_mgr) {
-		igmp_mgr* g_p_igmp_mgr_tmp = g_p_igmp_mgr;
-		g_p_igmp_mgr = NULL;
-		delete g_p_igmp_mgr_tmp;
-		usleep(50000);
-	}
-
 	if (g_p_event_handler_manager)
 		g_p_event_handler_manager->stop_thread();
 
@@ -327,6 +318,7 @@ void print_vma_global_settings()
 	vlog_printf(VLOG_INFO,"Cmd Line: %s\n", safe_mce_sys().app_name);
 
 	// Use DEBUG level logging with more details in RPM release builds
+	/* coverity[assigned_value] */
 	vlog_levels_t log_level = VLOG_DEBUG;
 #if !defined(PRJ_LIBRARY_RELEASE) || (PRJ_LIBRARY_RELEASE == 0)
 	// If non RPM (development builds) use more verbosity
@@ -406,9 +398,6 @@ void print_vma_global_settings()
 
 	VLOG_PARAM_NUMBER("Tx Mem Segs TCP", safe_mce_sys().tx_num_segs_tcp, MCE_DEFAULT_TX_NUM_SEGS_TCP, SYS_VAR_TX_NUM_SEGS_TCP);
 	VLOG_PARAM_NUMBER("Tx Mem Bufs", safe_mce_sys().tx_num_bufs, MCE_DEFAULT_TX_NUM_BUFS, SYS_VAR_TX_NUM_BUFS);
-#ifdef DEFINED_TSO
-	VLOG_PARAM_NUMBER("Tx Mem Buf size", safe_mce_sys().tx_buf_size, MCE_DEFAULT_TX_BUF_SIZE, SYS_VAR_TX_BUF_SIZE);
-#endif /* DEFINED_TSO */
 	VLOG_PARAM_NUMBER("Tx QP WRE", safe_mce_sys().tx_num_wr, MCE_DEFAULT_TX_NUM_WRE, SYS_VAR_TX_NUM_WRE);
 	VLOG_PARAM_NUMBER("Tx QP WRE Batching", safe_mce_sys().tx_num_wr_to_signal, MCE_DEFAULT_TX_NUM_WRE_TO_SIGNAL, SYS_VAR_TX_NUM_WRE_TO_SIGNAL);
 	VLOG_PARAM_NUMBER("Tx Max QP INLINE", safe_mce_sys().tx_max_inline, MCE_DEFAULT_TX_MAX_INLINE, SYS_VAR_TX_MAX_INLINE);
@@ -522,12 +511,7 @@ void print_vma_global_settings()
 	VLOG_PARAM_NUMBER("Num of UC ARPs", safe_mce_sys().neigh_uc_arp_quata, MCE_DEFAULT_NEIGH_UC_ARP_QUATA, SYS_VAR_NEIGH_UC_ARP_QUATA);
 	VLOG_PARAM_NUMBER("UC ARP delay (msec)", safe_mce_sys().neigh_wait_till_send_arp_msec, MCE_DEFAULT_NEIGH_UC_ARP_DELAY_MSEC, SYS_VAR_NEIGH_UC_ARP_DELAY_MSEC);
 	VLOG_PARAM_NUMBER("Num of neigh restart retries", safe_mce_sys().neigh_num_err_retries, MCE_DEFAULT_NEIGH_NUM_ERR_RETRIES, SYS_VAR_NEIGH_NUM_ERR_RETRIES );
-
-	VLOG_PARAM_STRING("IPOIB support", safe_mce_sys().enable_ipoib, MCE_DEFAULT_IPOIB_FLAG, SYS_VAR_IPOIB, safe_mce_sys().enable_ipoib ? "Enabled " : "Disabled");
 	VLOG_PARAM_STRING("SocketXtreme mode", safe_mce_sys().enable_socketxtreme, MCE_DEFAULT_SOCKETXTREME, SYS_VAR_SOCKETXTREME, safe_mce_sys().enable_socketxtreme ? "Enabled " : "Disabled");
-#ifdef DEFINED_TSO
-	VLOG_PARAM_STRING("TSO support", safe_mce_sys().enable_tso, MCE_DEFAULT_TSO, SYS_VAR_TSO, safe_mce_sys().enable_tso ? "Enabled " : "Disabled");
-#endif /* DEFINED_TSO */
 	VLOG_PARAM_STRING("BF (Blue Flame)", safe_mce_sys().handle_bf, MCE_DEFAULT_BF_FLAG, SYS_VAR_BF, safe_mce_sys().handle_bf ? "Enabled " : "Disabled");
 	VLOG_PARAM_STRING("fork() support", safe_mce_sys().handle_fork, MCE_DEFAULT_FORK_SUPPORT, SYS_VAR_FORK, safe_mce_sys().handle_fork ? "Enabled " : "Disabled");
 	VLOG_PARAM_STRING("close on dup2()", safe_mce_sys().close_on_dup2, MCE_DEFAULT_CLOSE_ON_DUP2, SYS_VAR_CLOSE_ON_DUP2, safe_mce_sys().close_on_dup2 ? "Enabled " : "Disabled");
@@ -673,8 +657,6 @@ static void do_global_ctors_helper()
 
 	NEW_CTOR(g_p_route_table_mgr, route_table_mgr());
 
-	NEW_CTOR(g_p_igmp_mgr, igmp_mgr());
-
 	NEW_CTOR(g_buffer_pool_rx, buffer_pool(safe_mce_sys().rx_num_bufs,
 			RX_BUF_SIZE(g_p_net_device_table_mgr->get_max_mtu()),
 			buffer_pool::free_rx_lwip_pbuf_custom,
@@ -682,25 +664,11 @@ static void do_global_ctors_helper()
 			(safe_mce_sys().m_ioctl.user_alloc.flags & VMA_IOCTL_USER_ALLOC_FLAG_RX ? safe_mce_sys().m_ioctl.user_alloc.memfree : NULL)));
  	g_buffer_pool_rx->set_RX_TX_for_stats(true);
 
-#ifdef DEFINED_TSO
-	safe_mce_sys().tx_buf_size = MIN((int)safe_mce_sys().tx_buf_size, (int)0xFF00);
- 	if (safe_mce_sys().tx_buf_size <= get_lwip_tcp_mss(g_p_net_device_table_mgr->get_max_mtu(), safe_mce_sys().lwip_mss)) {
- 		safe_mce_sys().tx_buf_size = 0;
- 	}
- 	NEW_CTOR(g_buffer_pool_tx, buffer_pool(safe_mce_sys().tx_num_bufs,
- 			TX_BUF_SIZE(safe_mce_sys().tx_buf_size ?
- 					safe_mce_sys().tx_buf_size :
-					get_lwip_tcp_mss(g_p_net_device_table_mgr->get_max_mtu(), safe_mce_sys().lwip_mss)),
-			buffer_pool::free_tx_lwip_pbuf_custom,
-			(safe_mce_sys().m_ioctl.user_alloc.flags & VMA_IOCTL_USER_ALLOC_FLAG_TX ? safe_mce_sys().m_ioctl.user_alloc.memalloc : NULL),
-			(safe_mce_sys().m_ioctl.user_alloc.flags & VMA_IOCTL_USER_ALLOC_FLAG_TX ? safe_mce_sys().m_ioctl.user_alloc.memfree : NULL)));
-#else
  	NEW_CTOR(g_buffer_pool_tx, buffer_pool(safe_mce_sys().tx_num_bufs,
 			TX_BUF_SIZE(get_lwip_tcp_mss(g_p_net_device_table_mgr->get_max_mtu(), safe_mce_sys().lwip_mss)),
 			buffer_pool::free_tx_lwip_pbuf_custom,
 			(safe_mce_sys().m_ioctl.user_alloc.flags & VMA_IOCTL_USER_ALLOC_FLAG_TX ? safe_mce_sys().m_ioctl.user_alloc.memalloc : NULL),
 			(safe_mce_sys().m_ioctl.user_alloc.flags & VMA_IOCTL_USER_ALLOC_FLAG_TX ? safe_mce_sys().m_ioctl.user_alloc.memfree : NULL)));
-#endif /* DEFINED_TSO */
  	g_buffer_pool_tx->set_RX_TX_for_stats(false);
 
  	NEW_CTOR(g_tcp_seg_pool,  tcp_seg_pool(safe_mce_sys().tx_num_segs_tcp));
@@ -780,7 +748,6 @@ int do_global_ctors()
 void reset_globals()
 {
 	g_p_fd_collection = NULL;
-	g_p_igmp_mgr = NULL;
 	g_p_ip_frag_manager = NULL;
 	g_buffer_pool_rx = NULL;
 	g_buffer_pool_tx = NULL;
