@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -54,25 +54,3 @@ const std::string ETH_addr::to_str() const
 		sprintf(s, ETH_HW_ADDR_PRINT_FMT, ETH_HW_ADDR_PRINT_ADDR(m_p_raw_address));
 	return (std::string(s));
 }
-
-const std::string IPoIB_addr::to_str() const
-{
-	char s[100] = "";
-	if (m_len > 0)
-		sprintf(s, IPOIB_HW_ADDR_PRINT_FMT, IPOIB_HW_ADDR_PRINT_ADDR(m_p_raw_address));
-	return (std::string(s));
-}
-
-void IPoIB_addr::extract_qpn()
-{
-	unsigned char rem_qpn[4];
-
-	rem_qpn[0] = m_p_raw_address[3];
-	rem_qpn[1] = m_p_raw_address[2];
-	rem_qpn[2] = m_p_raw_address[1];
-	rem_qpn[3] = 0;
-	memcpy(&m_qpn, rem_qpn, 4);
-	L2_logdbg("qpn = %#x", m_qpn);
-}
-
-
