@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -24,15 +24,8 @@ public:
 		      socket_data &data, resource_allocation_key &ring_alloc_logic);
 	virtual ~dst_entry_tcp();
 
-#ifdef DEFINED_TSO
-	ssize_t fast_send(const iovec* p_iov, const ssize_t sz_iov, vma_send_attr attr);
-	ssize_t slow_send(const iovec* p_iov, const ssize_t sz_iov, vma_send_attr attr,
-			struct vma_rate_limit_t &rate_limit, int flags = 0,
-			socket_fd_api* sock = 0, tx_call_t call_type = TX_UNDEF);
-#else
 	virtual ssize_t fast_send(const iovec* p_iov, const ssize_t sz_iov, bool is_dummy, bool b_blocked = true, bool is_rexmit = false);
 	ssize_t slow_send(const iovec* p_iov, size_t sz_iov, bool is_dummy, struct vma_rate_limit_t &rate_limit, bool b_blocked = true, bool is_rexmit = false, int flags = 0, socket_fd_api* sock = 0, tx_call_t call_type = TX_UNDEF);
-#endif /* DEFINED_TSO */
 	ssize_t slow_send_neigh(const iovec* p_iov, size_t sz_iov, struct vma_rate_limit_t &rate_limit);
 
 	mem_buf_desc_t* get_buffer(bool b_blocked = false);
@@ -51,11 +44,11 @@ protected:
 private:
 	const uint32_t       m_n_sysvar_tx_bufs_batch_tcp;
 
-	inline void		send_lwip_buffer(ring_user_id_t id, vma_ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr)
+	inline void		send_lwip_buffer(ring_user_id_t id, ibv_send_wr* p_send_wqe, vma_wr_tx_packet_attr attr)
 	{
 		if (unlikely(is_set(attr, VMA_TX_PACKET_DUMMY))) {
 			if (m_p_ring->get_hw_dummy_send_support(id, p_send_wqe)) {
-				vma_ibv_wr_opcode last_opcode = m_p_send_wqe_handler->set_opcode(*p_send_wqe, VMA_IBV_WR_NOP);
+				ibv_wr_opcode last_opcode = m_p_send_wqe_handler->set_opcode(*p_send_wqe, VMA_IBV_WR_NOP);
 				m_p_ring->send_lwip_buffer(id, p_send_wqe, attr);
 				m_p_send_wqe_handler->set_opcode(*p_send_wqe, last_opcode);
 			}
