@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -18,7 +18,6 @@
 #include "vma/proto/route_rule_table_key.h"
 #include "vma/proto/route_table_mgr.h"
 #include "vma/dev/wqe_send_handler.h"
-#include "vma/dev/wqe_send_ib_handler.h"
 
 //This include should be after vma includes
 #include <netinet/tcp.h>
@@ -62,7 +61,7 @@ inline int neigh_eth::build_mc_neigh_val()
 
 	if (m_val == NULL)
 		//This is the first time we are trying to allocate new val or it failed last time
-		m_val = new neigh_eth_val;
+		m_val = new neigh_val;
 
 	BULLSEYE_EXCLUDE_BLOCK_START
 	if (m_val == NULL) {
@@ -98,7 +97,7 @@ inline int neigh_eth::build_uc_neigh_val()
 
 	if (m_val == NULL) {
 		// This is the first time we are trying to allocate new val or it failed last time
-		m_val = new neigh_eth_val;
+		m_val = new neigh_val;
 	}
 
 	BULLSEYE_EXCLUDE_BLOCK_START
@@ -128,32 +127,12 @@ inline int neigh_eth::build_uc_neigh_val()
 	return 0;
 }
 
-neigh_val & neigh_ib_val::operator=(const neigh_val & val)
-{
-	IPoIB_addr* l2_addr = NULL;
-	neigh_val* tmp_val = const_cast<neigh_val *>(&val);
-	const neigh_ib_val* ib_val = dynamic_cast<const neigh_ib_val*>(tmp_val);
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if (ib_val == NULL) {
-		__log_panic("neigh_ib_val is NULL");
-	}
-	BULLSEYE_EXCLUDE_BLOCK_END
-	m_l2_address = new IPoIB_addr((ib_val->get_l2_address())->get_address());
-	l2_addr = (IPoIB_addr *)m_l2_address; //no need to do dynamic casting here
-	m_ah = ib_val->get_ah(); //TODO: we need to handle this - in case ah is used in post_send we cannot destroy it
-	m_qkey = ib_val->get_qkey();
-	l2_addr->set_qpn(ib_val->get_qpn());
-	m_ah_attr = ib_val->get_ah_attr();
-	return *this;
-}
-
-neigh_entry::neigh_entry(neigh_key key, transport_type_t _type, bool is_init_resources):
+neigh_entry::neigh_entry(neigh_key key, bool is_init_resources):
 	cache_entry_subject<neigh_key, neigh_val *>(key),
 	m_cma_id(NULL),
 	m_rdma_port_space((enum rdma_port_space)0),
 	m_state_machine(NULL),
 	m_type(UNKNOWN),
-	m_trans_type(_type),
 	m_state(false),
 	m_err_counter(0),
 	m_timer_handle(NULL),
@@ -161,7 +140,7 @@ neigh_entry::neigh_entry(neigh_key key, transport_type_t _type, bool is_init_res
 	m_p_dev(key.get_net_device_val()),
 	m_p_ring(NULL),
 	m_is_loopback(false),
-	m_to_str(std::string(priv_vma_transport_type_str(m_trans_type)) + ":" + get_key().to_str()), m_id(0),
+	m_to_str(get_key().to_str()), m_id(0),
 	m_is_first_send_arp(true), m_n_sysvar_neigh_wait_till_send_arp_msec(safe_mce_sys().neigh_wait_till_send_arp_msec),
 	m_n_sysvar_neigh_uc_arp_quata(safe_mce_sys().neigh_uc_arp_quata),
 	m_n_sysvar_neigh_num_err_retries(safe_mce_sys().neigh_num_err_retries)
@@ -222,7 +201,9 @@ neigh_entry::~neigh_entry()
 		m_state_machine = NULL;
 	}
 	if (m_p_dev && m_p_ring) {
-		m_p_dev->release_ring(m_ring_allocation_logic.get_key());
+		if (m_p_dev->release_ring(m_ring_allocation_logic.get_key())<0) {
+			neigh_logwarn("Failed to release ring for allocation key %s", m_ring_allocation_logic.get_key()->to_str());
+		}
 		m_p_ring = NULL;
 	}
 	if (m_val) {
@@ -332,7 +313,7 @@ void neigh_entry::handle_timer_expired(void* ctx)
 
 	if(!priv_is_failed(state)) {
 		//We want to verify that L2 address wasn't changed
-		unsigned char tmp[IPOIB_HW_ADDR_LEN];
+		unsigned char tmp[ETH_ALEN];
 		address_t l2_addr = (address_t)tmp;
 		if(!priv_get_neigh_l2(l2_addr)) {
 			return;
@@ -487,9 +468,6 @@ bool neigh_entry::post_send_udp(neigh_send_data *n_send_data)
 
 		m_sge.addr = (uintptr_t)(p_mem_buf_desc->p_buffer + (uint8_t)h->m_transport_header_tx_offset);
 		m_sge.length = sz_user_data_to_copy + hdr_len;
-#ifdef DEFINED_TSO
-		m_sge.lkey = m_p_ring->get_tx_lkey(m_id);
-#endif /* DEFINED_TSO */
 		m_send_wqe.wr_id = (uintptr_t)p_mem_buf_desc;
 
 		neigh_logdbg("%s packet_sz=%d, payload_sz=%zd, ip_offset=%d id=%d", h->to_str().c_str(),
@@ -555,9 +533,6 @@ bool neigh_entry::post_send_tcp(neigh_send_data *p_data)
 	size_t hdr_alignment_diff = h->m_aligned_l2_l3_len - h->m_total_hdr_len;
 	m_sge.addr = (uintptr_t)((uint8_t*)p_pkt + hdr_alignment_diff);
 	m_sge.length = total_packet_len;
-#ifdef DEFINED_TSO
-	m_sge.lkey = m_p_ring->get_tx_lkey(m_id);
-#endif /* DEFINED_TSO */
 
 	/* for DEBUG */
 	if ((uint8_t*)m_sge.addr < p_mem_buf_desc->p_buffer) {
@@ -689,6 +664,7 @@ void neigh_entry::handle_neigh_event(neigh_nl_event* nl_ev)
 		}
 
 		// Check if neigh L2 address changed (HA event) and restart the state machine
+		/* coverity[check_return] */
 		priv_handle_neigh_is_l2_changed(nl_info->lladdr);
 		break;
 	}
@@ -761,8 +737,6 @@ const char* neigh_entry::event_to_str(event_t event) const
 		return "EV_PATH_RESOLVED";
 	case EV_ERROR:
 		return "EV_ERROR";
-	case EV_TIMEOUT_EXPIRED:
-		return "EV_TIMEOUT_EXPIRED";
 	case EV_UNHANDLED:
 		return "EV_UNHANDELED";
 	BULLSEYE_EXCLUDE_BLOCK_START
@@ -816,7 +790,7 @@ neigh_entry::event_t neigh_entry::rdma_event_mapping(struct rdma_cm_event* p_rdm
 		return EV_UNHANDLED;
 	}
 	BULLSEYE_EXCLUDE_BLOCK_END
-	neigh_logdbg("Got event %s (%d)", priv_rdma_cm_event_type_str(p_rdma_cm_event->event), p_rdma_cm_event->event);
+	neigh_logdbg("Got event %s (%d)", rdma_event_str(p_rdma_cm_event->event), p_rdma_cm_event->event);
 
 	switch (p_rdma_cm_event->event)
 	{
@@ -1229,7 +1203,7 @@ void neigh_entry::priv_unregister_timer()
 //============================================================== neigh_eth ==================================================
 
 neigh_eth::neigh_eth(neigh_key key) :
-		neigh_entry(key, VMA_TRANSPORT_ETH)
+	neigh_entry(key)
 {
 	neigh_logdbg("");
 	m_rdma_port_space = RDMA_PS_UDP;
@@ -1428,14 +1402,14 @@ bool neigh_eth::post_send_arp(bool is_broadcast)
 		dst = m_p_dev->get_br_address();
 	}
 
-
-	const unsigned char* peer_mac = dst->get_address();
 	BULLSEYE_EXCLUDE_BLOCK_START
 	if (src == NULL || dst == NULL) {
 		neigh_logdbg("src or dst is NULL not sending ARP");
 		return false;
 	}
 	BULLSEYE_EXCLUDE_BLOCK_END
+	
+	const unsigned char* peer_mac = dst->get_address();
 
 	m_id = m_p_ring->generate_id(src->get_address(), dst->get_address(), netdevice_eth->get_vlan() ? htons(ETH_P_8021Q) : htons(ETH_P_ARP), htons(ETH_P_ARP), 0, 0, 0, 0);
 	mem_buf_desc_t* p_mem_buf_desc = m_p_ring->mem_buf_tx_get(m_id, false, 1);
@@ -1525,628 +1499,3 @@ ring_user_id_t neigh_eth::generate_ring_user_id(header * h /* = NULL */)
 				     h->m_header.hdr.m_udp_hdr.source,
 				     h->m_header.hdr.m_udp_hdr.dest);
 }
-
-//============================================================== neigh_ib ==================================================
-
-neigh_ib::neigh_ib(neigh_key key, bool is_init_resources) :
-				neigh_entry(key, VMA_TRANSPORT_IB, is_init_resources), m_pd(NULL), m_n_sysvar_wait_after_join_msec(safe_mce_sys().wait_after_join_msec)
-{
-	neigh_logdbg("");
-
-	m_rdma_port_space = RDMA_PS_IPOIB;
-
-	if(IS_BROADCAST_N(key.get_in_addr())) {
-		//In case of broadcast neigh we don't want to have state machine
-		m_type = MC;
-		return;
-	}
-
-	if (IN_MULTICAST_N(key.get_in_addr())) {
-		//This is Multicast neigh
-		m_type = MC;
-	}
-	else {
-		// This is Unicast neigh
-		m_type = UC;
-	}
-	//Do we need to handle case when we get EV_ERROR but in case this error is not related to the state
-	//Like Address Resolve Error when we at ST_ARP_RESOLVED or ST_PATH_RESOLVED ....
-
-	sm_short_table_line_t short_sm_table[] =
-	{
-			// 	{curr state,            event,                  next state,             action func   }
-			{ ST_NOT_ACTIVE, 	EV_KICK_START, 		ST_INIT,		NULL },
-			{ ST_ERROR, 		EV_KICK_START, 		ST_INIT, 		NULL },
-			{ ST_INIT, 		EV_START_RESOLUTION,	ST_INIT_RESOLUTION, 	NULL },
-			{ ST_INIT_RESOLUTION, 	EV_ADDR_RESOLVED,	ST_ARP_RESOLVED, 	NULL },
-			{ ST_ARP_RESOLVED, 	EV_PATH_RESOLVED,	ST_PATH_RESOLVED, 	NULL },
-			{ ST_PATH_RESOLVED, 	EV_TIMEOUT_EXPIRED,	ST_READY, 		NULL },
-			{ ST_PATH_RESOLVED, 	EV_ERROR,		ST_ERROR, 		NULL },
-			{ ST_ARP_RESOLVED, 	EV_ERROR,		ST_ERROR, 		NULL },
-			{ ST_READY, 		EV_ERROR, 		ST_ERROR,		NULL },
-			{ ST_INIT, 		EV_ERROR, 		ST_ERROR, 		NULL },
-			{ ST_INIT_RESOLUTION, 	EV_ERROR, 		ST_ERROR, 		NULL },
-			{ ST_ERROR, 		EV_ERROR, 		ST_NOT_ACTIVE, 		NULL },
-			//Entry functions
-			{ ST_INIT, 		SM_STATE_ENTRY, 	SM_NO_ST,		neigh_entry::dofunc_enter_init },
-			{ ST_INIT_RESOLUTION, 	SM_STATE_ENTRY, 	SM_NO_ST,		neigh_entry::dofunc_enter_init_resolution },
-			{ ST_ARP_RESOLVED, 	SM_STATE_ENTRY,		SM_NO_ST,		neigh_ib::dofunc_enter_arp_resolved },
-			{ ST_PATH_RESOLVED, 	SM_STATE_ENTRY,		SM_NO_ST, 		neigh_ib::dofunc_enter_path_resolved },
-			{ ST_READY, 		SM_STATE_ENTRY, 	SM_NO_ST,		neigh_entry::dofunc_enter_ready },
-			{ ST_NOT_ACTIVE, 	SM_STATE_ENTRY,		SM_NO_ST,		neigh_entry::dofunc_enter_not_active },
-			{ ST_ERROR, 		SM_STATE_ENTRY,		SM_NO_ST,		neigh_entry::dofunc_enter_error },
-			SM_TABLE_END };
-
-	// Create state_nachine
-	m_state_machine = new state_machine(this,		// app hndl
-			ST_NOT_ACTIVE,		// start state_t
-			ST_LAST,		// max states
-			EV_LAST,		// max events
-			short_sm_table,	// short table
-			general_st_entry,	// default entry function
-			general_st_leave,	// default leave function
-			NULL,		// default func
-			print_event_info	// debug function
-	);
-
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if (m_state_machine == NULL)
-		neigh_logpanic("Failed allocating state_machine");
-	BULLSEYE_EXCLUDE_BLOCK_END
-
-	priv_kick_start_sm();
-}
-
-neigh_ib::~neigh_ib()
-{
-	priv_enter_not_active();
-}
-
-void neigh_ib::handle_event_ibverbs_cb(void* ev_data, void* ctx)
-{
-	NOT_IN_USE(ctx);
-	event_t event = ibverbs_event_mapping(ev_data);
-	event_handler(event, ev_data);
-}
-
-// called when timer expired
-void neigh_ib::handle_timer_expired(void* ctx)
-{
-	neigh_logdbg("general timeout expired!");
-
-	m_sm_lock.lock();
-	int state = m_state_machine->get_curr_state();
-	m_sm_lock.unlock();
-
-	if(state == ST_PATH_RESOLVED) {
-		// Clear Timer Handler
-		m_timer_handle = NULL;
-		event_handler(EV_TIMEOUT_EXPIRED);
-	}
-	else if(state == ST_READY) {
-		neigh_entry::handle_timer_expired(ctx);
-	}
-	else if(state == ST_INIT) {
-		// Clear Timer Handler
-		m_timer_handle = NULL;
-		event_handler(EV_START_RESOLUTION);
-	}
-}
-
-bool neigh_ib::priv_handle_neigh_is_l2_changed(address_t new_l2_addr)
-{
-	auto_unlocker lock(m_lock);
-	IPoIB_addr new_l2_address(new_l2_addr);
-	if (m_val) {
-		if(m_val->get_l2_address()) {
-			if (!(m_val->get_l2_address()->compare(new_l2_address))) {
-				neigh_logdbg("l2 address was changed (%s => %s)", (m_val->get_l2_address())->to_str().c_str(), new_l2_address.to_str().c_str());
-				event_handler(EV_ERROR);
-				return true;
-			}
-			else {
-				neigh_logdbg("No change in l2 address");
-				return false;
-			}
-		}
-		else {
-			neigh_logdbg("l2 address is NULL\n");
-		}
-	}
-	else {
-		neigh_logerr("m_val is NULL");
-	}
-
-	event_handler(EV_ERROR);
-	return true;
-}
-
-bool neigh_ib::post_send_arp(bool is_broadcast)
-{
-	neigh_logdbg("Sending %s ARP", is_broadcast?"BC":"UC");
-
-	mem_buf_desc_t* p_mem_buf_desc = m_p_ring->mem_buf_tx_get(m_id, false, 1);
-	if (unlikely(p_mem_buf_desc == NULL)) {
-		neigh_logdbg("No free TX buffer, not sending ARP");
-		return false;
-	}
-
-	net_device_val_ib *netdevice_ib = dynamic_cast<net_device_val_ib*>(m_p_dev);
-	if (netdevice_ib == NULL) {
-		m_p_ring->mem_buf_tx_release(p_mem_buf_desc, true);
-		neigh_logdbg("Net dev is NULL not sending ARP");
-		return false;
-	}
-
-	const L2_address *src = netdevice_ib->get_l2_address();
-	const L2_address *dst;
-	neigh_ib_val br_neigh_val;
-	ibv_ah* ah = NULL;
-	uint32_t qpn;
-	uint32_t qkey;
-	const unsigned char* peer_mac = NULL;
-	if (!is_broadcast) {
-		dst = m_val->get_l2_address();
-		peer_mac = dst->get_address();
-		ah = ((neigh_ib_val *)m_val)->get_ah();
-		qpn = ((neigh_ib_val *)m_val)->get_qpn();
-		qkey = ((neigh_ib_val *)m_val)->get_qkey();
-	}
-	else {
-		dst = m_p_dev->get_br_address();
-		neigh_ib_broadcast * br_neigh = const_cast<neigh_ib_broadcast *>(((net_device_val_ib*)m_p_dev)->get_br_neigh());
-		bool ret = br_neigh->get_peer_info(&br_neigh_val);
-		if (ret) {
-			ah = br_neigh_val.get_ah();
-			qpn = br_neigh_val.get_qpn();
-			qkey = br_neigh_val.get_qkey();
-		}
-		else {
-			m_p_ring->mem_buf_tx_release(p_mem_buf_desc, true);
-			neigh_logdbg("BR Neigh is not valid, not sending BR ARP");
-			return false;
-		}
-	}
-
-	if (src == NULL || dst == NULL) {
-		m_p_ring->mem_buf_tx_release(p_mem_buf_desc, true);
-		neigh_logdbg("src or dst is NULL not sending ARP");
-		return false;
-	}
-
-	wqe_send_ib_handler wqe_sh;
-	wqe_sh.init_ib_wqe(m_send_wqe, &m_sge, 1, ah, qpn, qkey);
-	neigh_logdbg("ARP: ah=%p, qkey=%#x, qpn=%#x", ah ,qkey, qpn);
-	header h;
-	h.init();
-	h.configure_ipoib_headers(IPOIB_ARP_HEADER);
-
-
-	tx_packet_template_t *p_pkt = (tx_packet_template_t*)p_mem_buf_desc->p_buffer;
-	h.copy_l2_hdr(p_pkt);
-
-	ib_arp_hdr* p_arphdr = (ib_arp_hdr*) (p_mem_buf_desc->p_buffer + h.m_transport_header_tx_offset + h.m_total_hdr_len);
-	set_ib_arp_hdr(p_arphdr, m_p_dev->get_local_addr(), get_key().get_in_addr(), m_p_dev->get_l2_address()->get_address(), peer_mac);
-
-	m_sge.addr = (uintptr_t)(p_mem_buf_desc->p_buffer + (uint8_t)h.m_transport_header_tx_offset);
-	m_sge.length = sizeof(ib_arp_hdr) + h.m_total_hdr_len;
-	m_sge.lkey = p_mem_buf_desc->lkey;
-	p_mem_buf_desc->p_next_desc = NULL;
-	m_send_wqe.wr_id = (uintptr_t)p_mem_buf_desc;
-
-	m_p_ring->send_ring_buffer(m_id, &m_send_wqe, (vma_wr_tx_packet_attr)0);
-
-	neigh_logdbg("ARP Sent");
-	return true;
-}
-
-bool neigh_ib::prepare_to_send_packet(header * h)
-{
-	neigh_logdbg("");
-	wqe_send_ib_handler wqe_sh;
-	wqe_sh.init_ib_wqe(m_send_wqe, &m_sge , 1, ((neigh_ib_val *)m_val)->get_ah(), ((neigh_ib_val *)m_val)->get_qpn(), ((neigh_ib_val *)m_val)->get_qkey());
-	h->configure_ipoib_headers();
-
-	return true;
-}
-
-neigh_entry::event_t neigh_ib::ibverbs_event_mapping(void* p_event_info)
-{
-	struct ibv_async_event *ev = (struct ibv_async_event *) p_event_info;
-	neigh_logdbg("Got event %s (%d) ", priv_ibv_event_desc_str(ev->event_type), ev->event_type);
-
-	switch (ev->event_type)
-	{
-	case IBV_EVENT_SM_CHANGE:
-	case IBV_EVENT_CLIENT_REREGISTER:
-		return EV_ERROR;
-	default:
-		return EV_UNHANDLED;
-	}
-}
-
-void neigh_ib::dofunc_enter_arp_resolved(const sm_info_t& func_info)
-{
-	neigh_ib * my_neigh = (neigh_ib *) func_info.app_hndl;
-	neigh_entry::general_st_entry(func_info);
-
-	run_helper_func(priv_enter_arp_resolved(), EV_ERROR);
-}
-
-void neigh_ib::dofunc_enter_path_resolved(const sm_info_t& func_info)
-{
-	neigh_ib * my_neigh = (neigh_ib *) func_info.app_hndl;
-	neigh_entry::general_st_entry(func_info);
-
-	uint32_t wait_after_join_msec;
-
-	run_helper_func(priv_enter_path_resolved((struct rdma_cm_event*)func_info.ev_data, wait_after_join_msec),
-			EV_ERROR);
-	my_neigh->m_timer_handle = my_neigh->priv_register_timer_event(wait_after_join_msec, my_neigh, ONE_SHOT_TIMER, NULL);
-}
-
-int neigh_ib::priv_enter_arp_resolved()
-{
-	neigh_logfunc("");
-
-	if (m_cma_id->verbs == NULL) {
-		neigh_logdbg("m_cma_id->verbs is NULL");
-		return -1;
-	}
-
-	if (find_pd())
-		return -1;
-
-	//Register Verbs event in case there was Fabric change
-	if (m_cma_id->verbs) {
-		g_p_event_handler_manager->register_ibverbs_event(
-				m_cma_id->verbs->async_fd, this,
-				m_cma_id->verbs, 0);
-	}
-
-	if (m_type == UC)
-		return (handle_enter_arp_resolved_uc());
-	else
-		// MC
-		return (handle_enter_arp_resolved_mc());
-}
-
-int neigh_ib::priv_enter_path_resolved(struct rdma_cm_event* event_data,
-		uint32_t & wait_after_join_msec)
-{
-	neigh_logfunc("");
-
-	if (m_val == NULL)
-		//This is the first time we are trying to allocate new val or it failed last time
-		m_val = new neigh_ib_val;
-
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if (m_val == NULL)
-		return -1;
-	BULLSEYE_EXCLUDE_BLOCK_END
-
-	if (m_type == UC)
-		return (build_uc_neigh_val(event_data, wait_after_join_msec));
-	else
-		//MC
-		return (build_mc_neigh_val(event_data, wait_after_join_msec));
-}
-
-void  neigh_ib::priv_enter_error()
-{
-	auto_unlocker lock(m_lock);
-
-	m_state = false;
-	m_pd = NULL;
-
-	destroy_ah();
-	priv_unregister_timer();
-
-	if (m_cma_id && m_cma_id->verbs) {
-		neigh_logdbg("Unregister Verbs event");
-		g_p_event_handler_manager->unregister_ibverbs_event(m_cma_id->verbs->async_fd, this);
-	}
-
-	neigh_entry::priv_enter_error();
-}
-
-void neigh_ib::priv_enter_not_active()
-{
-	neigh_logfunc("");
-
-	auto_unlocker lock(m_lock);
-
-	m_state = false;
-	m_pd = NULL;
-
-	destroy_ah();
-
-	if (m_cma_id && m_cma_id->verbs) {
-		neigh_logdbg("Unregister Verbs event");
-		g_p_event_handler_manager->unregister_ibverbs_event(m_cma_id->verbs->async_fd, this);
-	}
-
-	neigh_entry::priv_enter_not_active();
-}
-
-int neigh_ib::priv_enter_ready()
-{
-	neigh_logfunc("");
-	priv_unregister_timer();
-	return (neigh_entry::priv_enter_ready());
-}
-
-int neigh_ib::handle_enter_arp_resolved_mc()
-{
-	neigh_logdbg("");
-
-	IF_RDMACM_FAILURE(rdma_join_multicast( m_cma_id, (struct sockaddr*)&m_dst_addr, (void *)this))
-	{
-		neigh_logdbg("Failed in rdma_join_multicast (errno=%d %m)", errno);
-		return -1;
-	} ENDIF_RDMACM_FAILURE;
-
-	return 0;
-}
-
-int neigh_ib::handle_enter_arp_resolved_uc()
-{
-	neigh_logdbg("");
-
-	IF_RDMACM_FAILURE(rdma_resolve_route(m_cma_id, RDMA_CM_TIMEOUT))
-	{
-		neigh_logdbg("Resolve address error (errno=%d %m)", errno);
-		return -1;
-	} ENDIF_RDMACM_FAILURE;
-
-	return 0;
-}
-
-int neigh_ib::build_mc_neigh_val(struct rdma_cm_event* event_data,
-		uint32_t & wait_after_join_msec)
-{
-	neigh_logdbg("");
-
-	m_val->m_l2_address = new IPoIB_addr(event_data->param.ud.qp_num, (address_t)event_data->param.ud.ah_attr.grh.dgid.raw);
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if (m_val->m_l2_address == NULL) {
-		neigh_logdbg("Failed allocating m_val->m_l2_address");
-		return -1;
-	}
-	BULLSEYE_EXCLUDE_BLOCK_END
-
-	((neigh_ib_val *) m_val)->m_qkey = event_data->param.ud.qkey;
-
-	memcpy(&((neigh_ib_val *) m_val)->m_ah_attr,
-			&event_data->param.ud.ah_attr,
-			sizeof(((neigh_ib_val *) m_val)->m_ah_attr));
-
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if (create_ah())
-		return -1;
-	BULLSEYE_EXCLUDE_BLOCK_END
-
-	neigh_logdbg("IB multicast neigh params are : ah=%p, qkey=%#x, sl=%#x, rate=%#x, port_num = %#x,  qpn=%#x dlid=%#x dgid = " IPOIB_HW_ADDR_PRINT_FMT_16,
-				((neigh_ib_val *) m_val)->m_ah, ((neigh_ib_val *) m_val)->m_qkey, ((neigh_ib_val *) m_val)->m_ah_attr.sl, ((neigh_ib_val *) m_val)->m_ah_attr.static_rate,
-				((neigh_ib_val *) m_val)->m_ah_attr.port_num, ((neigh_ib_val *) m_val)->get_qpn(), ((neigh_ib_val *) m_val)->m_ah_attr.dlid,
-				IPOIB_HW_ADDR_PRINT_ADDR_16(((neigh_ib_val *) m_val)->m_ah_attr.grh.dgid.raw));
-
-	wait_after_join_msec = m_n_sysvar_wait_after_join_msec;
-
-	return 0;
-}
-
-int neigh_ib::build_uc_neigh_val(struct rdma_cm_event* event_data,
-		uint32_t & wait_after_join_msec)
-{
-	NOT_IN_USE(event_data);
-	neigh_logdbg("");
-
-	// Find peer's IPoIB row address
-	unsigned char tmp[IPOIB_HW_ADDR_LEN];
-	address_t address = (address_t) tmp;
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if (!priv_get_neigh_l2(address)) {
-		neigh_logdbg("Failed in priv_get_neigh_l2()");
-		return -1;
-	}
-	BULLSEYE_EXCLUDE_BLOCK_END
-
-	m_val->m_l2_address = new IPoIB_addr(address);
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if (m_val->m_l2_address == NULL) {
-		neigh_logdbg("Failed creating m_val->m_l2_address");
-		return -1;
-	}
-	BULLSEYE_EXCLUDE_BLOCK_END
-	neigh_logdbg("IPoIB MAC = %s", m_val->m_l2_address->to_str().c_str());
-	// IPoIB qkey is hard coded in SM . Do we want to take it from event or leave it hard coded
-	//((neigh_ib_val *) m_val)->m_qkey = event_data->param.ud.qkey; //0x0b1b;
-	((neigh_ib_val *) m_val)->m_qkey = IPOIB_QKEY;
-
-	//memcpy(&m_val.ib_addr.m_ah_attr, &event_data->param.ud.ah_attr, sizeof(struct ibv_ah_attr));
-
-	if (!m_cma_id || m_cma_id->route.num_paths <= 0) {
-		neigh_logdbg("Can't prepare AH attr (cma_id=%p, num_paths=%d)", m_cma_id, m_cma_id ? m_cma_id->route.num_paths : 0);
-		return -1;
-	}
-
-	memset(&((neigh_ib_val *) m_val)->m_ah_attr, 0, sizeof(((neigh_ib_val *) m_val)->m_ah_attr));
-	((neigh_ib_val *) m_val)->m_ah_attr.dlid 	  = 	ntohs(m_cma_id->route.path_rec->dlid);
-	((neigh_ib_val *) m_val)->m_ah_attr.sl 	 	  = 	m_cma_id->route.path_rec->sl;
-	((neigh_ib_val *) m_val)->m_ah_attr.src_path_bits = 	0;
-	((neigh_ib_val *) m_val)->m_ah_attr.static_rate   = 	m_cma_id->route.path_rec->rate;
-	((neigh_ib_val *) m_val)->m_ah_attr.is_global 	  = 	0;
-	((neigh_ib_val *) m_val)->m_ah_attr.port_num 	  = 	m_cma_id->port_num;
-
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if (create_ah())
-		return -1;
-	BULLSEYE_EXCLUDE_BLOCK_END
-
-	neigh_logdbg("IB unicast neigh params  ah=%p, qkey=%#x, qpn=%#x, dlid=%#x", ((neigh_ib_val *) m_val)->m_ah,
-			((neigh_ib_val *) m_val)->m_qkey, ((neigh_ib_val *) m_val)->get_qpn(), ((neigh_ib_val *) m_val)->m_ah_attr.dlid);
-
-	wait_after_join_msec = 0;
-
-	return 0;
-}
-
-int neigh_ib::find_pd()
-{
-	neigh_logdbg("");
-
-	ib_ctx_handler* ib_ctx_h = g_p_ib_ctx_handler_collection->get_ib_ctx(m_p_dev->get_ifname_link());
-
-	if (ib_ctx_h) {
-		m_pd = ib_ctx_h->get_ibv_pd();
-		return 0;
-	}
-
-	return -1;
-}
-
-int neigh_ib::create_ah()
-{
-	neigh_logdbg("");
-
-	/*	if (((neigh_ib_val *) m_val)->m_ah) {
-		// if there's ah we want to destroy it - shouldn't happen
-		neigh_logerr("destroy ah %p (shouldn't happen)", ((neigh_ib_val *) m_val)->m_ah);
-		if (destroy_ah())
-			return -1;
-	}
-	 */
-	((neigh_ib_val *) m_val)->m_ah = ibv_create_ah(m_pd, &((neigh_ib_val *) m_val)->m_ah_attr);
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if (!((neigh_ib_val *) m_val)->m_ah) {
-		neigh_logdbg("failed creating address handler (errno=%d %m)", errno);
-		return -1;
-	}
-	BULLSEYE_EXCLUDE_BLOCK_END
-	return 0;
-}
-
-int neigh_ib::destroy_ah()
-{
-	neigh_logdbg("");
-	//For now we whouldn't destroy it
-	//We cannot destroy ah till each post_send with this ah has ended
-	//TODO: Need to think how to  handle this - for now there will be ah leak
-	return 0;
-#if 0 //unreachable code
-#ifndef __COVERITY__
-	if (m_val && ((neigh_ib_val *) m_val)->m_ah) {
-		IF_VERBS_FAILURE(ibv_destroy_ah(((neigh_ib_val *) m_val)->m_ah))
-		{
-			neigh_logdbg("failed destroying address handle (errno=%d %m)", errno);
-			return -1;
-		}ENDIF_VERBS_FAILURE;
-	}
-	return 0;
-#endif
-#endif
-}
-
-//==================================================================================================================
-
-neigh_ib_broadcast::neigh_ib_broadcast(neigh_key key) : neigh_ib(key, false)
-{
-	neigh_logdbg("Calling rdma_create_id");
-	IF_RDMACM_FAILURE(rdma_create_id(g_p_neigh_table_mgr->m_neigh_cma_event_channel, &m_cma_id, (void *)this, m_rdma_port_space))
-	{
-		neigh_logerr("Failed in rdma_create_id (errno=%d %m)", errno);
-		return;
-	} ENDIF_RDMACM_FAILURE;
-
-
-	neigh_logdbg("Calling rdma_bind_addr");
-	struct sockaddr_in local_sockaddr;
-	local_sockaddr.sin_family = AF_INET;
-	local_sockaddr.sin_port = INPORT_ANY;
-	local_sockaddr.sin_addr.s_addr = m_p_dev->get_local_addr();
-
-	IF_RDMACM_FAILURE(rdma_bind_addr(m_cma_id, (struct sockaddr*)&local_sockaddr)) {
-		neigh_logerr("Failed in rdma_bind_addr (src=%d.%d.%d.%d) (errno=%d %m)", NIPQUAD(m_p_dev->get_local_addr()), errno);
-		return;
-	} ENDIF_RDMACM_FAILURE;
-
-	build_mc_neigh_val();
-
-	m_state = true;
-}
-
-void neigh_ib_broadcast::build_mc_neigh_val()
-{
-	m_val = new neigh_ib_val;
-	if(m_val == NULL) {
-		neigh_logerr("Failed allocating m_val");
-		return;
-	}
-
-	if (m_cma_id->verbs == NULL) {
-		neigh_logdbg("m_cma_id->verbs is NULL");
-		return;
-	}
-
-	m_val->m_l2_address = new IPoIB_addr(((m_p_dev->get_br_address())->get_address()));
-	if (m_val->m_l2_address == NULL) {
-		neigh_logerr("Failed allocating m_val->m_l2_address");
-		return;
-	}
-
-	((neigh_ib_val *) m_val)->m_qkey = IPOIB_QKEY;
-
-	memset(&((neigh_ib_val *) m_val)->m_ah_attr, 0, sizeof(((neigh_ib_val *) m_val)->m_ah_attr));
-	memcpy( ((neigh_ib_val *) m_val)->m_ah_attr.grh.dgid.raw , &((m_val->m_l2_address->get_address())[4]), 16*sizeof(char));
-
-	((neigh_ib_val *) m_val)->m_ah_attr.dlid 	  = 	0xc000;
-	((neigh_ib_val *) m_val)->m_ah_attr.static_rate   = 	0x3;
-	((neigh_ib_val *) m_val)->m_ah_attr.port_num 	  = 	m_cma_id->port_num;
-	((neigh_ib_val *) m_val)->m_ah_attr.is_global	  =	0x1;
-
-	if(find_pd()) {
-		neigh_logerr("Failed find_pd()");
-		return;
-	}
-
-	if (create_ah())
-		return;
-
-	neigh_logdbg("IB broadcast neigh params are : ah=%p, qkey=%#x, sl=%#x, rate=%#x, port_num = %#x,  qpn=%#x,  dlid=%#x dgid = " IPOIB_HW_ADDR_PRINT_FMT_16,
-			((neigh_ib_val *) m_val)->m_ah, ((neigh_ib_val *) m_val)->m_qkey, ((neigh_ib_val *) m_val)->m_ah_attr.sl,
-			((neigh_ib_val *) m_val)->m_ah_attr.static_rate,((neigh_ib_val *) m_val)->m_ah_attr.port_num,
-			((neigh_ib_val *) m_val)->get_qpn(), ((neigh_ib_val *) m_val)->m_ah_attr.dlid, IPOIB_HW_ADDR_PRINT_ADDR_16(((neigh_ib_val *) m_val)->m_ah_attr.grh.dgid.raw) );
-
-
-}
-
-bool neigh_ib_broadcast::get_peer_info(neigh_val * p_val)
-{
-	neigh_logfunc("calling neigh_entry get_peer_info. state = %d", m_state);
-	if (p_val == NULL) {
-		neigh_logdbg("p_val is NULL, return false");
-		return false;
-	}
-
-	auto_unlocker lock(m_lock);
-	if (m_state) {
-		neigh_logdbg("There is a valid val");
-		*p_val = *m_val;
-		return m_state;
-	}
-
-	return false;
-}
-
-int neigh_ib_broadcast::send(neigh_send_info &s_info)
-{
-	NOT_IN_USE(s_info);
-	neigh_logerr("We should not call for this function, something is wrong");
-	return false;
-}
-
-void neigh_ib_broadcast::send_arp()
-{
-	neigh_logerr("We should not call for this function, something is wrong");
-}
