@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2016-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2016-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -34,8 +34,6 @@
 #include "vma/proto/vma_lwip.h"
 #include "vma/proto/route_table_mgr.h"
 #include "vma/proto/rule_table_mgr.h"
-#include "vma/proto/igmp_mgr.h"
-
 #include "vma/proto/neighbour_table_mgr.h"
 #include "vma/netlink/netlink_wrapper.h"
 #include "vma/event/command.h"
@@ -278,6 +276,10 @@ int mce_sys_var::env_to_cpuset(char *orig_start, cpu_set_t *cpu_set)
 	int ret;
 	char* start = strdup(orig_start); // save the caller string from strtok destruction.
 
+	if (!start) {
+        vlog_printf(VLOG_WARNING, "Failed to allocate memory for CPU set");
+        return -1;
+    }
 	/*
 	 * We expect a hex number or comma delimited cpulist.  Check for 
 	 * starting characters of "0x" or "0X" and if present then parse
@@ -298,8 +300,8 @@ int mce_sys_var::env_to_cpuset(char *orig_start, cpu_set_t *cpu_set)
 
 void mce_sys_var::read_env_variable_with_pid(char* mce_sys_name, size_t mce_sys_max_size, char* env_ptr)
 {
-	int n = -1;
-	char* d_pos = NULL;
+	int n;
+	char* d_pos;
 
 	if (NULL == env_ptr || NULL == mce_sys_name || mce_sys_max_size < 2) {
 		return ;
@@ -505,9 +507,6 @@ void mce_sys_var::get_env_params()
 
 	tx_num_segs_tcp         = MCE_DEFAULT_TX_NUM_SEGS_TCP;
 	tx_num_bufs             = MCE_DEFAULT_TX_NUM_BUFS;
-#ifdef DEFINED_TSO
-	tx_buf_size             = MCE_DEFAULT_TX_BUF_SIZE;
-#endif /* DEFINED_TSO */
 	tx_num_wr               = MCE_DEFAULT_TX_NUM_WRE;
 	tx_num_wr_to_signal     = MCE_DEFAULT_TX_NUM_WRE_TO_SIGNAL;
 	tx_max_inline		= MCE_DEFAULT_TX_MAX_INLINE;
@@ -574,11 +573,7 @@ void mce_sys_var::get_env_params()
 	thread_mode		= MCE_DEFAULT_THREAD_MODE;
 	buffer_batching_mode	= MCE_DEFAULT_BUFFER_BATCHING_MODE;
 	mem_alloc_type          = MCE_DEFAULT_MEM_ALLOC_TYPE;
-	enable_ipoib		= MCE_DEFAULT_IPOIB_FLAG;
 	enable_socketxtreme	= MCE_DEFAULT_SOCKETXTREME;
-#ifdef DEFINED_TSO
-	enable_tso		= MCE_DEFAULT_TSO;
-#endif /* DEFINED_TSO */
 	handle_fork		= MCE_DEFAULT_FORK_SUPPORT;
 	handle_bf		= MCE_DEFAULT_BF_FLAG;
 	close_on_dup2		= MCE_DEFAULT_CLOSE_ON_DUP2;
@@ -635,9 +630,6 @@ void mce_sys_var::get_env_params()
 		rx_num_wr               = 256; //MCE_DEFAULT_RX_NUM_WRE (16000)
 		rx_num_wr_to_post_recv  = 4; //MCE_DEFAULT_RX_NUM_WRE_TO_POST_RECV (64)
 		rx_poll_num             = -1; //MCE_DEFAULT_RX_NUM_POLLS
-#ifdef DEFINED_TSO
-		enable_tso              = false; //MCE_DEFAULT_TSO (true)
-#endif /* DEFINED_TSO */
 		rx_udp_poll_os_ratio    = 0; //MCE_DEFAULT_RX_UDP_POLL_OS_RATIO
 		rx_prefetch_bytes	= MCE_DEFAULT_RX_PREFETCH_BYTES; //(256)
 		rx_prefetch_bytes_before_poll = 256; //MCE_DEFAULT_RX_PREFETCH_BYTES_BEFORE_POLL 0
@@ -664,9 +656,6 @@ void mce_sys_var::get_env_params()
 		rx_num_wr               = 256; //MCE_DEFAULT_RX_NUM_WRE (16000)
 		rx_num_wr_to_post_recv  = 4;   //MCE_DEFAULT_RX_NUM_WRE_TO_POST_RECV (64)
 		rx_poll_num             = -1;  //MCE_DEFAULT_RX_NUM_POLLS (100000)
-#ifdef DEFINED_TSO
-		enable_tso              = false; //MCE_DEFAULT_TSO (true)
-#endif /* DEFINED_TSO */
 		rx_prefetch_bytes_before_poll = 256; //MCE_DEFAULT_RX_PREFETCH_BYTES_BEFORE_POLL (0)
 		select_poll_num         = -1;  //MCE_DEFAULT_SELECT_NUM_POLLS (100000)
 		avoid_sys_calls_on_tcp_fd = true; //MCE_DEFAULT_AVOID_SYS_CALLS_ON_TCP_FD (false)
@@ -816,11 +805,6 @@ void mce_sys_var::get_env_params()
 	if ((env_ptr = getenv(SYS_VAR_TX_NUM_BUFS)) != NULL)
 		tx_num_bufs = (uint32_t)atoi(env_ptr);
 
-#ifdef DEFINED_TSO
-	if ((env_ptr = getenv(SYS_VAR_TX_BUF_SIZE)) != NULL)
-		tx_buf_size = (uint32_t)atoi(env_ptr);
-#endif /* DEFINED_TSO */
-
 	if ((env_ptr = getenv(SYS_VAR_TX_NUM_WRE)) != NULL)
 		tx_num_wr = (uint32_t)atoi(env_ptr);
 
@@ -1215,20 +1199,6 @@ void mce_sys_var::get_env_params()
 	if ((env_ptr = getenv(SYS_VAR_FORK)) != NULL)
 		handle_fork = atoi(env_ptr) ? true : false;
 
-	if((env_ptr = getenv(SYS_VAR_IPOIB )) != NULL)
-		enable_ipoib = atoi(env_ptr) ? true : false;
-
-#ifdef DEFINED_TSO
-	if((env_ptr = getenv(SYS_VAR_TSO)) != NULL)
-		enable_tso = atoi(env_ptr) ? true : false;
-
-	if (enable_tso && (ring_migration_ratio_tx != -1)) {
-		ring_migration_ratio_tx = -1;
-		vlog_printf(VLOG_DEBUG,"%s parameter is forced to %d in case %s is enabled\n",
-				SYS_VAR_RING_MIGRATION_RATIO_TX, -1, SYS_VAR_TSO);
-	}
-#endif /* DEFINED_TSO */
-
 	if ((env_ptr = getenv(SYS_VAR_CLOSE_ON_DUP2)) != NULL)
 		close_on_dup2 = atoi(env_ptr) ? true : false;
 
@@ -1287,10 +1257,12 @@ void set_env_params()
 	if (safe_mce_sys().handle_bf) {
 		setenv("MLX4_POST_SEND_PREFER_BF", "1", 1);
 		setenv("MLX5_POST_SEND_PREFER_BF", "1", 1);
+		setenv("MLX5_SHUT_UP_BF", "0", 1);
 	} else {
 		/* todo - these seem not to work if inline is on, since libmlx is doing (inl || bf) when deciding to bf*/
 		setenv("MLX4_POST_SEND_PREFER_BF", "0", 1);
 		setenv("MLX5_POST_SEND_PREFER_BF", "0", 1);
+		setenv("MLX5_SHUT_UP_BF", "1", 1);
 	}
 
 	switch (safe_mce_sys().mem_alloc_type) {
