@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -100,26 +100,6 @@ typedef enum {
 	BUFFER_BATCHING_LAST,
 } buffer_batching_mode_t;
 
-// See ibv_transport_type for general verbs transport types
-typedef enum {
-	VMA_TRANSPORT_UNKNOWN	= -1,
-	VMA_TRANSPORT_IB	= 0,
-	VMA_TRANSPORT_ETH
-} transport_type_t;
-
-static inline const char* priv_vma_transport_type_str(transport_type_t transport_type)
-{
-	BULLSEYE_EXCLUDE_BLOCK_START
-	switch (transport_type) {
-	case VMA_TRANSPORT_IB: 			return "IB";
-	case VMA_TRANSPORT_ETH: 		return "ETH";
-	case VMA_TRANSPORT_UNKNOWN:
-	default:				break;
-	}
-	return "UNKNOWN";
-	BULLSEYE_EXCLUDE_BLOCK_END
-}
-
 typedef enum {
 	MSS_FOLLOW_MTU = 0
 } mss_mode_t;
@@ -291,9 +271,6 @@ public:
 
 	uint32_t 	tx_num_segs_tcp;
 	uint32_t 	tx_num_bufs;
-#ifdef DEFINED_TSO
-	uint32_t 	tx_buf_size;
-#endif /* DEFINED_TSO */
 	uint32_t 	tx_num_wr;
 	uint32_t	tx_num_wr_to_signal;
 	uint32_t 	tx_max_inline;
@@ -371,12 +348,7 @@ public:
 	cpu_set_t	internal_thread_affinity;
 	bool		internal_thread_arm_cq_enabled;
 	bool 		handle_bf;
-
-	bool 		enable_ipoib;
 	bool 		enable_socketxtreme;
-#ifdef DEFINED_TSO
-	bool 		enable_tso;
-#endif /* DEFINED_TSO */
 	uint32_t	timer_netlink_update_msec;
 
 	//Neigh parameters
@@ -447,9 +419,6 @@ extern mce_sys_var & safe_mce_sys();
 
 #define SYS_VAR_TX_NUM_SEGS_TCP				"VMA_TX_SEGS_TCP"
 #define SYS_VAR_TX_NUM_BUFS				"VMA_TX_BUFS"
-#ifdef DEFINED_TSO
-#define SYS_VAR_TX_BUF_SIZE				"VMA_TX_BUF_SIZE"
-#endif /* DEFINED_TSO */
 #define SYS_VAR_TX_NUM_WRE				"VMA_TX_WRE"
 #define SYS_VAR_TX_NUM_WRE_TO_SIGNAL			"VMA_TX_WRE_BATCHING"
 #define SYS_VAR_TX_MAX_INLINE				"VMA_TX_MAX_INLINE"
@@ -522,12 +491,7 @@ extern mce_sys_var & safe_mce_sys();
 #define SYS_VAR_SPEC					"VMA_SPEC"
 #define SYS_VAR_SPEC_PARAM1				"VMA_SPEC_PARAM1"
 #define SYS_VAR_SPEC_PARAM2				"VMA_SPEC_PARAM2"
-
-#define SYS_VAR_IPOIB					"VMA_IPOIB"
-#define SYS_VAR_SOCKETXTREME				"VMA_SOCKETXTREME"
-#ifdef DEFINED_TSO
-#define SYS_VAR_TSO					"VMA_TSO"
-#endif /* DEFINED_TSO */
+#define SYS_VAR_SOCKETXTREME			"VMA_SOCKETXTREME"
 
 #define SYS_VAR_INTERNAL_THREAD_AFFINITY		"VMA_INTERNAL_THREAD_AFFINITY"
 #define SYS_VAR_INTERNAL_THREAD_CPUSET			"VMA_INTERNAL_THREAD_CPUSET"
@@ -565,9 +529,6 @@ extern mce_sys_var & safe_mce_sys();
 #define MCE_DEFAULT_TCP_MAX_SYN_RATE                	(0)
 #define MCE_DEFAULT_TX_NUM_SEGS_TCP			(1000000)
 #define MCE_DEFAULT_TX_NUM_BUFS				(200000)
-#ifdef DEFINED_TSO
-#define MCE_DEFAULT_TX_BUF_SIZE				(0)
-#endif /* DEFINED_TSO */
 #define MCE_DEFAULT_TX_NUM_WRE				(2048)
 #define MCE_DEFAULT_TX_NUM_WRE_TO_SIGNAL		(64)
 #define MCE_DEFAULT_TX_MAX_INLINE			(204) //+18(always inline ETH header) = 222
@@ -635,7 +596,7 @@ extern mce_sys_var & safe_mce_sys();
 #define MCE_DEFAULT_WAIT_AFTER_JOIN_MSEC		(0)
 #define MCE_DEFAULT_THREAD_MODE				(THREAD_MODE_MULTI)
 #define MCE_DEFAULT_BUFFER_BATCHING_MODE		(BUFFER_BATCHING_WITH_RECLAIM)
-#ifndef VMA_IBV_ACCESS_ALLOCATE_MR
+#ifndef DEFINED_IBV_ACCESS_ALLOCATE_MR
 #define MCE_DEFAULT_MEM_ALLOC_TYPE			(ALLOC_TYPE_HUGEPAGES)
 #else
 #define MCE_DEFAULT_MEM_ALLOC_TYPE			(ALLOC_TYPE_CONTIG)
@@ -662,24 +623,20 @@ extern mce_sys_var & safe_mce_sys();
 #define MCE_MAX_NUM_SGE					(32)
 #define MCE_MIN_RX_NUM_POLLS				(-1)
 #define MCE_MAX_RX_NUM_POLLS				(100000000)
-#define MCE_MIN_RX_PREFETCH_BYTES			(32) /* Just enough for headers (IPoIB+IP+UDP)*/
+#define MCE_MIN_RX_PREFETCH_BYTES			(32)
 #define MCE_MAX_RX_PREFETCH_BYTES			(2044)
 #define MCE_RX_CQ_DRAIN_RATE_DISABLED			(0)
 #define MCE_CQ_DRAIN_INTERVAL_DISABLED			(0)
 #define MCE_CQ_ADAPTIVE_MODERATION_DISABLED		(0)
 #define MCE_MIN_CQ_POLL_BATCH				(1)
 #define MCE_MAX_CQ_POLL_BATCH				(128)
-#define MCE_DEFAULT_IPOIB_FLAG				(1)
 #define MCE_DEFAULT_SOCKETXTREME			(false)
-#ifdef DEFINED_TSO
-#define MCE_DEFAULT_TSO					(true)
-#endif /* DEFINED_TSO */
 #define MCE_DEFAULT_RX_POLL_ON_TX_TCP			(false)
 #define MCE_DEFAULT_TRIGGER_DUMMY_SEND_GETSOCKNAME	(false)
 #define MCE_DEFAULT_DEFERRED_CLOSE (false)
 
 #define MCE_ALIGNMENT					((unsigned long)63)
-#define RX_BUF_SIZE(mtu)				((mtu) + IPOIB_HDR_LEN + GRH_HDR_LEN) // RX buffers are larger in IB
+#define RX_BUF_SIZE(mtu)				((mtu) + ETH_VLAN_HDR_LEN)
 #define TX_BUF_SIZE(mtu)				((mtu) + 92) // Tx buffers are larger in Ethernet (they include L2 for RAW QP)
 #define NUM_TX_WRE_TO_SIGNAL_MAX			64
 #define NUM_RX_WRE_TO_POST_RECV_MAX			1024
@@ -688,7 +645,6 @@ extern mce_sys_var & safe_mce_sys();
 #define IFTYPE_PARAM_FILE				"/sys/class/net/%s/type"
 #define IFADDR_MTU_PARAM_FILE				"/sys/class/net/%s/mtu"
 #define UMCAST_PARAM_FILE				"/sys/class/net/%s/umcast"
-#define IPOIB_MODE_PARAM_FILE				"/sys/class/net/%s/mode"
 #define VERBS_DEVICE_PORT_PARAM_FILE			"/sys/class/net/%s/dev_port"
 #define VERBS_DEVICE_ID_PARAM_FILE			"/sys/class/net/%s/dev_id"
 #define BONDING_MODE_PARAM_FILE				"/sys/class/net/%s/bonding/mode"
@@ -702,9 +658,6 @@ extern mce_sys_var & safe_mce_sys();
 #define L2_ADDR_FILE_FMT                                "/sys/class/net/%.*s/address"
 #define L2_BR_ADDR_FILE_FMT                                   "/sys/class/net/%.*s/broadcast"
 #define OPER_STATE_PARAM_FILE				"/sys/class/net/%s/operstate"
-#if defined(DEFINED_VERBS_VERSION) && (DEFINED_VERBS_VERSION == 2)
-#define RAW_QP_PRIVLIGES_PARAM_FILE			"/sys/module/ib_uverbs/parameters/disable_raw_qp_enforcement"
-#endif /* DEFINED_VERBS_VERSION */
 #define FLOW_STEERING_MGM_ENTRY_SIZE_PARAM_FILE		"/sys/module/mlx4_core/parameters/log_num_mgm_entry_size"
 #define VIRTUAL_DEVICE_FOLDER			"/sys/devices/virtual/net/%s/"
 #define BOND_DEVICE_FILE				"/proc/net/bonding/%s"
