@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2001-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -94,8 +94,8 @@ int get_base_interface_name(const char *if_name, char *base_ifname, size_t sz_ba
 		return 0;
 	}
 
-	unsigned char vlan_if_address[MAX_L2_ADDR_LEN];
-	const size_t ADDR_LEN = get_local_ll_addr(if_name, vlan_if_address, MAX_L2_ADDR_LEN, false);
+	unsigned char vlan_if_address[ETH_ALEN];
+	const size_t ADDR_LEN = get_local_ll_addr(if_name, vlan_if_address, ETH_ALEN, false);
 	if (ADDR_LEN > 0) {
 		struct ifaddrs *ifaddr, *ifa;
 		int rc = getifaddrs(&ifaddr);
@@ -125,12 +125,9 @@ int get_base_interface_name(const char *if_name, char *base_ifname, size_t sz_ba
 
 			unsigned char tmp_mac[ADDR_LEN];
 			if (ADDR_LEN == get_local_ll_addr(ifa->ifa_name, tmp_mac, ADDR_LEN, false)) {
-				int size_to_compare;
-				if (ADDR_LEN == ETH_ALEN) size_to_compare = ETH_ALEN;
-				else size_to_compare = IPOIB_HW_ADDR_GID_LEN;
-				int offset = ADDR_LEN - size_to_compare;
-				if (0 == memcmp(vlan_if_address + offset, tmp_mac + offset, size_to_compare) && 0 == (ifa->ifa_flags & IFF_MASTER)) {
-					// A bond name cannot be a base name of an interface even if both have the same MAC(ethernet) or GID(IB) addresses
+				int offset = ADDR_LEN - ETH_ALEN;
+				if (0 == memcmp(vlan_if_address + offset, tmp_mac + offset, ETH_ALEN) && 0 == (ifa->ifa_flags & IFF_MASTER)) {
+					// A bond name cannot be a base name of an interface even if both have the same MAC(ethernet)
 					snprintf(base_ifname, sz_base_ifname, "%s" ,ifa->ifa_name);
 					freeifaddrs(ifaddr);
 					__log_dbg("Found base_ifname %s for interface %s", base_ifname, if_name);
@@ -145,7 +142,7 @@ int get_base_interface_name(const char *if_name, char *base_ifname, size_t sz_ba
 	return 0;
 }
 
-void print_roce_lag_warnings(char* interface, char* disable_path /* = NULL */, const char* port1 /* = NULL */, const char* port2 /* = NULL */)
+void print_roce_lag_warnings(char* interface, const char* port1 /* = NULL */, const char* port2 /* = NULL */)
 {
 	vlog_printf(VLOG_WARNING,"******************************************************************************************************\n");
 
@@ -158,11 +155,6 @@ void print_roce_lag_warnings(char* interface, char* disable_path /* = NULL */, c
 	}
 
 	vlog_printf(VLOG_WARNING,"* Please refer to VMA Release Notes for more info\n");
-
-	if (disable_path) {
-		vlog_printf(VLOG_WARNING,"* In order to disable RoCE LAG please use:\n");
-		vlog_printf(VLOG_WARNING,"* echo 0 > %s\n", disable_path);
-	}
 	vlog_printf(VLOG_WARNING,"******************************************************************************************************\n");
 }
 
@@ -394,7 +386,7 @@ const char* iphdr_protocol_type_to_str(const int type)
 
 int priv_read_file(const char *path, char *buf, size_t size, vlog_levels_t log_level /*= VLOG_ERROR*/)
 {
-	int len = -1;
+	int len;
 	int fd = open(path, O_RDONLY);
 	BULLSEYE_EXCLUDE_BLOCK_START
 	if (fd < 0) {
@@ -523,7 +515,7 @@ int get_iftype_from_ifname(const char* ifname)
 
 	char iftype_filename[100];
 	char iftype_value_str[32];
-	char base_ifname[32];
+	char base_ifname[32] = {0};
 	int iftype_value = -1;
 
 	get_base_interface_name(ifname, base_ifname, sizeof(base_ifname));
@@ -542,7 +534,7 @@ int get_if_mtu_from_ifname(const char* ifname)
 
 	char if_mtu_len_filename[100];
 	char if_mtu_value_str[32];
-	char base_ifname[32];
+	char base_ifname[32] = {0};
 	int if_mtu_value = 0;
 
 	/* initially try reading MTU from ifname. In case of failure (expected in alias ifnames) - try reading MTU from base ifname */
@@ -579,7 +571,7 @@ int get_window_scaling_factor(int tcp_rmem_max, int core_rmem_max)
 
 int get_ipv4_from_ifname(char *ifname, struct sockaddr_in *addr)
 {
-	int ret = -1;
+	int ret;
 	__log_func("find ip addr for ifname '%s'", ifname);
 
 	int fd = orig_os_api.socket(AF_INET, SOCK_DGRAM, 0);
@@ -758,19 +750,12 @@ size_t get_local_ll_addr(IN const char * ifname, OUT unsigned char* addr, IN int
 	if (addr_len < bytes_len) return 0; // error not enough room was provided by caller
 	BULLSEYE_EXCLUDE_BLOCK_END
 
-	if (bytes_len == IPOIB_HW_ADDR_LEN && addr_len >= IPOIB_HW_ADDR_LEN) { // addr_len >= IPOIB_HW_ADDR_LEN is just for silencing coverity
-		sscanf(buf, IPOIB_HW_ADDR_SSCAN_FMT, IPOIB_HW_ADDR_SSCAN(addr));
-		__log_dbg("found IB %s address " IPOIB_HW_ADDR_PRINT_FMT " for interface %s", is_broadcast?"BR":"UC", IPOIB_HW_ADDR_PRINT_ADDR(addr), ifname);
-	}
-	else if (bytes_len == ETH_ALEN) {
-		sscanf(buf, ETH_HW_ADDR_SSCAN_FMT, ETH_HW_ADDR_SSCAN(addr));
+	if (bytes_len == ETH_ALEN && (0 < sscanf(buf, ETH_HW_ADDR_SSCAN_FMT, ETH_HW_ADDR_SSCAN(addr)))) {
 		__log_dbg("found ETH %s address" ETH_HW_ADDR_PRINT_FMT " for interface %s", is_broadcast?"BR":"UC", ETH_HW_ADDR_PRINT_ADDR(addr), ifname);
-	}
-	else {
-		return 0; // error
+		return bytes_len; // Success
 	}
 
-	return bytes_len; // success
+	return 0; // Failure
 }
 
 bool check_bond_device_exist(const char* ifname)
@@ -823,7 +808,7 @@ out:
 bool get_bond_name(IN const char* ifname, OUT char* bond_name, IN int sz)
 {
 	char upper_path[256];
-	char base_ifname[IFNAMSIZ];
+	char base_ifname[IFNAMSIZ] = {0};
 	get_base_interface_name(ifname, base_ifname, sizeof(base_ifname));
 	struct ifaddrs *ifaddr, *ifa;
 	bool ret = false;
@@ -866,30 +851,10 @@ bool get_bond_active_slave_name(IN const char* bond_name, OUT char* active_slave
 	return true;
 }
 
-bool check_bond_roce_lag_exist(OUT char* bond_roce_lag_path, int sz, IN const char* slave_name)
-{
-#if defined(DEFINED_DIRECT_VERBS) && defined(DEFINED_VERBS_VERSION) && (DEFINED_VERBS_VERSION == 3)
-	NOT_IN_USE(bond_roce_lag_path);
-	NOT_IN_USE(sz);
-	NOT_IN_USE(slave_name);
-	return true;
-#else
-	char sys_res[1024] = {0};
-	snprintf(bond_roce_lag_path, sz, BONDING_ROCE_LAG_FILE, slave_name);
-	if (priv_read_file(bond_roce_lag_path, sys_res, 1024, VLOG_FUNC) > 0) {
-		if (strtol(sys_res, NULL,10) > 0 && errno != ERANGE) {
-			return true;
-		}
-	}
-#endif
-
-	return false;
-}
-
 bool get_netvsc_slave(IN const char* ifname, OUT char* slave_name, OUT unsigned int &slave_flags)
 {
 	char netvsc_path[256];
-	char base_ifname[IFNAMSIZ];
+	char base_ifname[IFNAMSIZ] = {0};
 	get_base_interface_name(ifname, base_ifname, sizeof(base_ifname));
 	struct ifaddrs *ifaddr, *ifa;
 	bool ret = false;
@@ -921,7 +886,7 @@ bool check_netvsc_device_exist(const char* ifname)
 {
 	int ret = -1;
 	char device_path[256] = {0};
-	char base_ifname[IFNAMSIZ];
+	char base_ifname[IFNAMSIZ] = {0};
 	get_base_interface_name(ifname, base_ifname, sizeof(base_ifname));
 	sprintf(device_path, NETVSC_DEVICE_CLASS_FILE, base_ifname);
 	char sys_res[1024] = {0};
@@ -968,9 +933,8 @@ bool check_device_exist(const char* ifname, const char *path)
 {
 	char device_path[256] = {0};
 	int fd = -1;
-	int n = -1;
+	int n = snprintf(device_path, sizeof(device_path), path, ifname);
 
-	n = snprintf(device_path, sizeof(device_path), path, ifname);
 	if (likely((0 < n) && (n < (int)sizeof(device_path)))) {
 		fd = orig_os_api.open(device_path, O_RDONLY);
 		if (fd >= 0) {
@@ -986,17 +950,16 @@ bool check_device_exist(const char* ifname, const char *path)
 
 bool check_device_name_ib_name(const char* ifname, const char* ibname)
 {
-	int n = -1;
-	int fd = -1;
+	int fd;
 	char ib_path[IBV_SYSFS_PATH_MAX]= {0};
 	const char *str_ifname = ifname;
-
+	int n = snprintf(ib_path, sizeof(ib_path), "/sys/class/infiniband/%s/device/net/%s/ifindex",
+	ibname, str_ifname);
 	/* Case #1:
 	 * Direct mapping between if device and ib device
 	 * For example: ens4f1 -> mlx5_3
 	 */
-	n = snprintf(ib_path, sizeof(ib_path), "/sys/class/infiniband/%s/device/net/%s/ifindex",
-			ibname, str_ifname);
+	
 	if (likely((0 < n) && (n < (int)sizeof(ib_path)))) {
 		fd = open(ib_path, O_RDONLY);
 		if (fd >= 0) {
@@ -1005,7 +968,6 @@ bool check_device_name_ib_name(const char* ifname, const char* ibname)
 		}
 	}
 
-#if (defined(DEFINED_DIRECT_VERBS) && defined(DEFINED_VERBS_VERSION) && (DEFINED_VERBS_VERSION == 3))
 	/* Case #2:
 	 * When device is a slave interface
 	 * For example: ens4f1(bondX) -> mlx5_bond_X
@@ -1034,7 +996,6 @@ bool check_device_name_ib_name(const char* ifname, const char* ibname)
 			}
 		}
 	}
-#endif
 
 	return false;
 }
@@ -1052,59 +1013,6 @@ bool get_interface_oper_state(IN const char* interface_name, OUT char* curr_stat
 	return true;
 }
 
-int validate_ipoib_prop(const char* ifname, unsigned int ifflags,
-		const char prop_file[], const char *expected_val,
-		int val_size, OUT char *filename, OUT char* base_ifname)
-{
-	char mode[10];
-	char ifname_tmp[IFNAMSIZ];
-	char active_slave_name[IFNAMSIZ];
-
-	// In case of alias (ib0:xx) take only the device name for that interface (ib0)
-	strncpy(ifname_tmp, ifname, sizeof(ifname_tmp) - 1);
-	ifname_tmp[sizeof(ifname_tmp) - 1] = '\0';
-	base_ifname = strtok(ifname_tmp, ":");
-
-	if (ifflags & IFF_MASTER) {
-		// this is a bond interface, let find the slave
-		BULLSEYE_EXCLUDE_BLOCK_START
-		if (!get_bond_active_slave_name(base_ifname, active_slave_name, IFNAMSIZ)) {
-			return -1;
-		}
-		BULLSEYE_EXCLUDE_BLOCK_END
-		sprintf(filename, prop_file, active_slave_name);
-	} else {
-		sprintf(filename, prop_file, base_ifname);
-	}
-
-	BULLSEYE_EXCLUDE_BLOCK_START
-	if (priv_read_file(filename, mode, val_size) <= 0) {
-		return -1;
-	}
-	BULLSEYE_EXCLUDE_BLOCK_END
-	if (strncmp(mode, expected_val, val_size)) {
-		return 1;
-	} else {
-		return 0;
-	}
-}
-
-#if defined(DEFINED_VERBS_VERSION) && (DEFINED_VERBS_VERSION == 2)
-//NOTE RAW_QP_PRIVLIGES_PARAM_FILE does not exist on upstream drivers
-int validate_raw_qp_privliges()
-{
-	// RAW_QP_PRIVLIGES_PARAM_FILE: "/sys/module/ib_uverbs/parameters/disable_raw_qp_enforcement"
-	char raw_qp_privliges_value = 0;
-	if (priv_read_file((const char*)RAW_QP_PRIVLIGES_PARAM_FILE, &raw_qp_privliges_value, 1, VLOG_DEBUG) <= 0) {
-		return -1;
-	}
-	if (raw_qp_privliges_value != '1') {
-		return 0;
-	}
-	return 1;
-}
-#endif /* DEFINED_VERBS_VERSION */
-
 bool validate_user_has_cap_net_raw_privliges()
 {
 #ifdef HAVE_SYS_CAPABILITY_H
@@ -1156,38 +1064,6 @@ size_t default_huge_page_size(void)
 	return hugepage_sz;
 }
 
-int validate_tso(int if_index)
-{
-#ifdef HAVE_LINUX_ETHTOOL_H
-	int ret = -1;
-	int fd = -1;
-	struct ifreq req;
-	struct ethtool_value eval;
-
- 	fd = orig_os_api.socket(AF_INET, SOCK_DGRAM, 0);
-	if (fd < 0) {
-		__log_err("ERROR from socket() (errno=%d %m)", errno);
-		return -1;
-	}
- 	memset(&req, 0, sizeof(req));
-	eval.cmd = ETHTOOL_GTSO;
-	req.ifr_ifindex = if_index;
-	if_indextoname(if_index, req.ifr_name);
-	req.ifr_data = (char *)&eval;
-	ret = orig_os_api.ioctl(fd, SIOCETHTOOL, &req);
-	if (ret < 0) {
-		__log_dbg("ioctl(SIOCETHTOOL) cmd=ETHTOOL_GTSO (errno=%d %m)", errno);
-	} else {
-		ret = eval.data;
-	}
- 	orig_os_api.close(fd);
-	return ret;
-#else
-	NOT_IN_USE(if_index);
-	return -1;
-#endif
-}
-
 loops_timer::loops_timer()
 	: m_start()
 	, m_elapsed(milliseconds(0))
