@@ -146,6 +146,13 @@ int main(int argc, char *argv[])
 		goto free_mem;
 	}
 
+	/* Negotiate parameters. */
+	if (negotiate_params(&ctx, &user_comm, &user_param)) {
+		fprintf(stderr, " Failed to negotiate parameters\n");
+		dealloc_ctx(&ctx, &user_param);
+		goto free_mem;
+	}
+
 	/* Create RDMA CM resources and connect through CM. */
 	if (user_param.work_rdma_cm == ON) {
 		rc = create_rdma_cm_connection(&ctx, &user_param, &user_comm,
@@ -222,6 +229,17 @@ int main(int argc, char *argv[])
 		ctx_print_pingpong_data(&rem_dest[i],&user_comm);
 	}
 
+	if (user_param.use_event) {
+		if (ibv_req_notify_cq(ctx.send_cq, 0)) {
+			fprintf(stderr, " Couldn't request CQ notification\n");
+			goto destroy_context;
+		}
+		if (ibv_req_notify_cq(ctx.recv_cq, 0)) {
+			fprintf(stderr, " Couldn't request CQ notification\n");
+			goto destroy_context;
+		}
+	}
+
 	/* An additional handshake is required after moving qp to RTR. */
 	if (ctx_hand_shake(&user_comm,&my_dest[0],&rem_dest[0])) {
 		fprintf(stderr," Failed to exchange data between server and clients\n");
@@ -246,7 +264,7 @@ int main(int argc, char *argv[])
 
 		if (ctx_hand_shake(&user_comm,&my_dest[0],&rem_dest[0])) {
 			fprintf(stderr," Failed to exchange data between server and clients\n");
-			goto free_mem;
+			goto destroy_context;
 		}
 
 		xchg_bw_reports(&user_comm, &my_bw_rep,&rem_bw_rep,atof(user_param.rem_version));
@@ -259,7 +277,7 @@ int main(int argc, char *argv[])
 
 		if (ctx_close_connection(&user_comm,&my_dest[0],&rem_dest[0])) {
 			fprintf(stderr,"Failed to close connection between server and client\n");
-			goto free_mem;
+			goto destroy_context;
 		}
 
 		if (user_param.output == FULL_VERBOSITY) {
@@ -312,7 +330,7 @@ int main(int argc, char *argv[])
 			    (user_param.machine == SERVER || user_param.duplex)) {
 				if (ctx_set_recv_wqes(&ctx,&user_param)) {
 					fprintf(stderr," Failed to post receive recv_wqes\n");
-					goto free_mem;
+					goto destroy_context;
 				}
 			}
 
@@ -323,14 +341,14 @@ int main(int argc, char *argv[])
 					fprintf(stderr, "Skipping\n");
 				} else if(perform_warm_up(&ctx, &user_param)) {
 					fprintf(stderr, "Problems with warm up\n");
-					goto free_mem;
+					goto destroy_context;
 				}
 			}
 
 			if(user_param.duplex || user_param.verb == WRITE_IMM) {
 				if (ctx_hand_shake(&user_comm,&my_dest[0],&rem_dest[0])) {
 					fprintf(stderr,"Failed to sync between server and client between different msg sizes\n");
-					goto free_mem;
+					goto destroy_context;
 				}
 			}
 
@@ -338,28 +356,28 @@ int main(int argc, char *argv[])
 
 				if(run_iter_bi(&ctx,&user_param)){
 					fprintf(stderr," Failed to complete run_iter_bi function successfully\n");
-					goto free_mem;
+					goto destroy_context;
 				}
 
 			} else if (user_param.machine == CLIENT || user_param.verb != WRITE_IMM) {
 
 				if(run_iter_bw(&ctx,&user_param)) {
 					fprintf(stderr," Failed to complete run_iter_bw function successfully\n");
-					goto free_mem;
+					goto destroy_context;
 				}
 
 			} else if (user_param.machine == SERVER) {
 
 				if(run_iter_bw_server(&ctx,&user_param)) {
 					fprintf(stderr," Failed to complete run_iter_bw_server function successfully\n");
-					goto free_mem;
+					goto destroy_context;
 				}
 			}
 
 			if (user_param.verb == WRITE_IMM || (user_param.duplex && (atof(user_param.version) >= 4.6))) {
 				if (ctx_hand_shake(&user_comm,&my_dest[0],&rem_dest[0])) {
 					fprintf(stderr,"Failed to sync between server and client between different msg sizes\n");
-					goto free_mem;
+					goto destroy_context;
 				}
 			}
 
@@ -380,7 +398,7 @@ int main(int argc, char *argv[])
 			    (user_param.machine == SERVER || user_param.duplex)) {
 			if (ctx_set_recv_wqes(&ctx,&user_param)) {
 				fprintf(stderr," Failed to post receive recv_wqes\n");
-				goto free_mem;
+				goto destroy_context;
 			}
 		}
 
@@ -389,7 +407,7 @@ int main(int argc, char *argv[])
 			if (user_param.perform_warm_up) {
 				if(perform_warm_up(&ctx, &user_param)) {
 					fprintf(stderr, "Problems with warm up\n");
-					goto free_mem;
+					goto destroy_context;
 				}
 			}
 		}
@@ -397,7 +415,7 @@ int main(int argc, char *argv[])
 		if(user_param.duplex || user_param.verb == WRITE_IMM) {
 			if (ctx_hand_shake(&user_comm,&my_dest[0],&rem_dest[0])) {
 				fprintf(stderr,"Failed to sync between server and client between different msg sizes\n");
-				goto free_mem;
+				goto destroy_context;
 			}
 		}
 
@@ -405,21 +423,21 @@ int main(int argc, char *argv[])
 
 			if(run_iter_bi(&ctx,&user_param)){
 				fprintf(stderr," Failed to complete run_iter_bi function successfully\n");
-				goto free_mem;
+				goto destroy_context;
 			}
 
 		} else if (user_param.machine == CLIENT || user_param.verb != WRITE_IMM) {
 
 			if(run_iter_bw(&ctx,&user_param)) {
 				fprintf(stderr," Failed to complete run_iter_bw function successfully\n");
-				goto free_mem;
+				goto destroy_context;
 			}
 
 		} else if (user_param.machine == SERVER) {
 
 			if(run_iter_bw_server(&ctx,&user_param)) {
 				fprintf(stderr," Failed to complete run_iter_bw_server function successfully\n");
-				goto free_mem;
+				goto destroy_context;
 			}
 		}
 
@@ -453,26 +471,26 @@ int main(int argc, char *argv[])
 		else if (user_param.machine == SERVER && user_param.verb == WRITE_IMM) {
 			if (ctx_set_recv_wqes(&ctx,&user_param)) {
 				fprintf(stderr," Failed to post receive recv_wqes\n");
-				goto free_mem;
+				goto destroy_context;
 			}
 		}
 
 		if (user_param.verb == WRITE_IMM) {
 			if (ctx_hand_shake(&user_comm,&my_dest[0],&rem_dest[0])) {
 				fprintf(stderr,"Failed to exchange data between server and clients\n");
-				goto free_mem;
+				goto destroy_context;
 			}
 		}
 
 		if (user_param.machine == CLIENT || user_param.verb == WRITE) {
 			if(run_iter_bw_infinitely(&ctx,&user_param)) {
 				fprintf(stderr," Error occurred while running infinitely! aborting ...\n");
-				goto free_mem;
+				goto destroy_context;
 			}
 		} else if (user_param.machine == SERVER && user_param.verb == WRITE_IMM) {
 			if(run_iter_bw_infinitely_server(&ctx,&user_param)) {
 				fprintf(stderr," Error occurred while running infinitely on server! aborting ...\n");
-				goto free_mem;
+				goto destroy_context;
 			}
 		}
 	}
@@ -488,7 +506,7 @@ int main(int argc, char *argv[])
 	if (user_param.machine == CLIENT && user_param.verb == WRITE && !user_param.duplex) {
 		if (ctx_hand_shake(&user_comm,&my_dest[0],&rem_dest[0])) {
 			fprintf(stderr," Failed to exchange data between server and clients\n");
-			goto free_mem;
+			goto destroy_context;
 		}
 
 		xchg_bw_reports(&user_comm, &my_bw_rep,&rem_bw_rep,atof(user_param.rem_version));
@@ -497,7 +515,7 @@ int main(int argc, char *argv[])
 	/* Closing connection. */
 	if (ctx_close_connection(&user_comm,&my_dest[0],&rem_dest[0])) {
 		fprintf(stderr,"Failed to close connection between server and client\n");
-		goto free_mem;
+		goto destroy_context;
 	}
 
 	if (!user_param.is_bw_limit_passed && (user_param.is_limit_bw == ON ) ) {
