@@ -288,8 +288,6 @@ nvmet_rdma_get_rsp(struct nvmet_rdma_queue *queue)
 
 #ifdef HAVE_SBITMAP_GET_1_PARAM
 	tag = sbitmap_get(&queue->rsp_tags);
-#elif defined(HAVE_SBITMAP_GET_2_PARAMS)
-	tag = sbitmap_get(&queue->rsp_tags, true);
 #else
 	tag = sbitmap_get(&queue->rsp_tags, true, false);
 #endif
@@ -531,9 +529,6 @@ nvmet_rdma_alloc_rsps(struct nvmet_rdma_queue *queue)
 #ifdef HAVE_SBITMAP_GET_1_PARAM
 	if (sbitmap_init_node(&queue->rsp_tags, nr_rsps, -1, GFP_KERNEL,
 			NUMA_NO_NODE, false, true))
-#elif defined(HAVE_SBITMAP_GET_2_PARAMS)
-	if (sbitmap_init_node(&queue->rsp_tags, nr_rsps, -1, GFP_KERNEL,
-			NUMA_NO_NODE, false))
 #else
 	if (sbitmap_init_node(&queue->rsp_tags, nr_rsps, -1, GFP_KERNEL,
 			NUMA_NO_NODE))
@@ -1063,8 +1058,7 @@ static void nvmet_rdma_handle_command(struct nvmet_rdma_queue *queue,
 		cmd->send_sge.addr, cmd->send_sge.length,
 		DMA_TO_DEVICE);
 
-	if (!nvmet_req_init(&cmd->req, &queue->nvme_cq,
-			&queue->nvme_sq, &nvmet_rdma_ops))
+	if (!nvmet_req_init(&cmd->req, &queue->nvme_sq, &nvmet_rdma_ops))
 		return;
 
 	status = nvmet_rdma_map_sgl(cmd);
@@ -1485,6 +1479,7 @@ static void nvmet_rdma_free_queue(struct nvmet_rdma_queue *queue,
 
 	if (drain_queue)
 		nvmet_sq_destroy(&queue->nvme_sq);
+	nvmet_cq_put(&queue->nvme_cq);
 
 	nvmet_rdma_destroy_queue_ib(queue, drain_queue);
 	if (!queue->nsrq && !queue->offload) {
@@ -1494,11 +1489,7 @@ static void nvmet_rdma_free_queue(struct nvmet_rdma_queue *queue,
 	}
 	nvmet_rdma_free_rsps(queue);
 	cancel_work_sync(&queue->disconnect_work);
-#ifdef HAVE_IDA_FREE
 	ida_free(&nvmet_rdma_queue_ida, queue->idx);
-#else
-	ida_simple_remove(&nvmet_rdma_queue_ida, queue->idx);
-#endif
 	kfree(queue);
 }
 
@@ -1620,7 +1611,8 @@ nvmet_rdma_alloc_queue(struct nvmet_rdma_device *ndev,
 		goto out_reject;
 	}
 
-	ret = nvmet_sq_init(&queue->nvme_sq);
+	nvmet_cq_init(&queue->nvme_cq);
+	ret = nvmet_sq_init(&queue->nvme_sq, &queue->nvme_cq);
 	if (ret) {
 		ret = NVME_RDMA_CM_NO_RSC;
 		goto out_free_queue;
@@ -1651,11 +1643,7 @@ nvmet_rdma_alloc_queue(struct nvmet_rdma_device *ndev,
 	spin_lock_init(&queue->rsp_wr_wait_lock);
 	INIT_LIST_HEAD(&queue->queue_list);
 
-#ifdef HAVE_IDA_ALLOC
 	queue->idx = ida_alloc(&nvmet_rdma_queue_ida, GFP_KERNEL);
-#else
-	queue->idx = ida_simple_get(&nvmet_rdma_queue_ida, 0, 0, GFP_KERNEL);
-#endif
 	if (queue->idx < 0) {
 		ret = NVME_RDMA_CM_NO_RSC;
 		goto out_destroy_sq;
@@ -1706,14 +1694,11 @@ out_free_cmds:
 out_free_responses:
 	nvmet_rdma_free_rsps(queue);
 out_ida_remove:
-#ifdef HAVE_IDA_FREE
 	ida_free(&nvmet_rdma_queue_ida, queue->idx);
-#else
-	ida_simple_remove(&nvmet_rdma_queue_ida, queue->idx);
-#endif
 out_destroy_sq:
 	nvmet_sq_destroy(&queue->nvme_sq);
 out_free_queue:
+	nvmet_cq_put(&queue->nvme_cq);
 	kfree(queue);
 out_reject:
 	nvmet_rdma_cm_reject(cm_id, ret);
@@ -2205,7 +2190,7 @@ static void nvmet_rdma_disc_port_addr(struct nvmet_req *req,
 	if (inet_addr_is_any(&cm_id->route.addr.src_addr)) {
 #else
 	/* Older kernels: cast required from sockaddr_storage to sockaddr */
-	if (inet_addr_is_any((struct sockaddr *)&cm_id->route.addr.src_addr)) {
+ 	if (inet_addr_is_any((struct sockaddr *)&cm_id->route.addr.src_addr)) {
 #endif
 		struct nvmet_rdma_rsp *rsp =
 			container_of(req, struct nvmet_rdma_rsp, req);
@@ -2437,9 +2422,6 @@ module_exit(nvmet_rdma_exit);
 
 MODULE_DESCRIPTION("NVMe target RDMA transport driver");
 MODULE_LICENSE("GPL v2");
-#ifdef RETPOLINE_MLNX
-MODULE_INFO(retpoline, "Y");
-#endif
 MODULE_ALIAS("nvmet-transport-1"); /* 1 == NVMF_TRTYPE_RDMA */
 
 #include "rdma_offload.c"
