@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -79,9 +79,8 @@ protected:
 
     sockaddr_store_t client_addr;
     sockaddr_store_t server_addr;
-    sockaddr_store_t remote_addr;
+    sockaddr_store_t remote_unreachable_addr;
     sockaddr_store_t bogus_addr;
-    bool def_gw_exists;
     static uint16_t m_port;
     static int m_family;
 
