@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -110,13 +110,11 @@ TEST_F(udp_bind, ti_2)
     int rc = EOK;
     int fd;
 
-    SKIP_TRUE(def_gw_exists, "No Default Gateway");
-
     fd = udp_base::sock_create();
     ASSERT_LE(0, fd);
 
     errno = EOK;
-    rc = bind(fd, (struct sockaddr *)&remote_addr, sizeof(remote_addr));
+    rc = bind(fd, (struct sockaddr *)&remote_unreachable_addr, sizeof(remote_unreachable_addr));
     EXPECT_EQ(EADDRNOTAVAIL, errno);
     EXPECT_GT(0, rc);
 
@@ -472,6 +470,13 @@ public:
         int fd = socket(AF_INET6, SOCK_DGRAM, IPPROTO_IP);
         EXPECT_GT(fd, 0) << "Socket failed to open";
 
+        /* Set receive timeout to prevent indefinite blocking */
+        struct timeval timeout;
+        timeout.tv_sec = 10;
+        timeout.tv_usec = 0;
+        EXPECT_EQ(setsockopt(fd, SOL_SOCKET, SO_RCVTIMEO, &timeout, sizeof(timeout)), 0)
+            << "Failed to set receive timeout";
+
         sockaddr_store_t server_any_sockaddr = {
             .addr6 = {
                 .sin6_family = AF_INET6,
@@ -531,7 +536,11 @@ TEST_F(pktinfo, check_recvmsg_returns_expected_pktinfo)
                 &client_addr.addr, sizeof(client_addr.addr), &vec, 1U, cbuf, sizeof(cbuf), 0
             };
 
-            ASSERT_GT(recvmsg(fd, &msg, 0), 0) << "Failed to receive the msg";
+            ssize_t ret;
+            do {
+                ret = recvmsg(fd, &msg, 0);
+            } while (ret < 0 && errno == EINTR);
+            ASSERT_GT(ret, 0) << "Failed to receive the msg, errno=" << errno;
 
             struct cmsghdr *cmsg = CMSG_FIRSTHDR(&msg);
 
@@ -549,5 +558,8 @@ TEST_F(pktinfo, check_recvmsg_returns_expected_pktinfo)
         };
 
         server_func(pid, server_code_under_test);
+
+        /* Wait for child process to complete and validate exit status */
+        ASSERT_EQ(0, wait_fork(pid));
     }
 }
