@@ -950,8 +950,8 @@ benchmark_thread_worker(void *aux_)
                 }
                 break;
             default:
-                fprintf(stderr, "[%u]: Reached step %s\n",
-                        tid, step_names[step]);
+                fprintf(stderr, "[%u]: Reached step %d\n",
+                        tid, step);
                 OVS_NOT_REACHED();
                 break;
             }
