@@ -1,6 +1,6 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
@@ -59,7 +59,7 @@ int proc_loop(void)
     while ((0 == daemon_cfg.sig) && (errno != EINTR)) {
         fd_set readfds;
         struct timeval tv;
-        int max_fd = -1;
+        int max_fd;
 
         FD_ZERO(&readfds);
         FD_SET(daemon_cfg.sock_fd, &readfds);
