@@ -153,27 +153,25 @@ static struct ctl_table xr_tunables_table[] = {
 		.mode		= 0644,
 		.proc_handler	= proc_dointvec,
 	},
+#ifdef HAVE_CTL_TABLE_CHILD
 	{ },
+#endif
 };
 
 #ifdef HAVE_CTL_TABLE_CHILD
 static struct ctl_table sunrpc_table[] = {
 	{
-		.procname	= "sunrpc",
-		.mode		= 0555,
-		.child		= xr_tunables_table
+		.procname       = "sunrpc",
+		.mode           = 0555,
+		.child          = xr_tunables_table
 	},
-	{ },
-};
+	{ }, 
+};      
 #endif
 
 #endif
 
-#ifdef HAVE_RPC_XPRT_OPS_CONST
 static const struct rpc_xprt_ops xprt_rdma_procs;
-#else
-static struct rpc_xprt_ops xprt_rdma_procs;
-#endif
 
 static void
 xprt_rdma_format_addresses4(struct rpc_xprt *xprt, struct sockaddr *sap)
@@ -359,14 +357,8 @@ xprt_setup_rdma(struct xprt_create *args)
 	if (!try_module_get(THIS_MODULE))
 		return ERR_PTR(-EIO);
 
-#ifdef HAVE_RPC_XPRT_OPS_FREE_SLOT
 	xprt = xprt_alloc(args->net, sizeof(struct rpcrdma_xprt), 0,
 			  xprt_rdma_slot_table_entries);
-#else
-	xprt = xprt_alloc(args->net, sizeof(struct rpcrdma_xprt),
-			xprt_rdma_slot_table_entries,
-			xprt_rdma_slot_table_entries);
-#endif
 	if (!xprt) {
 		module_put(THIS_MODULE);
 		return ERR_PTR(-ENOMEM);
@@ -570,7 +562,6 @@ xprt_rdma_connect(struct rpc_xprt *xprt, struct rpc_task *task)
 #endif
 }
 
-#ifdef HAVE_RPC_XPRT_OPS_FREE_SLOT
 /**
  * xprt_rdma_alloc_slot - allocate an rpc_rqst
  * @xprt: controlling RPC transport
@@ -628,7 +619,6 @@ xprt_rdma_free_slot(struct rpc_xprt *xprt, struct rpc_rqst *rqst)
 		clear_bit(XPRT_CONGESTED, &xprt->state);
 #endif
 }
-#endif
 
 static bool rpcrdma_check_regbuf(struct rpcrdma_xprt *r_xprt,
 				 struct rpcrdma_regbuf *rb, size_t size,
@@ -656,11 +646,7 @@ xprt_rdma_allocate(struct rpc_task *task)
 {
 	struct rpc_rqst *rqst = task->tk_rqstp;
 	struct rpcrdma_xprt *r_xprt = rpcx_to_rdmax(rqst->rq_xprt);
-#ifdef HAVE_RPC_XPRT_OPS_FREE_SLOT
 	struct rpcrdma_req *req = rpcr_to_rdmar(rqst);
-#else
-	struct rpcrdma_req *req;
-#endif
 #ifdef HAVE_RPC_TASK_GPF_MASK_EXPORTED
 	gfp_t flags = rpc_task_gfp_mask();
 #else
@@ -675,12 +661,6 @@ xprt_rdma_allocate(struct rpc_task *task)
 #endif
 #endif
 
-#ifndef HAVE_RPC_XPRT_OPS_FREE_SLOT
-	req = rpcrdma_buffer_get(&r_xprt->rx_buf);
-	if (req == NULL)
-		goto out_get;
-#endif
-
 	if (!rpcrdma_check_regbuf(r_xprt, req->rl_sendbuf, rqst->rq_callsize,
 				  flags))
 		goto out_fail;
@@ -688,19 +668,11 @@ xprt_rdma_allocate(struct rpc_task *task)
 				  flags))
 		goto out_fail;
 
-#ifndef HAVE_RPC_XPRT_OPS_FREE_SLOT
-	rpcrdma_set_xprtdata(rqst, req);
-#endif
-
 	rqst->rq_buffer = rdmab_data(req->rl_sendbuf);
 	rqst->rq_rbuffer = rdmab_data(req->rl_recvbuf);
 	return 0;
 
 out_fail:
-#ifndef HAVE_RPC_XPRT_OPS_FREE_SLOT
-	rpcrdma_buffer_put(&r_xprt->rx_buf, req);
-out_get:
-#endif
 	return -ENOMEM;
 }
 
@@ -714,15 +686,8 @@ static void
 xprt_rdma_free(struct rpc_task *task)
 {
 	struct rpc_rqst *rqst = task->tk_rqstp;
-#if !defined(HAVE_XPRT_PIN_RQST) || !defined(HAVE_RPC_XPRT_OPS_FREE_SLOT)
-	struct rpcrdma_xprt *r_xprt = rpcx_to_rdmax(rqst->rq_xprt);
-#endif
 	struct rpcrdma_req *req = rpcr_to_rdmar(rqst);
 
-#ifndef HAVE_XPRT_PIN_RQST
-	rpcrdma_remove_req(&r_xprt->rx_buf, req);
-#endif
-
 	if (unlikely(!list_empty(&req->rl_registered))) {
 #ifdef HAVE_TRACE_RPCRDMA_H
 		trace_xprtrdma_mrs_zap(task);
@@ -730,9 +695,6 @@ xprt_rdma_free(struct rpc_task *task)
 		frwr_unmap_sync(rpcx_to_rdmax(rqst->rq_xprt), req);
 	}
 
-#ifndef HAVE_RPC_XPRT_OPS_FREE_SLOT
-	rpcrdma_buffer_put(&r_xprt->rx_buf, req);
-#endif
 	/* XXX: If the RPC is completing because of a signal and
 	 * not because a reply was received, we ought to ensure
 	 * that the Send completion has fired, so that memory
@@ -797,13 +759,11 @@ xprt_rdma_send_request(struct rpc_task *task)
 
 	rqst->rq_xmit_bytes_sent += rqst->rq_snd_buf.len;
 
-#ifdef HAVE_RPC_REPLY_EXPECTED
 	/* An RPC with no reply will throw off credit accounting,
 	 * so drop the connection to reset the credit grant.
 	 */
 	if (!rpc_reply_expected(rqst->rq_task))
 		goto drop_connection;
-#endif
 	return 0;
 
 failed_marshal:
@@ -869,19 +829,11 @@ xprt_rdma_disable_swap(struct rpc_xprt *xprt)
 /*
  * Plumbing for rpc transport switch and kernel module
  */
-#ifdef HAVE_RPC_XPRT_OPS_CONST
 static const struct rpc_xprt_ops xprt_rdma_procs = {
-#else
-static struct rpc_xprt_ops xprt_rdma_procs = {
-#endif
 	.reserve_xprt		= xprt_reserve_xprt_cong,
 	.release_xprt		= xprt_release_xprt_cong, /* sunrpc/xprt.c */
-#ifdef HAVE_RPC_XPRT_OPS_FREE_SLOT
 	.alloc_slot		= xprt_rdma_alloc_slot,
 	.free_slot		= xprt_rdma_free_slot,
-#else
-	.alloc_slot		= xprt_alloc_slot,
-#endif
 	.release_request	= xprt_release_rqst_cong,       /* ditto */
 #ifdef HAVE_RPC_XPRT_OPS_SET_RETRANS_TIMEOUT
 	.set_retrans_timeout	= xprt_set_retrans_timeout_def, /* ditto */
