@@ -142,7 +142,7 @@ def open_spell_check_dict():
                           'shlibs', 'runtime', 'uflow', 'groupid', 'userid',
                           'bootc', 'cirrusci', 'punix', 'ptcp', 'globals',
                           'quiescing', 'wrt', 'connmgr', 'userdata', 'proto',
-                          'ttl',
+                          'ttl', 'ethdev', 'arborescence',
                           ]
 
         global spell_check_dict
@@ -277,7 +277,7 @@ skip_committer_signoff_check = False
 #
 # Python isn't checked as flake8 performs these checks during build.
 line_length_ignore_list = re.compile(
-    r'\.(am|at|etc|in|m4|mk|patch|py|yml)$|^debian/.*$')
+    r'\.(am|at|etc|in|m4|mk|patch|py|rst|yml)$|^debian/.*$')
 
 # Don't enforce a requirement that leading whitespace be all spaces on
 # files that include these characters in their name, since these kinds
