@@ -942,12 +942,27 @@ static struct virtio_fs_req *virtio_fs_req_get(struct virtio_fs_vq *fsvq,
 	struct virtio_fs_req *fs_req;
 
 	spin_lock(&fsvq->lock);
-	if (!force && !fsvq->initialized) {
-		reinit_completion(&fsvq->init_comp);
-		spin_unlock(&fsvq->lock);
-		wait_for_completion(&fsvq->init_comp);
-		spin_lock(&fsvq->lock);
+	if (!force) {
+		if (!fsvq->initialized) {
+			reinit_completion(&fsvq->init_comp);
+			spin_unlock(&fsvq->lock);
+			wait_for_completion(&fsvq->init_comp);
+			spin_lock(&fsvq->lock);
+		}
+		/* Matches smp_wmb() in fuse_set_initialized() */
+		smp_rmb();
+		if (fm && fm->fc) {
+			if (!fm->fc->connected) {
+				spin_unlock(&fsvq->lock);
+				return ERR_PTR(-ENOTCONN);
+			}
+			if (fm->fc->conn_error) {
+				spin_unlock(&fsvq->lock);
+				return ERR_PTR(-ECONNREFUSED);
+			}
+		}
 	}
+
 	fs_req = list_first_entry_or_null(&fsvq->free_reqs,
 					  struct virtio_fs_req, entry);
 	if (likely(fs_req)) {
@@ -1704,9 +1719,10 @@ static long virtio_fs_direct_access(struct dax_device *dax_dev, pgoff_t pgoff,
 #else
 		/* Older kernels: use pfn_t with flags */
 		*pfn = phys_to_pfn_t(fs->window_phys_addr + offset,
-				     PFN_DEV | PFN_MAP);
+					PFN_DEV | PFN_MAP);
 #endif
 	}
+
 	return nr_pages > max_nr_pages ? max_nr_pages : nr_pages;
 }
 
@@ -2359,8 +2375,8 @@ static void virtio_fs_notify_send_req(struct virtio_fs *fs)
 	struct virtio_fs_req *fs_req;
 
 	fs_req = virtio_fs_req_get(&fs->vqs[VQ_NOTIFY], NULL, GFP_KERNEL, true);
-	if (!fs_req) {
-		pr_err("virtio-fs: cannot get notify request\n");
+	if (IS_ERR_OR_NULL(fs_req)) {
+		pr_err("virtio-fs: cannot get notify request: %ld\n", PTR_ERR(fs_req));
 		return;
 	}
 	if (fs_req->allocated) {
@@ -2539,8 +2555,8 @@ static int virtio_fs_simple_background(struct fuse_mount *fm,
 	fsvq = &fs->vqs[queue_id];
 
 	fs_req = virtio_fs_req_get(fsvq, fm, gfp_flags, args->force);
-	if (!fs_req)
-		return -ENOMEM;
+	if (IS_ERR_OR_NULL(fs_req))
+		return fs_req ? PTR_ERR(fs_req) : -ENOMEM;
 
 	req = &fs_req->req;
 	if (args->force) {
@@ -2584,8 +2600,8 @@ static ssize_t virtio_fs_simple_request(struct fuse_mount *fm,
 		gfp_flags |= __GFP_NOFAIL;
 
 	fs_req = virtio_fs_req_get(fsvq, fm, gfp_flags, args->force);
-	if (!fs_req)
-		return -ENOMEM;
+	if (IS_ERR_OR_NULL(fs_req))
+		return fs_req ? PTR_ERR(fs_req) : -ENOMEM;
 
 	req = &fs_req->req;
 	if (args->force) {
@@ -2630,8 +2646,8 @@ static int __virtio_fs_simple_notify_reply(struct virtio_fs *fs, struct fuse_mou
 	fsvq = &fs->vqs[queue_id];
 
 	fs_req = virtio_fs_req_get(fsvq, fm, GFP_KERNEL, args->force);
-	if (!fs_req)
-		return -ENOMEM;
+	if (IS_ERR_OR_NULL(fs_req))
+		return fs_req ? PTR_ERR(fs_req) : -ENOMEM;
 
 	req = &fs_req->req;
 	if (!args->nocreds) {
