@@ -287,7 +287,7 @@ ndu_get_dpif(void)
     cfg = ovsrec_open_vswitch_first(ndu_ctx.idl);
     br = cfg->bridges;
     for (i = 0; i < cfg->n_bridges; i++) {
-        if (!strcmp(br[i]->datapath_type, NDU_DPIF_TYPE)) {
+        if (!strcmp(ofproto_normalize_type(br[i]->datapath_type), NDU_DPIF_TYPE)) {
             dpif = ofproto_dpif_lookup_by_name(br[i]->name);
             break;
         }
